<?php

class CertificadoAgendamentosCalendarForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'CertificadoAgendamentos';
    private static $primaryKey = 'id';
    private static $formName = 'form_CertificadoAgendamentosCalendarForm';
    private static $startDateField = 'data_hora_inicial';
    private static $endDateField = 'data_hora_final';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Agendamentos de Certificado");

        $view = new THidden('view');

        $criteria_certificado_venda_id = new TCriteria();

        $filterVar = CertificadoVendaSituacao::AguardandoAtendimento;
        $criteria_certificado_venda_id->add(new TFilter('certificado_venda_situacao_id', '=', $filterVar)); 

        // $criteria_certificado_venda_id = new TCriteria();

        $criteria_certificado_venda_id->add(new TFilter('cliente_id', 'in', "(SELECT id FROM cliente WHERE deletado_em is NULL )")); 

        $id = new TEntry('id');
        $certificado_venda_id = new TDBCombo('certificado_venda_id', 'kraftsoft', 'CertificadoVenda', 'id', 'id:{id} - {cliente->nome}  - {produto->nome}','id asc' , $criteria_certificado_venda_id );
        $titulo = new TEntry('titulo');
        $data_hora_inicial = new TDateTime('data_hora_inicial');
        $data_hora_final = new TDateTime('data_hora_final');
        $cor = new TColor('cor');
        $observacao = new TText('observacao');
        $criado_em = new TDateTime('criado_em');
        $criado_por = new TEntry('criado_por');
        $alterado_em = new TDateTime('alterado_em');
        $alterado_por = new TEntry('alterado_por');

        $certificado_venda_id->setChangeAction(new TAction([$this,'onExitVendaCertificado']));

        $certificado_venda_id->addValidation("Certificado venda id", new TRequiredValidator()); 
        $titulo->addValidation("Titulo", new TRequiredValidator()); 
        $data_hora_inicial->addValidation("Data Hora Inicial", new TRequiredValidator()); 
        $data_hora_final->addValidation("Data Hora Final", new TRequiredValidator()); 
        $cor->addValidation("Cor", new TRequiredValidator()); 

        $certificado_venda_id->enableSearch();
        $cor->setValue('#2196F3');
        $criado_em->setMask('dd/mm/yyyy hh:ii');
        $alterado_em->setMask('dd/mm/yyyy hh:ii');
        $data_hora_final->setMask('dd/mm/yyyy hh:ii');
        $data_hora_inicial->setMask('dd/mm/yyyy hh:ii');

        $criado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $alterado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_hora_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_hora_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');

        $id->setEditable(false);
        $titulo->setEditable(false);
        $criado_em->setEditable(false);
        $criado_por->setEditable(false);
        $alterado_em->setEditable(false);
        $alterado_por->setEditable(false);

        $id->setSize(100);
        $cor->setSize('100%');
        $titulo->setSize('100%');
        $criado_em->setSize(150);
        $alterado_em->setSize(150);
        $criado_por->setSize('100%');
        $alterado_por->setSize('100%');
        $observacao->setSize('100%', 70);
        $data_hora_final->setSize('100%');
        $data_hora_inicial->setSize('100%');
        $certificado_venda_id->setSize('100%');


        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Venda do Certificado:", '#ff0000', '14px', null, '100%'),$certificado_venda_id]);
        $row1->layout = [' col-sm-2',' col-sm-10'];

        $row2 = $this->form->addFields([new TLabel("Titulo:", '#ff0000', '14px', null, '100%'),$titulo]);
        $row2->layout = [' col-sm-12'];

        $row3 = $this->form->addFields([new TLabel("Data Hora Inicial:", '#ff0000', '14px', null, '100%'),$data_hora_inicial],[new TLabel("Data Hora Final:", '#ff0000', '14px', null, '100%'),$data_hora_final]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Cor:", '#ff0000', '14px', null, '100%'),$cor]);
        $row4->layout = [' col-sm-12'];

        $row5 = $this->form->addFields([new TLabel("Observação:", null, '14px', null, '100%'),$observacao]);
        $row5->layout = [' col-sm-12'];

        $bcontainer_6315212859170 = new BContainer('bcontainer_6315212859170');
        $this->bcontainer_6315212859170 = $bcontainer_6315212859170;

        $bcontainer_6315212859170->setTitle("Histórico de Alterações", '#333', '18px', '', '#fff');
        $bcontainer_6315212859170->setBorderColor('#c0c0c0');

        $row6 = $bcontainer_6315212859170->addFields([new TLabel("Criado Em:", null, '14px', null, '100%'),$criado_em],[new TLabel("Criado Por:", null, '14px', null, '100%'),$criado_por],[new TLabel("Alterado Em:", null, '14px', null, '100%'),$alterado_em],[new TLabel("Alterado Por:", null, '14px', null, '100%'),$alterado_por]);
        $row6->layout = [' col-sm-4',' col-sm-8',' col-sm-4',' col-sm-8'];

        $row7 = $this->form->addFields([$bcontainer_6315212859170]);
        $row7->layout = [' col-sm-12'];

        $this->form->addFields([$view]);

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onclear = $this->form->addAction("Limpar formulário", new TAction([$this, 'onClear']), 'fas:eraser #dd5a43');
        $this->btn_onclear = $btn_onclear;

        $btn_ondelete = $this->form->addAction("Excluir", new TAction([$this, 'onDelete']), 'fas:trash-alt #dd5a43');
        $this->btn_ondelete = $btn_ondelete;

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=CertificadoAgendamentosCalendarForm]');
        $style->width = '50% !important';   
        $style->show(true);

    }

    public static function onExitVendaCertificado($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database); 
            $certvenda = CertificadoVenda::find($param['certificado_venda_id']);  

            // if ($certvenda->certificado_tipo_atendimento_id==CertificadoTipoAtendimento::)
            // {
            //     $titulo =  'Emissão externa agendada, cliente: ';               
            // }
            // else
            // {
            //     $titulo =  'Emissão interna agendada, cliente: ';                               
            // }

            $titulo = 'Emissão '.CertificadoTipoAtendimento::ObterTextoTipoAtendimento($certvenda->certificado_tipo_atendimento_id)
                      .' agendada, cliente: '; 

            $obj = new stdClass();
            $obj->titulo = $titulo.$certvenda->cliente->nome;

            TForm::sendData(self::$formName, $obj);    

            TTransaction::close();            

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new CertificadoAgendamentos(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            //verificando se já existe algum agendamento para esta venda..
            $certagendamento = CertificadoAgendamentos::where('certificado_venda_id', '=', $object->certificado_venda_id)->first();

            if ($certagendamento->id<>$object->id)
            {
              throw new Exception("Já existe um agendamento para esta emissão!");                   
            }

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);

            $object->store(); // save the object 

            $messageAction = new TAction(['CertificadoAgendamentosCalendarFormView', 'onReload']);
            $messageAction->setParameter('view', $data->view);
            $messageAction->setParameter('date', explode(' ', $data->data_hora_inicial)[0]);

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            //atualizando a venda da certificação com a data de agendamento
            $certvenda = CertificadoVenda::find($object->certificado_venda_id); 
            $certvenda->data_hora_agendamento = $object->data_hora_inicial;
            $certvenda->store();

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            //evento
            TTransaction::open(self::$database);

            Funcoes::VerificaNotificacaoEvento(TSession::getValue('userid'),
                                               CertificadoVenda::Evento_VendaCertificadoAgendada,
                                               $certvenda,
                                               'Venda de certificado cliente '.$certvenda->cliente->nome.' , '.
                                               'Produto: '.$certvenda->produto->nome.' , '.
                                               'Valor: R$ '.number_format($certvenda->valor_total,2, ',', '.').' , '.
                                               'Data agendamento: '.$object->data_hora_inicial); 
            TTransaction::close();               

            TToast::show('success', "Agendamento Salvo!", 'topRight', 'far:check-circle'); 

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }
    public function onDelete($param = null) 
    {
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                $key = $param[self::$primaryKey];

                // open a transaction with database
                TTransaction::open(self::$database);

                $class = self::$activeRecord;

                // instantiates object
                $object = new $class($key, FALSE);

                // deletes the object from the database
                $object->delete();

                //removendo a data e hora do agendamento da venda...
                if ($object->vcertificado_venda_id != null)
                {
                    $certvenda = CertificadoVenda::find($object->vcertificado_venda_id); 
                    $certvenda->data_hora_agendamento = null;
                    $certvenda->store();                
                }

                // close the transaction
                TTransaction::close();

                $messageAction = new TAction(array(__CLASS__.'View', 'onReload'));
                $messageAction->setParameter('view', $param['view']);
                $messageAction->setParameter('date', explode(' ',$param[self::$startDateField])[0]);

                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'), $messageAction);
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters((array) $this->form->getData());
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new CertificadoAgendamentos($key); // instantiates the Active Record 

                                $object->view = !empty($param['view']) ? $param['view'] : 'agendaWeek'; 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public function onStartEdit($param)
    {

        $this->form->clear(true);

        $data = new stdClass;
        $data->view = $param['view'] ?? 'agendaWeek'; // calendar view
        $data->cor = '#3a87ad';

        if (!empty($param['date']))
        {
            if(strlen($param['date']) == '10')
                $param['date'].= ' 09:00';

            $data->data_hora_inicial = str_replace('T', ' ', $param['date']);

            $data_hora_final = new DateTime($data->data_hora_inicial);
            $data_hora_final->add(new DateInterval('PT1H'));
            $data->data_hora_final = $data_hora_final->format('Y-m-d H:i:s');

        }

        $this->form->setData( $data );
    }

    public static function onUpdateEvent($param)
    {
        try
        {
            if (isset($param['id']))
            {
                TTransaction::open(self::$database);

                $class = self::$activeRecord;
                $object = new $class($param['id']);

                $object->data_hora_inicial = str_replace('T', ' ', $param['start_time']);
                $object->data_hora_final   = str_replace('T', ' ', $param['end_time']);

                $certvenda = CertificadoVenda::find($object->certificado_venda_id); 
                $certvenda->data_hora_agendamento = $object->data_hora_inicial;
                $certvenda->store();

                $object->store();

                // close the transaction
                TTransaction::close();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', '<b>Error</b> ' . $e->getMessage());
            TTransaction::rollback();
        }
    }

    public static function getFormName()
    {
        return self::$formName;
    }

}

