<?php

class CertificadoAgenteRegistroForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'CertificadoAgenteRegistro';
    private static $primaryKey = 'id';
    private static $formName = 'form_CertificadoAgenteRegistroForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Cadastro de Agente de Registro");


        $id = new TEntry('id');
        $parceiro_id = new TDBCombo('parceiro_id', 'kraftsoft', 'Parceiro', 'id', '{nome}','nome asc'  );
        $nome = new TEntry('nome');
        $cpf = new TEntry('cpf');
        $fone = new TEntry('fone');
        $email = new TEntry('email');
        $percentual_comissao = new TNumeric('percentual_comissao', '2', ',', '.' );
        $criado_em = new TDateTime('criado_em');
        $criado_por = new TEntry('criado_por');
        $alterado_em = new TDateTime('alterado_em');
        $alterado_por = new TEntry('alterado_por');

        $nome->addValidation("Nome", new TRequiredValidator()); 

        $parceiro_id->enableSearch();
        $criado_em->setMask('dd/mm/yyyy hh:ii');
        $alterado_em->setMask('dd/mm/yyyy hh:ii');

        $criado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $alterado_em->setDatabaseMask('yyyy-mm-dd hh:ii');

        $id->setEditable(false);
        $criado_em->setEditable(false);
        $criado_por->setEditable(false);
        $alterado_em->setEditable(false);
        $alterado_por->setEditable(false);

        $id->setSize(100);
        $cpf->setSize('100%');
        $nome->setSize('100%');
        $fone->setSize('100%');
        $email->setSize('100%');
        $criado_em->setSize(150);
        $alterado_em->setSize(150);
        $criado_por->setSize('100%');
        $parceiro_id->setSize('100%');
        $alterado_por->setSize('100%');
        $percentual_comissao->setSize('100%');


        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Parceiro:", null, '14px', null, '100%'),$parceiro_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Nome:", '#ff0000', '14px', null, '100%'),$nome],[new TLabel("CPF:", null, '14px', null, '100%'),$cpf]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Fone:", null, '14px', null, '100%'),$fone],[new TLabel("E-Mail:", null, '14px', null, '100%'),$email]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Comissão (%):", null, '14px', null, '100%'),$percentual_comissao]);
        $row4->layout = ['col-sm-6'];

        $bcontainer_630d3682a6fe9 = new BContainer('bcontainer_630d3682a6fe9');
        $this->bcontainer_630d3682a6fe9 = $bcontainer_630d3682a6fe9;

        $bcontainer_630d3682a6fe9->setTitle("Histórico de Alterações", '#333', '18px', '', '#fff');
        $bcontainer_630d3682a6fe9->setBorderColor('#c0c0c0');

        $row5 = $bcontainer_630d3682a6fe9->addFields([new TLabel("Criado Em:", null, '14px', null, '100%'),$criado_em],[new TLabel("Criado Por:", null, '14px', null, '100%'),$criado_por],[new TLabel("Alterado Em:", null, '14px', null, '100%'),$alterado_em],[new TLabel("Alterado Por:", null, '14px', null, '100%'),$alterado_por]);
        $row5->layout = [' col-sm-2',' col-sm-4',' col-sm-2',' col-sm-4'];

        $row6 = $this->form->addFields([$bcontainer_630d3682a6fe9]);
        $row6->layout = [' col-sm-12'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onclear = $this->form->addAction("Limpar formulário", new TAction([$this, 'onClear']), 'fas:eraser #dd5a43');
        $this->btn_onclear = $btn_onclear;

        $btn_onshow = $this->form->addAction("Voltar", new TAction(['CertificadoAgenteRegistroList', 'onShow']), 'fas:arrow-left #000000');
        $this->btn_onshow = $btn_onshow;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Certificação Digital","Cadastro de Agente de Registro"]));
        }
        $container->add($this->form);

        parent::add($container);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new CertificadoAgenteRegistro(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);

            $object->store(); // save the object 

            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('CertificadoAgenteRegistroList', 'onShow', $loadPageParam); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new CertificadoAgenteRegistro($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

