<?php

class CertificadoClienteReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'CertificadoCliente';
    private static $primaryKey = 'id';
    private static $formName = 'form_CertificadoClienteReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Vencimentos de Certificados ");

        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{nome}','nome asc'  );
        $certificado_cliente_situacao_id = new TDBCombo('certificado_cliente_situacao_id', 'kraftsoft', 'CertificadoClienteSituacao', 'id', '{descricao}','descricao asc'  );
        $parceiro_id = new TDBCombo('parceiro_id', 'kraftsoft', 'Parceiro', 'id', '{nome}','nome asc'  );
        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc'  );
        $data_vencimento_inicial = new TDate('data_vencimento_inicial');
        $data_vencimento_final = new TDate('data_vencimento_final');

        $data_vencimento_final->setMask('dd/mm/yyyy');
        $data_vencimento_inicial->setMask('dd/mm/yyyy');

        $data_vencimento_final->setDatabaseMask('yyyy-mm-dd');
        $data_vencimento_inicial->setDatabaseMask('yyyy-mm-dd');

        $cliente_id->enableSearch();
        $produto_id->enableSearch();
        $parceiro_id->enableSearch();
        $certificado_cliente_situacao_id->enableSearch();

        $cliente_id->setSize('100%');
        $produto_id->setSize('100%');
        $parceiro_id->setSize('100%');
        $data_vencimento_final->setSize('100%');
        $data_vencimento_inicial->setSize('100%');
        $certificado_cliente_situacao_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null, '100%'),$cliente_id],[new TLabel("Situação:", null, '14px', null, '100%'),$certificado_cliente_situacao_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Parceiro:", null, '14px', null, '100%'),$parceiro_id],[new TLabel("Produto:", null, '14px', null, '100%'),$produto_id]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Data Vencimento Inicial:", '#EC0808', '14px', null, '100%'),$data_vencimento_inicial],[new TLabel("Data Vencimento Final:", '#EC0808', '14px', null),$data_vencimento_final]);
        $row3->layout = [' col-sm-3',' col-sm-3'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratehtml = $this->form->addAction("Gerar HTML", new TAction([$this, 'onGenerateHtml']), 'far:file-code #ffffff');
        $this->btn_ongeneratehtml = $btn_ongeneratehtml;
        $btn_ongeneratehtml->addStyleClass('btn-primary'); 

        $btn_ongeneratepdf = $this->form->addAction("Gerar PDF", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        $btn_ongeneratexls = $this->form->addAction("Gerar XLS", new TAction([$this, 'onGenerateXls']), 'far:file-excel #00a65a');
        $this->btn_ongeneratexls = $btn_ongeneratexls;

        $btn_ongeneratertf = $this->form->addAction("Gerar RTF", new TAction([$this, 'onGenerateRtf']), 'far:file-alt #324bcc');
        $this->btn_ongeneratertf = $btn_ongeneratertf;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Certificação Digital","Vencimentos de Certificados "]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }
    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }
    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }
    public function onGenerateRtf($param = null) 
    {
        $this->onGenerate('rtf');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }
        if (isset($data->certificado_cliente_situacao_id) AND ( (is_scalar($data->certificado_cliente_situacao_id) AND $data->certificado_cliente_situacao_id !== '') OR (is_array($data->certificado_cliente_situacao_id) AND (!empty($data->certificado_cliente_situacao_id)) )) )
        {

            $filters[] = new TFilter('certificado_cliente_situacao_id', '=', $data->certificado_cliente_situacao_id);// create the filter 
        }
        if (isset($data->parceiro_id) AND ( (is_scalar($data->parceiro_id) AND $data->parceiro_id !== '') OR (is_array($data->parceiro_id) AND (!empty($data->parceiro_id)) )) )
        {

            $filters[] = new TFilter('parceiro_id', '=', $data->parceiro_id);// create the filter 
        }
        if (isset($data->produto_id) AND ( (is_scalar($data->produto_id) AND $data->produto_id !== '') OR (is_array($data->produto_id) AND (!empty($data->produto_id)) )) )
        {

            $filters[] = new TFilter('produto_id', '=', $data->produto_id);// create the filter 
        }
        if (isset($data->data_vencimento_inicial) AND ( (is_scalar($data->data_vencimento_inicial) AND $data->data_vencimento_inicial !== '') OR (is_array($data->data_vencimento_inicial) AND (!empty($data->data_vencimento_inicial)) )) )
        {

            $filters[] = new TFilter('data_vencimento_certificado', '>=', $data->data_vencimento_inicial);// create the filter 
        }
        if (isset($data->data_vencimento_final) AND ( (is_scalar($data->data_vencimento_final) AND $data->data_vencimento_final !== '') OR (is_array($data->data_vencimento_final) AND (!empty($data->data_vencimento_final)) )) )
        {

            $filters[] = new TFilter('data_vencimento_certificado', '<=', $data->data_vencimento_final);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            $param = [];
            // creates a repository for CertificadoCliente
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            $criteria->setProperties($param);

            if ($filters)
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            $criteria->add(new TFilter('cliente_id', 'in', "(SELECT id FROM cliente WHERE deletado_em is NULL )")); 

            $data = $this->form->getData();

            if ($data->data_vencimento_inicial==null or $data->data_vencimento_final==null)
            {
               throw new Exception("Informe a data inicial e final de vencimento!");  
            }    

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            if ($objects)
            {
                $widths = array(200,200,200,200,200,200,200,200,200);
                $reportExtension = 'pdf';
                switch ($format)
                {
                    case 'html':
                        $tr = new TTableWriterHTML($widths);
                        $reportExtension = 'html';
                        break;
                    case 'xls':
                        $tr = new TTableWriterXLS($widths);
                        $reportExtension = 'xls';
                        break;
                    case 'pdf':
                        $tr = new TTableWriterPDF($widths, 'L', 'A4');
                        $reportExtension = 'pdf';
                        break;
                    case 'htmlPdf':
                        $reportExtension = 'pdf';
                        $tr = new BTableWriterHtmlPDF($widths, 'L', 'A4');
                        break;
                    case 'rtf':
                        if (!class_exists('PHPRtfLite_Autoloader'))
                        {
                            PHPRtfLite::registerAutoloader();
                        }
                        $reportExtension = 'rtf';
                        $tr = new TTableWriterRTF($widths, 'L', 'A4');
                        break;
                }

                if (!empty($tr))
                {
                    // create the document styles
                    $tr->addStyle('title', 'Helvetica', '10', 'B',   '#ffffff', '#96c0e6');
                    $tr->addStyle('datap', 'Arial', '10', '',    '#000000', '#f0f0f0');
                    $tr->addStyle('datai', 'Arial', '10', '',    '#000000', '#ffffff');
                    $tr->addStyle('header', 'Helvetica', '16', 'B',   '#4589c5', '#494D90');
                    $tr->addStyle('footer', 'Helvetica', '10', 'B',  '#4589c5', '#B1B1EA');
                    $tr->addStyle('break', 'Helvetica', '10', 'B',  '#ffffff', '#8da4bc');
                    $tr->addStyle('total', 'Helvetica', '10', 'I',  '#000000', '#b2d1dc');
                    $tr->addStyle('breakTotal', 'Helvetica', '10', 'I',  '#000000', '#b3d5f3');

                    // Quando o relatório a ser gerado for PDF
                    if ($format == 'pdf')
                    {
                        // Adiciona o cabeçalho
                        $tr->setHeaderCallback(
                            function($tr)
                            {
                                $pdf = $tr->getNativeWriter();

                                // Define a fonte/ estilos
                                $pdf->SetFont('Arial','B',15); 

                                // Define o posicionamento do texto
                                $pdf->Cell(80); 

                                // Texto do cabeçalho
                                $pdf->Cell(140,10, utf8_decode('Relatório de Vencimentos de Certificados') ,0,0,'C'); 
                                $pdf->Ln(10); 

                                $data = $this->form->getData();

                                TTransaction::open('kraftsoft');
                                $filial = SystemUnit::find( $data->system_unit_id);

                                if ($filial)
                                {
                                    $NomeFilial = $filial->name;
                                    $numcell = 405;
                                }
                                else
                                {
                                    $NomeFilial = 'Todas';
                                    $numcell = 80;
                                }

                                TTransaction::close();         

                                $date1 = new DateTime($data->data_vencimento_inicial);
                                $DataInicial = $date1->format('d/m/Y H:i');

                                $date2 = new DateTime($data->data_vencimento_final);
                                $DataFinal = $date2->format('d/m/Y H:i');                                

                                $pdf->SetFont('Arial','B',13); 
                                $pdf->Cell($numcell,40, utf8_decode('Filial: '.$NomeFilial) ,0,0,'C'); 

                                $pdf->Ln(3); 
                                $pdf->Cell(370,60, utf8_decode('Período de vencimento: '.$DataInicial. ' a '.$DataFinal ) ,0,0,'C');                                                                 

                                // Line break 
                                $pdf->Ln(5); 

                            }
                        );

                        // Adiciona o footer do relatório
                        $tr->setFooterCallback(
                            function($tr)
                            {
                                $pdf = $tr->getNativeWriter();

                                // Necessário para obter o número total de páginas
                                $pdf->AliasNbPages();

                                // Posiciona o footer no final da página
                                $pdf->SetY(-40);

                                // Define o estilho do footer
                                $pdf->SetFont('Arial'   ,'B',12); 
                                $pdf->Cell(110); 

                                // Obtém o número da página atual
                                $numero = $pdf->PageNo();

                                // Footer
                                $pdf->Cell(0,10, utf8_decode("Página: {$numero}/{nb}") ,0,0,'R');

                                // Line break 
                                $pdf->Ln(20); 
                            }
                        );
                    }
                    else // Para os demais formatos
                    {
                         $tr->setHeaderCallback(
                            function($tr)
                            {
                                $tr->addRow();
                                $tr->addCell('Relatório de Vencimentos de Certificados', 'center', 'header', 100);
                            }
                        );

                        $tr->setFooterCallback(
                            function($tr)
                            {
                                $tr->addRow();
                                $tr->addCell(date('d/m/Y h:i:s'), 'center', 'footer', 100);
                            }
                        );
                    }

                    // add titles row
                    $tr->addRow();
                    $tr->addCell("Cliente", 'center', 'title');
                    $tr->addCell("CPF/CNPJ", 'center', 'title');
                    $tr->addCell("Fone", 'center', 'title');
                    $tr->addCell("E-Mail", 'center', 'title');
                    $tr->addCell("Parceiro", 'center', 'title');
                    $tr->addCell("Produto", 'center', 'title');
                    $tr->addCell("Situação", 'center', 'title');
                    $tr->addCell("Vencimento Certificado", 'center', 'title');
                    $tr->addCell("Id Venda Cert.", 'center', 'title');

                    $grandTotal = [];
                    $breakTotal = [];
                    $breakValue = null;
                    $firstRow = true;

                    // controls the background filling
                    $colour = false;                
                    foreach ($objects as $object)
                    {
                        $style = $colour ? 'datap' : 'datai';

                        $firstRow = false;

                        $object->data_vencimento_certificado = call_user_func(function($value, $object, $row) 
                        {
                            if(!empty(trim($value)))
                            {
                                try
                                {
                                    $date = new DateTime($value);
                                    return $date->format('d/m/Y');
                                }
                                catch (Exception $e)
                                {
                                    return $value;
                                }
                            }
                        }, $object->data_vencimento_certificado, $object, null);

                        $tr->addRow();

                        $tr->addCell($object->cliente->nome, 'left', $style);
                        $tr->addCell($object->cliente->cpf_cnpj, 'left', $style);
                        $tr->addCell($object->cliente->fone, 'left', $style);
                        $tr->addCell($object->cliente->email, 'left', $style);
                        $tr->addCell($object->parceiro->nome, 'left', $style);
                        $tr->addCell($object->produto->nome, 'left', $style);
                        $tr->addCell($object->certificado_cliente_situacao->descricao, 'left', $style);
                        $tr->addCell($object->data_vencimento_certificado, 'left', $style);
                        $tr->addCell($object->certificado_venda_id, 'left', $style);

                        $colour = !$colour;

                    }

                    $file = 'report_'.uniqid().".{$reportExtension}";
                    // stores the file
                    if (!file_exists("app/output/{$file}") || is_writable("app/output/{$file}"))
                    {
                        $tr->save("app/output/{$file}");
                    }
                    else
                    {
                        throw new Exception(_t('Permission denied') . ': ' . "app/output/{$file}");
                    }

                    parent::openFile("app/output/{$file}");

                    // shows the success message
                    new TMessage('info', _t('Report generated. Please, enable popups'));
                }
            }
            else
            {
                new TMessage('error', _t('No records found'));
            }

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

        $this->btn_ongeneratepdf->style = 'display: none';
    }


}

