<?php

class CertificadoVendaAGRReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'CertificadoVenda';
    private static $primaryKey = 'id';
    private static $formName = 'form_CertificadoVendaAGRReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Relatório de Agente de Registro");

        $system_unit_id = new TDBCombo('system_unit_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{nome}','nome asc'  );
        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc'  );
        $parceiro_id = new TDBCombo('parceiro_id', 'kraftsoft', 'Parceiro', 'id', '{nome}','nome asc'  );
        $certificado_agente_registro_id = new TDBCombo('certificado_agente_registro_id', 'kraftsoft', 'CertificadoAgenteRegistro', 'id', '{nome}','nome asc'  );
        $certificado_venda_situacao_id = new TDBCombo('certificado_venda_situacao_id', 'kraftsoft', 'CertificadoVendaSituacao', 'id', '{descricao}','descricao asc'  );
        $certificado_venda_modalidade_id = new TDBCombo('certificado_venda_modalidade_id', 'kraftsoft', 'CertificadoVendaModalidade', 'id', '{descricao}','descricao asc'  );
        $certificado_venda_tipomodalidade_id = new TDBCombo('certificado_venda_tipomodalidade_id', 'kraftsoft', 'CertificadoVendaTipomodalidade', 'id', '{descricao}','descricao asc'  );
        $certificado_tipo_repasse_id = new TDBCombo('certificado_tipo_repasse_id', 'kraftsoft', 'CertificadoTipoRepasse', 'id', '{descricao}','descricao asc'  );
        $data_emissao_inicial = new TDate('data_emissao_inicial');
        $data_emissao_final = new TDate('data_emissao_final');

        $data_emissao_inicial->addValidation("Data Criação Inicial", new TRequiredValidator()); 
        $data_emissao_final->addValidation("Data Criação Final", new TRequiredValidator()); 

        $data_emissao_final->setMask('dd/mm/yyyy');
        $data_emissao_inicial->setMask('dd/mm/yyyy');

        $data_emissao_final->setDatabaseMask('yyyy-mm-dd');
        $data_emissao_inicial->setDatabaseMask('yyyy-mm-dd');

        $cliente_id->enableSearch();
        $produto_id->enableSearch();
        $parceiro_id->enableSearch();
        $system_unit_id->enableSearch();
        $certificado_tipo_repasse_id->enableSearch();
        $certificado_venda_situacao_id->enableSearch();
        $certificado_agente_registro_id->enableSearch();
        $certificado_venda_modalidade_id->enableSearch();
        $certificado_venda_tipomodalidade_id->enableSearch();

        $cliente_id->setSize('100%');
        $produto_id->setSize('100%');
        $parceiro_id->setSize('100%');
        $system_unit_id->setSize('100%');
        $data_emissao_final->setSize('100%');
        $data_emissao_inicial->setSize('100%');
        $certificado_tipo_repasse_id->setSize('100%');
        $certificado_venda_situacao_id->setSize('100%');
        $certificado_agente_registro_id->setSize('100%');
        $certificado_venda_modalidade_id->setSize('100%');
        $certificado_venda_tipomodalidade_id->setSize('100%');


        $row1 = $this->form->addFields([new TLabel("Filial:", null, '14px', null, '100%'),$system_unit_id]);
        $row1->layout = ['col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null, '100%'),$cliente_id],[new TLabel("Produto:", null, '14px', null, '100%'),$produto_id]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Parceiro:", null, '14px', null, '100%'),$parceiro_id],[new TLabel("Agente de Registro:", '#E10606', '14px', null, '100%'),$certificado_agente_registro_id]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Situação:", null, '14px', null, '100%'),$certificado_venda_situacao_id],[new TLabel("Modalidade:", null, '14px', null, '100%'),$certificado_venda_modalidade_id]);
        $row4->layout = ['col-sm-6','col-sm-6'];

        $row5 = $this->form->addFields([new TLabel("Tipo Modalidade:", null, '14px', null, '100%'),$certificado_venda_tipomodalidade_id],[new TLabel("Tipo Repasse:", null, '14px', null, '100%'),$certificado_tipo_repasse_id]);
        $row5->layout = ['col-sm-6','col-sm-6'];

        $row6 = $this->form->addFields([new TLabel("Data/Hora Emissão Inicial:", '#E10606', '14px', null, '100%'),$data_emissao_inicial],[new TLabel("Data/Hora Emissão Final:", '#E10606', '14px', null, '100%'),$data_emissao_final]);
        $row6->layout = ['col-sm-3','col-sm-3'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratepdf = $this->form->addAction("Gerar PDF", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        $btn_ongeneratexls = $this->form->addAction("Gerar XLS", new TAction([$this, 'onGenerateXls']), 'far:file-excel #00a65a');
        $this->btn_ongeneratexls = $btn_ongeneratexls;

        $btn_ongeneratehtml = $this->form->addAction("Gerar HTML", new TAction([$this, 'onGenerateHtml']), 'far:file-code #ffffff');
        $this->btn_ongeneratehtml = $btn_ongeneratehtml;
        $btn_ongeneratehtml->addStyleClass('btn-primary'); 

        $btn_ongeneratertf = $this->form->addAction("Gerar RTF", new TAction([$this, 'onGenerateRtf']), 'far:file-alt #324bcc');
        $this->btn_ongeneratertf = $btn_ongeneratertf;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Certificação Digital","Relatório de Agente de Registro"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }
    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }
    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }
    public function onGenerateRtf($param = null) 
    {
        $this->onGenerate('rtf');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->system_unit_id) AND ( (is_scalar($data->system_unit_id) AND $data->system_unit_id !== '') OR (is_array($data->system_unit_id) AND (!empty($data->system_unit_id)) )) )
        {

            $filters[] = new TFilter('system_unit_id', '=', $data->system_unit_id);// create the filter 
        }
        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }
        if (isset($data->produto_id) AND ( (is_scalar($data->produto_id) AND $data->produto_id !== '') OR (is_array($data->produto_id) AND (!empty($data->produto_id)) )) )
        {

            $filters[] = new TFilter('produto_id', '=', $data->produto_id);// create the filter 
        }
        if (isset($data->parceiro_id) AND ( (is_scalar($data->parceiro_id) AND $data->parceiro_id !== '') OR (is_array($data->parceiro_id) AND (!empty($data->parceiro_id)) )) )
        {

            $filters[] = new TFilter('parceiro_id', '=', $data->parceiro_id);// create the filter 
        }
        if (isset($data->certificado_agente_registro_id) AND ( (is_scalar($data->certificado_agente_registro_id) AND $data->certificado_agente_registro_id !== '') OR (is_array($data->certificado_agente_registro_id) AND (!empty($data->certificado_agente_registro_id)) )) )
        {

            $filters[] = new TFilter('certificado_agente_registro_id', '=', $data->certificado_agente_registro_id);// create the filter 
        }
        if (isset($data->certificado_venda_situacao_id) AND ( (is_scalar($data->certificado_venda_situacao_id) AND $data->certificado_venda_situacao_id !== '') OR (is_array($data->certificado_venda_situacao_id) AND (!empty($data->certificado_venda_situacao_id)) )) )
        {

            $filters[] = new TFilter('certificado_venda_situacao_id', '=', $data->certificado_venda_situacao_id);// create the filter 
        }
        if (isset($data->certificado_venda_modalidade_id) AND ( (is_scalar($data->certificado_venda_modalidade_id) AND $data->certificado_venda_modalidade_id !== '') OR (is_array($data->certificado_venda_modalidade_id) AND (!empty($data->certificado_venda_modalidade_id)) )) )
        {

            $filters[] = new TFilter('certificado_venda_modalidade_id', '=', $data->certificado_venda_modalidade_id);// create the filter 
        }
        if (isset($data->certificado_venda_tipomodalidade_id) AND ( (is_scalar($data->certificado_venda_tipomodalidade_id) AND $data->certificado_venda_tipomodalidade_id !== '') OR (is_array($data->certificado_venda_tipomodalidade_id) AND (!empty($data->certificado_venda_tipomodalidade_id)) )) )
        {

            $filters[] = new TFilter('certificado_venda_tipomodalidade_id', '=', $data->certificado_venda_tipomodalidade_id);// create the filter 
        }
        if (isset($data->certificado_tipo_repasse_id) AND ( (is_scalar($data->certificado_tipo_repasse_id) AND $data->certificado_tipo_repasse_id !== '') OR (is_array($data->certificado_tipo_repasse_id) AND (!empty($data->certificado_tipo_repasse_id)) )) )
        {

            $filters[] = new TFilter('certificado_tipo_repasse_id', '=', $data->certificado_tipo_repasse_id);// create the filter 
        }
        if (isset($data->data_emissao_inicial) AND ( (is_scalar($data->data_emissao_inicial) AND $data->data_emissao_inicial !== '') OR (is_array($data->data_emissao_inicial) AND (!empty($data->data_emissao_inicial)) )) )
        {

            $filters[] = new TFilter('data_emissao', '>=', $data->data_emissao_inicial);// create the filter 
        }
        if (isset($data->data_emissao_final) AND ( (is_scalar($data->data_emissao_final) AND $data->data_emissao_final !== '') OR (is_array($data->data_emissao_final) AND (!empty($data->data_emissao_final)) )) )
        {

            $filters[] = new TFilter('data_emissao', '<=', $data->data_emissao_final);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            $param = [];
            // creates a repository for CertificadoVenda
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            $param['order'] = 'certificado_agente_registro_id';
            $param['direction'] = 'asc';

            $criteria->setProperties($param);

            if ($filters)
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            $data = $this->form->getData();

            if ($data->data_emissao_inicial==null or $data->data_emissao_final==null)
            {
               throw new Exception("Informe a data inicial e final de emissão!");  
            }     

            if ($data->certificado_agente_registro_id==null)
            {
               throw new Exception("Informe o Agente!");  
            }     

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            if ($objects)
            {
                $widths = array(200,200,200,200,200,200,200);
                $reportExtension = 'pdf';
                switch ($format)
                {
                    case 'html':
                        $tr = new TTableWriterHTML($widths);
                        $reportExtension = 'html';
                        break;
                    case 'xls':
                        $tr = new TTableWriterXLS($widths);
                        $reportExtension = 'xls';
                        break;
                    case 'pdf':
                        $tr = new TTableWriterPDF($widths, 'L', 'A4');
                        $reportExtension = 'pdf';
                        break;
                    case 'htmlPdf':
                        $reportExtension = 'pdf';
                        $tr = new BTableWriterHtmlPDF($widths, 'L', 'A4');
                        break;
                    case 'rtf':
                        if (!class_exists('PHPRtfLite_Autoloader'))
                        {
                            PHPRtfLite::registerAutoloader();
                        }
                        $reportExtension = 'rtf';
                        $tr = new TTableWriterRTF($widths, 'L', 'A4');
                        break;
                }

                if (!empty($tr))
                {
                    // create the document styles
                    $tr->addStyle('title', 'Helvetica', '10', 'B',   '#ffffff', '#96c0e6');
                    $tr->addStyle('datap', 'Arial', '10', '',    '#000000', '#f0f0f0');
                    $tr->addStyle('datai', 'Arial', '10', '',    '#000000', '#ffffff');
                    $tr->addStyle('header', 'Helvetica', '16', 'B',   '#4589c5', '#494D90');
                    $tr->addStyle('footer', 'Helvetica', '10', 'B',  '#4589c5', '#B1B1EA');
                    $tr->addStyle('break', 'Helvetica', '10', 'B',  '#ffffff', '#8da4bc');
                    $tr->addStyle('total', 'Helvetica', '10', 'I',  '#000000', '#b2d1dc');
                    $tr->addStyle('breakTotal', 'Helvetica', '10', 'I',  '#000000', '#b3d5f3');

                    // Quando o relatório a ser gerado for PDF
                    if ($format == 'pdf')
                    {
                        // Adiciona o cabeçalho
                        $tr->setHeaderCallback(
                            function($tr)
                            {
                                $pdf = $tr->getNativeWriter();

                                // Define a fonte/ estilos
                                $pdf->SetFont('Arial','B',15); 

                                // Define o posicionamento do texto
                                $pdf->Cell(80); 

                                // Texto do cabeçalho
                                $pdf->Cell(170,10, utf8_decode('Relatório de Vendas de Certificados - Agente de Registro') ,0,0,'C'); 
                                $pdf->Ln(10); 

                                //$data = $this->form->getData();

                                $data = $this->form->getData();
                                echo 'filial: '.$data->system_unit_id;

                                TTransaction::open('kraftsoft');
                                $filial = SystemUnit::find( $data->system_unit_id);

                                if ($filial)
                                {
                                    $NomeFilial = $filial->name;
                                    $numcell = 405;
                                }
                                else
                                {
                                    $NomeFilial = 'Todas';
                                    $numcell = 80;
                                }

                                TTransaction::close();         

                                $date1 = new DateTime($data->data_emissao_inicial);
                                $DataInicial = $date1->format('d/m/Y H:i');

                                $date2 = new DateTime($data->data_emissao_final);
                                $DataFinal = $date2->format('d/m/Y H:i');                                

                                $pdf->SetFont('Arial','B',13); 
                                $pdf->Cell($numcell,40, utf8_decode('Filial: '.$NomeFilial) ,0,0,'C'); 

                                $pdf->Ln(3); 
                                $pdf->Cell(294,60, utf8_decode('Período de emissão: '.$DataInicial. ' a '.$DataFinal ) ,0,0,'C');                                                                 

                                // Line break 
                                $pdf->Ln(5); 

                            }
                        );

                        // Adiciona o footer do relatório
                        $tr->setFooterCallback(
                            function($tr)
                            {
                                $pdf = $tr->getNativeWriter();

                                // Necessário para obter o número total de páginas
                                $pdf->AliasNbPages();

                                // Posiciona o footer no final da página
                                $pdf->SetY(-40);

                                // Define o estilho do footer
                                $pdf->SetFont('Arial'   ,'B',12); 
                                $pdf->Cell(110); 

                                // Obtém o número da página atual
                                $numero = $pdf->PageNo();

                                // Footer
                                $pdf->Cell(0,10, utf8_decode("Página: {$numero}/{nb}") ,0,0,'R');

                                // Line break 
                                $pdf->Ln(20); 
                            }
                        );
                    }
                    else // Para os demais formatos
                    {
                         $tr->setHeaderCallback(
                            function($tr)
                            {
                                $tr->addRow();
                                $tr->addCell('Relatório de Vendas de Certificados - Agente de Registro', 'center', 'header', 5);
                            }
                        );

                        $tr->setFooterCallback(
                            function($tr)
                            {
                                $tr->addRow();
                                $tr->addCell(date('Y-m-d h:i:s'), 'center', 'footer', 5);
                            }
                        );
                    }

                    // add titles row
                    $tr->addRow();
                    $tr->addCell("Id", 'center', 'title');
                    $tr->addCell("Data Emissão", 'center', 'title');
                    $tr->addCell("Cliente", 'center', 'title');
                    $tr->addCell("Produto", 'center', 'title');
                    $tr->addCell("Protocolo Emissão", 'center', 'title');
                    $tr->addCell("Valor Comissão", 'center', 'title');
                    $tr->addCell("Situação", 'left', 'title');

                    $grandTotal = [];
                    $breakTotal = [];
                    $breakValue = null;
                    $firstRow = true;

                    // controls the background filling
                    $colour = false;                
                    foreach ($objects as $object)
                    {
                        $style = $colour ? 'datap' : 'datai';

                        if ($object->certificado_agente_registro_id !== $breakValue)
                        {
                            if (!$firstRow)
                            {
                                $tr->addRow();

                                $breakTotal_agente_comissao_valor = array_sum($breakTotal['agente_comissao_valor']);

                                $breakTotal_agente_comissao_valor = call_user_func(function($value)
                                {
                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return "R$ " . number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }
                                }, $breakTotal_agente_comissao_valor); 

                                $tr->addCell('', 'center', 'breakTotal');
                                $tr->addCell('', 'center', 'breakTotal');
                                $tr->addCell('', 'center', 'breakTotal');
                                $tr->addCell('', 'center', 'breakTotal');
                                $tr->addCell('', 'center', 'breakTotal');
                                $tr->addCell($breakTotal_agente_comissao_valor, 'left', 'breakTotal');
                                $tr->addCell('', 'center', 'breakTotal');
                            }
                            $tr->addRow();
                            $tr->addCell($object->render('{certificado_agente_registro->nome}'), 'left', 'break', 7);
                            $breakTotal = [];
                        }
                        $breakValue = $object->certificado_agente_registro_id;

                        $grandTotal['agente_comissao_valor'][] = $object->agente_comissao_valor;
                        $breakTotal['agente_comissao_valor'][] = $object->agente_comissao_valor;

                        $firstRow = false;

                        $object->data_emissao = call_user_func(function($value, $object, $row) 
                        {
                            if(!empty(trim($value)))
                            {
                                try
                                {
                                    $date = new DateTime($value);
                                    return $date->format('d/m/Y');
                                }
                                catch (Exception $e)
                                {
                                    return $value;
                                }
                            }
                        }, $object->data_emissao, $object, null);

                        $object->agente_comissao_valor = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->agente_comissao_valor, $object, null);

                        $tr->addRow();

                        $tr->addCell($object->id, 'left', $style);
                        $tr->addCell($object->data_emissao, 'left', $style);
                        $tr->addCell($object->cliente->nome, 'left', $style);
                        $tr->addCell($object->produto->nome, 'left', $style);
                        $tr->addCell($object->protocolo_emissao, 'left', $style);
                        $tr->addCell($object->agente_comissao_valor, 'left', $style);
                        $tr->addCell($object->certificado_venda_situacao->descricao, 'left', $style);

                        $colour = !$colour;

                    }

                    $tr->addRow();

                    $breakTotal_agente_comissao_valor = array_sum($breakTotal['agente_comissao_valor']);

                    $breakTotal_agente_comissao_valor = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $breakTotal_agente_comissao_valor); 

                    $tr->addCell('', 'center', 'breakTotal');
                    $tr->addCell('', 'center', 'breakTotal');
                    $tr->addCell('', 'center', 'breakTotal');
                    $tr->addCell('', 'center', 'breakTotal');
                    $tr->addCell('', 'center', 'breakTotal');
                    $tr->addCell($breakTotal_agente_comissao_valor, 'left', 'breakTotal');
                    $tr->addCell('', 'center', 'breakTotal');

                    $tr->addRow();

                    $grandTotal_agente_comissao_valor = array_sum($grandTotal['agente_comissao_valor']);

                    $grandTotal_agente_comissao_valor = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $grandTotal_agente_comissao_valor); 

                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell($grandTotal_agente_comissao_valor, 'left', 'total');
                    $tr->addCell('', 'center', 'total');

                    $file = 'report_'.uniqid().".{$reportExtension}";
                    // stores the file
                    if (!file_exists("app/output/{$file}") || is_writable("app/output/{$file}"))
                    {
                        $tr->save("app/output/{$file}");
                    }
                    else
                    {
                        throw new Exception(_t('Permission denied') . ': ' . "app/output/{$file}");
                    }

                    parent::openFile("app/output/{$file}");

                    // shows the success message
                    new TMessage('info', _t('Report generated. Please, enable popups'));
                }
            }
            else
            {
                new TMessage('error', _t('No records found'));
            }

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }


}

