<?php

class CertificadoVendaList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'CertificadoVenda';
    private static $primaryKey = 'id';
    private static $formName = 'form_CertificadoVendaList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de vendas de certificado");
        $this->limit = 100;

        $id = new TEntry('id');
        $system_unit_id = new TDBCombo('system_unit_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{nome}','nome asc'  );
        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc'  );
        $parceiro_id = new TDBCombo('parceiro_id', 'kraftsoft', 'Parceiro', 'id', '{nome}','nome asc'  );
        $certificado_agente_registro_id = new TDBCombo('certificado_agente_registro_id', 'kraftsoft', 'CertificadoAgenteRegistro', 'id', '{nome}','nome asc'  );
        $certificado_venda_situacao_id = new TDBCombo('certificado_venda_situacao_id', 'kraftsoft', 'CertificadoVendaSituacao', 'id', '{descricao}','descricao asc'  );
        $certificado_venda_modalidade_id = new TDBCombo('certificado_venda_modalidade_id', 'kraftsoft', 'CertificadoVendaModalidade', 'id', '{descricao}','descricao asc'  );
        $certificado_venda_tipomodalidade_id = new TDBCombo('certificado_venda_tipomodalidade_id', 'kraftsoft', 'CertificadoVendaTipomodalidade', 'id', '{descricao}','descricao asc'  );
        $certificado_tipo_repasse_id = new TDBCombo('certificado_tipo_repasse_id', 'kraftsoft', 'CertificadoTipoRepasse', 'id', '{descricao}','descricao asc'  );
        $vendedor_id = new TDBCombo('vendedor_id', 'kraftsoft', 'Vendedor', 'id', '{nome}','nome asc'  );
        $protocolo_emissao = new TEntry('protocolo_emissao');
        $data_agendamento_inicial = new TDateTime('data_agendamento_inicial');
        $data_agendamento_final = new TDateTime('data_agendamento_final');
        $data_vencimento_certificado_inicial = new TDate('data_vencimento_certificado_inicial');
        $data_vencimento_certificado_final = new TDate('data_vencimento_certificado_final');
        $data_emissao_inicial = new TDate('data_emissao_inicial');
        $data_emissao_final = new TDate('data_emissao_final');
        $certificadora_id = new TDBCombo('certificadora_id', 'kraftsoft', 'Certificadora', 'id', '{nome}','nome asc'  );
        $data_cadastro_inicial = new TDate('data_cadastro_inicial');
        $data_cadastro_final = new TDate('data_cadastro_final');


        $data_emissao_final->setMask('dd/mm/yyyy');
        $data_cadastro_final->setMask('dd/mm/yyyy');
        $data_emissao_inicial->setMask('dd/mm/yyyy');
        $data_cadastro_inicial->setMask('dd/mm/yyyy');
        $data_agendamento_final->setMask('dd/mm/yyyy hh:ii');
        $data_agendamento_inicial->setMask('dd/mm/yyyy hh:ii');
        $data_vencimento_certificado_final->setMask('dd/mm/yyyy');
        $data_vencimento_certificado_inicial->setMask('dd/mm/yyyy');

        $data_emissao_final->setDatabaseMask('yyyy-mm-dd');
        $data_cadastro_final->setDatabaseMask('yyyy-mm-dd');
        $data_emissao_inicial->setDatabaseMask('yyyy-mm-dd');
        $data_cadastro_inicial->setDatabaseMask('yyyy-mm-dd');
        $data_agendamento_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_agendamento_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_vencimento_certificado_final->setDatabaseMask('yyyy-mm-dd');
        $data_vencimento_certificado_inicial->setDatabaseMask('yyyy-mm-dd');

        $cliente_id->enableSearch();
        $produto_id->enableSearch();
        $parceiro_id->enableSearch();
        $vendedor_id->enableSearch();
        $system_unit_id->enableSearch();
        $certificadora_id->enableSearch();
        $certificado_tipo_repasse_id->enableSearch();
        $certificado_venda_situacao_id->enableSearch();
        $certificado_agente_registro_id->enableSearch();
        $certificado_venda_modalidade_id->enableSearch();
        $certificado_venda_tipomodalidade_id->enableSearch();

        $id->setSize(100);
        $cliente_id->setSize('100%');
        $produto_id->setSize('100%');
        $parceiro_id->setSize('100%');
        $vendedor_id->setSize('100%');
        $system_unit_id->setSize('100%');
        $certificadora_id->setSize('100%');
        $protocolo_emissao->setSize('100%');
        $data_emissao_final->setSize('100%');
        $data_cadastro_final->setSize('100%');
        $data_emissao_inicial->setSize('100%');
        $data_cadastro_inicial->setSize('100%');
        $data_agendamento_final->setSize('100%');
        $data_agendamento_inicial->setSize('100%');
        $certificado_tipo_repasse_id->setSize('100%');
        $certificado_venda_situacao_id->setSize('100%');
        $certificado_agente_registro_id->setSize('100%');
        $certificado_venda_modalidade_id->setSize('100%');
        $data_vencimento_certificado_final->setSize('100%');
        $certificado_venda_tipomodalidade_id->setSize('100%');
        $data_vencimento_certificado_inicial->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Filial:", null, '14px', null, '100%'),$system_unit_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null, '100%'),$cliente_id],[new TLabel("Produto:", null, '14px', null, '100%'),$produto_id]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Parceiro:", null, '14px', null, '100%'),$parceiro_id],[new TLabel("Agente de Registro:", null, '14px', null, '100%'),$certificado_agente_registro_id]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Situação:", null, '14px', null, '100%'),$certificado_venda_situacao_id],[new TLabel("Modalidade:", null, '14px', null, '100%'),$certificado_venda_modalidade_id]);
        $row4->layout = ['col-sm-6','col-sm-6'];

        $row5 = $this->form->addFields([new TLabel("Tipo Modalidade:", null, '14px', null, '100%'),$certificado_venda_tipomodalidade_id],[new TLabel("Tipo Repasse:", null, '14px', null, '100%'),$certificado_tipo_repasse_id]);
        $row5->layout = ['col-sm-6','col-sm-6'];

        $row6 = $this->form->addFields([new TLabel("Vendedor:", null, '14px', null, '100%'),$vendedor_id],[new TLabel("Protocolo Emissão:", null, '14px', null),$protocolo_emissao]);
        $row6->layout = ['col-sm-6',' col-sm-6'];

        $row7 = $this->form->addFields([new TLabel("Data/Hora Agendamento Inicial:", null, '14px', null, '100%'),$data_agendamento_inicial],[new TLabel("Data/Hora Agendamento Final:", null, '14px', null, '100%'),$data_agendamento_final],[new TLabel("Data Vencimento Certificado Inicial:", null, '14px', null, '100%'),$data_vencimento_certificado_inicial],[new TLabel("Data Vencimento Certificado Final:", null, '14px', null, '100%'),$data_vencimento_certificado_final]);
        $row7->layout = [' col-sm-3',' col-sm-3',' col-sm-3',' col-sm-3'];

        $row8 = $this->form->addFields([new TLabel("Data emissão inicial:", null, '14px', null, '100%'),$data_emissao_inicial],[new TLabel("Data emissão final:", null, '14px', null, '100%'),$data_emissao_final],[new TLabel("Certificadora:", null, '14px', null, '100%'),$certificadora_id]);
        $row8->layout = ['col-sm-3','col-sm-3',' col-sm-6'];

        $row9 = $this->form->addFields([new TLabel("Data cadastro Inicial:", null, '14px', null, '100%'),$data_cadastro_inicial],[new TLabel("Data cadastro final:", null, '14px', null, '100%'),$data_cadastro_final],[]);
        $row9->layout = ['col-sm-3','col-sm-3',' col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_criado_em_transformed = new TDataGridColumn('criado_em', "Criado Em", 'left');
        $column_data_emissao_transformed = new TDataGridColumn('data_emissao', "Data Emissão", 'left');
        $column_cliente_nome = new TDataGridColumn('cliente->nome', "Cliente", 'left');
        $column_produto_nome = new TDataGridColumn('produto->nome', "Produto", 'left');
        $column_data_hora_agendamento_transformed = new TDataGridColumn('data_hora_agendamento', "Data/Hora Agendamento", 'left');
        $column_certificado_agente_registro_nome = new TDataGridColumn('certificado_agente_registro->nome', "Agente Registro", 'left');
        $column_parceiro_nome = new TDataGridColumn('parceiro->nome', "Parceiro", 'left');
        $column_valor_total_transformed = new TDataGridColumn('valor_total', "Valor Total", 'left');
        $column_certificado_venda_situacao_descricao_transformed = new TDataGridColumn('certificado_venda_situacao->descricao', "Situação", 'left');

        $column_criado_em_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_data_emissao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_data_hora_agendamento_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_valor_total_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_certificado_venda_situacao_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
                $label = $object->certificado_venda_situacao->descricao;
                $cor = $object->certificado_venda_situacao->cor;

                return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

                // $div = new TElement('div');
                // $div->style="text-shadow:none; font-size:14px; font-weight:bold; color:#FFFFFF; width:160px; text-align:center;background-color: $cor;padding:10px";
                // $div->add($label);

                // return $div;    //code here

        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_criado_em_transformed);
        $this->datagrid->addColumn($column_data_emissao_transformed);
        $this->datagrid->addColumn($column_cliente_nome);
        $this->datagrid->addColumn($column_produto_nome);
        $this->datagrid->addColumn($column_data_hora_agendamento_transformed);
        $this->datagrid->addColumn($column_certificado_agente_registro_nome);
        $this->datagrid->addColumn($column_parceiro_nome);
        $this->datagrid->addColumn($column_valor_total_transformed);
        $this->datagrid->addColumn($column_certificado_venda_situacao_descricao_transformed);

        $action_group = new TDataGridActionGroup("Opções", 'fas:cog');
        $action_group->addHeader('');

        $action_onShow = new TDataGridAction(array('CertificadoVendaFormView', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Abrir");
        $action_onShow->setImage('fas:book-open #00BCD4');
        $action_onShow->setField(self::$primaryKey);

        $action_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_onShow);

        $action_onEdit = new TDataGridAction(array('CertificadoVendaForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('CertificadoVendaList::onMostrarEditar');

        $action_group->addAction($action_onEdit);

        $action_onAguardandoAprovacao = new TDataGridAction(array('CertificadoVendaList', 'onAguardandoAprovacao'));
        $action_onAguardandoAprovacao->setUseButton(TRUE);
        $action_onAguardandoAprovacao->setButtonClass('btn btn-default');
        $action_onAguardandoAprovacao->setLabel("Aguardando Aprovação");
        $action_onAguardandoAprovacao->setImage('fas:clock #FFC107');
        $action_onAguardandoAprovacao->setField(self::$primaryKey);
        $action_onAguardandoAprovacao->setDisplayCondition('CertificadoVendaList::onMostrarAguardandoAprovacao');

        $action_group->addAction($action_onAguardandoAprovacao);

        $action_onFinalizarAtendimento = new TDataGridAction(array('CertificadoVendaList', 'onFinalizarAtendimento'));
        $action_onFinalizarAtendimento->setUseButton(TRUE);
        $action_onFinalizarAtendimento->setButtonClass('btn btn-default');
        $action_onFinalizarAtendimento->setLabel("Finalizar Atendimento");
        $action_onFinalizarAtendimento->setImage('fas:fingerprint #009688');
        $action_onFinalizarAtendimento->setField(self::$primaryKey);
        $action_onFinalizarAtendimento->setDisplayCondition('CertificadoVendaList::onMostrarFinalizarAtendimento');

        $action_group->addAction($action_onFinalizarAtendimento);

        $action_onDelete = new TDataGridAction(array('CertificadoVendaList', 'onDelete'));
        $action_onDelete->setUseButton(TRUE);
        $action_onDelete->setButtonClass('btn btn-default');
        $action_onDelete->setLabel("Cancelar");
        $action_onDelete->setImage('fas:window-close #CE0D0D');
        $action_onDelete->setField(self::$primaryKey);
        $action_onDelete->setDisplayCondition('CertificadoVendaList::onMostrarCancelar');

        $action_group->addAction($action_onDelete);

        $action_onCopiarProtocolo = new TDataGridAction(array('CertificadoVendaList', 'onCopiarProtocolo'));
        $action_onCopiarProtocolo->setUseButton(TRUE);
        $action_onCopiarProtocolo->setButtonClass('btn btn-default');
        $action_onCopiarProtocolo->setLabel("Copiar Protocolo");
        $action_onCopiarProtocolo->setImage('fas:copy #CDDC39');
        $action_onCopiarProtocolo->setField(self::$primaryKey);
        $action_onCopiarProtocolo->setDisplayCondition('CertificadoVendaList::onMostrarCopiarProtocolo');

        $action_group->addAction($action_onCopiarProtocolo);

        $action_onDesfazerAtendimento = new TDataGridAction(array('CertificadoVendaList', 'onDesfazerAtendimento'));
        $action_onDesfazerAtendimento->setUseButton(TRUE);
        $action_onDesfazerAtendimento->setButtonClass('btn btn-default');
        $action_onDesfazerAtendimento->setLabel("Desfazer Atendimento");
        $action_onDesfazerAtendimento->setImage('fas:backward #9C27B0');
        $action_onDesfazerAtendimento->setField(self::$primaryKey);
        $action_onDesfazerAtendimento->setDisplayCondition('CertificadoVendaList::onMostrarDesfazerAtendimento');

        $action_group->addAction($action_onDesfazerAtendimento);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup("Listagem de vendas de certificado");
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $button_cadastrar = new TButton('button_button_cadastrar');
        $button_cadastrar->setAction(new TAction(['CertificadoVendaForm', 'onShow']), "Cadastrar");
        $button_cadastrar->addStyleClass('btn-default');
        $button_cadastrar->setImage('fas:plus #69aa46');

        $this->datagrid_form->addField($button_cadastrar);

        $btnShowCurtainFilters = new TButton('button_btnShowCurtainFilters');
        $btnShowCurtainFilters->setAction(new TAction(['CertificadoVendaList', 'onShowCurtainFilters']), "Filtros");
        $btnShowCurtainFilters->addStyleClass('btn-default');
        $btnShowCurtainFilters->setImage('fas:filter #000000');

        $this->datagrid_form->addField($btnShowCurtainFilters);

        $button_limpar_filtros = new TButton('button_button_limpar_filtros');
        $button_limpar_filtros->setAction(new TAction(['CertificadoVendaList', 'onClearFilters']), "Limpar filtros");
        $button_limpar_filtros->addStyleClass('btn-default');
        $button_limpar_filtros->setImage('fas:eraser #f44336');

        $this->datagrid_form->addField($button_limpar_filtros);

        $button_atualizar = new TButton('button_button_atualizar');
        $button_atualizar->setAction(new TAction(['CertificadoVendaList', 'onRefresh']), "Atualizar");
        $button_atualizar->addStyleClass('btn-default');
        $button_atualizar->setImage('fas:sync-alt #03a9f4');

        $this->datagrid_form->addField($button_atualizar);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['CertificadoVendaList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['CertificadoVendaList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['CertificadoVendaList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['CertificadoVendaList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_left_actions->add($button_cadastrar);
        $head_left_actions->add($btnShowCurtainFilters);
        $head_left_actions->add($button_limpar_filtros);
        $head_left_actions->add($button_atualizar);

        $head_right_actions->add($dropdown_button_exportar);

        $this->btnShowCurtainFilters = $btnShowCurtainFilters;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Certificação Digital","Vendas de Certificado"]));
        }

        $container->add($panel);

        parent::add($container);

    }

    public static function onMostrarEditar($object)
    {
        try 
        {
            if($object->certificado_venda_situacao_id==CertificadoVendaSituacao::Atendido
            or $object->certificado_venda_situacao_id==CertificadoVendaSituacao::Cancelado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onAguardandoAprovacao($param = null) 
    {
        try 
        {
            new TQuestion("Confirma aguardando aprovação?", new TAction([__CLASS__, 'onAguardandoAprovacaoYes'], $param), new TAction([__CLASS__, 'onAguardandoAprovacaoNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarAguardandoAprovacao($object)
    {
        try 
        {
            if($object->certificado_venda_situacao_id==CertificadoVendaSituacao::AguardandoAtendimento)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onFinalizarAtendimento($param = null) 
    {
        try 
        {
            new TQuestion("Deseja finalizar o atendimento?", new TAction([__CLASS__, 'onFinalizarAtendimentoYes'], $param), new TAction([__CLASS__, 'onFinalizarAtendimentoNo'], $param));
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarFinalizarAtendimento($object)
    {
        try 
        {
            if($object->certificado_venda_situacao_id==CertificadoVendaSituacao::Atendido
            or $object->certificado_venda_situacao_id==CertificadoVendaSituacao::Cancelado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onDelete($param = null) 
    { 
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new CertificadoVenda($key, FALSE); 

                // deletes the object from the database
                $object->certificado_venda_situacao_id=CertificadoVendaSituacao::Cancelado;
                $object->store();

                // close the transaction
                TTransaction::close();

                //evento
                TTransaction::open(self::$database);

                Funcoes::VerificaNotificacaoEvento(TSession::getValue('userid'),
                                                   CertificadoVenda::Evento_VendaCertificadoCancelada,
                                                   $object,
                                                   'Venda de certificado cliente '.$object->cliente->nome.' , '.
                                                   'Produto: '.$object->produto->nome.' , '.
                                                   'Valor: R$ '.number_format($object->valor_total,2, ',', '.')); 
                TTransaction::close();                      

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', 'Operação realizada com sucesso!');
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion('Deseja cancelar a venda?', $action);   
        }
    }
    public static function onMostrarCancelar($object)
    {
        try 
        {
            if($object->certificado_venda_situacao_id==CertificadoVendaSituacao::Atendido
            or $object->certificado_venda_situacao_id==CertificadoVendaSituacao::Cancelado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onCopiarProtocolo($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $certvenda = CertificadoVenda::find($param['id']);     

            if($certvenda) 
            {            
                $texto = $certvenda->certificadora->texto_emissao;

                $texto = str_replace('{protocolo_emissao}',$certvenda->protocolo_emissao,$texto);
                $texto = str_replace('{cliente_nome}',$certvenda->cliente->nome,$texto);
                $texto = str_replace('{cliente_cpf_cnpj}',Funcoes::mascaraCpfCnpj($certvenda->cliente->cpf_cnpj),$texto);
                $texto = str_replace('{produto_nome}',$certvenda->produto->nome,$texto);                

                // $texto =   "Informamos que o seu Certificado Digital foi aprovado com sucesso.".  
                //             " Vá em (https://safeweb.com.br/suporte) para acessar o passo a passo e proceder com a instalação.".
                //             " DADOS DO CERTIFICADO DIGITAL:". 
                //             " Nome Completo: ".$certvenda->cliente->nome.
                //             " CPF/CNPJ: ".$certvenda->cliente->cpf_cnpj.
                //             " Protocolo: ".$certvenda->protocolo_emissao.
                //             " Tipo: ".$certvenda->produto->nome.";";

                TScript::create("navigator.clipboard.writeText('$texto');");     

                TToast::show('success', "Mensagem copiada com sucesso!", 'topRight', 'far:check-circle');  
            }

           TTransaction::close();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarCopiarProtocolo($object)
    {
        try 
        {
            if($object->certificado_venda_situacao_id==CertificadoVendaSituacao::Atendido)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onDesfazerAtendimento($param = null) 
    {
        try 
        {
            new TQuestion("Deseja desfazer o atedimento?", new TAction([__CLASS__, 'onDesfazerAtendimentoYes'], $param), new TAction([__CLASS__, 'onDesfazerAtendimentoNo'], $param));
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarDesfazerAtendimento($object)
    {
        try 
        {
            if($object->certificado_venda_situacao_id==CertificadoVendaSituacao::Atendido)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('object');
                $object->data  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }
    public static function onShowCurtainFilters($param = null) 
    {
        try 
        {
            //code here

                        $filter = new self([]);

            $btnClose = new TButton('closeCurtain');
            $btnClose->class = 'btn btn-sm btn-default';
            $btnClose->style = 'margin-right:10px;';
            $btnClose->onClick = "Template.closeRightPanel();";
            $btnClose->setLabel("Fechar");
            $btnClose->setImage('fas:times');

            $filter->form->addHeaderWidget($btnClose);

            $page = new TPage();
            $page->setTargetContainer('adianti_right_panel');
            $page->setProperty('page-name', 'CertificadoVendaListSearch');
            $page->setProperty('page_name', 'CertificadoVendaListSearch');
            $page->adianti_target_container = 'adianti_right_panel';
            $page->target_container = 'adianti_right_panel';
            $page->add($filter->form);
            $page->setIsWrapped(true);
            $page->show();

            $style = new TStyle('right-panel > .container-part[page-name=CertificadoVendaListSearch]');
            $style->width = '70% !important';
            $style->show(true);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onClearFilters($param = null) 
    {
        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }
    public function onRefresh($param = null) 
    {
        $this->onReload([]);
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->system_unit_id) AND ( (is_scalar($data->system_unit_id) AND $data->system_unit_id !== '') OR (is_array($data->system_unit_id) AND (!empty($data->system_unit_id)) )) )
        {

            $filters[] = new TFilter('system_unit_id', '=', $data->system_unit_id);// create the filter 
        }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }

        if (isset($data->produto_id) AND ( (is_scalar($data->produto_id) AND $data->produto_id !== '') OR (is_array($data->produto_id) AND (!empty($data->produto_id)) )) )
        {

            $filters[] = new TFilter('produto_id', '=', $data->produto_id);// create the filter 
        }

        if (isset($data->parceiro_id) AND ( (is_scalar($data->parceiro_id) AND $data->parceiro_id !== '') OR (is_array($data->parceiro_id) AND (!empty($data->parceiro_id)) )) )
        {

            $filters[] = new TFilter('parceiro_id', '=', $data->parceiro_id);// create the filter 
        }

        if (isset($data->certificado_agente_registro_id) AND ( (is_scalar($data->certificado_agente_registro_id) AND $data->certificado_agente_registro_id !== '') OR (is_array($data->certificado_agente_registro_id) AND (!empty($data->certificado_agente_registro_id)) )) )
        {

            $filters[] = new TFilter('certificado_agente_registro_id', '=', $data->certificado_agente_registro_id);// create the filter 
        }

        if (isset($data->certificado_venda_situacao_id) AND ( (is_scalar($data->certificado_venda_situacao_id) AND $data->certificado_venda_situacao_id !== '') OR (is_array($data->certificado_venda_situacao_id) AND (!empty($data->certificado_venda_situacao_id)) )) )
        {

            $filters[] = new TFilter('certificado_venda_situacao_id', '=', $data->certificado_venda_situacao_id);// create the filter 
        }

        if (isset($data->certificado_venda_modalidade_id) AND ( (is_scalar($data->certificado_venda_modalidade_id) AND $data->certificado_venda_modalidade_id !== '') OR (is_array($data->certificado_venda_modalidade_id) AND (!empty($data->certificado_venda_modalidade_id)) )) )
        {

            $filters[] = new TFilter('certificado_venda_modalidade_id', '=', $data->certificado_venda_modalidade_id);// create the filter 
        }

        if (isset($data->certificado_venda_tipomodalidade_id) AND ( (is_scalar($data->certificado_venda_tipomodalidade_id) AND $data->certificado_venda_tipomodalidade_id !== '') OR (is_array($data->certificado_venda_tipomodalidade_id) AND (!empty($data->certificado_venda_tipomodalidade_id)) )) )
        {

            $filters[] = new TFilter('certificado_venda_tipomodalidade_id', '=', $data->certificado_venda_tipomodalidade_id);// create the filter 
        }

        if (isset($data->certificado_tipo_repasse_id) AND ( (is_scalar($data->certificado_tipo_repasse_id) AND $data->certificado_tipo_repasse_id !== '') OR (is_array($data->certificado_tipo_repasse_id) AND (!empty($data->certificado_tipo_repasse_id)) )) )
        {

            $filters[] = new TFilter('certificado_tipo_repasse_id', '=', $data->certificado_tipo_repasse_id);// create the filter 
        }

        if (isset($data->vendedor_id) AND ( (is_scalar($data->vendedor_id) AND $data->vendedor_id !== '') OR (is_array($data->vendedor_id) AND (!empty($data->vendedor_id)) )) )
        {

            $filters[] = new TFilter('vendedor_id', '=', $data->vendedor_id);// create the filter 
        }

        if (isset($data->protocolo_emissao) AND ( (is_scalar($data->protocolo_emissao) AND $data->protocolo_emissao !== '') OR (is_array($data->protocolo_emissao) AND (!empty($data->protocolo_emissao)) )) )
        {

            $filters[] = new TFilter('protocolo_emissao', 'like', "%{$data->protocolo_emissao}%");// create the filter 
        }

        if (isset($data->data_agendamento_inicial) AND ( (is_scalar($data->data_agendamento_inicial) AND $data->data_agendamento_inicial !== '') OR (is_array($data->data_agendamento_inicial) AND (!empty($data->data_agendamento_inicial)) )) )
        {

            $filters[] = new TFilter('data_hora_agendamento', '>=', $data->data_agendamento_inicial);// create the filter 
        }

        if (isset($data->data_agendamento_final) AND ( (is_scalar($data->data_agendamento_final) AND $data->data_agendamento_final !== '') OR (is_array($data->data_agendamento_final) AND (!empty($data->data_agendamento_final)) )) )
        {

            $filters[] = new TFilter('data_hora_agendamento', '<=', $data->data_agendamento_final);// create the filter 
        }

        if (isset($data->data_vencimento_certificado_inicial) AND ( (is_scalar($data->data_vencimento_certificado_inicial) AND $data->data_vencimento_certificado_inicial !== '') OR (is_array($data->data_vencimento_certificado_inicial) AND (!empty($data->data_vencimento_certificado_inicial)) )) )
        {

            $filters[] = new TFilter('data_vencimento_certificado', '>=', $data->data_vencimento_certificado_inicial);// create the filter 
        }

        if (isset($data->data_vencimento_certificado_final) AND ( (is_scalar($data->data_vencimento_certificado_final) AND $data->data_vencimento_certificado_final !== '') OR (is_array($data->data_vencimento_certificado_final) AND (!empty($data->data_vencimento_certificado_final)) )) )
        {

            $filters[] = new TFilter('data_vencimento_certificado', '<=', $data->data_vencimento_certificado_final);// create the filter 
        }

        if (isset($data->data_emissao_inicial) AND ( (is_scalar($data->data_emissao_inicial) AND $data->data_emissao_inicial !== '') OR (is_array($data->data_emissao_inicial) AND (!empty($data->data_emissao_inicial)) )) )
        {

            $filters[] = new TFilter('data_emissao', '>=', $data->data_emissao_inicial);// create the filter 
        }

        if (isset($data->data_emissao_final) AND ( (is_scalar($data->data_emissao_final) AND $data->data_emissao_final !== '') OR (is_array($data->data_emissao_final) AND (!empty($data->data_emissao_final)) )) )
        {

            $filters[] = new TFilter('data_emissao', '<=', $data->data_emissao_final);// create the filter 
        }

        if (isset($data->certificadora_id) AND ( (is_scalar($data->certificadora_id) AND $data->certificadora_id !== '') OR (is_array($data->certificadora_id) AND (!empty($data->certificadora_id)) )) )
        {

            $filters[] = new TFilter('certificadora_id', '=', $data->certificadora_id);// create the filter 
        }

        if (isset($data->data_cadastro_inicial) AND ( (is_scalar($data->data_cadastro_inicial) AND $data->data_cadastro_inicial !== '') OR (is_array($data->data_cadastro_inicial) AND (!empty($data->data_cadastro_inicial)) )) )
        {

            $filters[] = new TFilter('criado_em', '>=', $data->data_cadastro_inicial);// create the filter 
        }

        if (isset($data->data_cadastro_final) AND ( (is_scalar($data->data_cadastro_final) AND $data->data_cadastro_final !== '') OR (is_array($data->data_cadastro_final) AND (!empty($data->data_cadastro_final)) )) )
        {

            $filters[] = new TFilter('criado_em', '<=', $data->data_cadastro_final);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for CertificadoVenda
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            //</blockLine><btnShowCurtainFiltersAutoCode>
            if(!empty($this->btnShowCurtainFilters) && empty($this->btnShowCurtainFiltersAdjusted))
            {
                $this->btnShowCurtainFiltersAdjusted = true;
                $this->btnShowCurtainFilters->style = 'position: relative';
                $countFilters = count($filters ?? []);
                $this->btnShowCurtainFilters->setLabel($this->btnShowCurtainFilters->getLabel(). "<span class='badge badge-success' style='position: absolute'>{$countFilters}<span>");
            }
            //</blockLine></btnShowCurtainFiltersAutoCode>

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function onFinalizarAtendimentoYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $certvenda = CertificadoVenda::find($param['id']);     

            if($certvenda) 
            {
                //fazendo validações...
                if ($certvenda->data_vencimento_certificado==null)
                {
                   throw new Exception("Informe a data de vencimento do certificado!");  
                }  

                $listacertformapgto = CertificadoVendaFormapgto::where('certificado_venda_id', '=', $certvenda->id)->load();

                if ($certvenda->parceiro_id != null) 
                {
                    if ((!$listacertformapgto) && ($certvenda->certificado_tipo_repasse_id==CertificadoTipoRepasse::PrePago))
                    {
                       throw new Exception("Forma de pagamento não informada!");  
                    }                      
                }

                if (!$listacertformapgto && $certvenda->parceiro_id == null)
                {
                   throw new Exception("Forma de pagamento não informada!");  
                }

                $certvenda->certificado_venda_situacao_id=CertificadoVendaSituacao::Atendido;
                $certvenda->store();

                //salvando o historico na tabela de certificado cliente
                $certificadocliente = new CertificadoCliente;
                $certificadocliente->cliente_id                       = $certvenda->cliente_id;
                $certificadocliente->parceiro_id                      = $certvenda->parceiro_id;
                $certificadocliente->produto_id                       = $certvenda->produto_id;
                $certificadocliente->data_vencimento_certificado      = $certvenda->data_vencimento_certificado;                
                $certificadocliente->certificado_venda_id             = $certvenda->id;                                
                $certificadocliente->certificado_cliente_situacao_id  = CertificadoClienteSituacao::AguardandoRenovacao;                                                
                $certificadocliente->store();                     
            }

            TTransaction::close();

            //evento
            TTransaction::open(self::$database);

            Funcoes::VerificaNotificacaoEvento(TSession::getValue('userid'),
                                               CertificadoVenda::Evento_VendaCertificadoFinalizada,
                                               $certvenda,
                                               'Venda de certificado cliente '.$certvenda->cliente->nome.' , '.
                                               'Produto: '.$certvenda->produto->nome.' , '.
                                               'Valor: R$ '.number_format($certvenda->valor_total,2, ',', '.').' , '.
                                               'Data emissão: '.TDate::date2br($certvenda->data_emissao).' , '.
                                               'Protocolo: '.$certvenda->protocolo_emissao); 
            TTransaction::close();                 

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');              

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onFinalizarAtendimentoNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onDesfazerAtendimentoYes($param = null) 
    {
        try 
        {
            //validando a permissão...
            SystemProgramPermissoes::ValidaPermissaoEspecial('CertificadoVendaList',
                                                             CertificadoVenda::Permissao_DesfazerAtendimento,
                                                             TSession::getValue('userid'),
                                                             TSession::getValue('userunitid'));                     

            TTransaction::open(self::$database);

            $certvenda = CertificadoVenda::find($param['id']);     

            if($certvenda) 
            {
                $certvenda->certificado_venda_situacao_id=CertificadoVendaSituacao::AguardandoAtendimento;
                $certvenda->store();

                //deletando o registro do certificado do cliente...
                $certificadocliente = CertificadoCliente::where('certificado_venda_id', '=', $certvenda->id)->first();

                if ($certificadocliente)
                {
                    $certificadocliente->delete();
                }
            }

            TTransaction::close();

            //evento
            TTransaction::open(self::$database);

            Funcoes::VerificaNotificacaoEvento(TSession::getValue('userid'),
                                               CertificadoVenda::Evento_VendaCertificadoDesfeita,
                                               $certvenda,
                                               'Venda de certificado cliente '.$certvenda->cliente->nome.' , '.
                                               'Produto: '.$certvenda->produto->nome.' , '.
                                               'Valor: R$ '.number_format($certvenda->valor_total,2, ',', '.')); 
            TTransaction::close();               

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');  
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onDesfazerAtendimentoNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onAguardandoAprovacaoYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $certvenda = CertificadoVenda::find($param['id']);     

            if($certvenda) 
            {
                $certvenda->certificado_venda_situacao_id=CertificadoVendaSituacao::AguardandoAprovacao;
                $certvenda->store();
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');   
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onAguardandoAprovacaoNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

}

