<?php

class ClienteHistoricoLimiteForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'ClienteHistoricoLimite';
    private static $primaryKey = 'id';
    private static $formName = 'form_ClienteHistoricoLimiteForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Cadastro limite de crédito de cliente ");


        $id = new THidden('id');
        $cliente_id = new THidden('cliente_id');
        $data_hora = new TDateTime('data_hora');
        $valor = new TNumeric('valor', '2', ',', '.' );

        $data_hora->addValidation("Data/Hora", new TRequiredValidator()); 
        $valor->addValidation("Valor", new TRequiredValidator()); 

        $data_hora->setEditable(false);
        $data_hora->setMask('dd/mm/yyyy hh:ii');
        $data_hora->setDatabaseMask('yyyy-mm-dd hh:ii');
        $id->setValue('kraftsoft');
        $data_hora->setValue(date('Y-m-d H:i'));
        $cliente_id->setValue(TSession::getValue("cliente_id"));

        $id->setSize(200);
        $valor->setSize('100%');
        $cliente_id->setSize(200);
        $data_hora->setSize('100%');

        $row1 = $this->form->addFields([$id],[$cliente_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Data/Hora:", '#ff0000', '14px', null, '100%'),$data_hora],[new TLabel("Valor:", '#ff0000', '14px', null, '100%'),$valor]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=ClienteHistoricoLimiteForm]');
        $style->width = '50% !important';   
        $style->show(true);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new ClienteHistoricoLimite(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            Funcoes::SalvarUsuarioCadastro($object);

            $object->store(); // save the object 

            //atualizando o cadastro de cliente...
            $cliente = Cliente::find($object->cliente_id);
            $cliente->limite_credito = $object->valor;
            $cliente->store();

            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('ClienteHistoricoLimiteHeaderList', 'onShow', $loadPageParam); 

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new ClienteHistoricoLimite($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

