<?php

class ClienteList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Cliente';
    private static $primaryKey = 'id';
    private static $formName = 'form_ClienteList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Clientes");
        $this->limit = 100;

        $id = new TEntry('id');
        $system_unit_id = new TDBCombo('system_unit_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $nome = new TEntry('nome');
        $cpf_cnpj = new TEntry('cpf_cnpj');
        $cliente_classificacao_id = new TDBCombo('cliente_classificacao_id', 'kraftsoft', 'ClienteClassificacao', 'id', '{descricao}','descricao asc'  );
        $cliente_classificacao_subgrupo_id = new TCombo('cliente_classificacao_subgrupo_id');
        $razao_social = new TEntry('razao_social');

        $cliente_classificacao_id->setChangeAction(new TAction([$this,'onChangecliente_classificacao_id']));

        $id->setSize('100%');
        $nome->setSize('100%');
        $cpf_cnpj->setSize('100%');
        $razao_social->setSize('100%');
        $system_unit_id->setSize('100%');
        $cliente_classificacao_id->setSize('100%');
        $cliente_classificacao_subgrupo_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null),$id],[new TLabel("Filial de Origem:", null, '14px', null),$system_unit_id]);
        $row1->layout = [' col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Nome:", null, '14px', null, '100%'),$nome],[new TLabel("CPF/CNPJ:", null, '14px', null, '100%'),$cpf_cnpj]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Classificação:", null, '14px', null),$cliente_classificacao_id],[new TLabel("Sub-Grupo:", null, '14px', null),$cliente_classificacao_subgrupo_id]);
        $row3->layout = [' col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Razão Social:", null, '14px', null),$razao_social]);
        $row4->layout = ['col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );
        $this->fireEvents( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Cadastrar", new TAction(['ClienteForm', 'onShow']), 'fas:plus #69aa46');
        $this->btn_onshow = $btn_onshow;

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_nome = new TDataGridColumn('nome', "Nome", 'left');
        $column_razao_social = new TDataGridColumn('razao_social', "Razão Social", 'left');
        $column_cpf_cnpj_transformed = new TDataGridColumn('cpf_cnpj', "CPF/CNPJ", 'left');
        $column_vendedor_nome = new TDataGridColumn('vendedor->nome', "Vendedor", 'left');
        $column_system_unit_name = new TDataGridColumn('system_unit->name', "Filial Origem", 'left');
        $column_cliente_situacao_descricao_transformed = new TDataGridColumn('cliente_situacao->descricao', "Situação", 'left');

        $column_cpf_cnpj_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            return Funcoes::mascaraCpfCnpj($value);
        });

        $column_cliente_situacao_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            $label = $object->cliente_situacao->descricao;

            if ($object->cliente_situacao_id == ClienteSituacao::Ativo)
            {
                $cor = '#4CAF50';
            }

            if ($object->cliente_situacao_id == ClienteSituacao::Bloqueado)
            {
                $cor = '#FF9800';
            }

            if ($object->cliente_situacao_id == ClienteSituacao::Desativado)
            {
                $cor = '#F44336';
            }

             return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

            // $div = new TElement('div');
            // $div->style="text-shadow:none; font-size:14px; font-weight:bold; color:#FFFFFF; width:160px; text-align:center;background-color: $cor;padding:10px"; 
            // $div->add($label);

            // return $div;    //code here

        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_nome);
        $this->datagrid->addColumn($column_razao_social);
        $this->datagrid->addColumn($column_cpf_cnpj_transformed);
        $this->datagrid->addColumn($column_vendedor_nome);
        $this->datagrid->addColumn($column_system_unit_name);
        $this->datagrid->addColumn($column_cliente_situacao_descricao_transformed);

        $action_group = new TDataGridActionGroup("Opcões", 'fas:cog');
        $action_group->addHeader('');

        $action_onEdit = new TDataGridAction(array('ClienteForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);

        $action_group->addAction($action_onEdit);

        $action_onDelete = new TDataGridAction(array('ClienteList', 'onDelete'));
        $action_onDelete->setUseButton(TRUE);
        $action_onDelete->setButtonClass('btn btn-default');
        $action_onDelete->setLabel("Excluir");
        $action_onDelete->setImage('fas:trash-alt #dd5a43');
        $action_onDelete->setField(self::$primaryKey);

        $action_group->addAction($action_onDelete);

        $action_ClienteEspelhoFinanceiro_onEdit = new TDataGridAction(array('ClienteEspelhoFinanceiro', 'onEdit'));
        $action_ClienteEspelhoFinanceiro_onEdit->setUseButton(TRUE);
        $action_ClienteEspelhoFinanceiro_onEdit->setButtonClass('btn btn-default');
        $action_ClienteEspelhoFinanceiro_onEdit->setLabel("Espelho Financeiro");
        $action_ClienteEspelhoFinanceiro_onEdit->setImage('fas:money-bill #4CAF50');
        $action_ClienteEspelhoFinanceiro_onEdit->setField(self::$primaryKey);

        $action_group->addAction($action_ClienteEspelhoFinanceiro_onEdit);

        $action_onCertificadosDigitais = new TDataGridAction(array('ClienteList', 'onCertificadosDigitais'));
        $action_onCertificadosDigitais->setUseButton(TRUE);
        $action_onCertificadosDigitais->setButtonClass('btn btn-default');
        $action_onCertificadosDigitais->setLabel("Certificados Digitais");
        $action_onCertificadosDigitais->setImage('fas:fingerprint #009688');
        $action_onCertificadosDigitais->setField(self::$primaryKey);

        $action_group->addAction($action_onCertificadosDigitais);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['ClienteList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:table #00b894' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['ClienteList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['ClienteList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_right_actions->add($dropdown_button_exportar);

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Cliente","Clientes"]));
        }
        $container->add($this->form);
        $container->add($panel);

        parent::add($container);

    }

    public static function onChangecliente_classificacao_id($param)
    {
        try
        {

            if (isset($param['cliente_classificacao_id']) && $param['cliente_classificacao_id'])
            { 
                $criteria = TCriteria::create(['cliente_classificacao_id' => $param['cliente_classificacao_id']]);
                TDBCombo::reloadFromModel(self::$formName, 'cliente_classificacao_subgrupo_id', 'kraftsoft', 'ClienteClassificacaoSubgrupo', 'id', '{descricao}', 'descricao asc', $criteria, TRUE); 
            } 
            else 
            { 
                TCombo::clearField(self::$formName, 'cliente_classificacao_subgrupo_id'); 
            }  

        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    } 

    public function onDelete($param = null) 
    { 
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {

                //validando a permissão...
                SystemProgramPermissoes::ValidaPermissaoEspecial('ClienteList',
                                                                 Cliente::Permissao_ExcluirCliente,
                                                                 TSession::getValue('userid'),
                                                                 TSession::getValue('userunitid'));                              

                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new Cliente($key, FALSE); 

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }
    public function onCertificadosDigitais($param = null) 
    {
        try 
        {
            TSession::setValue('clientelist_id', $param['id']);

            TApplication::loadPage('CertificadoClienteFormList', 'OnShow', $param);
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('object');
                $object->data  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function fireEvents( $object )
    {
        $obj = new stdClass;
        if(is_object($object) && get_class($object) == 'stdClass')
        {
            if(isset($object->cliente_classificacao_id))
            {
                $value = $object->cliente_classificacao_id;

                $obj->cliente_classificacao_id = $value;
            }
            if(isset($object->cliente_classificacao_subgrupo_id))
            {
                $value = $object->cliente_classificacao_subgrupo_id;

                $obj->cliente_classificacao_subgrupo_id = $value;
            }
        }
        elseif(is_object($object))
        {
            if(isset($object->cliente_classificacao_id))
            {
                $value = $object->cliente_classificacao_id;

                $obj->cliente_classificacao_id = $value;
            }
            if(isset($object->cliente_classificacao_subgrupo_id))
            {
                $value = $object->cliente_classificacao_subgrupo_id;

                $obj->cliente_classificacao_subgrupo_id = $value;
            }
        }
        TForm::sendData(self::$formName, $obj);
    }  

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->system_unit_id) AND ( (is_scalar($data->system_unit_id) AND $data->system_unit_id !== '') OR (is_array($data->system_unit_id) AND (!empty($data->system_unit_id)) )) )
        {

            $filters[] = new TFilter('system_unit_id', '=', $data->system_unit_id);// create the filter 
        }

        if (isset($data->nome) AND ( (is_scalar($data->nome) AND $data->nome !== '') OR (is_array($data->nome) AND (!empty($data->nome)) )) )
        {

            $filters[] = new TFilter('nome', 'like', "%{$data->nome}%");// create the filter 
        }

        if (isset($data->cpf_cnpj) AND ( (is_scalar($data->cpf_cnpj) AND $data->cpf_cnpj !== '') OR (is_array($data->cpf_cnpj) AND (!empty($data->cpf_cnpj)) )) )
        {

            $filters[] = new TFilter('cpf_cnpj', 'like', "%{$data->cpf_cnpj}%");// create the filter 
        }

        if (isset($data->cliente_classificacao_id) AND ( (is_scalar($data->cliente_classificacao_id) AND $data->cliente_classificacao_id !== '') OR (is_array($data->cliente_classificacao_id) AND (!empty($data->cliente_classificacao_id)) )) )
        {

            $filters[] = new TFilter('cliente_classificacao_id', '=', $data->cliente_classificacao_id);// create the filter 
        }

        if (isset($data->cliente_classificacao_subgrupo_id) AND ( (is_scalar($data->cliente_classificacao_subgrupo_id) AND $data->cliente_classificacao_subgrupo_id !== '') OR (is_array($data->cliente_classificacao_subgrupo_id) AND (!empty($data->cliente_classificacao_subgrupo_id)) )) )
        {

            $filters[] = new TFilter('cliente_classificacao_subgrupo_id', '=', $data->cliente_classificacao_subgrupo_id);// create the filter 
        }

        if (isset($data->razao_social) AND ( (is_scalar($data->razao_social) AND $data->razao_social !== '') OR (is_array($data->razao_social) AND (!empty($data->razao_social)) )) )
        {

            $filters[] = new TFilter('razao_social', 'like', "%{$data->razao_social}%");// create the filter 
        }

        $this->fireEvents($data);

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Cliente
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

}

