<?php

class NotaFiscalCompraReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'NotaFiscalCompra';
    private static $primaryKey = 'id';
    private static $formName = 'form_NotaFiscalCompraReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Relatório de Notas Fiscais de Compra");

        $fornecedor_id = new TDBCombo('fornecedor_id', 'kraftsoft', 'Fornecedor', 'id', '{nome}','nome asc'  );
        $numero = new TEntry('numero');
        $data_emissao_inicial = new TDateTime('data_emissao_inicial');
        $data_emissao_final = new TDateTime('data_emissao_final');
        $data_cadastro_inicial = new TDateTime('data_cadastro_inicial');
        $data_cadastro_final = new TDateTime('data_cadastro_final');

        $fornecedor_id->enableSearch();
        $data_emissao_final->setMask('dd/mm/yyyy hh:ii');
        $data_cadastro_final->setMask('dd/mm/yyyy hh:ii');
        $data_emissao_inicial->setMask('dd/mm/yyyy hh:ii');
        $data_cadastro_inicial->setMask('dd/mm/yyyy hh:ii');

        $data_emissao_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_cadastro_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_emissao_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_cadastro_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');

        $numero->setSize('100%');
        $fornecedor_id->setSize('100%');
        $data_emissao_final->setSize('100%');
        $data_cadastro_final->setSize('100%');
        $data_emissao_inicial->setSize('100%');
        $data_cadastro_inicial->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Fornecedor:", null, '14px', null),$fornecedor_id],[new TLabel("Número:", null, '14px', null, '100%'),$numero]);
        $row1->layout = [' col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Data emissão Inicial:", null, '14px', null, '100%'),$data_emissao_inicial],[new TLabel("Data emissão Final:", null, '14px', null, '100%'),$data_emissao_final],[new TLabel("Data cadastro Inicial:", null, '14px', null, '100%'),$data_cadastro_inicial],[new TLabel("Data cadastro Final:", null, '14px', null, '100%'),$data_cadastro_final]);
        $row2->layout = ['col-sm-3','col-sm-3',' col-sm-3',' col-sm-3'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratehtml = $this->form->addAction("Gerar HTML", new TAction([$this, 'onGenerateHtml']), 'far:file-code #ffffff');
        $this->btn_ongeneratehtml = $btn_ongeneratehtml;
        $btn_ongeneratehtml->addStyleClass('btn-primary'); 

        $btn_ongeneratepdf = $this->form->addAction("Gerar PDF", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        $btn_ongeneratexls = $this->form->addAction("Gerar XLS", new TAction([$this, 'onGenerateXls']), 'far:file-excel #00a65a');
        $this->btn_ongeneratexls = $btn_ongeneratexls;

        $btn_ongeneratertf = $this->form->addAction("Gerar RTF", new TAction([$this, 'onGenerateRtf']), 'far:file-alt #324bcc');
        $this->btn_ongeneratertf = $btn_ongeneratertf;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Compras","Relatório de Notas Fiscais de Compra"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }

    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }

    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }

    public function onGenerateRtf($param = null) 
    {
        $this->onGenerate('rtf');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->fornecedor_id) AND ( (is_scalar($data->fornecedor_id) AND $data->fornecedor_id !== '') OR (is_array($data->fornecedor_id) AND (!empty($data->fornecedor_id)) )) )
        {

            $filters[] = new TFilter('fornecedor_id', '=', $data->fornecedor_id);// create the filter 
        }
        if (isset($data->numero) AND ( (is_scalar($data->numero) AND $data->numero !== '') OR (is_array($data->numero) AND (!empty($data->numero)) )) )
        {

            $filters[] = new TFilter('numero', 'like', "%{$data->numero}%");// create the filter 
        }
        if (isset($data->data_emissao_inicial) AND ( (is_scalar($data->data_emissao_inicial) AND $data->data_emissao_inicial !== '') OR (is_array($data->data_emissao_inicial) AND (!empty($data->data_emissao_inicial)) )) )
        {

            $filters[] = new TFilter('data_emissao', '>=', $data->data_emissao_inicial);// create the filter 
        }
        if (isset($data->data_emissao_final) AND ( (is_scalar($data->data_emissao_final) AND $data->data_emissao_final !== '') OR (is_array($data->data_emissao_final) AND (!empty($data->data_emissao_final)) )) )
        {

            $filters[] = new TFilter('data_emissao', '<=', $data->data_emissao_final);// create the filter 
        }
        if (isset($data->data_cadastro_inicial) AND ( (is_scalar($data->data_cadastro_inicial) AND $data->data_cadastro_inicial !== '') OR (is_array($data->data_cadastro_inicial) AND (!empty($data->data_cadastro_inicial)) )) )
        {

            $filters[] = new TFilter('criado_em', '>=', $data->data_cadastro_inicial);// create the filter 
        }
        if (isset($data->data_cadastro_final) AND ( (is_scalar($data->data_cadastro_final) AND $data->data_cadastro_final !== '') OR (is_array($data->data_cadastro_final) AND (!empty($data->data_cadastro_final)) )) )
        {

            $filters[] = new TFilter('criado_em', '<=', $data->data_cadastro_final);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            $param = [];
            // creates a repository for NotaFiscalCompra
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            $criteria->setProperties($param);

            if ($filters)
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            if ($objects)
            {
                $widths = array(200,200,200,200,200,200,200,200,200,200,200,200,200,200);
                $reportExtension = 'pdf';
                switch ($format)
                {
                    case 'html':
                        $tr = new TTableWriterHTML($widths);
                        $reportExtension = 'html';
                        break;
                    case 'xls':
                        $tr = new TTableWriterXLS($widths);
                        $reportExtension = 'xls';
                        break;
                    case 'pdf':
                        $tr = new TTableWriterPDF($widths, 'L', 'A4');
                        $reportExtension = 'pdf';
                        break;
                    case 'htmlPdf':
                        $reportExtension = 'pdf';
                        $tr = new BTableWriterHtmlPDF($widths, 'L', 'A4');
                        break;
                    case 'rtf':
                        if (!class_exists('PHPRtfLite_Autoloader'))
                        {
                            PHPRtfLite::registerAutoloader();
                        }
                        $reportExtension = 'rtf';
                        $tr = new TTableWriterRTF($widths, 'L', 'A4');
                        break;
                }

                if (!empty($tr))
                {
                    // create the document styles
                    $tr->addStyle('title', 'Helvetica', '10', 'B',   '#ffffff', '#96c0e6');
                    $tr->addStyle('datap', 'Arial', '10', '',    '#000000', '#f0f0f0');
                    $tr->addStyle('datai', 'Arial', '10', '',    '#000000', '#ffffff');
                    $tr->addStyle('header', 'Helvetica', '16', 'B',   '#4589c5', '#494D90');
                    $tr->addStyle('footer', 'Helvetica', '10', 'B',  '#4589c5', '#B1B1EA');
                    $tr->addStyle('break', 'Helvetica', '10', 'B',  '#ffffff', '#8da4bc');
                    $tr->addStyle('total', 'Helvetica', '10', 'I',  '#000000', '#b2d1dc');
                    $tr->addStyle('breakTotal', 'Helvetica', '10', 'I',  '#000000', '#b3d5f3');

                    // add titles row
                    $tr->addRow();
                    $tr->addCell("Número", 'left', 'title');
                    $tr->addCell("Criado Em", 'left', 'title');
                    $tr->addCell("Data Emissão", 'left', 'title');
                    $tr->addCell("Fornecedor", 'left', 'title');
                    $tr->addCell("CPF/CNPJ", 'left', 'title');
                    $tr->addCell("Base ICMS", 'left', 'title');
                    $tr->addCell("Valor ICMS", 'left', 'title');
                    $tr->addCell("Base ICMS ST", 'left', 'title');
                    $tr->addCell("Valor ICMS ST", 'left', 'title');
                    $tr->addCell("Frete", 'left', 'title');
                    $tr->addCell("IPI", 'left', 'title');
                    $tr->addCell("Outras Despesas", 'left', 'title');
                    $tr->addCell("Total Produtos", 'left', 'title');
                    $tr->addCell("Total Nota", 'left', 'title');

                    $grandTotal = [];
                    $breakTotal = [];
                    $breakValue = null;
                    $firstRow = true;

                    // controls the background filling
                    $colour = false;                
                    foreach ($objects as $object)
                    {
                        $style = $colour ? 'datap' : 'datai';

                        $grandTotal['numero'][] = $object->numero;
                        $breakTotal['numero'][] = $object->numero;
                        $grandTotal['icms_basecalc'][] = $object->icms_basecalc;
                        $breakTotal['icms_basecalc'][] = $object->icms_basecalc;
                        $grandTotal['icms_valor'][] = $object->icms_valor;
                        $breakTotal['icms_valor'][] = $object->icms_valor;
                        $grandTotal['icms_subst_base_calc'][] = $object->icms_subst_base_calc;
                        $breakTotal['icms_subst_base_calc'][] = $object->icms_subst_base_calc;
                        $grandTotal['icms_subst_valor'][] = $object->icms_subst_valor;
                        $breakTotal['icms_subst_valor'][] = $object->icms_subst_valor;
                        $grandTotal['frete_valor'][] = $object->frete_valor;
                        $breakTotal['frete_valor'][] = $object->frete_valor;
                        $grandTotal['ipi_valor'][] = $object->ipi_valor;
                        $breakTotal['ipi_valor'][] = $object->ipi_valor;
                        $grandTotal['outras_despesas'][] = $object->outras_despesas;
                        $breakTotal['outras_despesas'][] = $object->outras_despesas;
                        $grandTotal['valor_total_produtos'][] = $object->valor_total_produtos;
                        $breakTotal['valor_total_produtos'][] = $object->valor_total_produtos;
                        $grandTotal['valor_total_nota'][] = $object->valor_total_nota;
                        $breakTotal['valor_total_nota'][] = $object->valor_total_nota;

                        $firstRow = false;

                        $object->criado_em = call_user_func(function($value, $object, $row)
                        {
                            if(!empty(trim($value)))
                            {
                                try
                                {
                                    $date = new DateTime($value);
                                    return $date->format('d/m/Y H:i');
                                }
                                catch (Exception $e)
                                {
                                    return $value;
                                }
                            }
                        }, $object->criado_em, $object, null);

                        $object->data_emissao = call_user_func(function($value, $object, $row)
                        {
                            if(!empty(trim($value)))
                            {
                                try
                                {
                                    $date = new DateTime($value);
                                    return $date->format('d/m/Y H:i');
                                }
                                catch (Exception $e)
                                {
                                    return $value;
                                }
                            }
                        }, $object->data_emissao, $object, null);

                        $object->icms_basecalc = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->icms_basecalc, $object, null);

                        $object->icms_valor = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->icms_valor, $object, null);

                        $object->icms_subst_base_calc = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->icms_subst_base_calc, $object, null);

                        $object->icms_subst_valor = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->icms_subst_valor, $object, null);

                        $object->frete_valor = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->frete_valor, $object, null);

                        $object->ipi_valor = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->ipi_valor, $object, null);

                        $object->outras_despesas = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->outras_despesas, $object, null);

                        $object->valor_total_produtos = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->valor_total_produtos, $object, null);

                        $object->valor_total_nota = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->valor_total_nota, $object, null);

                        $tr->addRow();

                        $tr->addCell($object->numero, 'left', $style);
                        $tr->addCell($object->criado_em, 'left', $style);
                        $tr->addCell($object->data_emissao, 'left', $style);
                        $tr->addCell($object->fornecedor->nome, 'left', $style);
                        $tr->addCell($object->cliente_cpfcnpj, 'left', $style);
                        $tr->addCell($object->icms_basecalc, 'right', $style);
                        $tr->addCell($object->icms_valor, 'right', $style);
                        $tr->addCell($object->icms_subst_base_calc, 'right', $style);
                        $tr->addCell($object->icms_subst_valor, 'right', $style);
                        $tr->addCell($object->frete_valor, 'right', $style);
                        $tr->addCell($object->ipi_valor, 'right', $style);
                        $tr->addCell($object->outras_despesas, 'right', $style);
                        $tr->addCell($object->valor_total_produtos, 'right', $style);
                        $tr->addCell($object->valor_total_nota, 'right', $style);

                        $colour = !$colour;

                    }

                    $tr->addRow();

                    $grandTotal_numero = count($grandTotal['numero']);
                    $grandTotal_icms_basecalc = array_sum($grandTotal['icms_basecalc']);
                    $grandTotal_icms_valor = array_sum($grandTotal['icms_valor']);
                    $grandTotal_icms_subst_base_calc = array_sum($grandTotal['icms_subst_base_calc']);
                    $grandTotal_icms_subst_valor = array_sum($grandTotal['icms_subst_valor']);
                    $grandTotal_frete_valor = array_sum($grandTotal['frete_valor']);
                    $grandTotal_ipi_valor = array_sum($grandTotal['ipi_valor']);
                    $grandTotal_outras_despesas = array_sum($grandTotal['outras_despesas']);
                    $grandTotal_valor_total_produtos = array_sum($grandTotal['valor_total_produtos']);
                    $grandTotal_valor_total_nota = array_sum($grandTotal['valor_total_nota']);

                    $grandTotal_icms_basecalc = call_user_func(function($value)
                    {

                                if(!$value)
                                {
                                    $value = 0;
                                }

                                if(is_numeric($value))
                                {
                                    return number_format($value, 2, ",", ".");
                                }
                                else
                                {
                                    return $value;
                                }

                    }, $grandTotal_icms_basecalc); 

                    $grandTotal_icms_valor = call_user_func(function($value)
                    {

                                if(!$value)
                                {
                                    $value = 0;
                                }

                                if(is_numeric($value))
                                {
                                    return number_format($value, 2, ",", ".");
                                }
                                else
                                {
                                    return $value;
                                }

                    }, $grandTotal_icms_valor); 

                    $grandTotal_icms_subst_base_calc = call_user_func(function($value)
                    {

                                if(!$value)
                                {
                                    $value = 0;
                                }

                                if(is_numeric($value))
                                {
                                    return number_format($value, 2, ",", ".");
                                }
                                else
                                {
                                    return $value;
                                }

                    }, $grandTotal_icms_subst_base_calc); 

                    $grandTotal_icms_subst_valor = call_user_func(function($value)
                    {

                                if(!$value)
                                {
                                    $value = 0;
                                }

                                if(is_numeric($value))
                                {
                                    return number_format($value, 2, ",", ".");
                                }
                                else
                                {
                                    return $value;
                                }

                    }, $grandTotal_icms_subst_valor); 

                    $grandTotal_frete_valor = call_user_func(function($value)
                    {

                                if(!$value)
                                {
                                    $value = 0;
                                }

                                if(is_numeric($value))
                                {
                                    return number_format($value, 2, ",", ".");
                                }
                                else
                                {
                                    return $value;
                                }

                    }, $grandTotal_frete_valor); 

                    $grandTotal_ipi_valor = call_user_func(function($value)
                    {

                                if(!$value)
                                {
                                    $value = 0;
                                }

                                if(is_numeric($value))
                                {
                                    return number_format($value, 2, ",", ".");
                                }
                                else
                                {
                                    return $value;
                                }

                    }, $grandTotal_ipi_valor); 

                    $grandTotal_outras_despesas = call_user_func(function($value)
                    {

                                if(!$value)
                                {
                                    $value = 0;
                                }

                                if(is_numeric($value))
                                {
                                    return number_format($value, 2, ",", ".");
                                }
                                else
                                {
                                    return $value;
                                }

                    }, $grandTotal_outras_despesas); 

                    $grandTotal_valor_total_produtos = call_user_func(function($value)
                    {

                                if(!$value)
                                {
                                    $value = 0;
                                }

                                if(is_numeric($value))
                                {
                                    return number_format($value, 2, ",", ".");
                                }
                                else
                                {
                                    return $value;
                                }

                    }, $grandTotal_valor_total_produtos); 

                    $grandTotal_valor_total_nota = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $grandTotal_valor_total_nota); 

                    $tr->addCell($grandTotal_numero, 'left', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell($grandTotal_icms_basecalc, 'right', 'total');
                    $tr->addCell($grandTotal_icms_valor, 'right', 'total');
                    $tr->addCell($grandTotal_icms_subst_base_calc, 'right', 'total');
                    $tr->addCell($grandTotal_icms_subst_valor, 'right', 'total');
                    $tr->addCell($grandTotal_frete_valor, 'right', 'total');
                    $tr->addCell($grandTotal_ipi_valor, 'right', 'total');
                    $tr->addCell($grandTotal_outras_despesas, 'right', 'total');
                    $tr->addCell($grandTotal_valor_total_produtos, 'right', 'total');
                    $tr->addCell($grandTotal_valor_total_nota, 'right', 'total');

                    $file = 'report_'.uniqid().".{$reportExtension}";
                    // stores the file
                    if (!file_exists("app/output/{$file}") || is_writable("app/output/{$file}"))
                    {
                        $tr->save("app/output/{$file}");
                    }
                    else
                    {
                        throw new Exception(_t('Permission denied') . ': ' . "app/output/{$file}");
                    }

                    parent::openFile("app/output/{$file}");

                    // shows the success message
                    new TMessage('info', _t('Report generated. Please, enable popups'));
                }
            }
            else
            {
                new TMessage('error', _t('No records found'));
            }

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }


}

