<?php

class NotaFiscalCompraVincularProduto extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = '';
    private static $activeRecord = '';
    private static $primaryKey = '';
    private static $formName = 'form_NotaFiscalCompraVincularProduto';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Vincular produto a nota fiscal de compra");

        $criteria_produto_id = new TCriteria();

        $filterVar = TipoRecurso::Produto;
        $criteria_produto_id->add(new TFilter('tipo_recurso_id', '=', $filterVar)); 

        $nome = new TEntry('nome');
        $ean = new TEntry('ean');
        $ncm = new TEntry('ncm');
        $unidade_medida_id = new TDBCombo('unidade_medida_id', 'kraftsoft', 'UnidadeMedida', 'id', '{sigla}','nome asc'  );
        $produto_grupo_id = new TDBCombo('produto_grupo_id', 'kraftsoft', 'ProdutoGrupo', 'id', '{descricao}','descricao asc'  );
        $estoque_deposito_id = new TDBCombo('estoque_deposito_id', 'kraftsoft', 'EstoqueDeposito', 'id', '{descricao}','descricao asc'  );
        $button_opcao_1_cadastrar_novo_produto = new TButton('button_opcao_1_cadastrar_novo_produto');
        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{id} -{nome}','nome asc' , $criteria_produto_id );
        $button_opcao_2_vincular_produto_ja_cadastrado = new TButton('button_opcao_2_vincular_produto_ja_cadastrado');
        $nota_fiscal_compra_item_id = new THidden('nota_fiscal_compra_item_id');

        $nome->addValidation("Nome Produto", new TRequiredValidator()); 
        $unidade_medida_id->addValidation("Unidade", new TRequiredValidator()); 
        $produto_grupo_id->addValidation("Grupo", new TRequiredValidator()); 
        $estoque_deposito_id->addValidation("Depósito", new TRequiredValidator()); 

        $button_opcao_1_cadastrar_novo_produto->setAction(new TAction([$this, 'onBtnCadastrarNovoProduto']), "Opção 1: Cadastrar novo produto");
        $button_opcao_2_vincular_produto_ja_cadastrado->setAction(new TAction([$this, 'onBtnVincularProdutoCadastrado']), "Opção 2: Vincular produto já cadastrado");

        $button_opcao_1_cadastrar_novo_produto->addStyleClass('btn-primary');
        $button_opcao_2_vincular_produto_ja_cadastrado->addStyleClass('btn-success');

        $button_opcao_1_cadastrar_novo_produto->setImage('fas:plus-circle #FFFFFF');
        $button_opcao_2_vincular_produto_ja_cadastrado->setImage('fas:exchange-alt #FFFFFF');

        $produto_id->enableSearch();
        $produto_grupo_id->enableSearch();
        $unidade_medida_id->enableSearch();
        $estoque_deposito_id->enableSearch();

        $ean->setSize('100%');
        $ncm->setSize('100%');
        $nome->setSize('100%');
        $produto_id->setSize('100%');
        $produto_grupo_id->setSize('100%');
        $unidade_medida_id->setSize('100%');
        $estoque_deposito_id->setSize('100%');
        $nota_fiscal_compra_item_id->setSize(200);


        $row1 = $this->form->addContent([new TFormSeparator("Opção 1: Cadastrar novo produto", '#E91E63', '18', '#eee')]);
        $row2 = $this->form->addFields([new TLabel("Nome produto:", '#F44336', '14px', null, '100%'),$nome],[new TLabel("Código barras (EAN):", null, '14px', null),$ean]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("NCM:", null, '14px', null),$ncm],[new TLabel("Unidade:", '#F44336', '14px', null),$unidade_medida_id]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Grupo:", '#F44336', '14px', null),$produto_grupo_id],[new TLabel("Depósito padrão:", '#F44336', '14px', null),$estoque_deposito_id]);
        $row4->layout = ['col-sm-6','col-sm-6'];

        $row5 = $this->form->addFields([$button_opcao_1_cadastrar_novo_produto],[]);
        $row5->layout = ['col-sm-6','col-sm-6'];

        $row6 = $this->form->addContent([new TFormSeparator("", '#333', '18', '#eee')]);
        $row7 = $this->form->addContent([new TFormSeparator("Opção 2 - Vincular produto já cadastrado", '#E91E63', '18', '#EEEEEE')]);
        $row8 = $this->form->addFields([new TLabel("Produto:", null, '14px', null),$produto_id]);
        $row8->layout = [' col-sm-12'];

        $row9 = $this->form->addFields([$button_opcao_2_vincular_produto_ja_cadastrado]);
        $row9->layout = [' col-sm-12'];

        $row10 = $this->form->addFields([$nota_fiscal_compra_item_id]);
        $row10->layout = [' col-sm-12'];

        // create the form actions

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=NotaFiscalCompraVincularProduto]');
        $style->width = '70% !important';   
        $style->show(true);

    }

    public  function onBtnCadastrarNovoProduto($param = null) 
    {
        try 
        {
            TTransaction::open('kraftsoft'); // open a transaction

            $this->form->validate(); // validate form data         
            $data = $this->form->getData(); // get form data as array

            $produto = new Produto();
            $produto->permite_desconto = Resposta::Nao;
            $produto->tipo_recurso_id = TipoRecurso::Produto;
            $produto->notafiscal_item_orig_id = NotafiscalItemOrig::Zero;
            $produto->produto_tipo_id = ProdutoTipo::Simples;
            $produto->habilitado_id = Resposta::Sim;            

            $produto->nome = $data->nome;                 
            $produto->ean = $data->ean;
            $produto->ncm = $data->ncm;   
            $produto->unidade_medida_id = $data->unidade_medida_id;
            $produto->produto_grupo_id = $data->produto_grupo_id;                       
            Funcoes::SalvarUsuarioCadastro($produto);   

            $produto->store();

            //pegar a margem padrão se tiver...
            $margem = 0;
            $config_filial = ConfiguracoesFilial::where('system_unit_id', '=', TSession::getValue('userunitid'))->first();

            if($config_filial)
            {
                $margem = $config_filial->produto_margem_padrao;
            }

            //incluindo produto_filial..
            $prod_filial = new ProdutoFilial();
            $prod_filial->system_unit_id = TSession::getValue('userunitid');
            $prod_filial->produto_id = $produto->id;
            $prod_filial->estoque_deposito_padrao_id = $data->estoque_deposito_id;
            $prod_filial->margem = $margem;
            $prod_filial->preco_venda = 0;
            $prod_filial->store();

            //vincular o produto ao item da nota fiscal de compra..
            $notacompraitem = NotaFiscalCompraItens::find($param['nota_fiscal_compra_item_id']);
            $notacompraitem->produto_id = $produto->id;
            $notacompraitem->estoque_deposito_id =  $data->estoque_deposito_id;
            $notacompraitem->store();

            //analisar a situação da nota...
            NotaFiscalCompra::AnalisaVinculosNota($notacompraitem->nota_fiscal_compra_id);

            TTransaction::close();

            TToast::show('success', "Produto criado e vinculado com sucesso!", 'topRight', 'far:check-circle');

            $pageParam = ['key' => $notacompraitem->nota_fiscal_compra_id]; // ex.: = ['key' => 10]

            TApplication::loadPage('NotaFiscalCompraFormView', 'OnShow', $pageParam);    

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onBtnVincularProdutoCadastrado($param = null) 
    {
        try 
        {
            TTransaction::open('kraftsoft'); // open a transaction

            (new TRequiredValidator)->validate('Produto', $param['produto_id'] );

            //vincular o produto ao item da nota fiscal de compra..
            $notacompraitem = NotaFiscalCompraItens::find($param['nota_fiscal_compra_item_id']);
            $notacompraitem->produto_id = $param['produto_id'];
            $notacompraitem->store();

            //analisar a situação da nota...
            NotaFiscalCompra::AnalisaVinculosNota($notacompraitem->nota_fiscal_compra_id);

            TTransaction::close();

            TToast::show('success', "Produto vinculado com sucesso!", 'topRight', 'far:check-circle');

            $pageParam = ['key' => $notacompraitem->nota_fiscal_compra_id]; // ex.: = ['key' => 10]

            TApplication::loadPage('NotaFiscalCompraFormView', 'OnShow', $pageParam);    

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public function onShow($param = null)
    {               

        TTransaction::open('kraftsoft');

        $notacompraitem = NotaFiscalCompraItens::find($param['key']);

        if ($notacompraitem)
        {
            $object = new stdClass();

            $object->nota_fiscal_compra_item_id = $param['key'];    
            $object->nome                  = $notacompraitem->xprod;
            $object->ean                   = $notacompraitem->ean;
            $object->ncm                   = $notacompraitem->ncm;

            //tentando buscar a unidade de medida
            $unid = UnidadeMedida::where('sigla', '=', $notacompraitem->ucom)->first();

            if ($unid)
            {
              $object->unidade_medida_id = $unid->id;    
            }

            TTransaction::close();

            TForm::sendData(self::$formName, $object);

        }
        else 
        {
            throw new Exception('Nota fiscal de compra não encontrada!');
        }

    } 

}

