<?php

class NotafiscalTpagForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'NotafiscalTpag';
    private static $primaryKey = 'id';
    private static $formName = 'form_NotafiscalTpagForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Cadastro de Tipo Pagamento NF-e");


        $tpag = new TEntry('tpag');
        $id = new THidden('id');
        $descricao = new TEntry('descricao');
        $criado_em = new TDateTime('criado_em');
        $criado_por = new TEntry('criado_por');
        $alterado_em = new TDateTime('alterado_em');
        $alterado_por = new TEntry('alterado_por');

        $tpag->addValidation("tpag", new TRequiredValidator()); 
        $descricao->addValidation("Descrição", new TRequiredValidator()); 

        $criado_em->setMask('dd/mm/yyyy hh:ii');
        $alterado_em->setMask('dd/mm/yyyy hh:ii');

        $criado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $alterado_em->setDatabaseMask('yyyy-mm-dd hh:ii');

        $criado_em->setEditable(false);
        $criado_por->setEditable(false);
        $alterado_em->setEditable(false);
        $alterado_por->setEditable(false);

        $id->setSize(200);
        $tpag->setSize('100%');
        $descricao->setSize('100%');
        $criado_em->setSize('100%');
        $criado_por->setSize('100%');
        $alterado_em->setSize('100%');
        $alterado_por->setSize('100%');

        $criado_em->style = Funcoes::retornaCSSCamposDesabilitados();
        $alterado_em->style = Funcoes::retornaCSSCamposDesabilitados();
        $criado_por->style = Funcoes::retornaCSSCamposDesabilitados();
        $alterado_por->style = Funcoes::retornaCSSCamposDesabilitados();

        $row1 = $this->form->addFields([new TLabel("Código:", '#ff0000', '14px', null, '100%'),$tpag],[$id]);
        $row1->layout = ['col-sm-2','col-sm-2'];

        $row2 = $this->form->addFields([new TLabel("Descrição:", '#ff0000', '14px', null, '100%'),$descricao]);
        $row2->layout = [' col-sm-6'];

        $row3 = $this->form->addContent([new TFormSeparator("Histórico de Alterações", '#333', '18', '#eee')]);
        $row4 = $this->form->addFields([new TLabel("Criado Em:", null, '14px', null, '100%'),$criado_em],[new TLabel("Criado Por:", null, '14px', null, '100%'),$criado_por],[new TLabel("Alterado Em:", null, '14px', null, '100%'),$alterado_em],[new TLabel("Alterado Por:", null, '14px', null, '100%'),$alterado_por]);
        $row4->layout = [' col-sm-2',' col-sm-4',' col-sm-2',' col-sm-4'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onclear = $this->form->addAction("Limpar formulário", new TAction([$this, 'onClear']), 'fas:eraser #dd5a43');
        $this->btn_onclear = $btn_onclear;

        $btn_onshow = $this->form->addAction("Voltar", new TAction(['NotafiscalTpagList', 'onShow']), 'fas:arrow-alt-circle-left #000000');
        $this->btn_onshow = $btn_onshow;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Configurações","Cadastro de Tipo Pagamento NF-e"]));
        }
        $container->add($this->form);

        parent::add($container);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new NotafiscalTpag(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            Funcoes::SalvarUsuarioCadastro($object);

            $object->store(); // save the object 

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            new TMessage('info', "Registro salvo", $messageAction); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new NotafiscalTpag($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

