<?php

class NegocioAtividadeCalendarForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'NegocioAtividade';
    private static $primaryKey = 'id';
    private static $formName = 'form_NegocioAtividadeCalendarForm';
    private static $startDateField = 'horario_inicial';
    private static $endDateField = 'horario_final';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Atividades do Negócio");

        $view = new THidden('view');

        $id = new THidden('id');
        $negocio_id = new THidden('negocio_id');
        $negocio_tipo_atividade_id = new TDBCombo('negocio_tipo_atividade_id', 'kraftsoft', 'NegocioTipoAtividade', 'id', '{icone_formatado} {descricao}','descricao asc'  );
        $descricao = new TEntry('descricao');
        $horario_inicial = new TDateTime('horario_inicial');
        $horario_final = new TDateTime('horario_final');
        $observacao = new THtmlEditor('observacao');
        $criado_em = new TDateTime('criado_em');
        $criado_por = new TEntry('criado_por');
        $alterado_em = new TDateTime('alterado_em');
        $alterado_por = new TEntry('alterado_por');

        $negocio_tipo_atividade_id->addValidation("Negocio tipo atividade id", new TRequiredValidator()); 
        $descricao->addValidation("Descrição", new TRequiredValidator()); 

        $negocio_id->setValue(TSession::getValue("negocio_id"));
        $negocio_tipo_atividade_id->enableSearch();
        $criado_em->setMask('dd/mm/yyyy hh:ii');
        $alterado_em->setMask('dd/mm/yyyy hh:ii');
        $horario_final->setMask('dd/mm/yyyy hh:ii');
        $horario_inicial->setMask('dd/mm/yyyy hh:ii');

        $criado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $alterado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $horario_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $horario_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');

        $criado_em->setEditable(false);
        $criado_por->setEditable(false);
        $alterado_em->setEditable(false);
        $alterado_por->setEditable(false);

        $id->setSize(200);
        $negocio_id->setSize(200);
        $descricao->setSize('100%');
        $criado_em->setSize('100%');
        $criado_por->setSize('100%');
        $alterado_em->setSize('100%');
        $alterado_por->setSize('100%');
        $horario_final->setSize('100%');
        $horario_inicial->setSize('100%');
        $observacao->setSize('100%', 100);
        $negocio_tipo_atividade_id->setSize('100%');


        $row1 = $this->form->addFields([$id],[$negocio_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Atividade", '#ff0000', '14px', null, '100%'),$negocio_tipo_atividade_id],[new TLabel("Descrição:", '#ff0000', '14px', null, '100%'),$descricao]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Horário Inicial:", null, '14px', null, '100%'),$horario_inicial],[new TLabel("Horário Final:", null, '14px', null, '100%'),$horario_final]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Observação:", null, '14px', null, '100%'),$observacao]);
        $row4->layout = [' col-sm-12'];

        $bcontainer_63f53bf4f36a2 = new BContainer('bcontainer_63f53bf4f36a2');
        $this->bcontainer_63f53bf4f36a2 = $bcontainer_63f53bf4f36a2;

        $bcontainer_63f53bf4f36a2->setTitle("Histórico de Atualizações", '#333', '18px', '', '#fff');
        $bcontainer_63f53bf4f36a2->setBorderColor('#c0c0c0');

        $row5 = $bcontainer_63f53bf4f36a2->addFields([new TLabel("Criado Em:", null, '14px', null, '100%'),$criado_em],[new TLabel("Criado Por:", null, '14px', null, '100%'),$criado_por],[new TLabel("Alterado Em:", null, '14px', null, '100%'),$alterado_em],[new TLabel("Alterado Por:", null, '14px', null, '100%'),$alterado_por]);
        $row5->layout = [' col-sm-2',' col-sm-4',' col-sm-2',' col-sm-4'];

        $row6 = $this->form->addFields([$bcontainer_63f53bf4f36a2]);
        $row6->layout = [' col-sm-12'];

        $this->form->addFields([$view]);

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onclear = $this->form->addAction("Limpar formulário", new TAction([$this, 'onClear']), 'fas:eraser #dd5a43');
        $this->btn_onclear = $btn_onclear;

        $btn_ondelete = $this->form->addAction("Excluir", new TAction([$this, 'onDelete']), 'fas:trash-alt #dd5a43');
        $this->btn_ondelete = $btn_ondelete;

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=NegocioAtividadeCalendarForm]');
        $style->width = '70% !important';   
        $style->show(true);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new NegocioAtividade(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            Funcoes::SalvarUsuarioCadastro($object);    

            $object->store(); // save the object 

            $messageAction = new TAction(['NegocioAtividadeCalendarFormView', 'onReload']);
            $messageAction->setParameter('view', $data->view);
            $messageAction->setParameter('date', explode(' ', $data->horario_inicial)[0]);

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            new TMessage('info', "Registro salvo", $messageAction); 

            $paramTimeline = [
                'target_container' => 'container_timeline',
                'negocio_id' => $object->negocio_id
                ]; 

            TApplication::loadPage('NegocioTimeline', 'onShow', $paramTimeline);

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }
    public function onDelete($param = null) 
    {
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                $key = $param[self::$primaryKey];

                // open a transaction with database
                TTransaction::open(self::$database);

                $class = self::$activeRecord;

                // instantiates object
                $object = new $class($key, FALSE);

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                $messageAction = new TAction(array(__CLASS__.'View', 'onReload'));
                $messageAction->setParameter('view', $param['view']);
                $messageAction->setParameter('date', explode(' ',$param[self::$startDateField])[0]);

                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'), $messageAction);
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters((array) $this->form->getData());
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new NegocioAtividade($key); // instantiates the Active Record 

                                $object->view = !empty($param['view']) ? $param['view'] : 'agendaWeek'; 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public function onStartEdit($param)
    {

        $this->form->clear(true);

        $data = new stdClass;
        $data->view = $param['view'] ?? 'agendaWeek'; // calendar view
        $data->negocio_tipo_atividade = new stdClass();
        $data->negocio_tipo_atividade->cor = '#3a87ad';

        if (!empty($param['date']))
        {
            if(strlen($param['date']) == '10')
                $param['date'].= ' 09:00';

            $data->horario_inicial = str_replace('T', ' ', $param['date']);

            $horario_final = new DateTime($data->horario_inicial);
            $horario_final->add(new DateInterval('PT1H'));
            $data->horario_final = $horario_final->format('Y-m-d H:i:s');

        }

        $this->form->setData( $data );
    }

    public static function onUpdateEvent($param)
    {
        try
        {
            if (isset($param['id']))
            {
                TTransaction::open(self::$database);

                $class = self::$activeRecord;
                $object = new $class($param['id']);

                $object->horario_inicial = str_replace('T', ' ', $param['start_time']);
                $object->horario_final   = str_replace('T', ' ', $param['end_time']);

                $object->store();

                // close the transaction
                TTransaction::close();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', '<b>Error</b> ' . $e->getMessage());
            TTransaction::rollback();
        }
    }

    public static function getFormName()
    {
        return self::$formName;
    }

}

