<?php

class NegocioEmailForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = '';
    private static $activeRecord = '';
    private static $primaryKey = '';
    private static $formName = 'form_NegocioEmailForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Enviar e-mail de negócio");

        $criteria_negocios_id = new TCriteria();

        $filterVar = self::getNegocios_id($param);
        $criteria_negocios_id->add(new TFilter('id', 'in', $filterVar)); 

        $negocios_id = new TCheckList('negocios_id');
        $email_template_id = new TDBCombo('email_template_id', 'kraftsoft', 'EmailTemplate', 'id', '{titulo}','titulo asc'  );
        $mensagem = new THtmlEditor('mensagem');
        $negocio_id = new THidden('negocio_id');

        $email_template_id->setChangeAction(new TAction([$this,'onChangeTemplateEmail']));

        $negocios_id->addValidation("Negócios", new TRequiredValidator()); 
        $email_template_id->addValidation("Template de e-mail", new TRequiredValidator()); 

        $email_template_id->enableSearch();
        $negocio_id->setValue($param["negocio_id"] ?? "");
        $negocios_id->setValue(self::getNegocios_id($param));

        $negocio_id->setSize(200);
        $mensagem->setSize('100%', 200);
        $email_template_id->setSize('100%');

        $negocios_id->setIdColumn('id');

        $column_negocios_id_id = $negocios_id->addColumn('id', "Id", 'center' , '14%');
        $column_negocios_id_funil_descricao = $negocios_id->addColumn('funil->descricao', "Funil", 'center' , '14%');
        $column_negocios_id_funil_etapas_descricao = $negocios_id->addColumn('funil_etapas->descricao', "Etapa", 'center' , '14%');
        $column_negocios_id_negocio_origem_descricao = $negocios_id->addColumn('negocio_origem->descricao', "Origem", 'center' , '14%');
        $column_negocios_id_titulo = $negocios_id->addColumn('titulo', "Titulo", 'center' , '14%');
        $column_negocios_id_valor_transformed = $negocios_id->addColumn('valor', "Valor", 'center' , '14%');
        $column_negocios_id_data_abertura_transformed = $negocios_id->addColumn('data_abertura', "Data Abertura", 'center' , '14%');

        $column_negocios_id_valor_transformed->setTransformer(function($value, $object, $row) 
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_negocios_id_data_abertura_transformed->setTransformer(function($value, $object, $row) 
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });        

        $negocios_id->setHeight(200);
        $negocios_id->makeScrollable();

        $negocios_id->fillWith('kraftsoft', 'Negocio', 'id', 'id desc' , $criteria_negocios_id);


        $row1 = $this->form->addFields([new TLabel("Negócios:", '#F44336', '14px', null, '100%'),$negocios_id]);
        $row1->layout = [' col-sm-12'];

        $row2 = $this->form->addFields([new TLabel("Template de e-mail:", '#F44336', '14px', null, '100%'),$email_template_id]);
        $row2->layout = ['col-sm-12'];

        $row3 = $this->form->addFields([new TLabel("Mensagem:", '#F44336', '14px', null),$mensagem,$negocio_id]);
        $row3->layout = ['col-sm-12'];

        // create the form actions
        $btn_onenviaremail = $this->form->addAction("Enviar", new TAction([$this, 'onEnviarEmail']), 'fas:rocket #ffffff');
        $this->btn_onenviaremail = $btn_onenviaremail;
        $btn_onenviaremail->addStyleClass('btn-primary'); 

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=NegocioEmailForm]');
        $style->width = '60% !important';   
        $style->show(true);

    }

    public static function onChangeTemplateEmail($param = null) 
    {
        try 
        {
            if(!empty($param['key']))
            {
                TTransaction::open('kraftsoft');

                $EmailTemplate = new EmailTemplate($param['key']);

                TTransaction::close();

                $obj = new stdClass();
                $obj->mensagem = $EmailTemplate->mensagem;

                TForm::sendData(self::$formName, $obj);

            }

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public function onEnviarEmail($param = null) 
    {
        try
        {
           $this->form->validate();
           $data = $this->form->getData();

        //   var_dump($data);

           $mensagem = $data->mensagem;

           if($data->negocios_id)
           {
                TTransaction::open('kraftsoft');
                $EmailTemplate = new EmailTemplate($data->email_template_id);               

                foreach($data->negocios_id as $negocio_id)
                {
                    $negocio = new Negocio($negocio_id);

                    $mensagem = str_replace('{contato_nome}', $negocio->contato_nome, $mensagem);
                    $EmailTemplate->titulo = str_replace('{contato_nome}', $negocio->contato_nome, $EmailTemplate->titulo);

                    if($negocio->cliente->email)
                    {
                        MailService::send($negocio->cliente->email, $EmailTemplate->titulo, $mensagem, 'html');
                    }
                    else if ($negocio->contato_email)
                    {
                        MailService::send($negocio->contato_email, $EmailTemplate->titulo, $mensagem, 'html');
                    }

                }

                TTransaction::close();
           }

           $this->form->setData($data);
           new TMessage('info', 'Email(s) enviado(s)!');

           if (!$data->negocios_id)
           {
               TSession::setValue('NegocioListbuilder_datagrid_check', null);

              TApplication::loadPage('NegocioList', 'onShow');
           }

           //fecha a cortina lateral...
          TScript::create("Template.closeRightPanel();");
        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    }

    public function onShow($param = null)
    {               

    } 

    public static function getNegocios_id($param)
    {
        if(!empty($param['negocio_id']))
        {
            return [ $param['negocio_id'] ];
        }
        else if(TSession::getValue('NegocioListbuilder_datagrid_check'))
        {
            return TSession::getValue('NegocioListbuilder_datagrid_check');                     
        }

        return [-1];  
    }

}

