<?php

class NegocioForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Negocio';
    private static $primaryKey = 'id';
    private static $formName = 'form_NegocioForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Negócio");


        $id = new TEntry('id');
        $negocio_origem_id = new TDBCombo('negocio_origem_id', 'kraftsoft', 'NegocioOrigem', 'id', '{descricao}','descricao asc'  );
        $funil_id = new TDBCombo('funil_id', 'kraftsoft', 'Funil', 'id', '{descricao}','descricao asc'  );
        $funil_etapas_id = new TCombo('funil_etapas_id');
        $data_abertura = new TDate('data_abertura');
        $data_fechamento_esperada = new TDate('data_fechamento_esperada');
        $contato_nome = new TEntry('contato_nome');
        $contato_fone = new TEntry('contato_fone');
        $contato_email = new TEntry('contato_email');
        $titulo = new TEntry('titulo');
        $valor = new TNumeric('valor', '2', ',', '.' );
        $cliente_id = new TDBUniqueSearch('cliente_id', 'kraftsoft', 'Cliente', 'id', 'nome','nome asc'  );
        $vendedor_id = new TDBCombo('vendedor_id', 'kraftsoft', 'Vendedor', 'id', '{nome}','nome asc'  );
        $criado_em = new TDateTime('criado_em');
        $criado_por = new TEntry('criado_por');
        $alterado_em = new TDateTime('alterado_em');
        $alterado_por = new TEntry('alterado_por');
        $negocio_motivo_perda_id = new TDBCombo('negocio_motivo_perda_id', 'kraftsoft', 'NegocioMotivoPerda', 'id', '{descricao}','descricao asc'  );
        $comentarios_motivo_perda = new THtmlEditor('comentarios_motivo_perda');

        $funil_id->setChangeAction(new TAction([$this,'onChangefunil_id']));

        $negocio_origem_id->addValidation("Origem", new TRequiredValidator()); 
        $funil_id->addValidation("Funil", new TRequiredValidator()); 
        $funil_etapas_id->addValidation("Etapa Funil", new TRequiredValidator()); 
        $data_abertura->addValidation("Data Abertura", new TRequiredValidator()); 
        $contato_nome->addValidation("Nome Contato", new TRequiredValidator()); 
        $contato_fone->addValidation("Fone Contato", new TRequiredValidator()); 
        $titulo->addValidation("Titulo", new TRequiredValidator()); 

        $data_abertura->setValue(date('d/m/Y'));
        $cliente_id->setMinLength(2);
        $cliente_id->setFilterColumns(["cpf_cnpj","razao_social","nome"]);
        $funil_id->enableSearch();
        $vendedor_id->enableSearch();
        $funil_etapas_id->enableSearch();
        $negocio_origem_id->enableSearch();

        $data_abertura->setDatabaseMask('yyyy-mm-dd');
        $criado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $alterado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_fechamento_esperada->setDatabaseMask('yyyy-mm-dd');

        $cliente_id->setMask('{nome}');
        $data_abertura->setMask('dd/mm/yyyy');
        $criado_em->setMask('dd/mm/yyyy hh:ii');
        $alterado_em->setMask('dd/mm/yyyy hh:ii');
        $data_fechamento_esperada->setMask('dd/mm/yyyy');

        $id->setEditable(false);
        $valor->setEditable(false);
        $criado_em->setEditable(false);
        $criado_por->setEditable(false);
        $alterado_em->setEditable(false);
        $alterado_por->setEditable(false);

        $id->setSize(100);
        $valor->setSize('100%');
        $titulo->setSize('100%');
        $funil_id->setSize('100%');
        $criado_em->setSize('100%');
        $cliente_id->setSize('100%');
        $criado_por->setSize('100%');
        $vendedor_id->setSize('100%');
        $alterado_em->setSize('100%');
        $contato_nome->setSize('100%');
        $contato_fone->setSize('100%');
        $alterado_por->setSize('100%');
        $data_abertura->setSize('100%');
        $contato_email->setSize('100%');
        $funil_etapas_id->setSize('100%');
        $negocio_origem_id->setSize('100%');
        $negocio_motivo_perda_id->setSize('100%');
        $data_fechamento_esperada->setSize('100%');
        $comentarios_motivo_perda->setSize('100%', 110);


        $this->form->appendPage("Principal");

        $this->form->addFields([new THidden('current_tab')]);
        $this->form->setTabFunction("$('[name=current_tab]').val($(this).attr('data-current_page'));");

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Origem:", '#ff0000', '14px', null, '100%'),$negocio_origem_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Funil:", '#FF0000', '14px', null, '100%'),$funil_id],[new TLabel("Etapa Funil:", '#FF0000', '14px', null, '100%'),$funil_etapas_id]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Data abertura:", '#ff0000', '14px', null, '100%'),$data_abertura],[new TLabel("Data esperada fechamento:", null, '14px', null, '100%'),$data_fechamento_esperada]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Nome contato:", '#FF0000', '14px', null, '100%'),$contato_nome],[new TLabel("Fone contato:", '#FF0000', '14px', null, '100%'),$contato_fone]);
        $row4->layout = ['col-sm-6','col-sm-6'];

        $row5 = $this->form->addFields([new TLabel("E-mail contato:", null, '14px', null),$contato_email],[]);
        $row5->layout = ['col-sm-6','col-sm-6'];

        $row6 = $this->form->addFields([new TLabel("Titulo:", '#ff0000', '14px', null, '100%'),$titulo],[new TLabel("Valor:", null, '14px', null, '100%'),$valor]);
        $row6->layout = ['col-sm-6','col-sm-6'];

        $row7 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null),$cliente_id],[new TLabel("Vendedor:", null, '14px', null),$vendedor_id]);
        $row7->layout = ['col-sm-6','col-sm-6'];

        $bcontainer_63f52f22cd2b3 = new BContainer('bcontainer_63f52f22cd2b3');
        $this->bcontainer_63f52f22cd2b3 = $bcontainer_63f52f22cd2b3;

        $bcontainer_63f52f22cd2b3->setTitle("Histórico de Atualizações", '#333', '18px', '', '#fff');
        $bcontainer_63f52f22cd2b3->setBorderColor('#c0c0c0');

        $row8 = $bcontainer_63f52f22cd2b3->addFields([new TLabel("Criado Em:", null, '14px', null, '100%'),$criado_em],[new TLabel("Criado Por:", null, '14px', null, '100%'),$criado_por],[new TLabel("Alterado Em:", null, '14px', null, '100%'),$alterado_em],[new TLabel("Alterado Por:", null, '14px', null, '100%'),$alterado_por]);
        $row8->layout = [' col-sm-2',' col-sm-4',' col-sm-2',' col-sm-4'];

        $row9 = $this->form->addFields([$bcontainer_63f52f22cd2b3]);
        $row9->layout = [' col-sm-12'];

        $this->form->appendPage("Perda");
        $row10 = $this->form->addFields([new TLabel("Motivo Perda:", null, '14px', null, '100%'),$negocio_motivo_perda_id]);
        $row10->layout = [' col-sm-12'];

        $row11 = $this->form->addFields([new TLabel("Comentário Motivo Perda:", null, '14px', null, '100%'),$comentarios_motivo_perda]);
        $row11->layout = [' col-sm-12'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onclear = $this->form->addAction("Limpar formulário", new TAction([$this, 'onClear']), 'fas:eraser #dd5a43');
        $this->btn_onclear = $btn_onclear;

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=NegocioForm]');
        $style->width = '80% !important';   
        $style->show(true);

    }

    public static function onChangefunil_id($param)
    {
        try
        {

            if (isset($param['funil_id']) && $param['funil_id'])
            { 
                $criteria = TCriteria::create(['funil_id' => $param['funil_id']]);
                TDBCombo::reloadFromModel(self::$formName, 'funil_etapas_id', 'kraftsoft', 'FunilEtapas', 'id', '{descricao}', 'numero_etapa asc', $criteria, TRUE); 
            } 
            else 
            { 
                TCombo::clearField(self::$formName, 'funil_etapas_id'); 
            }  

        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    } 

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new Negocio(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);

            if (!$data->id)
            {
                $negocio = Negocio::select('max(ordem) as ordem')->where('system_unit_id', '=', $object->system_unit_id)
                         ->where('funil_etapas_id', '=', $object->funil_etapas_id)
                         ->where('ativo_id', '=', Ativo::Ativo)->first();

                if ($negocio)
                {
                    $object->ordem = $negocio->ordem + 1;
                }

                $object->system_unit_id = TSession::getValue('userunitid');     
                $object->arquivado = Resposta::Nao;
                $object->ativo_id = Ativo::Ativo;
            }

            $object->store(); // save the object 

            if (!$data->id)
            {
                //historico
                $negocioEtapaHistorico = new NegocioEtapasHistorico();
                $negocioEtapaHistorico->funil_etapas_id = $object->funil_etapas_id;
                $negocioEtapaHistorico->negocio_id = $object->id;

                Funcoes::SalvarUsuarioCadastro($negocioEtapaHistorico);

                $negocioEtapaHistorico->store();     
            }

            $this->fireEvents($object);

            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            if(!empty($object->id))
            {
                $loadPageParam["key"] = $object->id;
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            //evento
            TTransaction::open(self::$database);

            Funcoes::VerificaNotificacaoEvento(TSession::getValue('userid'),
                                               Negocio::Evento_NegocioIncluidoOuAlterado,
                                               $object,
                                               'Título: '.$object->titulo.
                                               ', Nome Contato: '.$object->contato_nome.
                                               ', Origem: '.$object->negocio_origem->descricao); 

            TTransaction::close();     

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('NegocioFormView', 'onShow', $loadPageParam); 

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new Negocio($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                $this->fireEvents($object);

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

        if (!empty($param['key']))
        {
            $obj = new stdClass;
            $obj->funil_etapas_id = $param['key'];

            $this->form->setData($obj);
        }

    } 

    public function fireEvents( $object )
    {
        $obj = new stdClass;
        if(is_object($object) && get_class($object) == 'stdClass')
        {
            if(isset($object->funil_id))
            {
                $value = $object->funil_id;

                $obj->funil_id = $value;
            }
            if(isset($object->funil_etapas_id))
            {
                $value = $object->funil_etapas_id;

                $obj->funil_etapas_id = $value;
            }
        }
        elseif(is_object($object))
        {
            if(isset($object->funil_id))
            {
                $value = $object->funil_id;

                $obj->funil_id = $value;
            }
            if(isset($object->funil_etapas_id))
            {
                $value = $object->funil_etapas_id;

                $obj->funil_etapas_id = $value;
            }
        }
        TForm::sendData(self::$formName, $obj);
    }  

    public static function getFormName()
    {
        return self::$formName;
    }

}

