<?php

class NegocioItensForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'NegocioItens';
    private static $primaryKey = 'id';
    private static $formName = 'form_NegocioItensForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Cadastro de itens de negócio");


        $id = new THidden('id');
        $negocio_id = new THidden('negocio_id');
        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc'  );
        $valor_unitario = new TNumeric('valor_unitario', '2', ',', '.' );
        $quantidade = new TNumeric('quantidade', '2', ',', '.' );
        $valor_total = new TNumeric('valor_total', '2', ',', '.' );

        $produto_id->setChangeAction(new TAction([$this,'onExitProduto']));

        $valor_unitario->setExitAction(new TAction([$this,'onExitValorUnitario']));
        $quantidade->setExitAction(new TAction([$this,'onExitQuantidade']));

        $produto_id->addValidation("Produto", new TRequiredValidator()); 
        $valor_unitario->addValidation("Valor Unitário", new TRequiredValidator()); 
        $quantidade->addValidation("Quantidade", new TRequiredValidator()); 
        $valor_total->addValidation("Valor Total", new TRequiredValidator()); 

        $negocio_id->setValue(TSession::getValue("negocio_id"));
        $produto_id->enableSearch();
        $valor_total->setEditable(false);
        $id->setSize(200);
        $negocio_id->setSize(200);
        $produto_id->setSize('100%');
        $quantidade->setSize('100%');
        $valor_total->setSize('100%');
        $valor_unitario->setSize('100%');

        $row1 = $this->form->addFields([$id],[$negocio_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Produto:", '#ff0000', '14px', null, '100%'),$produto_id]);
        $row2->layout = [' col-sm-12'];

        $row3 = $this->form->addFields([new TLabel("Valor Unitário:", '#ff0000', '14px', null, '100%'),$valor_unitario],[new TLabel("Quantidade:", '#ff0000', '14px', null, '100%'),$quantidade],[new TLabel("Valor Total:", '#ff0000', '14px', null, '100%'),$valor_total]);
        $row3->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=NegocioItensForm]');
        $style->width = '60% !important';   
        $style->show(true);

    }

    public static function onExitValorUnitario($param = null) 
    {
        try 
        {
           self::onExitQuantidade($param);

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onExitQuantidade($param = null) 
    {
        try 
        {
            if (!empty($param['quantidade']) && !empty($param['valor_unitario']))
            {
                $valor_unitario = (double) str_replace(',', '.', str_replace('.', '', $param['valor_unitario']));
                $quantidade = (double) str_replace(',', '.', str_replace('.', '', $param['quantidade']));

                $valor_total = $valor_unitario * $quantidade ;
                $object = new stdClass();
                $object->valor_total = number_format($valor_total, 2, ',', '.');
                TForm::sendData(self::$formName, $object);
            }

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onExitProduto($param = null) 
    {
        try 
        {
            //code here
            if (isset($param['produto_id']) && $param['produto_id'])
            {
                TTransaction::open(self::$database);

                $object = new stdClass();

                //trazer o preço da filial..
                $produtofilial = ProdutoFilial::where('produto_id', '=', $param['produto_id'])
                                              ->where('system_unit_id', '=', TSession::getValue('userunitid'))
                                              ->first();

                if($produtofilial)
                {                        
                    $valorproduto = $produtofilial->preco_venda;

                    $object->quantidade = 1;
                    $object->valor_unitario = number_format($valorproduto, 2, ',', '.');  
                    $object->valor_total = number_format($valorproduto, 2, ',', '.');  
                }
                else
                {
                    new TMessage('error', 'Este produto não tem vinculo configurado para esta filial!');
                    $object->produto_id = '';
                }

                TTransaction::close();       

                TForm::sendData(self::$formName, $object);
            }

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new NegocioItens(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            $object->store(); // save the object 

            Negocio::TotalizaNegocio($object->negocio_id);

            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('NegocioItensHeaderList', 'onShow', $loadPageParam); 

            $paramTimeline = [
                'target_container' => 'container_timeline',
                'negocio_id' => $object->negocio_id
                ]; 

            TApplication::loadPage('NegocioTimeline', 'onShow', $paramTimeline);

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new NegocioItens($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

