<?php

class NegocioList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Negocio';
    private static $primaryKey = 'id';
    private static $formName = 'form_NegocioList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Consultar Negócios");
        $this->limit = 100;

        $criteria_funil_etapas_funil_id = new TCriteria();

        $funil_etapas_funil_id = new TDBCombo('funil_etapas_funil_id', 'kraftsoft', 'Funil', 'id', '{descricao}','descricao asc' , $criteria_funil_etapas_funil_id );
        $funil_etapas_id = new TCombo('funil_etapas_id');
        $titulo = new TEntry('titulo');
        $contato_nome = new TEntry('contato_nome');
        $cliente_id = new TDBUniqueSearch('cliente_id', 'kraftsoft', 'Cliente', 'id', 'nome','nome asc'  );
        $vendedor_id = new TDBCombo('vendedor_id', 'kraftsoft', 'Vendedor', 'id', '{nome}','nome asc'  );
        $negocio_origem_id = new TDBCombo('negocio_origem_id', 'kraftsoft', 'NegocioOrigem', 'id', '{descricao}','descricao asc'  );
        $arquivado = new TDBCombo('arquivado', 'kraftsoft', 'Resposta', 'id', '{descricao}','descricao asc'  );
        $data_abertura_inicial = new TDate('data_abertura_inicial');
        $data_abertura_final = new TDate('data_abertura_final');
        $data_fechamento_inicial = new TDate('data_fechamento_inicial');
        $data_fechamento_final = new TDate('data_fechamento_final');

        $funil_etapas_funil_id->setChangeAction(new TAction([$this,'onChangefunil_etapas_funil_id']));

        $cliente_id->setMinLength(2);
        $cliente_id->setFilterColumns(["cpf_cnpj","razao_social","nome"]);
        $data_abertura_final->setDatabaseMask('yyyy-mm-dd');
        $data_abertura_inicial->setDatabaseMask('yyyy-mm-dd');
        $data_fechamento_final->setDatabaseMask('yyyy-mm-dd');
        $data_fechamento_inicial->setDatabaseMask('yyyy-mm-dd');

        $arquivado->enableSearch();
        $vendedor_id->enableSearch();
        $funil_etapas_id->enableSearch();
        $negocio_origem_id->enableSearch();
        $funil_etapas_funil_id->enableSearch();

        $cliente_id->setMask('{nome}');
        $data_abertura_final->setMask('dd/mm/yyyy');
        $data_abertura_inicial->setMask('dd/mm/yyyy');
        $data_fechamento_final->setMask('dd/mm/yyyy');
        $data_fechamento_inicial->setMask('dd/mm/yyyy');

        $titulo->setSize('100%');
        $arquivado->setSize('100%');
        $cliente_id->setSize('100%');
        $vendedor_id->setSize('100%');
        $contato_nome->setSize('100%');
        $funil_etapas_id->setSize('100%');
        $negocio_origem_id->setSize('100%');
        $data_abertura_final->setSize('100%');
        $funil_etapas_funil_id->setSize('100%');
        $data_abertura_inicial->setSize('100%');
        $data_fechamento_final->setSize('100%');
        $data_fechamento_inicial->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Funil:", null, '14px', null),$funil_etapas_funil_id],[new TLabel("Etapa Funil:", null, '14px', null),$funil_etapas_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Titulo:", null, '14px', null, '100%'),$titulo],[new TLabel("Contato:", null, '14px', null, '100%'),$contato_nome]);
        $row2->layout = ['col-sm-6',' col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null, '100%'),$cliente_id],[new TLabel(" Vendedor:", null, '14px', null, '100%'),$vendedor_id]);
        $row3->layout = ['col-sm-6',' col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Origem:", null, '14px', null),$negocio_origem_id],[new TLabel("Arquivado:", null, '14px', null),$arquivado]);
        $row4->layout = ['col-sm-6',' col-sm-6'];

        $row5 = $this->form->addFields([new TLabel("Data abertura inicial:", null, '14px', null, '100%'),$data_abertura_inicial],[new TLabel("Data abertura final:", null, '14px', null, '100%'),$data_abertura_final],[new TLabel("Data fechamento inicial:", null, '14px', null, '100%'),$data_fechamento_inicial],[new TLabel("Data fechamento final:", null, '14px', null, '100%'),$data_fechamento_final]);
        $row5->layout = [' col-sm-3',' col-sm-3',' col-sm-3',' col-sm-3'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );
        $this->fireEvents( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_orcamento_id = new TDataGridColumn('orcamento_id', "Orçamento", 'left');
        $column_data_abertura_transformed = new TDataGridColumn('data_abertura', "Data Abertura", 'left');
        $column_negocio_origem_descricao = new TDataGridColumn('negocio_origem->descricao', "Origem", 'left');
        $column_titulo = new TDataGridColumn('titulo', "Titulo", 'left');
        $column_contato_nome = new TDataGridColumn('contato_nome', "Nome Contato", 'left');
        $column_data_fechamento_transformed = new TDataGridColumn('data_fechamento', "Data Fechamento", 'left');
        $column_vendedor_nome = new TDataGridColumn('vendedor->nome', "Vendedor", 'left');
        $column_valor_transformed = new TDataGridColumn('valor', "Valor", 'left');
        $column_funil_etapas_funil_descricao = new TDataGridColumn('funil_etapas->funil->descricao', "Funil", 'left');
        $column_funil_etapas_descricao_transformed = new TDataGridColumn('funil_etapas->descricao', "Etapa", 'left');
        $column_fk_arquivado_descricao_transformed = new TDataGridColumn('fk_arquivado->descricao', "Arquivado", 'left');

        $column_data_abertura_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_data_fechamento_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_valor_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_funil_etapas_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {

            return "<span class='label' style='background-color:{$object->funil_etapas->cor_etapa}'>{$object->funil_etapas->descricao}</span>";

        });

        $column_fk_arquivado_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {

            $cor = '#F44336';
            $label = Resposta::find($object->arquivado)->descricao;

            if ($object->arquivado == Resposta::Sim)
            {
                $cor = '#4CAF50';
            }

            if ($object->arquivado == Resposta::Nao)
            {
                $cor = '#F44336';
            }

            return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->builder_datagrid_check_all = new TCheckButton('builder_datagrid_check_all');
        $this->builder_datagrid_check_all->setIndexValue('on');
        $this->builder_datagrid_check_all->onclick = "Builder.checkAll(this)";
        $this->builder_datagrid_check_all->style = 'cursor:pointer';
        $this->builder_datagrid_check_all->setProperty('class', 'filled-in');
        $this->builder_datagrid_check_all->id = 'builder_datagrid_check_all';

        $label = new TLabel('');
        $label->style = 'margin:0';
        $label->class = 'checklist-label';
        $this->builder_datagrid_check_all->after($label);
        $label->for = 'builder_datagrid_check_all';

        $this->builder_datagrid_check = $this->datagrid->addColumn( new TDataGridColumn('builder_datagrid_check', $this->builder_datagrid_check_all, 'center',  '1%') );

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_orcamento_id);
        $this->datagrid->addColumn($column_data_abertura_transformed);
        $this->datagrid->addColumn($column_negocio_origem_descricao);
        $this->datagrid->addColumn($column_titulo);
        $this->datagrid->addColumn($column_contato_nome);
        $this->datagrid->addColumn($column_data_fechamento_transformed);
        $this->datagrid->addColumn($column_vendedor_nome);
        $this->datagrid->addColumn($column_valor_transformed);
        $this->datagrid->addColumn($column_funil_etapas_funil_descricao);
        $this->datagrid->addColumn($column_funil_etapas_descricao_transformed);
        $this->datagrid->addColumn($column_fk_arquivado_descricao_transformed);

        $action_onShow = new TDataGridAction(array('NegocioFormView', 'onShow'));
        $action_onShow->setUseButton(true);
        $action_onShow->setButtonClass('btn btn-default btn-sm');
        $action_onShow->setLabel("Abrir");
        $action_onShow->setImage('fas:search-plus #03A9F4');
        $action_onShow->setField(self::$primaryKey);

        $action_onShow->setParameter('key', '{id}');

        $this->datagrid->addAction($action_onShow);

        $action_onEdit = new TDataGridAction(array('NegocioForm', 'onEdit'));
        $action_onEdit->setUseButton(true);
        $action_onEdit->setButtonClass('btn btn-default btn-sm');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('NegocioList::onExibirEditar');

        $this->datagrid->addAction($action_onEdit);

        $action_onDelete = new TDataGridAction(array('NegocioList', 'onDelete'));
        $action_onDelete->setUseButton(true);
        $action_onDelete->setButtonClass('btn btn-default btn-sm');
        $action_onDelete->setLabel("Excluir");
        $action_onDelete->setImage('fas:trash-alt #dd5a43');
        $action_onDelete->setField(self::$primaryKey);
        $action_onDelete->setDisplayCondition('NegocioList::onExibirExcluir');

        $this->datagrid->addAction($action_onDelete);

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup("Consultar Negócios");
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $button_cadastrar = new TButton('button_button_cadastrar');
        $button_cadastrar->setAction(new TAction(['NegocioForm', 'onShow']), "Cadastrar");
        $button_cadastrar->addStyleClass('btn-default');
        $button_cadastrar->setImage('fas:plus #69aa46');

        $this->datagrid_form->addField($button_cadastrar);

        $btnShowCurtainFilters = new TButton('button_btnShowCurtainFilters');
        $btnShowCurtainFilters->setAction(new TAction(['NegocioList', 'onShowCurtainFilters']), "Filtros");
        $btnShowCurtainFilters->addStyleClass('btn-default');
        $btnShowCurtainFilters->setImage('fas:filter #000000');

        $this->datagrid_form->addField($btnShowCurtainFilters);

        $button_limpar_filtros = new TButton('button_button_limpar_filtros');
        $button_limpar_filtros->setAction(new TAction(['NegocioList', 'onClearFilters']), "Limpar filtros");
        $button_limpar_filtros->addStyleClass('btn-default');
        $button_limpar_filtros->setImage('fas:eraser #f44336');

        $this->datagrid_form->addField($button_limpar_filtros);

        $button_atualizar = new TButton('button_button_atualizar');
        $button_atualizar->setAction(new TAction(['NegocioList', 'onRefresh']), "Atualizar");
        $button_atualizar->addStyleClass('btn-default');
        $button_atualizar->setImage('fas:sync-alt #03a9f4');

        $this->datagrid_form->addField($button_atualizar);

        $button_enviar_e_mails = new TButton('button_button_enviar_e_mails');
        $button_enviar_e_mails->setAction(new TAction(['NegocioEmailForm', 'onShow']), "Enviar E-mails");
        $button_enviar_e_mails->addStyleClass('btn-default');
        $button_enviar_e_mails->setImage('fas:envelope #E91E63');

        $this->datagrid_form->addField($button_enviar_e_mails);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['NegocioList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['NegocioList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['NegocioList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['NegocioList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_left_actions->add($button_cadastrar);
        $head_left_actions->add($btnShowCurtainFilters);
        $head_left_actions->add($button_limpar_filtros);
        $head_left_actions->add($button_atualizar);
        $head_left_actions->add($button_enviar_e_mails);

        $head_right_actions->add($dropdown_button_exportar);

        $this->btnShowCurtainFilters = $btnShowCurtainFilters;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["CRM","Consultar Negócios"]));
        }

        $container->add($panel);

        parent::add($container);

    }

    public static function onChangefunil_etapas_funil_id($param)
    {
        try
        {

            if (isset($param['funil_etapas_funil_id']) && $param['funil_etapas_funil_id'])
            { 
                $criteria = TCriteria::create(['funil_id' => $param['funil_etapas_funil_id']]);
                TDBCombo::reloadFromModel(self::$formName, 'funil_etapas_id', 'kraftsoft', 'FunilEtapas', 'id', '{descricao}', 'descricao asc', $criteria, TRUE); 
            } 
            else 
            { 
                TCombo::clearField(self::$formName, 'funil_etapas_id'); 
            }  

        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    } 

    public static function onExibirEditar($object)
    {
        try 
        {
            // if($object)
            // {
            //     return true;
            // }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onDelete($param = null) 
    { 
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new Negocio($key, FALSE); 

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }
    public static function onExibirExcluir($object)
    {
        try 
        {
            // if($object)
            // {
            //     return true;
            // }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('iframe');
                $object->src  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }
    public static function onShowCurtainFilters($param = null) 
    {
        try 
        {
            //code here

                        $filter = new self([]);

            $btnClose = new TButton('closeCurtain');
            $btnClose->class = 'btn btn-sm btn-default';
            $btnClose->style = 'margin-right:10px;';
            $btnClose->onClick = "Template.closeRightPanel();";
            $btnClose->setLabel("Fechar");
            $btnClose->setImage('fas:times');

            $filter->form->addHeaderWidget($btnClose);

            $page = new TPage();
            $page->setTargetContainer('adianti_right_panel');
            $page->setProperty('page-name', 'NegocioListSearch');
            $page->setProperty('page_name', 'NegocioListSearch');
            $page->adianti_target_container = 'adianti_right_panel';
            $page->target_container = 'adianti_right_panel';
            $page->add($filter->form);
            $page->setIsWrapped(true);
            $page->show();

            $style = new TStyle('right-panel > .container-part[page-name=NegocioListSearch]');
            $style->width = '60% !important';
            $style->show(true);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onClearFilters($param = null) 
    {
        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }
    public function onRefresh($param = null) 
    {
        $this->onReload([]);
    }

    public function fireEvents( $object )
    {
        $obj = new stdClass;
        if(is_object($object) && get_class($object) == 'stdClass')
        {
            if(isset($object->funil_etapas_funil_id))
            {
                $value = $object->funil_etapas_funil_id;

                $obj->funil_etapas_funil_id = $value;
            }
            if(isset($object->funil_etapas_id))
            {
                $value = $object->funil_etapas_id;

                $obj->funil_etapas_id = $value;
            }
        }
        elseif(is_object($object))
        {
            if(isset($object->funil_etapas->funil_id))
            {
                $value = $object->funil_etapas->funil_id;

                $obj->funil_etapas_funil_id = $value;
            }
            if(isset($object->funil_etapas_id))
            {
                $value = $object->funil_etapas_id;

                $obj->funil_etapas_id = $value;
            }
        }
        TForm::sendData(self::$formName, $obj);
    }  

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->funil_etapas_funil_id) AND ( (is_scalar($data->funil_etapas_funil_id) AND $data->funil_etapas_funil_id !== '') OR (is_array($data->funil_etapas_funil_id) AND (!empty($data->funil_etapas_funil_id)) )) )
        {

            $filters[] = new TFilter('funil_etapas_id', 'in', "(SELECT id FROM funil_etapas WHERE funil_id = '{$data->funil_etapas_funil_id}')");// create the filter 
        }

        if (isset($data->funil_etapas_id) AND ( (is_scalar($data->funil_etapas_id) AND $data->funil_etapas_id !== '') OR (is_array($data->funil_etapas_id) AND (!empty($data->funil_etapas_id)) )) )
        {

            $filters[] = new TFilter('funil_etapas_id', '=', $data->funil_etapas_id);// create the filter 
        }

        if (isset($data->titulo) AND ( (is_scalar($data->titulo) AND $data->titulo !== '') OR (is_array($data->titulo) AND (!empty($data->titulo)) )) )
        {

            $filters[] = new TFilter('titulo', 'like', "%{$data->titulo}%");// create the filter 
        }

        if (isset($data->contato_nome) AND ( (is_scalar($data->contato_nome) AND $data->contato_nome !== '') OR (is_array($data->contato_nome) AND (!empty($data->contato_nome)) )) )
        {

            $filters[] = new TFilter('contato_nome', '=', $data->contato_nome);// create the filter 
        }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }

        if (isset($data->vendedor_id) AND ( (is_scalar($data->vendedor_id) AND $data->vendedor_id !== '') OR (is_array($data->vendedor_id) AND (!empty($data->vendedor_id)) )) )
        {

            $filters[] = new TFilter('vendedor_id', '=', $data->vendedor_id);// create the filter 
        }

        if (isset($data->negocio_origem_id) AND ( (is_scalar($data->negocio_origem_id) AND $data->negocio_origem_id !== '') OR (is_array($data->negocio_origem_id) AND (!empty($data->negocio_origem_id)) )) )
        {

            $filters[] = new TFilter('negocio_origem_id', '=', $data->negocio_origem_id);// create the filter 
        }

        if (isset($data->arquivado) AND ( (is_scalar($data->arquivado) AND $data->arquivado !== '') OR (is_array($data->arquivado) AND (!empty($data->arquivado)) )) )
        {

            $filters[] = new TFilter('arquivado', '=', $data->arquivado);// create the filter 
        }

        if (isset($data->data_abertura_inicial) AND ( (is_scalar($data->data_abertura_inicial) AND $data->data_abertura_inicial !== '') OR (is_array($data->data_abertura_inicial) AND (!empty($data->data_abertura_inicial)) )) )
        {

            $filters[] = new TFilter('data_abertura', '>=', $data->data_abertura_inicial);// create the filter 
        }

        if (isset($data->data_abertura_final) AND ( (is_scalar($data->data_abertura_final) AND $data->data_abertura_final !== '') OR (is_array($data->data_abertura_final) AND (!empty($data->data_abertura_final)) )) )
        {

            $filters[] = new TFilter('data_abertura', '<=', $data->data_abertura_final);// create the filter 
        }

        if (isset($data->data_fechamento_inicial) AND ( (is_scalar($data->data_fechamento_inicial) AND $data->data_fechamento_inicial !== '') OR (is_array($data->data_fechamento_inicial) AND (!empty($data->data_fechamento_inicial)) )) )
        {

            $filters[] = new TFilter('data_fechamento', '>=', $data->data_fechamento_inicial);// create the filter 
        }

        if (isset($data->data_fechamento_final) AND ( (is_scalar($data->data_fechamento_final) AND $data->data_fechamento_final !== '') OR (is_array($data->data_fechamento_final) AND (!empty($data->data_fechamento_final)) )) )
        {

            $filters[] = new TFilter('data_fechamento', '<=', $data->data_fechamento_final);// create the filter 
        }

        $this->fireEvents($data);

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Negocio
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            $session_checks = TSession::getValue(__CLASS__.'builder_datagrid_check');

            //</blockLine><btnShowCurtainFiltersAutoCode>
            if(!empty($this->btnShowCurtainFilters) && empty($this->btnShowCurtainFiltersAdjusted))
            {
                $this->btnShowCurtainFiltersAdjusted = true;
                $this->btnShowCurtainFilters->style = 'position: relative';
                $countFilters = count($filters ?? []);
                $this->btnShowCurtainFilters->setLabel($this->btnShowCurtainFilters->getLabel(). "<span class='badge badge-success' style='position: absolute'>{$countFilters}<span>");
            }
            //</blockLine></btnShowCurtainFiltersAutoCode>

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {
                    $check = new TCheckGroup('builder_datagrid_check');
                    $check->addItems([$object->id => '']);
                    $check->getButtons()[$object->id]->onclick = 'event.stopPropagation()';

                    if(!$this->datagrid_form->getField('builder_datagrid_check[]'))
                    {
                        $this->datagrid_form->setFields([$check]);
                    }

                    $check->setChangeAction(new TAction([$this, 'builderSelectCheck']));
                    $object->builder_datagrid_check = $check;

                    if(!empty($session_checks[$object->id]))
                    {
                        $object->builder_datagrid_check->setValue([$object->id=>$object->id]);
                    }

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function builderSelectCheck($param)
    {
        $session_checks = TSession::getValue(__CLASS__.'builder_datagrid_check');

        $valueOn = null;
        if(!empty($param['_field_data_json']))
        {
            $obj = json_decode($param['_field_data_json']);
            if($obj)
            {
                $valueOn = $obj->valueOn;
            }
        }

        $key = empty($param['key']) ? $valueOn : $param['key'];

        if(empty($param['builder_datagrid_check']) && !empty($session_checks[$key]))
        {
            unset($session_checks[$key]);
        }
        elseif(!empty($param['builder_datagrid_check']) && !in_array($key, $param['builder_datagrid_check']) && !empty($session_checks[$key]))
        {
            unset($session_checks[$key]);
        }
        elseif(!empty($param['builder_datagrid_check']) && in_array($key, $param['builder_datagrid_check']))
        {
            $session_checks[$key] = $key;
        }

        TSession::setValue(__CLASS__.'builder_datagrid_check', $session_checks);
    }

}

