<?php

class EstoqueLancamentosForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'EstoqueLancamentos';
    private static $primaryKey = 'id';
    private static $formName = 'form_EstoqueLancamentosForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Cadastro de lançamentos de estoque");

        $criteria_estoque_deposito_id = new TCriteria();

        $filterVar = TSession::getValue("userunitid");
        $criteria_estoque_deposito_id->add(new TFilter('system_unit_id', '=', $filterVar)); 

        $id = new THidden('id');
        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc'  );
        $estoque_tipo_movimentacao_id = new TDBCombo('estoque_tipo_movimentacao_id', 'kraftsoft', 'EstoqueTipoMovimentacao', 'id', '{descricao}','descricao asc'  );
        $estoque_deposito_id = new TDBCombo('estoque_deposito_id', 'kraftsoft', 'EstoqueDeposito', 'id', '{descricao}','descricao asc' , $criteria_estoque_deposito_id );
        $produto_lote_id = new TCombo('produto_lote_id');
        $button_novo_lote = new TButton('button_novo_lote');
        $quantidade = new TNumeric('quantidade', '2', ',', '.' );
        $observacao = new TText('observacao');

        $produto_id->setChangeAction(new TAction([$this,'onChangeproduto_id']));

        $produto_id->addValidation("Produto", new TRequiredValidator()); 
        $estoque_tipo_movimentacao_id->addValidation("Tipo Movimentação", new TRequiredValidator()); 
        $estoque_deposito_id->addValidation("Depósito", new TRequiredValidator()); 
        $quantidade->addValidation("Quantidade", new TRequiredValidator()); 
        $observacao->addValidation("Observação", new TRequiredValidator()); 

        $button_novo_lote->setAction(new TAction([$this, 'onNovoLote']), "Novo Lote");
        $button_novo_lote->addStyleClass('btn-default');
        $button_novo_lote->setImage('fas:boxes #673AB7');
        $produto_id->enableSearch();
        $produto_lote_id->enableSearch();
        $estoque_deposito_id->enableSearch();
        $estoque_tipo_movimentacao_id->enableSearch();

        $id->setSize(200);
        $produto_id->setSize('100%');
        $quantidade->setSize('100%');
        $produto_lote_id->setSize(450);
        $observacao->setSize('100%', 70);
        $estoque_deposito_id->setSize('100%');
        $estoque_tipo_movimentacao_id->setSize('100%');

        $row1 = $this->form->addFields([$id]);
        $row1->layout = ['col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Produto:", '#ff0000', '14px', null, '100%'),$produto_id]);
        $row2->layout = [' col-sm-12'];

        $row3 = $this->form->addFields([new TLabel("Tipo Movimentação:", '#ff0000', '14px', null, '100%'),$estoque_tipo_movimentacao_id],[new TLabel("Depósito:", '#ff0000', '14px', null, '100%'),$estoque_deposito_id]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Lote:", null, '14px', null, '100%'),$produto_lote_id,$button_novo_lote],[new TLabel("Quantidade:", '#ff0000', '14px', null, '100%'),$quantidade]);
        $row4->layout = ['col-sm-6','col-sm-6'];

        $row5 = $this->form->addFields([new TLabel("Observação:", '#FF0000', '14px', null, '100%'),$observacao]);
        $row5->layout = [' col-sm-12'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Voltar", new TAction(['EstoqueLancamentosList', 'onShow']), 'fas:arrow-left #000000');
        $this->btn_onshow = $btn_onshow;

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=EstoqueLancamentosForm]');
        $style->width = '80% !important';   
        $style->show(true);

    }

    public static function onChangeproduto_id($param)
    {
        try
        {

            if (isset($param['produto_id']) && $param['produto_id'])
            { 
                $criteria = TCriteria::create(['produto_id' => $param['produto_id']]);
                TDBCombo::reloadFromModel(self::$formName, 'produto_lote_id', 'kraftsoft', 'ProdutoLote', 'id', '{numero}', 'numero asc', $criteria, TRUE); 
            } 
            else 
            { 
                TCombo::clearField(self::$formName, 'produto_lote_id'); 
            }  

        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    } 

    public static function onNovoLote($param = null) 
    {
        try 
        {
            if(!$param['produto_id'])
            {
                throw new Exception(AdiantiCoreTranslator::translate('The field ^1 is required', "Produto"));
            }            

            TSession::setValue('estoque_lancamento_form_produto_id', $param['produto_id']);

            TApplication::loadPage('ProdutoLoteFormAvulso', 'onShow');

            //recarregando o combo do lote..
            // $idprod = $param['produto_id'];

            // $object = new stdClass();
            // $object->produto_id = '';
            // TForm::sendData(self::$formName, $object);                   

            $criteria = TCriteria::create(['produto_id' => $param['produto_id']]);
            TDBCombo::reloadFromModel(self::$formName, 'produto_lote_id', 'kraftsoft', 'ProdutoLote', 'id', '{numero}', 'numero asc', $criteria, TRUE);             

            //EstoqueLancamentosForm::onChangeproduto_id();

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new EstoqueLancamentos(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            if ($object->estoque_tipo_movimentacao_id==EstoqueTipoMovimentacao::Entrada)
            {
                $object->estoque_origem_id = EstoqueOrigem::EntradaAvulsa;
            }
            else
            {
                $object->estoque_origem_id = EstoqueOrigem::SaidaAvulsa;
            }

            $object->system_unit_id = TSession::getValue('userunitid');
            $object->data_lancamento = date('Y-m-d');

            //pegando o preço de custo e venda
            $produtofilial = ProdutoFilial::where('produto_id', '=', $object->produto_id)
                                          ->where('system_unit_id', '=', $object->system_unit_id)->first();

            if ($produtofilial)
            {
                $object->preco_venda = $produtofilial->preco_venda;
                $object->preco_custo = $produtofilial->preco_custo;
            }

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);

            $object->store(); // save the object 

            $this->fireEvents($object);

            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('EstoqueLancamentosList', 'onShow', $loadPageParam); 

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode>  

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new EstoqueLancamentos($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                $this->fireEvents($object);

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public function fireEvents( $object )
    {
        $obj = new stdClass;
        if(is_object($object) && get_class($object) == 'stdClass')
        {
            if(isset($object->produto_id))
            {
                $value = $object->produto_id;

                $obj->produto_id = $value;
            }
            if(isset($object->produto_lote_id))
            {
                $value = $object->produto_lote_id;

                $obj->produto_lote_id = $value;
            }
        }
        elseif(is_object($object))
        {
            if(isset($object->produto_id))
            {
                $value = $object->produto_id;

                $obj->produto_id = $value;
            }
            if(isset($object->produto_lote_id))
            {
                $value = $object->produto_lote_id;

                $obj->produto_lote_id = $value;
            }
        }
        TForm::sendData(self::$formName, $obj);
    }  

    public static function getFormName()
    {
        return self::$formName;
    }

}

