<?php

class EstoqueLancamentosList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'EstoqueLancamentos';
    private static $primaryKey = 'id';
    private static $formName = 'form_EstoqueLancamentosList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de lançamentos de estoque ");
        $this->limit = 50;

        $criteria_estoque_deposito_id = new TCriteria();

        $filterVar = TSession::getValue("userunitid");
        $criteria_estoque_deposito_id->add(new TFilter('system_unit_id', '=', $filterVar)); 

        $produto_id = new TSeekButton('produto_id');
        $produto_nome = new TEntry('produto_nome');
        $estoque_origem_id = new TDBCombo('estoque_origem_id', 'kraftsoft', 'EstoqueOrigem', 'id', '{descricao}','descricao asc'  );
        $estoque_deposito_id = new TDBCombo('estoque_deposito_id', 'kraftsoft', 'EstoqueDeposito', 'id', '{descricao}','descricao asc' , $criteria_estoque_deposito_id );
        $produto_lote_id = new TDBCombo('produto_lote_id', 'kraftsoft', 'ProdutoLote', 'id', '{numero}','numero asc'  );
        $data_inicial = new TDate('data_inicial');
        $data_final = new TDate('data_final');
        $quant_disponivel = new TNumeric('quant_disponivel', '2', ',', '.' );
        $quant_reservada = new TNumeric('quant_reservada', '2', ',', '.' );
        $saldo_anterior = new TNumeric('saldo_anterior', '2', ',', '.' );
        $preco_medio = new TNumeric('preco_medio', '2', ',', '.' );

        $produto_id->setExitAction(new TAction([$this,'onExitProduto']));

        $produto_id->addValidation("Produto", new TRequiredValidator()); 

        $data_final->setMask('dd/mm/yyyy');
        $data_inicial->setMask('dd/mm/yyyy');

        $data_final->setValue(date('d/m/Y'));
        $data_inicial->setValue(date('d/m/Y'));

        $data_final->setDatabaseMask('yyyy-mm-dd');
        $data_inicial->setDatabaseMask('yyyy-mm-dd');

        $produto_lote_id->enableSearch();
        $estoque_origem_id->enableSearch();
        $estoque_deposito_id->enableSearch();

        $preco_medio->setEditable(false);
        $produto_nome->setEditable(false);
        $saldo_anterior->setEditable(false);
        $quant_reservada->setEditable(false);
        $quant_disponivel->setEditable(false);

        $produto_id->setSize(110);
        $produto_nome->setSize(540);
        $data_final->setSize('100%');
        $preco_medio->setSize('100%');
        $data_inicial->setSize('100%');
        $saldo_anterior->setSize('100%');
        $produto_lote_id->setSize('100%');
        $quant_reservada->setSize('100%');
        $quant_disponivel->setSize('100%');
        $estoque_origem_id->setSize('100%');
        $estoque_deposito_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Produto:", '#E30A0A', '14px', null, '100%'),$produto_id,$produto_nome],[new TLabel("Origem:", null, '14px', null, '100%'),$estoque_origem_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Deposito:", null, '14px', null, '100%'),$estoque_deposito_id],[new TLabel("Lote:", null, '14px', null, '100%'),$produto_lote_id]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Data Lançamento Inicial:", '#E30A0A', '14px', null, '100%'),$data_inicial],[new TLabel("Data Lançamento Final:", '#E30A0A', '14px', null, '100%'),$data_final]);
        $row3->layout = [' col-sm-6','col-sm-6'];

        $bcontainer_632d9eb31c0aa = new BContainer('bcontainer_632d9eb31c0aa');
        $this->bcontainer_632d9eb31c0aa = $bcontainer_632d9eb31c0aa;

        $bcontainer_632d9eb31c0aa->setTitle("Saldos", '#333', '18px', '', '#fff');
        $bcontainer_632d9eb31c0aa->setBorderColor('#c0c0c0');

        $row4 = $bcontainer_632d9eb31c0aa->addFields([new TLabel("Quantidade Disponível:", null, '14px', null),$quant_disponivel],[new TLabel("Quantidade Reservada:", null, '14px', null),$quant_reservada],[new TLabel("Saldo Anterior:", null, '14px', null),$saldo_anterior],[new TLabel("Preço Médio:", null, '14px', null),$preco_medio]);
        $row4->layout = ['col-sm-3','col-sm-3',' col-sm-3',' col-sm-3'];

        $row5 = $this->form->addFields([$bcontainer_632d9eb31c0aa]);
        $row5->layout = [' col-sm-12'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );
        $this->fireEvents( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Lançamento Avulso", new TAction(['EstoqueLancamentosForm', 'onShow']), 'fas:plus #69aa46');
        $this->btn_onshow = $btn_onshow;

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_data_lancamento_transformed = new TDataGridColumn('data_lancamento', "Data Lançamento", 'left');
        $column_estoque_tipo_movimentacao_descricao = new TDataGridColumn('estoque_tipo_movimentacao->descricao', "Tipo Movimentação", 'left');
        $column_estoque_origem_descricao = new TDataGridColumn('estoque_origem->descricao', "Origem", 'left');
        $column_observacao = new TDataGridColumn('observacao', "Observação", 'left');
        $column_estoque_deposito_descricao = new TDataGridColumn('estoque_deposito->descricao', "Depósito", 'left');
        $column_produto_lote_numero = new TDataGridColumn('produto_lote->numero', "Lote", 'left');
        $column_quantidade_transformed = new TDataGridColumn('quantidade', "Quantidade", 'left');
        $column_criado_por = new TDataGridColumn('criado_por', "Usuário", 'left');

        $column_data_lancamento_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_quantidade_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {

                    if(!$value)
                    {
                        $value = 0;
                    }

                    if(is_numeric($value))
                    {
                        return number_format($value, 2, ",", ".");
                    }
                    else
                    {
                        return $value;
                    }

        });        

        $this->datagrid->addColumn($column_data_lancamento_transformed);
        $this->datagrid->addColumn($column_estoque_tipo_movimentacao_descricao);
        $this->datagrid->addColumn($column_estoque_origem_descricao);
        $this->datagrid->addColumn($column_observacao);
        $this->datagrid->addColumn($column_estoque_deposito_descricao);
        $this->datagrid->addColumn($column_produto_lote_numero);
        $this->datagrid->addColumn($column_quantidade_transformed);
        $this->datagrid->addColumn($column_criado_por);

        $action_group = new TDataGridActionGroup("", 'fas:cog');
        $action_group->addHeader('');

        $action_onEdit = new TDataGridAction(array('EstoqueLancamentosForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('EstoqueLancamentosList::onExibeEditar');

        $action_group->addAction($action_onEdit);

        $action_onDelete = new TDataGridAction(array('EstoqueLancamentosList', 'onDelete'));
        $action_onDelete->setUseButton(TRUE);
        $action_onDelete->setButtonClass('btn btn-default');
        $action_onDelete->setLabel("Excluir");
        $action_onDelete->setImage('fas:trash-alt #dd5a43');
        $action_onDelete->setField(self::$primaryKey);
        $action_onDelete->setDisplayCondition('EstoqueLancamentosList::onExibeExcluir');

        $action_group->addAction($action_onDelete);

        $action_onVerOrigem = new TDataGridAction(array('EstoqueLancamentosList', 'onVerOrigem'));
        $action_onVerOrigem->setUseButton(TRUE);
        $action_onVerOrigem->setButtonClass('btn btn-default');
        $action_onVerOrigem->setLabel("Ver Origem");
        $action_onVerOrigem->setImage('fas:atom #4CAF50');
        $action_onVerOrigem->setField(self::$primaryKey);

        $action_group->addAction($action_onVerOrigem);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['EstoqueLancamentosList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['EstoqueLancamentosList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['EstoqueLancamentosList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['EstoqueLancamentosList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_right_actions->add($dropdown_button_exportar);

        $seed = AdiantiApplicationConfig::get()['general']['seed'];
        $produto_id_seekAction = new TAction(['ProdutoSeekWindow', 'onShow']);
        $seekFilters = [];
        $seekFields = base64_encode(serialize([
            ['name'=> 'produto_id', 'column'=>'{id}'],
            ['name'=> 'produto_nome', 'column'=>'{nome}']
        ]));

        $seekFilters = base64_encode(serialize($seekFilters));
        $produto_id_seekAction->setParameter('_seek_fields', $seekFields);
        $produto_id_seekAction->setParameter('_seek_filters', $seekFilters);
        $produto_id_seekAction->setParameter('_seek_hash', md5($seed.$seekFields.$seekFilters));
        $produto_id->setAction($produto_id_seekAction);

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Estoque","Lançamentos de Estoque "]));
        }
        $container->add($this->form);
        $container->add($panel);

        parent::add($container);

    }

    public static function onExitProduto($param = null) 
    {
        try 
        {
            if (isset($param['produto_id']) && $param['produto_id'])
            { 
                $criteria = TCriteria::create(['produto_id' => $param['produto_id']]);
                TDBCombo::reloadFromModel(self::$formName, 'produto_lote_id', 'kraftsoft', 'ProdutoLote', 'id', '{numero}', 'numero asc', $criteria, TRUE); 
            } 
            else 
            { 
                TCombo::clearField(self::$formName, 'produto_lote_id'); 
            }  

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onExibeEditar($object)
    {
        try 
        {
            // if($object)
            // {
            //     return true;
            // }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onDelete($param = null) 
    { 
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new EstoqueLancamentos($key, FALSE); 

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }
    public static function onExibeExcluir($object)
    {
        try 
        {
            // if($object)
            // {
            //     return true;
            // }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onVerOrigem($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);   
            $estoquelancamento = EstoqueLancamentos::find($param['key']);
            TTransaction::close();

            if ($estoquelancamento->estoque_origem_id == EstoqueOrigem::Vendas)
            {
                 $pageParam = ['key' => $estoquelancamento->venda_id]; // ex.: = ['key' => 10]

                 TApplication::loadPage('VendaFormView', 'OnShow', $pageParam);                
            }
            else
            if ($estoquelancamento->estoque_origem_id == EstoqueOrigem::Compras)
            {
                 $pageParam = ['key' => $estoquelancamento->nota_fiscal_compra_id]; // ex.: = ['key' => 10]

                 TApplication::loadPage('NotaFiscalCompraFormView', 'OnShow', $pageParam);                
            }   
            else
            if ($estoquelancamento->estoque_origem_id == EstoqueOrigem::ProducaoAcabados)
            {
                 $pageParam = ['key' => $estoquelancamento->ordem_producao_id]; // ex.: = ['key' => 10]

                 TApplication::loadPage('OrdemProducaoFormView2', 'OnShow', $pageParam);                
            }   
            else            
            if ($estoquelancamento->estoque_origem_id == EstoqueOrigem::ProducaoMateriasPrimas)
            {
                 $pageParam = ['key' => $estoquelancamento->ordem_producao_id]; // ex.: = ['key' => 10]

                 TApplication::loadPage('OrdemProducaoFormView2', 'OnShow', $pageParam);                
            }   
            else                        
            {
              new TMessage('info', 'Este lançamento é de origem avulsa!');  
            }

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('object');
                $object->data  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function fireEvents( $object )
    {
        $obj = new stdClass;
        if(is_object($object) && get_class($object) == 'stdClass')
        {
            if(isset($object->produto_id))
            {
                $value = $object->produto_id;

                $obj->produto_id = $value;
            }
        }
        elseif(is_object($object))
        {
            if(isset($object->produto_id))
            {
                $value = $object->produto_id;

                $obj->produto_id = $value;
            }
        }
        TForm::sendData(self::$formName, $obj);
    }  

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        // TApplication::loadPage('EstoqueLancamentosList', 'onShow');

        // (new TRequiredValidator)->validate('Produto', $data->produto_id );
        // (new TRequiredValidator)->validate('data de lançamento inicial', $data->data_inicial );
        // (new TRequiredValidator)->validate('data de lançamento final', $data->data_final );

        if ($data->produto_id==null)
        {
            //throw new Exception('Informe o produto!');
            new TMessage('error','Informe o produto!');
            TApplication::loadPage('EstoqueLancamentosList', 'onShow');
        }    

        if ($data->data_inicial==null or $data->data_final==null)
        {
          new TMessage('error','Informe a data de lançamento inicial e final!');  
          TApplication::loadPage('EstoqueLancamentosList', 'onShow');
        }            

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->produto_id) AND ( (is_scalar($data->produto_id) AND $data->produto_id !== '') OR (is_array($data->produto_id) AND (!empty($data->produto_id)) )) )
        {

            $filters[] = new TFilter('produto_id', '=', $data->produto_id);// create the filter 
        }

        if (isset($data->estoque_origem_id) AND ( (is_scalar($data->estoque_origem_id) AND $data->estoque_origem_id !== '') OR (is_array($data->estoque_origem_id) AND (!empty($data->estoque_origem_id)) )) )
        {

            $filters[] = new TFilter('estoque_origem_id', '=', $data->estoque_origem_id);// create the filter 
        }

        if (isset($data->estoque_deposito_id) AND ( (is_scalar($data->estoque_deposito_id) AND $data->estoque_deposito_id !== '') OR (is_array($data->estoque_deposito_id) AND (!empty($data->estoque_deposito_id)) )) )
        {

            $filters[] = new TFilter('estoque_deposito_id', '=', $data->estoque_deposito_id);// create the filter 
        }

        if (isset($data->produto_lote_id) AND ( (is_scalar($data->produto_lote_id) AND $data->produto_lote_id !== '') OR (is_array($data->produto_lote_id) AND (!empty($data->produto_lote_id)) )) )
        {

            $filters[] = new TFilter('produto_lote_id', '=', $data->produto_lote_id);// create the filter 
        }

        if (isset($data->data_inicial) AND ( (is_scalar($data->data_inicial) AND $data->data_inicial !== '') OR (is_array($data->data_inicial) AND (!empty($data->data_inicial)) )) )
        {

            $filters[] = new TFilter('data_lancamento', '>=', $data->data_inicial);// create the filter 
        }

        if (isset($data->data_final) AND ( (is_scalar($data->data_final) AND $data->data_final !== '') OR (is_array($data->data_final) AND (!empty($data->data_final)) )) )
        {

            $filters[] = new TFilter('data_lancamento', '<=', $data->data_final);// create the filter 
        }

        $this->fireEvents($data);

        TTransaction::open(self::$database);

        $repository = new TRepository('EstoqueSaldos');

        $criteria = new TCriteria();
        $criteria->add(new TFilter('produto_id', '=', $param['produto_id']));         
        $criteria->add(new TFilter('system_unit_id', '=', TSession::getValue('userunitid')));  

        if (isset($data->estoque_deposito_id) AND ( (is_scalar($data->estoque_deposito_id) AND $data->estoque_deposito_id !== '') OR (is_array($data->estoque_deposito_id) AND (!empty($data->estoque_deposito_id)) )) )
        {
            $criteria->add(new TFilter('estoque_deposito_id', '=', $param['estoque_deposito_id']));     
        }   

        if (isset($data->produto_lote_id) AND ( (is_scalar($data->produto_lote_id) AND $data->produto_lote_id !== '') OR (is_array($data->produto_lote_id) AND (!empty($data->produto_lote_id)) )) )
        {
            $criteria->add(new TFilter('produto_lote_id', '=', $param['produto_lote_id']));
        }        

        $lista_estoquesaldo = $repository->load($criteria); 

        $quant_disponivel = 0;
        $quant_reservada = 0;
        $custo_medio = 0;
        $preco_medio = 0;
        $saldo_anterior = 0;

        foreach ($lista_estoquesaldo as $estoquesaldo)
        {
            $quant_disponivel += $estoquesaldo->quant_disponivel;
            $quant_reservada += $estoquesaldo->quant_reservada;   

            // $custo_medio += $estoquesaldo->preco_medio_custo;               

             if ($preco_medio==0)
             {
                $preco_medio += $estoquesaldo->preco_medio_venda;                           
             }
        }

        //saldo anterior..
        $system_unit_id = TSession::getValue('userunitid');
        $produto_id = $param['produto_id'];
        $estoque_deposito_id = $param['estoque_deposito_id'];

        if ($estoque_deposito_id=='') 
        {
          $estoque_deposito_id = 0;  
        }

        $produto_lote_id = $param['produto_lote_id'];

        if ($produto_lote_id=='') 
        {
          $produto_lote_id = 0;  
        }        

        $data_inicial = $param['data_inicial'];

        $saldo_anterior = EstoqueLancamentos::ObterSaldoAnterior(TSession::getValue('userunitid'),
                                                                 $produto_id, 
                                                                 $estoque_deposito_id, 
                                                                 $produto_lote_id, 
                                                                 $data_inicial);    

        // $sql = 'call obter_saldo_anterior('.$system_unit_id.', 
        //                                   '.$produto_id.',
        //                                   '.$estoque_deposito_id.', 
        //                                   '.$produto_lote_id.', 
        //                                   '.$data_inicial.', 
        //                                   @saldo);
        //         '; 

        //echo 'sql '.$sql;        

        // $connection = TTransaction::get();
        // $query = $connection->query($sql);
        // $query = $connection->query('select @saldo');

        // $saldo_anterior = $query->saldo;

        TTransaction::close();                                     

        $object = new stdClass();
        $object->quant_disponivel = number_format($quant_disponivel, 2, ',', '.');
        $object->quant_reservada = number_format($quant_reservada, 2, ',', '.');
        $object->saldo_anterior = number_format($saldo_anterior, 2, ',', '.');
        //$object->custo_medio = number_format($custo_medio, 2, ',', '.');
        $object->preco_medio = number_format($preco_medio, 2, ',', '.');        
        TForm::sendData(self::$formName, $object);

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            if (empty($_REQUEST['method']) || ($_REQUEST['method'] == 'onShow'))
            {
                return;
            }
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for EstoqueLancamentos
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

}

