<?php

class ProdutoLoteFormAvulso extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'ProdutoLote';
    private static $primaryKey = 'id';
    private static $formName = 'form_ProdutoLoteFormAvulso';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Lote de Produto");


        $id = new TEntry('id');
        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc'  );
        $numero = new TEntry('numero');
        $validade = new TDate('validade');

        $produto_id->addValidation("Produto", new TRequiredValidator()); 
        $numero->addValidation("Número", new TRequiredValidator()); 

        $validade->setMask('dd/mm/yyyy');
        $validade->setDatabaseMask('yyyy-mm-dd');
        $id->setEditable(false);
        $produto_id->setEditable(false);

        $id->setSize(100);
        $numero->setSize('100%');
        $validade->setSize('100%');
        $produto_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Produto:", '#ff0000', '14px', null, '100%'),$produto_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Número:", '#ff0000', '14px', null, '100%'),$numero],[new TLabel("Validade:", null, '14px', null, '100%'),$validade]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=ProdutoLoteFormAvulso]');
        $style->width = '80% !important';   
        $style->show(true);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new ProdutoLote(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);

            $object->store(); // save the object 

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            new TMessage('info', "Registro salvo", $messageAction); 

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new ProdutoLote($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

        $idproduto = TSession::getValue('estoque_lancamento_form_produto_id');

        if ($idproduto<>0)
        {
            $object = new stdClass();
            $object->produto_id = $idproduto;
            TForm::sendData(self::$formName, $object);       

            //TDBCombo::disableField(self::$formName, 'produto_id');
        }
    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

