<?php

class VwProdutoEstoqueReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'VwProdutoEstoque';
    private static $primaryKey = 'id';
    private static $formName = 'form_VwProdutoEstoqueReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Estoque de produtos");

        $id = new TEntry('id');
        $nome = new TEntry('nome');
        $referencia = new TEntry('referencia');
        $codigo_sku = new TEntry('codigo_sku');
        $grupo = new TEntry('grupo');
        $subgrupo = new TEntry('subgrupo');
        $ean = new TEntry('ean');
        $combo_tipo_filtro = new TCombo('combo_tipo_filtro');

        $combo_tipo_filtro->addItems(["abaixominimo"=>"Produtos abaixo do estoque mínimo","acimaestoque"=>"Produtos acima do estoque máximo"]);
        $combo_tipo_filtro->enableSearch();
        $id->setSize(100);
        $ean->setSize('100%');
        $nome->setSize('100%');
        $grupo->setSize('100%');
        $subgrupo->setSize('100%');
        $referencia->setSize('100%');
        $codigo_sku->setSize('100%');
        $combo_tipo_filtro->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Nome:", null, '14px', null, '100%'),$nome]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Referencia:", null, '14px', null, '100%'),$referencia],[new TLabel("Código SKU:", null, '14px', null, '100%'),$codigo_sku]);
        $row2->layout = ['col-sm-6',' col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Grupo:", null, '14px', null, '100%'),$grupo],[new TLabel("Subgrupo:", null, '14px', null, '100%'),$subgrupo]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("EAN (Código de barras):", null, '14px', null, '100%'),$ean],[new TLabel("Tipo de Filtro:", null, '14px', null, '100%'),$combo_tipo_filtro]);
        $row4->layout = ['col-sm-6','col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratehtml = $this->form->addAction("Gerar HTML", new TAction([$this, 'onGenerateHtml']), 'far:file-code #ffffff');
        $this->btn_ongeneratehtml = $btn_ongeneratehtml;
        $btn_ongeneratehtml->addStyleClass('btn-primary'); 

        $btn_ongeneratepdf = $this->form->addAction("Gerar PDF", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        $btn_ongeneratexls = $this->form->addAction("Gerar XLS", new TAction([$this, 'onGenerateXls']), 'far:file-excel #00a65a');
        $this->btn_ongeneratexls = $btn_ongeneratexls;

        $btn_ongeneratertf = $this->form->addAction("Gerar RTF", new TAction([$this, 'onGenerateRtf']), 'far:file-alt #324bcc');
        $this->btn_ongeneratertf = $btn_ongeneratertf;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Estoque","Estoque de produtos"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }
    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }
    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }
    public function onGenerateRtf($param = null) 
    {
        $this->onGenerate('rtf');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }
        if (isset($data->nome) AND ( (is_scalar($data->nome) AND $data->nome !== '') OR (is_array($data->nome) AND (!empty($data->nome)) )) )
        {

            $filters[] = new TFilter('nome', 'like', "%{$data->nome}%");// create the filter 
        }
        if (isset($data->referencia) AND ( (is_scalar($data->referencia) AND $data->referencia !== '') OR (is_array($data->referencia) AND (!empty($data->referencia)) )) )
        {

            $filters[] = new TFilter('referencia', 'like', "%{$data->referencia}%");// create the filter 
        }
        if (isset($data->codigo_sku) AND ( (is_scalar($data->codigo_sku) AND $data->codigo_sku !== '') OR (is_array($data->codigo_sku) AND (!empty($data->codigo_sku)) )) )
        {

            $filters[] = new TFilter('SKU', 'like', "%{$data->codigo_sku}%");// create the filter 
        }
        if (isset($data->grupo) AND ( (is_scalar($data->grupo) AND $data->grupo !== '') OR (is_array($data->grupo) AND (!empty($data->grupo)) )) )
        {

            $filters[] = new TFilter('grupo', 'like', "%{$data->grupo}%");// create the filter 
        }
        if (isset($data->subgrupo) AND ( (is_scalar($data->subgrupo) AND $data->subgrupo !== '') OR (is_array($data->subgrupo) AND (!empty($data->subgrupo)) )) )
        {

            $filters[] = new TFilter('subgrupo', 'like', "%{$data->subgrupo}%");// create the filter 
        }
        if (isset($data->ean) AND ( (is_scalar($data->ean) AND $data->ean !== '') OR (is_array($data->ean) AND (!empty($data->ean)) )) )
        {

            $filters[] = new TFilter('ean', 'like', "%{$data->ean}%");// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            $param = [];
            // creates a repository for VwProdutoEstoque
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            $filterVar = TSession::getValue("userunitid");
            $criteria->add(new TFilter('system_unit_id', '=', $filterVar));

            $criteria->setProperties($param);

            if ($filters)
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            $data = $this->form->getData();

            if ($data->combo_tipo_filtro=='abaixominimo') 
            {
                $criteria->add(new TFilter('Quantidade_Disponivel', '<=', "(SELECT estoque_minimo FROM produto_filial WHERE id = produto_filial_id)"));
            }
            elseif ($data->combo_tipo_filtro=='acimaestoque')
            {
                $criteria->add(new TFilter('Quantidade_Disponivel', '>=', "(SELECT estoque_maximo FROM produto_filial WHERE id = produto_filial_id)"));
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            if ($objects)
            {
                $widths = array(200,200,200,200,200,200,200,200,200,200,200,200,200);
                $reportExtension = 'pdf';
                switch ($format)
                {
                    case 'html':
                        $tr = new TTableWriterHTML($widths);
                        $reportExtension = 'html';
                        break;
                    case 'xls':
                        $tr = new TTableWriterXLS($widths);
                        $reportExtension = 'xls';
                        break;
                    case 'pdf':
                        $tr = new TTableWriterPDF($widths, 'L', 'A4');
                        $reportExtension = 'pdf';
                        break;
                    case 'htmlPdf':
                        $reportExtension = 'pdf';
                        $tr = new BTableWriterHtmlPDF($widths, 'L', 'A4');
                        break;
                    case 'rtf':
                        if (!class_exists('PHPRtfLite_Autoloader'))
                        {
                            PHPRtfLite::registerAutoloader();
                        }
                        $reportExtension = 'rtf';
                        $tr = new TTableWriterRTF($widths, 'L', 'A4');
                        break;
                }

                if (!empty($tr))
                {
                    // create the document styles
                    $tr->addStyle('title', 'Helvetica', '10', 'B',   '#ffffff', '#96c0e6');
                    $tr->addStyle('datap', 'Arial', '10', '',    '#000000', '#f0f0f0');
                    $tr->addStyle('datai', 'Arial', '10', '',    '#000000', '#ffffff');
                    $tr->addStyle('header', 'Helvetica', '16', 'B',   '#4589c5', '#494D90');
                    $tr->addStyle('footer', 'Helvetica', '10', 'B',  '#4589c5', '#B1B1EA');
                    $tr->addStyle('break', 'Helvetica', '10', 'B',  '#ffffff', '#8da4bc');
                    $tr->addStyle('total', 'Helvetica', '10', 'I',  '#000000', '#b2d1dc');
                    $tr->addStyle('breakTotal', 'Helvetica', '10', 'I',  '#000000', '#b3d5f3');

                    // add titles row
                    $tr->addRow();
                    $tr->addCell("Id", 'left', 'title');
                    $tr->addCell("SKU", 'left', 'title');
                    $tr->addCell("Referência", 'left', 'title');
                    $tr->addCell("Nome", 'left', 'title');
                    $tr->addCell("Grupo", 'left', 'title');
                    $tr->addCell("Subgrupo", 'left', 'title');
                    $tr->addCell("Sigla", 'left', 'title');
                    $tr->addCell("Ean", 'left', 'title');
                    $tr->addCell("Quant. Disponivel", 'left', 'title');
                    $tr->addCell("Quant. Reservada", 'left', 'title');
                    $tr->addCell("Estoque Mínimo", 'left', 'title');
                    $tr->addCell("Estoque Máximo", 'left', 'title');
                    $tr->addCell("Deposito", 'left', 'title');

                    $grandTotal = [];
                    $breakTotal = [];
                    $breakValue = null;
                    $firstRow = true;

                    // controls the background filling
                    $colour = false;                
                    foreach ($objects as $object)
                    {
                        $style = $colour ? 'datap' : 'datai';

                        $firstRow = false;

                        $object->Quantidade_Disponivel = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->Quantidade_Disponivel, $object, null);

                        $object->Quantidade_Reservada = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->Quantidade_Reservada, $object, null);

                        $object->estoque_minimo = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->estoque_minimo, $object, null);

                        $object->estoque_maximo = call_user_func(function($value, $object, $row)
                        {

                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }

                        }, $object->estoque_maximo, $object, null);

                        $tr->addRow();

                        $tr->addCell($object->id, 'left', $style);
                        $tr->addCell($object->SKU, 'left', $style);
                        $tr->addCell($object->referencia, 'left', $style);
                        $tr->addCell($object->nome, 'left', $style);
                        $tr->addCell($object->grupo, 'left', $style);
                        $tr->addCell($object->subgrupo, 'left', $style);
                        $tr->addCell($object->sigla, 'left', $style);
                        $tr->addCell($object->ean, 'left', $style);
                        $tr->addCell($object->Quantidade_Disponivel, 'left', $style);
                        $tr->addCell($object->Quantidade_Reservada, 'left', $style);
                        $tr->addCell($object->estoque_minimo, 'left', $style);
                        $tr->addCell($object->estoque_maximo, 'left', $style);
                        $tr->addCell($object->deposito, 'left', $style);

                        $colour = !$colour;

                    }

                    $file = 'report_'.uniqid().".{$reportExtension}";
                    // stores the file
                    if (!file_exists("app/output/{$file}") || is_writable("app/output/{$file}"))
                    {
                        $tr->save("app/output/{$file}");
                    }
                    else
                    {
                        throw new Exception(_t('Permission denied') . ': ' . "app/output/{$file}");
                    }

                    parent::openFile("app/output/{$file}");

                    // shows the success message
                    new TMessage('info', _t('Report generated. Please, enable popups'));
                }
            }
            else
            {
                new TMessage('error', _t('No records found'));
            }

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }


}

