<?php

class AberturaCaixaForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Caixa';
    private static $primaryKey = 'id';
    private static $formName = 'form_AberturaCaixaForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Abertura de Caixa");


        $system_users_abertura_id = new TDBCombo('system_users_abertura_id', 'kraftsoft', 'SystemUsers', 'id', '{name}','name asc'  );
        $data_hora_abertura = new TDateTime('data_hora_abertura');
        $system_users_operador_id = new TDBCombo('system_users_operador_id', 'kraftsoft', 'SystemUsers', 'id', '{name}','name asc'  );
        $valor_fundo_cx = new TNumeric('valor_fundo_cx', '2', ',', '.' );
        $id = new THidden('id');

        $system_users_abertura_id->addValidation("system_users_abertura_id", new TRequiredValidator()); 
        $system_users_operador_id->addValidation("Operador", new TRequiredValidator()); 

        $data_hora_abertura->setMask('dd/mm/yyyy hh:ii');
        $data_hora_abertura->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_hora_abertura->setEditable(false);
        $system_users_abertura_id->setEditable(false);

        $system_users_abertura_id->enableSearch();
        $system_users_operador_id->enableSearch();

        $id->setSize(200);
        $valor_fundo_cx->setSize('100%');
        $data_hora_abertura->setSize('100%');
        $system_users_abertura_id->setSize('100%');
        $system_users_operador_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Usuário Abertura:", '#ff0000', '14px', null, '100%'),$system_users_abertura_id]);
        $row1->layout = [' col-sm-12'];

        $row2 = $this->form->addFields([new TLabel("Data Hora Abertura:", null, '14px', null, '100%'),$data_hora_abertura],[new TLabel("Operador:", '#ff0000', '14px', null, '100%'),$system_users_operador_id]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Valor fundo de caixa:", null, '14px', null),$valor_fundo_cx],[$id]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Voltar", new TAction(['CaixaList', 'onShow']), 'fas:arrow-left #000000');
        $this->btn_onshow = $btn_onshow;

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new Caixa(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            $object->caixa_situacao_id = CaixaSituacao::Aberto;
            $object->system_unit_id = TSession::getValue('userunitid');
            //$object->system_user_id_operador = $data->operador_id;
            //$object->system_user_id_abertura = TSession::getValue('userid');
            $object->sincronizado_local = 0;
            $object->data_hora_abertura = date('Y-m-d H:i:s');

            $object->store(); // save the object 

            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            //lançando o valor do fundo de caixa.. 
            if(($data->valor_fundo_cx > 0)&&(!empty($data->valor_fundo_cx)))
            {
                $cxlancamento = new CaixaLancamentos();
                $cxlancamento->caixa_id = $object->id;
                $cxlancamento->caixa_tipo_lancamento_id = CaixaTipoLancamento::FundoCaixa;
                $cxlancamento->system_users_id = TSession::getValue('userid');

                $idfpcaixa = Funcoes::retornaFormaPagamentoPadraoDinheiroCaixa();

                if ($idfpcaixa==0)
                {
                  throw new Exception('Forma de pagamento dinheiro não configurada para esta filial!');  
                }

                $cxlancamento->forma_pagamento_id = $idfpcaixa;
                $cxlancamento->valor = $data->valor_fundo_cx;
                $cxlancamento->data_hora_lancamento = date('Y-m-d H:i:s');
                $cxlancamento->store();
            }

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('CaixaList', 'onShow', $loadPageParam); 

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new Caixa($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

        $object = new stdClass();
        $object->system_users_abertura_id = TSession::getValue('userid');
        $object->data_hora_abertura = date('Y-m-d H:i:s');

        TForm::sendData(self::$formName, $object);
    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

