<?php

class CaixaList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Caixa';
    private static $primaryKey = 'id';
    private static $formName = 'form_CaixaList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de caixa monitor");
        $this->limit = 20;

        $id = new TEntry('id');
        $caixa_situacao_id = new TDBCombo('caixa_situacao_id', 'kraftsoft', 'CaixaSituacao', 'id', '{descricao}','descricao asc'  );
        $data_hora_abertura_inicial = new TDateTime('data_hora_abertura_inicial');
        $data_hora_abertura_final = new TDateTime('data_hora_abertura_final');
        $data_hora_fechamento_inicial = new TDateTime('data_hora_fechamento_inicial');
        $data_hora_fechamento_final = new TDateTime('data_hora_fechamento_final');
        $system_users_abertura_id = new TDBCombo('system_users_abertura_id', 'kraftsoft', 'SystemUsers', 'id', '{name}','name asc'  );
        $system_users_operador_id = new TDBCombo('system_users_operador_id', 'kraftsoft', 'SystemUsers', 'id', '{name}','name asc'  );
        $system_users_fechamento_id = new TDBCombo('system_users_fechamento_id', 'kraftsoft', 'SystemUsers', 'id', '{name}','name asc'  );


        $caixa_situacao_id->enableSearch();
        $system_users_abertura_id->enableSearch();
        $system_users_operador_id->enableSearch();
        $system_users_fechamento_id->enableSearch();

        $data_hora_abertura_final->setMask('dd/mm/yyyy hh:ii');
        $data_hora_abertura_inicial->setMask('dd/mm/yyyy hh:ii');
        $data_hora_fechamento_final->setMask('dd/mm/yyyy hh:ii');
        $data_hora_fechamento_inicial->setMask('dd/mm/yyyy hh:ii');

        $data_hora_abertura_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_hora_abertura_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_hora_fechamento_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_hora_fechamento_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');

        $id->setSize(100);
        $caixa_situacao_id->setSize('100%');
        $data_hora_abertura_final->setSize('100%');
        $system_users_abertura_id->setSize('100%');
        $system_users_operador_id->setSize('100%');
        $data_hora_abertura_inicial->setSize('100%');
        $data_hora_fechamento_final->setSize('100%');
        $system_users_fechamento_id->setSize('100%');
        $data_hora_fechamento_inicial->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Situação:", null, '14px', null, '100%'),$caixa_situacao_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Data Hora Abertura Inicial:", null, '14px', null, '100%'),$data_hora_abertura_inicial],[new TLabel("Data Hora Abertura Final:", null, '14px', null, '100%'),$data_hora_abertura_final],[new TLabel("Data Hora Fechamento:", null, '14px', null, '100%'),$data_hora_fechamento_inicial],[new TLabel("Data Hora Fechamento:", null, '14px', null, '100%'),$data_hora_fechamento_final]);
        $row2->layout = [' col-sm-3',' col-sm-3',' col-sm-3',' col-sm-3'];

        $row3 = $this->form->addFields([new TLabel("Usuário Abertura:", null, '14px', null, '100%'),$system_users_abertura_id],[new TLabel("Operador:", null, '14px', null, '100%'),$system_users_operador_id]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Usuário Fechamento:", null, '14px', null, '100%'),$system_users_fechamento_id]);
        $row4->layout = ['col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Abrir Caixa", new TAction(['AberturaCaixaForm', 'onShow']), 'fas:plus #69aa46');
        $this->btn_onshow = $btn_onshow;

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_data_hora_abertura_transformed = new TDataGridColumn('data_hora_abertura', "Data Hora Abertura", 'left');
        $column_data_hora_fechamento_transformed = new TDataGridColumn('data_hora_fechamento', "Data Hora Fechamento", 'left');
        $column_system_users_operador_name = new TDataGridColumn('system_users_operador->name', "Operador", 'left');
        $column_system_users_abertura_name = new TDataGridColumn('system_users_abertura->name', "Usuário Abertura", 'left');
        $column_system_users_fechamento_name = new TDataGridColumn('system_users_fechamento->name', "Usuário Fechamento", 'left');
        $column_caixa_situacao_descricao = new TDataGridColumn('caixa_situacao->descricao', "Situação", 'left');

        $column_data_hora_abertura_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_data_hora_fechamento_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_data_hora_abertura_transformed);
        $this->datagrid->addColumn($column_data_hora_fechamento_transformed);
        $this->datagrid->addColumn($column_system_users_operador_name);
        $this->datagrid->addColumn($column_system_users_abertura_name);
        $this->datagrid->addColumn($column_system_users_fechamento_name);
        $this->datagrid->addColumn($column_caixa_situacao_descricao);

        $action_group = new TDataGridActionGroup("Opções", 'fas:cog');
        $action_group->addHeader('');

        $action_onResumo = new TDataGridAction(array('CaixaList', 'onResumo'));
        $action_onResumo->setUseButton(TRUE);
        $action_onResumo->setButtonClass('btn btn-default');
        $action_onResumo->setLabel("Resumo Formas Pgto.");
        $action_onResumo->setImage('fas:align-justify #478fca');
        $action_onResumo->setField(self::$primaryKey);

        $action_group->addAction($action_onResumo);

        $action_onShow = new TDataGridAction(array('CaixaLancamentosSimpleList', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Detalhes Lançamentos");
        $action_onShow->setImage('fas:book #00BCD4');
        $action_onShow->setField(self::$primaryKey);

        $action_onShow->setParameter('caixa_id', '{id}');

        $action_group->addAction($action_onShow);

        $action_SangriaSuprimentoForm_onShow = new TDataGridAction(array('SangriaSuprimentoForm', 'onShow'));
        $action_SangriaSuprimentoForm_onShow->setUseButton(TRUE);
        $action_SangriaSuprimentoForm_onShow->setButtonClass('btn btn-default');
        $action_SangriaSuprimentoForm_onShow->setLabel("Sangria/Suprimento");
        $action_SangriaSuprimentoForm_onShow->setImage('fas:money-bill #8BC34A');
        $action_SangriaSuprimentoForm_onShow->setField(self::$primaryKey);
        $action_SangriaSuprimentoForm_onShow->setDisplayCondition('CaixaList::onExibeSangria');
        $action_SangriaSuprimentoForm_onShow->setParameter('caixa_id', '{id}');

        $action_group->addAction($action_SangriaSuprimentoForm_onShow);

        $action_onFecharCaixa = new TDataGridAction(array('CaixaList', 'onFecharCaixa'));
        $action_onFecharCaixa->setUseButton(TRUE);
        $action_onFecharCaixa->setButtonClass('btn btn-default');
        $action_onFecharCaixa->setLabel("Fechar Caixa");
        $action_onFecharCaixa->setImage('fas:window-close #DE0E0E');
        $action_onFecharCaixa->setField(self::$primaryKey);
        $action_onFecharCaixa->setDisplayCondition('CaixaList::onExibeFecharCaixa');

        $action_group->addAction($action_onFecharCaixa);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['CaixaList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['CaixaList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['CaixaList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['CaixaList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_right_actions->add($dropdown_button_exportar);

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Financeiro","Caixa Monitor"]));
        }
        $container->add($this->form);
        $container->add($panel);

        parent::add($container);

    }

    public function onResumo($param = null) 
    {
        try 
        {
            $pageParam = ['caixa_id' => $param['id']]; // ex.: = ['key' => 10]

            TSession::setValue('caixa_id', $param['id']);
            TSession::setValue('caixa_fechar', false);

            TApplication::loadPage('CaixaResumoFormasSimpleList', 'onReload', $pageParam);
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeSangria($object)
    {
        try 
        {
            if($object->caixa_situacao_id == CaixaSituacao::Aberto)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onFecharCaixa($param = null) 
    {
        try 
        {
            $pageParam = ['caixa_id' => $param['id']]; // ex.: = ['key' => 10]

            TSession::setValue('caixa_id', $param['id']);
            TSession::setValue('caixa_fechar', true);

            TApplication::loadPage('CaixaResumoFormasSimpleList', 'onReload', $pageParam);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeFecharCaixa($object)
    {
        try 
        {
            if($object->caixa_situacao_id==CaixaSituacao::Aberto)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('object');
                $object->data  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->caixa_situacao_id) AND ( (is_scalar($data->caixa_situacao_id) AND $data->caixa_situacao_id !== '') OR (is_array($data->caixa_situacao_id) AND (!empty($data->caixa_situacao_id)) )) )
        {

            $filters[] = new TFilter('caixa_situacao_id', '=', $data->caixa_situacao_id);// create the filter 
        }

        if (isset($data->data_hora_abertura_inicial) AND ( (is_scalar($data->data_hora_abertura_inicial) AND $data->data_hora_abertura_inicial !== '') OR (is_array($data->data_hora_abertura_inicial) AND (!empty($data->data_hora_abertura_inicial)) )) )
        {

            $filters[] = new TFilter('data_hora_abertura', '>=', $data->data_hora_abertura_inicial);// create the filter 
        }

        if (isset($data->data_hora_abertura_final) AND ( (is_scalar($data->data_hora_abertura_final) AND $data->data_hora_abertura_final !== '') OR (is_array($data->data_hora_abertura_final) AND (!empty($data->data_hora_abertura_final)) )) )
        {

            $filters[] = new TFilter('data_hora_abertura', '<=', $data->data_hora_abertura_final);// create the filter 
        }

        if (isset($data->data_hora_fechamento_inicial) AND ( (is_scalar($data->data_hora_fechamento_inicial) AND $data->data_hora_fechamento_inicial !== '') OR (is_array($data->data_hora_fechamento_inicial) AND (!empty($data->data_hora_fechamento_inicial)) )) )
        {

            $filters[] = new TFilter('data_hora_fechamento', '>=', $data->data_hora_fechamento_inicial);// create the filter 
        }

        if (isset($data->data_hora_fechamento_final) AND ( (is_scalar($data->data_hora_fechamento_final) AND $data->data_hora_fechamento_final !== '') OR (is_array($data->data_hora_fechamento_final) AND (!empty($data->data_hora_fechamento_final)) )) )
        {

            $filters[] = new TFilter('data_hora_fechamento', '<=', $data->data_hora_fechamento_final);// create the filter 
        }

        if (isset($data->system_users_abertura_id) AND ( (is_scalar($data->system_users_abertura_id) AND $data->system_users_abertura_id !== '') OR (is_array($data->system_users_abertura_id) AND (!empty($data->system_users_abertura_id)) )) )
        {

            $filters[] = new TFilter('system_users_abertura_id', '=', $data->system_users_abertura_id);// create the filter 
        }

        if (isset($data->system_users_operador_id) AND ( (is_scalar($data->system_users_operador_id) AND $data->system_users_operador_id !== '') OR (is_array($data->system_users_operador_id) AND (!empty($data->system_users_operador_id)) )) )
        {

            $filters[] = new TFilter('system_users_operador_id', '=', $data->system_users_operador_id);// create the filter 
        }

        if (isset($data->system_users_fechamento_id) AND ( (is_scalar($data->system_users_fechamento_id) AND $data->system_users_fechamento_id !== '') OR (is_array($data->system_users_fechamento_id) AND (!empty($data->system_users_fechamento_id)) )) )
        {

            $filters[] = new TFilter('system_users_fechamento_id', '=', $data->system_users_fechamento_id);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Caixa
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

}

