<?php

class CentroCustoForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'CentroCusto';
    private static $primaryKey = 'id';
    private static $formName = 'form_CentroCustoForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Cadastro de centro de custo");

        $criteria_centro_custo_pai_id = new TCriteria();

        $filterVar = CentroCustoTipo::Sintetico;
        $criteria_centro_custo_pai_id->add(new TFilter('centro_custo_tipo_id', '=', $filterVar)); 
        $filterVar = TSession::getValue("userunitid");
        $criteria_centro_custo_pai_id->add(new TFilter('system_unit_id', '=', $filterVar)); 

        $id = new TEntry('id');
        $numero = new TEntry('numero');
        $centro_custo_tipo_id = new TDBCombo('centro_custo_tipo_id', 'kraftsoft', 'CentroCustoTipo', 'id', '{descricao}','descricao asc'  );
        $centro_custo_pai_id = new TDBCombo('centro_custo_pai_id', 'kraftsoft', 'CentroCusto', 'id', '{descricao}','descricao asc' , $criteria_centro_custo_pai_id );
        $descricao = new TEntry('descricao');
        $criado_em = new TDateTime('criado_em');
        $criado_por = new TEntry('criado_por');
        $alterado_em = new TDateTime('alterado_em');
        $alterado_por = new TEntry('alterado_por');

        $numero->addValidation("Número", new TRequiredValidator()); 
        $centro_custo_tipo_id->addValidation("Centro custo tipo id", new TRequiredValidator()); 
        $descricao->addValidation("Descrição", new TRequiredValidator()); 

        $centro_custo_pai_id->enableSearch();
        $centro_custo_tipo_id->enableSearch();

        $criado_em->setMask('dd/mm/yyyy hh:ii');
        $alterado_em->setMask('dd/mm/yyyy hh:ii');

        $criado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $alterado_em->setDatabaseMask('yyyy-mm-dd hh:ii');

        $id->setEditable(false);
        $criado_em->setEditable(false);
        $criado_por->setEditable(false);
        $alterado_em->setEditable(false);
        $alterado_por->setEditable(false);

        $id->setSize(100);
        $numero->setSize('100%');
        $descricao->setSize('100%');
        $criado_em->setSize('100%');
        $criado_por->setSize('100%');
        $alterado_em->setSize('100%');
        $alterado_por->setSize('100%');
        $centro_custo_pai_id->setSize('100%');
        $centro_custo_tipo_id->setSize('100%');

        TTransaction::open(self::$database);
        $config_filial = ConfiguracoesFilial::where('system_unit_id', '=', TSession::getValue('userunitid'))->first();
        TTransaction::close();

        if ($config_filial)                                    
        {
            $numero->setMask($config_filial->centro_custo_mascara);
        }

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Número:", '#ff0000', '14px', null, '100%'),$numero]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Tipo:", '#ff0000', '14px', null, '100%'),$centro_custo_tipo_id],[new TLabel("Centro de Custo PAI SINTÉTICO:", null, '14px', null, '100%'),$centro_custo_pai_id]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Descrição:", '#FF0000', '14px', null),$descricao]);
        $row3->layout = [' col-sm-12'];

        $bcontainer_6346ab154027f = new BContainer('bcontainer_6346ab154027f');
        $this->bcontainer_6346ab154027f = $bcontainer_6346ab154027f;

        $bcontainer_6346ab154027f->setTitle("Histórico de Alterações", '#333', '18px', '', '#fff');
        $bcontainer_6346ab154027f->setBorderColor('#c0c0c0');

        $row4 = $bcontainer_6346ab154027f->addFields([new TLabel("Criado Em:", null, '14px', null, '100%'),$criado_em],[new TLabel("Criado Por:", null, '14px', null, '100%'),$criado_por],[new TLabel("Alterado Em:", null, '14px', null, '100%'),$alterado_em],[new TLabel("Alterado Por:", null, '14px', null, '100%'),$alterado_por]);
        $row4->layout = [' col-sm-2',' col-sm-4',' col-sm-2',' col-sm-4'];

        $row5 = $this->form->addFields([$bcontainer_6346ab154027f]);
        $row5->layout = [' col-sm-12'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onclear = $this->form->addAction("Limpar formulário", new TAction([$this, 'onClear']), 'fas:eraser #dd5a43');
        $this->btn_onclear = $btn_onclear;

        $btn_onshow = $this->form->addAction("Voltar", new TAction(['CentroCustoList', 'onShow']), 'fas:arrow-left #000000');
        $this->btn_onshow = $btn_onshow;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Financeiro","Cadastro de centro de custo"]));
        }
        $container->add($this->form);

        parent::add($container);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new CentroCusto(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);

            $object->system_unit_id = TSession::getValue('userunitid');

            $object->store(); // save the object 

            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('CentroCustoList', 'onShow', $loadPageParam); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new CentroCusto($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

