<?php

class ContaCorrenteLancamentosList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'ContaCorrenteLancamentos';
    private static $primaryKey = 'id';
    private static $formName = 'form_ContaCorrenteLancamentosList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de lançamentos de conta corrente");
        $this->limit = 50;

        $criteria_conta_corrente_id = new TCriteria();

        $filterVar = TSession::getValue("userunitid");
        $criteria_conta_corrente_id->add(new TFilter('system_unit_id', '=', $filterVar)); 

        $conta_corrente_id = new TDBCombo('conta_corrente_id', 'kraftsoft', 'ContaCorrente', 'id', '{nome}','nome asc' , $criteria_conta_corrente_id );
        $conta_corrente_lanc_tipo_id = new TDBCombo('conta_corrente_lanc_tipo_id', 'kraftsoft', 'ContaCorrenteLancTipo', 'id', '{descricao}','descricao asc'  );
        $data_lancamento_inicial = new TDate('data_lancamento_inicial');
        $data_lancamento_final = new TDate('data_lancamento_final');
        $documento = new TEntry('documento');
        $conta_corrente_lanc_origem_id = new TDBCombo('conta_corrente_lanc_origem_id', 'kraftsoft', 'ContaCorrenteLancOrigem', 'id', '{descricao}','descricao asc'  );
        $saldo_conta_geral = new TNumeric('saldo_conta_geral', '2', ',', '.' );
        $saldo_conta_periodo = new TNumeric('saldo_conta_periodo', '2', ',', '.' );
        $total_entradas_geral = new TNumeric('total_entradas_geral', '2', ',', '.' );
        $total_saidas_geral = new TNumeric('total_saidas_geral', '2', ',', '.' );


        $data_lancamento_final->setMask('dd/mm/yyyy');
        $data_lancamento_inicial->setMask('dd/mm/yyyy');

        $data_lancamento_final->setDatabaseMask('yyyy-mm-dd');
        $data_lancamento_inicial->setDatabaseMask('yyyy-mm-dd');

        $conta_corrente_id->enableSearch();
        $conta_corrente_lanc_tipo_id->enableSearch();
        $conta_corrente_lanc_origem_id->enableSearch();

        $saldo_conta_geral->setEditable(false);
        $total_saidas_geral->setEditable(false);
        $saldo_conta_periodo->setEditable(false);
        $total_entradas_geral->setEditable(false);

        $saldo_conta_geral->setValue('0,00');
        $total_saidas_geral->setValue('0,00');
        $saldo_conta_periodo->setValue('0,00');
        $total_entradas_geral->setValue('0,00');

        $documento->setSize('100%');
        $conta_corrente_id->setSize('100%');
        $saldo_conta_geral->setSize('100%');
        $total_saidas_geral->setSize('100%');
        $saldo_conta_periodo->setSize('100%');
        $total_entradas_geral->setSize('100%');
        $data_lancamento_final->setSize('100%');
        $data_lancamento_inicial->setSize('100%');
        $conta_corrente_lanc_tipo_id->setSize('100%');
        $conta_corrente_lanc_origem_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Conta Corrente:", null, '14px', null),$conta_corrente_id],[new TLabel("Tipo Lançamento:", null, '14px', null),$conta_corrente_lanc_tipo_id]);
        $row1->layout = [' col-sm-6',' col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Data Lançamento Inicial:", null, '14px', null, '100%'),$data_lancamento_inicial],[new TLabel("Data Lançamento Final:", null, '14px', null, '100%'),$data_lancamento_final],[new TLabel("Documento:", null, '14px', null, '100%'),$documento],[new TLabel("Origem:", null, '14px', null),$conta_corrente_lanc_origem_id]);
        $row2->layout = ['col-sm-3','col-sm-3',' col-sm-3',' col-sm-3'];

        $bcontainer_63492d5b11f43 = new BContainer('bcontainer_63492d5b11f43');
        $this->bcontainer_63492d5b11f43 = $bcontainer_63492d5b11f43;

        $bcontainer_63492d5b11f43->setTitle("Saldos", '#333', '18px', '', '#fff');
        $bcontainer_63492d5b11f43->setBorderColor('#c0c0c0');

        $row3 = $bcontainer_63492d5b11f43->addFields([new TLabel("Saldo conta geral:", null, '14px', null),$saldo_conta_geral],[new TLabel("Saldo conta no periodo:", null, '14px', null),$saldo_conta_periodo],[new TLabel("Entradas:", null, '14px', null),$total_entradas_geral],[new TLabel("Saídas:", null, '14px', null),$total_saidas_geral]);
        $row3->layout = [' col-sm-3',' col-sm-3',' col-sm-3',' col-sm-3'];

        $row4 = $this->form->addFields([$bcontainer_63492d5b11f43]);
        $row4->layout = [' col-sm-12'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Cadastrar", new TAction(['ContaCorrenteLancamentosForm', 'onShow']), 'fas:plus #69aa46');
        $this->btn_onshow = $btn_onshow;

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('conta_corrente_id', 'in', "(SELECT id FROM conta_corrente WHERE system_unit_id = '{$filterVar}')"));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'left');
        $column_conta_corrente_lanc_origem_descricao = new TDataGridColumn('conta_corrente_lanc_origem->descricao', "Origem", 'left');
        $column_conta_corrente_nome = new TDataGridColumn('conta_corrente->nome', "Conta corrente", 'left');
        $column_documento = new TDataGridColumn('documento', "Documento", 'left');
        $column_data_lancamento_transformed = new TDataGridColumn('data_lancamento', "Data Lançamento", 'left');
        $column_historico = new TDataGridColumn('historico', "Histórico", 'left');
        $column_valor_transformed = new TDataGridColumn('valor', "Valor", 'right');

        $column_data_lancamento_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_valor_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {

                if ($object->conta_corrente_lanc_tipo_id==ContaCorrenteLancTipo::Debito)
                {
                    return '<font color="red"><b>-'.number_format($object->valor, 2, ',', '.').'</b></font>';
                }
                else
                {
                    return '<b>'.number_format($object->valor, 2, ',', '.').'<b>';
                }
        });        

        $order_valor_transformed = new TAction(array($this, 'onReload'));
        $order_valor_transformed->setParameter('order', 'valor');
        $column_valor_transformed->setAction($order_valor_transformed);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_conta_corrente_lanc_origem_descricao);
        $this->datagrid->addColumn($column_conta_corrente_nome);
        $this->datagrid->addColumn($column_documento);
        $this->datagrid->addColumn($column_data_lancamento_transformed);
        $this->datagrid->addColumn($column_historico);
        $this->datagrid->addColumn($column_valor_transformed);

        $action_group = new TDataGridActionGroup("", 'fas:cog');
        $action_group->addHeader('');

        $action_onEdit = new TDataGridAction(array('ContaCorrenteLancamentosForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('ContaCorrenteLancamentosList::onExibeEditar');

        $action_group->addAction($action_onEdit);

        $action_onDelete = new TDataGridAction(array('ContaCorrenteLancamentosList', 'onDelete'));
        $action_onDelete->setUseButton(TRUE);
        $action_onDelete->setButtonClass('btn btn-default');
        $action_onDelete->setLabel("Excluir");
        $action_onDelete->setImage('fas:trash-alt #dd5a43');
        $action_onDelete->setField(self::$primaryKey);
        $action_onDelete->setDisplayCondition('ContaCorrenteLancamentosList::onExibeExcluir');

        $action_group->addAction($action_onDelete);

        $action_onVerOrigem = new TDataGridAction(array('ContaCorrenteLancamentosList', 'onVerOrigem'));
        $action_onVerOrigem->setUseButton(TRUE);
        $action_onVerOrigem->setButtonClass('btn btn-default');
        $action_onVerOrigem->setLabel("Ver Origem");
        $action_onVerOrigem->setImage('fas:atom #4CAF50');
        $action_onVerOrigem->setField(self::$primaryKey);

        $action_group->addAction($action_onVerOrigem);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['ContaCorrenteLancamentosList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['ContaCorrenteLancamentosList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['ContaCorrenteLancamentosList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['ContaCorrenteLancamentosList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_right_actions->add($dropdown_button_exportar);

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Financeiro","Lançamentos de conta corrente"]));
        }
        $container->add($this->form);
        $container->add($panel);

        parent::add($container);

    }

    public static function onExibeEditar($object)
    {
        try 
        {
            if($object->conta_corrente_lanc_origem_id==ContaCorrenteLancOrigem::Avulso)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onDelete($param = null) 
    { 
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new ContaCorrenteLancamentos($key, FALSE); 

                //deletando o lançamento contabil de tiver
                $lanc_contabil = ContabilLancamentos::where('conta_corrente_lancamentos_id','=',$object->id)->first(); 

                if ($lanc_contabil)
                {
                    $lanc_contabil->delete();                
                }                

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }
    public static function onExibeExcluir($object)
    {
        try 
        {
            if($object->conta_corrente_lanc_origem_id==ContaCorrenteLancOrigem::Avulso)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onVerOrigem($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);   
            $cc_lancamento = ContaCorrenteLancamentos::find($param['key']);

            switch ($cc_lancamento->conta_corrente_lanc_origem_id)
            {
                case ContaCorrenteLancOrigem::ContasPag:
                {    
                     //$cpag_baixas = ContaspagBaixas::find($cc_lancamento->contaspag_baixas_id);
                     $pageParam = ['key' => $cc_lancamento->contaspag_baixas->contaspag_id]; // ex.: = ['key' => 10]

                     TApplication::loadPage('ContaspagFormView', 'OnShow', $pageParam);                
                    break;

                }

                case ContaCorrenteLancOrigem::ContasRec:
                {    
                     $pageParam = ['key' => $cc_lancamento->contasrec_baixas->contasrec_id]; // ex.: = ['key' => 10]

                     TApplication::loadPage('ContasrecFormView', 'OnShow', $pageParam);                
                    break;

                }                

                case  ContaCorrenteLancOrigem::Avulso:
                    new TMessage('info', 'Este título é de origem avulsa!');                  
                break;    
            }

            // if ($contaspag->compras_id<>null)
            // {
            //      $pageParam = ['key' => $estoquelancamento->venda_id]; // ex.: = ['key' => 10]

            //      TApplication::loadPage('VendaFormView', 'OnShow', $pageParam);                
            // }
            // else
            {

            }

            TTransaction::close();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('object');
                $object->data  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->conta_corrente_id) AND ( (is_scalar($data->conta_corrente_id) AND $data->conta_corrente_id !== '') OR (is_array($data->conta_corrente_id) AND (!empty($data->conta_corrente_id)) )) )
        {

            $filters[] = new TFilter('conta_corrente_id', '=', $data->conta_corrente_id);// create the filter 
        }

        if (isset($data->conta_corrente_lanc_tipo_id) AND ( (is_scalar($data->conta_corrente_lanc_tipo_id) AND $data->conta_corrente_lanc_tipo_id !== '') OR (is_array($data->conta_corrente_lanc_tipo_id) AND (!empty($data->conta_corrente_lanc_tipo_id)) )) )
        {

            $filters[] = new TFilter('conta_corrente_lanc_tipo_id', '=', $data->conta_corrente_lanc_tipo_id);// create the filter 
        }

        if (isset($data->data_lancamento_inicial) AND ( (is_scalar($data->data_lancamento_inicial) AND $data->data_lancamento_inicial !== '') OR (is_array($data->data_lancamento_inicial) AND (!empty($data->data_lancamento_inicial)) )) )
        {

            $filters[] = new TFilter('data_lancamento', '>=', $data->data_lancamento_inicial);// create the filter 
        }

        if (isset($data->data_lancamento_final) AND ( (is_scalar($data->data_lancamento_final) AND $data->data_lancamento_final !== '') OR (is_array($data->data_lancamento_final) AND (!empty($data->data_lancamento_final)) )) )
        {

            $filters[] = new TFilter('data_lancamento', '<=', $data->data_lancamento_final);// create the filter 
        }

        if (isset($data->documento) AND ( (is_scalar($data->documento) AND $data->documento !== '') OR (is_array($data->documento) AND (!empty($data->documento)) )) )
        {

            $filters[] = new TFilter('documento', 'like', "%{$data->documento}%");// create the filter 
        }

        if (isset($data->conta_corrente_lanc_origem_id) AND ( (is_scalar($data->conta_corrente_lanc_origem_id) AND $data->conta_corrente_lanc_origem_id !== '') OR (is_array($data->conta_corrente_lanc_origem_id) AND (!empty($data->conta_corrente_lanc_origem_id)) )) )
        {

            $filters[] = new TFilter('conta_corrente_lanc_origem_id', '=', $data->conta_corrente_lanc_origem_id);// create the filter 
        }

        $entradas = 0;
        $saidas = 0;

        TTransaction::open('kraftsoft');

        $saldo_geral = ContaCorrenteLancamentos::ObterSaldoGeral($param['conta_corrente_id'], $entradas,$saidas,TSession::getValue('userunitid'));

        $saldo_periodo = ContaCorrenteLancamentos::ObterSaldoPeriodo($param['conta_corrente_id'],
                                                                     $param['data_lancamento_inicial'],
                                                                     $param['data_lancamento_final'], 
                                                                     $entradas,
                                                                     $saidas,
                                                                     TSession::getValue('userunitid'));

       TTransaction::close();                                                                     

        $object = new stdClass();
        $object->saldo_conta_geral = number_format($saldo_geral, 2, ',', '.');
        $object->saldo_conta_periodo = number_format($saldo_periodo, 2, ',', '.');
        // $object->total_entradas_geral = number_format($entradas, 2, ',', '.');
        // $object->total_saidas_geral = number_format($saidas, 2, ',', '.');        
        TForm::sendData(self::$formName, $object);

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            if (empty($_REQUEST['method']) || ($_REQUEST['method'] == 'onShow'))
            {
                return;
            }
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for ContaCorrenteLancamentos
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            $entradas = 0;
            $saidas = 0;

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                       if ($object->conta_corrente_lanc_tipo_id==ContaCorrenteLancTipo::Debito)
                       {
                           $saidas += $object->valor;
                       }
                       else
                       {
                           $entradas += $object->valor;
                       }                             

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            $object = new stdClass();
            $object->total_entradas_geral = number_format($entradas, 2, ',', '.');
            $object->total_saidas_geral = number_format($saidas, 2, ',', '.');        
            TForm::sendData(self::$formName, $object);            

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

}

