<?php

class ContabilLancamentosReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'ContabilLancamentos';
    private static $primaryKey = 'id';
    private static $formName = 'form_ContabilLancamentosReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Relatório Plano de Contas");

        $data_lancamento = new TDate('data_lancamento');
        $data_lancamento_final = new TDate('data_lancamento_final');
        $historico = new TEntry('historico');
    
        $fornecedor_id = new TDBCombo('fornecedor_id', 'kraftsoft', 'Fornecedor', 'id', '{nome}','nome asc'  );
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{nome}','nome asc'  );
        
        $fornecedor_id->enableSearch();
        $cliente_id->enableSearch();

        $data_lancamento->setDatabaseMask('yyyy-mm-dd');
        $data_lancamento->setMask('dd/mm/yyyy');

        $data_lancamento_final->setDatabaseMask('yyyy-mm-dd');
        $data_lancamento_final->setMask('dd/mm/yyyy');
        
        $tipo_impressao = new TCombo('tipo_impressao');
        $tipo_impressao->addItems(['S' => 'Sintético', 'A' => 'Analitico']);

        $tipo_periodo = new TCombo('tipo_periodo');
        $tipo_periodo->addItems(['V' => 'Vencimento', 'P' => 'Pagamento']);

        $tipo_impressao->setSize('100%');
        $fornecedor_id->setSize('100%');
        $cliente_id->setSize('100%');
        $tipo_periodo->setSize('100%');
        
        $tipo_periodo->addValidation("Tipo Período", new TRequiredValidator());
        $data_lancamento_final->addValidation("Período Final", new TRequiredValidator()); 
        $data_lancamento->addValidation("Período Inicial", new TRequiredValidator()); 
        $tipo_impressao->addValidation("Tipo", new TRequiredValidator()); 
        
        $historico->setSize('100%');
        $data_lancamento->setSize(150);
        $data_lancamento_final->setSize(150);

        $row1 = $this->form->addFields([new TLabel("Tipo:", null, '14px', null, '100%'),$tipo_impressao],[]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row1 = $this->form->addFields([new TLabel("Tipo Período:", null, '14px', null, '100%'),$tipo_periodo],[]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row1 = $this->form->addFields([new TLabel("Período:", null, '14px', null, '100%'),$data_lancamento, $data_lancamento_final],[]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row1 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null, '100%'),$cliente_id],[]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row1 = $this->form->addFields([new TLabel("Fornecedor:", null, '14px', null, '100%'),$fornecedor_id],[]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratepdf = $this->form->addAction("Gerar", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Financeiro","Relatório Plano de Contas"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }

    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }

    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }

    public function onGenerateRtf($param = null) 
    {
        $this->onGenerate('rtf');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->data_lancamento) AND ( (is_scalar($data->data_lancamento) AND $data->data_lancamento !== '') OR (is_array($data->data_lancamento) AND (!empty($data->data_lancamento)) )) )
        {

            $filters[] = new TFilter('data_lancamento', '=', $data->data_lancamento);// create the filter 
        }
        if (isset($data->historico) AND ( (is_scalar($data->historico) AND $data->historico !== '') OR (is_array($data->historico) AND (!empty($data->historico)) )) )
        {

            $filters[] = new TFilter('historico', 'like', "%{$data->historico}%");// create the filter 
        }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {
            $filters[] = new TFilter('contasrec_id', 'in', "(select id from contasrec where deletado_em is null and cliente_id = $data->cliente_id)"); 
        }

        if (isset($data->fornecedor_id) AND ( (is_scalar($data->fornecedor_id) AND $data->fornecedor_id !== '') OR (is_array($data->fornecedor_id) AND (!empty($data->fornecedor_id)) )) )
        {
            $filters[] = new TFilter('contaspag_contabil_id', 'in', "(select id from contaspag where deletado_em is null and fornecedor_id = $data->fornecedor_id)"); 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            
            $data = $this->form->getData();
            
            $this->form->validate();
   
            // $param = [];
            // // creates a repository for ContabilLancamentos
            // $repository = new TRepository(self::$activeRecord);
            // // creates a criteria
            // $criteria = new TCriteria;

            // $criteria->setProperties($param);

            // if ($filters)
            // {
            //     foreach ($filters as $filter) 
            //     {
            //         $criteria->add($filter);       
            //     }
            // }

            // // load the objects according to criteria
            // $objects = $repository->load($criteria, FALSE);

            // if ($objects)
            // {
                    
            // }
            // else
            // {
            //     new TMessage('error', _t('No records found'));
            // }
            
            $html = new AdiantiHTMLDocumentParser('app/resources/relatorio_plano_conta.html', 'A4', 'portrait'); 
            $output = $html->getContents();      
            
            $grid = null;
            
            $grid .= "
                <table class=\"grid\">
                    <tr style=\"text-align: center; background-color: #2C3E50; color: white; font-style:bold;\">
                        <td>DESCRIÇÃO</td>
                        <td>NÃO LIQUIDADO</td>
                        <td>LIQUIDADO</td>
                    </tr>    
            ";  
            
            $plano_contas = PlanoConta::where('system_unit_id', '=', TSession::getValue('userunitid'))->orderBy('numero')->load();
            
            foreach($plano_contas as $plano_conta){

                    $valor = self::SomaValor($plano_conta->numero, $data, ContabilLancamentosSituacao::NaoLiquidado);
                    $valor_2 = self::SomaValor($plano_conta->numero, $data, ContabilLancamentosSituacao::Liquidado);

                    if($valor == 0 and $valor_2 == 0){
                        continue;
                    }
                    
                    $valor = number_format($valor, 2, ',', '.');
                    $valor_2 = number_format($valor_2, 2, ',', '.');
                    
                    $vezes = substr_count($plano_conta->numero, '.');
                    
                    if($vezes == 0){
                        $grid .= "
                                <tr style=\"background-color: #22A7F0; color: white;\">
                                    <td style=\"padding-left:10px\"><b>".$plano_conta->numero.' '.$plano_conta->descricao."</b></td>
                                    <td style=\"text-align: right;\"><b>R$ $valor</b></td>
                                    <td style=\"text-align: right;\"><b>R$ $valor_2</b></td>
                                </tr>
                        ";
                    }else{
                        $padding = 10 * ($vezes + 1);
                        
                        $color = $vezes == 1 ? "#BFBFBF" : "white";
                        
                        $grid .= "
                                <tr style=\"background-color: $color; color: black;\">
                                    <td style=\"padding-left:".$padding."px\"><b>".$plano_conta->numero.' '.$plano_conta->descricao."</b></td>
                                    <td style=\"text-align: right;\"><b>R$ $valor</b></td>
                                    <td style=\"text-align: right;\"><b>R$ $valor_2</b></td>
                                </tr>
                        ";
                    }     
                    
                    if($plano_conta->plano_conta_tipo_id == PlanoContaTipo::Analitico and $data->tipo_impressao == 'A'){
                        $lancamentos = self::BuscaLancamentos($plano_conta->id, $data);

                        foreach($lancamentos as $lancamento){
                            $padding = 10 * ($vezes + 3);
                            
                            if($lancamento->contabil_lancamentos_situacao_id == ContabilLancamentosSituacao::Liquidado){
                                $valor_2 = $lancamento->valor;
                                $valor = 0;
                            }else{
                                $valor = $lancamento->valor;
                                $valor_2 = 0;
                            }
                            
                            $grid .= "
                                    <tr style=\"background-color: white; color: black; font-style:italic\">
                                        <td style=\"padding-left:".$padding."px\">".TDate::date2br($lancamento->data_lancamento).' - '.$lancamento->historico."</td>
                                        <td style=\"text-align: right;\">R$ ".number_format($valor, 2, ',', '.')."</td>
                                        <td style=\"text-align: right;\">R$ ".number_format($valor_2, 2, ',', '.')."</td>
                                    </tr>
                            ";
                        }
                    }
                    
            }
            

            $grid .= "</table>";
            
            $output    = str_replace('{dados}', $grid, $output);
            $output    = str_replace('{tipo}', $tipo = $data->tipo_impressao == 'S' ? 'Sintético' : 'Analitico', $output);
            $output    = str_replace('{periodo}', "Período ".TDate::date2br($data->data_lancamento)." - ".TDate::date2br($data->data_lancamento_final), $output);
            
            $output    = str_replace('{nome_unidade}', TSession::getValue('userunitname'), $output);
            
            $date = new DateTime(date('Y-m-d H:i:s'));
            $date = $date->format('d/m/Y H:i');
            
            $output    = str_replace('{data_hora}', $date, $output);

            $document = 'tmp/' . uniqid() . '.pdf';
    
            $html = AdiantiHTMLDocumentParser::newFromString($output);
    
            $html->saveAsPDF($document);
    
            parent::openFile($document);            

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }
    
    public static function SomaValor($numero, $data, $situacao){
        
        $numero = trim($numero)."%";
        
        //var_dump($numero);
        
        $plano_contas = PlanoConta::where('numero', 'like', $numero)
                                  ->where('plano_conta_tipo_id', '=', PlanoContaTipo::Analitico)
                                  ->where('system_unit_id', '=', TSession::getValue('userunitid'))
                                  ->getIndexedArray('id', 'id');
                                  

        if(count($plano_contas) == 0){
            return 0;    
        }   

        $repository = new TRepository(self::$activeRecord);
        $criteria = new TCriteria;
        
        // if (isset($data->data_lancamento) AND ( (is_scalar($data->data_lancamento) AND $data->data_lancamento !== '') OR (is_array($data->data_lancamento) AND (!empty($data->data_lancamento)) )) )
        // {
        //     $criteria->add(new TFilter('data_lancamento', '>=', $data->data_lancamento));
        // }

        // if (isset($data->data_lancamento_final) AND ( (is_scalar($data->data_lancamento_final) AND $data->data_lancamento_final !== '') OR (is_array($data->data_lancamento_final) AND (!empty($data->data_lancamento_final)) )) )
        // {
        //     $criteria->add(new TFilter('data_lancamento', '<=', $data->data_lancamento_final));
        // }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {
            $criteria->add(new TFilter('contasrec_id', 'in', "(select id from contasrec where deletado_em is null and cliente_id = $data->cliente_id)")); 
        }

        if (isset($data->fornecedor_id) AND ( (is_scalar($data->fornecedor_id) AND $data->fornecedor_id !== '') OR (is_array($data->fornecedor_id) AND (!empty($data->fornecedor_id)) )) )
        {
            $criteria->add(new TFilter('contaspag_contabil_id', 'in', "(select id from contaspag_contabil where deletado_em is null and contaspag_id in (select id from contaspag where fornecedor_id = $data->fornecedor_id))")); 
        }
        
        if($data->tipo_periodo == 'L'){
            $filtro = 'data_lancamento';    
        }elseif($data->tipo_periodo == 'V'){
            $filtro = 'data_vencimento';
        }else{
            $filtro = 'data_pagamento';
        }
        
        if (isset($data->data_lancamento) AND ( (is_scalar($data->data_lancamento) AND $data->data_lancamento !== '') OR (is_array($data->data_lancamento) AND (!empty($data->data_lancamento)) )) )
        {
            $criteria->add(new TFilter('id', 'in', "(select contabil_lancamento_id from view_consulta_contabil_por_data where $filtro >= '$data->data_lancamento')")); 
        }

        if (isset($data->data_lancamento_final) AND ( (is_scalar($data->data_lancamento_final) AND $data->data_lancamento_final !== '') OR (is_array($data->data_lancamento_final) AND (!empty($data->data_lancamento_final)) )) )
        {
            $criteria->add(new TFilter('id', 'in', "(select contabil_lancamento_id from view_consulta_contabil_por_data where $filtro <= '$data->data_lancamento_final')")); 
        }
        
        $criteria->add(new TFilter('plano_conta_id', 'in', $plano_contas));
        $criteria->add(new TFilter('contabil_lancamentos_situacao_id', '=', $situacao));
        
        $lancamentos = $repository->load($criteria, FALSE);
        
        $valor = 0;
        
        foreach($lancamentos as $lancamento){
            $valor += $lancamento->valor;    
        }
        
        return $valor;

    }

    public static function BuscaLancamentos($plano_conta_id, $data){
        $repository = new TRepository(self::$activeRecord);
        $criteria = new TCriteria;
        
        // if (isset($data->data_lancamento) AND ( (is_scalar($data->data_lancamento) AND $data->data_lancamento !== '') OR (is_array($data->data_lancamento) AND (!empty($data->data_lancamento)) )) )
        // {
        //     $criteria->add(new TFilter('data_lancamento', '>=', $data->data_lancamento));
        // }

        // if (isset($data->data_lancamento_final) AND ( (is_scalar($data->data_lancamento_final) AND $data->data_lancamento_final !== '') OR (is_array($data->data_lancamento_final) AND (!empty($data->data_lancamento_final)) )) )
        // {
        //     $criteria->add(new TFilter('data_lancamento', '<=', $data->data_lancamento_final));
        // }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {
            $criteria->add(new TFilter('contasrec_id', 'in', "(select id from contasrec where deletado_em is null and cliente_id = $data->cliente_id)")); 
        }

        if (isset($data->fornecedor_id) AND ( (is_scalar($data->fornecedor_id) AND $data->fornecedor_id !== '') OR (is_array($data->fornecedor_id) AND (!empty($data->fornecedor_id)) )) )
        {
            $criteria->add(new TFilter('contaspag_contabil_id', 'in', "(select id from contaspag_contabil where deletado_em is null and contaspag_id in (select id from contaspag where fornecedor_id = $data->fornecedor_id))")); 
        }
        
        if($data->tipo_periodo == 'L'){
            $filtro = 'data_lancamento';    
        }elseif($data->tipo_periodo == 'V'){
            $filtro = 'data_vencimento';
        }else{
            $filtro = 'data_pagamento';
        }

        if (isset($data->data_lancamento) AND ( (is_scalar($data->data_lancamento) AND $data->data_lancamento !== '') OR (is_array($data->data_lancamento) AND (!empty($data->data_lancamento)) )) )
        {
            $criteria->add(new TFilter('id', 'in', "(select contabil_lancamento_id from view_consulta_contabil_por_data where $filtro >= '$data->data_lancamento')")); 
        }

        if (isset($data->data_lancamento_final) AND ( (is_scalar($data->data_lancamento_final) AND $data->data_lancamento_final !== '') OR (is_array($data->data_lancamento_final) AND (!empty($data->data_lancamento_final)) )) )
        {
            $criteria->add(new TFilter('id', 'in', "(select contabil_lancamento_id from view_consulta_contabil_por_data where $filtro <= '$data->data_lancamento_final')")); 
        }

        
        $criteria->add(new TFilter('plano_conta_id', '=', $plano_conta_id));
        
        $lancamentos = $repository->load($criteria, FALSE);
        
        return $lancamentos;
        
    }

    public function onShow($param = null)
    {
  // Código gerado pelo snippet: "Enviar dados para campo"
        $object = new stdClass();
        $object->data_lancamento = TDate::date2br(date("Y-m-01"));
        $object->data_lancamento_final = TDate::date2br(date("Y-m-t"));
        $object->tipo_impressao = 'S';
        $object->tipo_periodo = 'L';
      
        TForm::sendData(self::$formName, $object);
        // -----
    }


}

