<?php

class ContaspagAnexosForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'ContaspagAnexos';
    private static $primaryKey = 'id';
    private static $formName = 'form_ContaspagAnexosForm';

    use Adianti\Base\AdiantiFileSaveTrait;

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Cadastro de anexos de contas a pagar");


        $id = new THidden('id');
        $contaspag_id = new THidden('contaspag_id');
        $titulos_tipo_anexo_id = new TDBCombo('titulos_tipo_anexo_id', 'kraftsoft', 'TitulosTipoAnexo', 'id', '{descricao}','descricao asc'  );
        $descricao = new TEntry('descricao');
        $arquivo = new TFile('arquivo');
        $criado_em = new TDateTime('criado_em');
        $criado_por = new TEntry('criado_por');
        $alterado_em = new TDateTime('alterado_em');
        $alterado_por = new TEntry('alterado_por');

        $titulos_tipo_anexo_id->addValidation("Titulos tipo anexo id", new TRequiredValidator()); 
        $descricao->addValidation("Descrição", new TRequiredValidator()); 
        $arquivo->addValidation("Arquivo", new TRequiredValidator()); 

        $contaspag_id->setValue(TSession::getValue("contaspagformview_contaspag_id"));
        $titulos_tipo_anexo_id->enableSearch();
        $arquivo->enableFileHandling();
        $arquivo->setLimitUploadSize(2);
        $criado_em->setMask('dd/mm/yyyy hh:ii');
        $alterado_em->setMask('dd/mm/yyyy hh:ii');

        $criado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $alterado_em->setDatabaseMask('yyyy-mm-dd hh:ii');

        $criado_em->setEditable(false);
        $criado_por->setEditable(false);
        $alterado_em->setEditable(false);
        $alterado_por->setEditable(false);

        $id->setSize(200);
        $arquivo->setSize('100%');
        $contaspag_id->setSize(200);
        $descricao->setSize('100%');
        $criado_em->setSize('100%');
        $criado_por->setSize('100%');
        $alterado_em->setSize('100%');
        $alterado_por->setSize('100%');
        $titulos_tipo_anexo_id->setSize('100%');

        $row1 = $this->form->addFields([$id],[$contaspag_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Tipo de Anexo", '#ff0000', '14px', null, '100%'),$titulos_tipo_anexo_id],[new TLabel("Descrição:", '#ff0000', '14px', null, '100%'),$descricao]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Arquivo:", '#ff0000', '14px', null, '100%'),$arquivo]);
        $row3->layout = [' col-sm-12'];

        $bcontainer_63f2922fc1889 = new BContainer('bcontainer_63f2922fc1889');
        $this->bcontainer_63f2922fc1889 = $bcontainer_63f2922fc1889;

        $bcontainer_63f2922fc1889->setTitle("Histórico de Atualizações", '#333', '18px', '', '#fff');
        $bcontainer_63f2922fc1889->setBorderColor('#c0c0c0');

        $row4 = $bcontainer_63f2922fc1889->addFields([new TLabel("Criado Em:", null, '14px', null, '100%'),$criado_em],[new TLabel("Criado Por:", null, '14px', null, '100%'),$criado_por],[new TLabel("Alterado Em:", null, '14px', null, '100%'),$alterado_em],[new TLabel("Alterado Por:", null, '14px', null, '100%'),$alterado_por]);
        $row4->layout = [' col-sm-2',' col-sm-4',' col-sm-2',' col-sm-4'];

        $row5 = $this->form->addFields([$bcontainer_63f2922fc1889]);
        $row5->layout = [' col-sm-12'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=ContaspagAnexosForm]');
        $style->width = '85% !important';   
        $style->show(true);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new ContaspagAnexos(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            $arquivo_dir = 'app/files/kraft_arquivos/cpag_anexos';  

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);            

            $object->store(); // save the object 

            $this->saveFile($object, $data, 'arquivo', $arquivo_dir);
            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            if(!empty($object->contaspag->id))
            {
                $loadPageParam["key"] = $object->contaspag->id;
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('ContaspagFormView', 'onShow', $loadPageParam); 

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new ContaspagAnexos($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

