<?php

class ContasrecHistoricoCobrancaFormList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'ContasrecHistoricoCobranca';
    private static $primaryKey = 'id';
    private static $formName = 'form_ContasrecHistoricoCobrancaFormList';
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param)
    {
        parent::__construct();
        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Histórico de cobranças");
        $this->limit = 20;


        $contasrec_id = new TDBCombo('contasrec_id', 'kraftsoft', 'Contasrec', 'id', '{id}','id asc'  );
        $id = new THidden('id');
        $data_cobranca = new TDate('data_cobranca');
        $historico = new TText('historico');

        $contasrec_id->addValidation("Contas a Receber ID", new TRequiredValidator()); 
        $data_cobranca->addValidation("Data Cobrança", new TRequiredValidator()); 
        $historico->addValidation("Histórico", new TRequiredValidator()); 

        $contasrec_id->setEditable(false);
        $contasrec_id->enableSearch();
        $data_cobranca->setMask('dd/mm/yyyy');
        $data_cobranca->setValue(date('d/m/Y'));
        $data_cobranca->setDatabaseMask('yyyy-mm-dd');
        $id->setSize(200);
        $contasrec_id->setSize('100%');
        $data_cobranca->setSize('100%');
        $historico->setSize('100%', 70);

        $row1 = $this->form->addFields([new TLabel("Contas a Receber Id:", '#ff0000', '14px', null, '100%'),$contasrec_id,$id],[new TLabel("Data Cobrança:", '#FF0000', '14px', null, '100%'),$data_cobranca]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Histórico:", '#ff0000', '14px', null, '100%'),$historico]);
        $row2->layout = [' col-sm-12'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onclear = $this->form->addAction("Limpar formulário", new TAction([$this, 'onClear']), 'fas:eraser #dd5a43');
        $this->btn_onclear = $btn_onclear;

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("key_formhistoricocobranca");
        $this->filter_criteria->add(new TFilter('contasrec_id', '=', $filterVar));

        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(320);

        $column_data_cobranca_transformed = new TDataGridColumn('data_cobranca', "Data Cobrança", 'left');
        $column_historico = new TDataGridColumn('historico', "Histórico", 'left');
        $column_criado_em_transformed = new TDataGridColumn('criado_em', "Criado Em", 'left');
        $column_criado_por = new TDataGridColumn('criado_por', "Criado Por", 'left');
        $column_alterado_em_transformed = new TDataGridColumn('alterado_em', "Alterado Em", 'left');
        $column_alterado_por = new TDataGridColumn('alterado_por', "Alterado Por", 'left');

        $column_data_cobranca_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_criado_em_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_alterado_em_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });        

        $this->datagrid->addColumn($column_data_cobranca_transformed);
        $this->datagrid->addColumn($column_historico);
        $this->datagrid->addColumn($column_criado_em_transformed);
        $this->datagrid->addColumn($column_criado_por);
        $this->datagrid->addColumn($column_alterado_em_transformed);
        $this->datagrid->addColumn($column_alterado_por);

        $action_onEdit = new TDataGridAction(array('ContasrecHistoricoCobrancaFormList', 'onEdit'));
        $action_onEdit->setUseButton(true);
        $action_onEdit->setButtonClass('btn btn-default btn-sm');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);

        $this->datagrid->addAction($action_onEdit);

        $action_onDelete = new TDataGridAction(array('ContasrecHistoricoCobrancaFormList', 'onDelete'));
        $action_onDelete->setUseButton(true);
        $action_onDelete->setButtonClass('btn btn-default btn-sm');
        $action_onDelete->setLabel("Excluir");
        $action_onDelete->setImage('fas:trash-alt #dd5a43');
        $action_onDelete->setField(self::$primaryKey);

        $this->datagrid->addAction($action_onDelete);

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);
        parent::add($panel);

        $style = new TStyle('right-panel > .container-part[page-name=ContasrecHistoricoCobrancaFormList]');
        $style->width = '80% !important';   
        $style->show(true);

    }

    public function onDelete($param = null) 
    { 
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new ContasrecHistoricoCobranca($key, FALSE); 

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );

                $object = new stdClass();
                $object->contasrec_id = TSession::getValue('key_formhistoricocobranca');
                TForm::sendData(self::$formName, $object);

                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }
    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new ContasrecHistoricoCobranca(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);            

            $object->store(); // save the object 

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            //evento
            TTransaction::open(self::$database);

            Funcoes::VerificaNotificacaoEvento(TSession::getValue('userid'),
                                               Contasrec::Evento_ContasRecHistoricoCobrancasIncluidoOuAlterado,
                                               $object,
                                               'Contas a receber do cliente '.$object->contasrec->cliente->nome.' , '.
                                               'Valor do título: R$ '.number_format($object->contasrec->valor,2, ',', '.').' , '.
                                               'Vencimento: '.TDate::date2br($object->contasrec->data_vencimento).' , '.
                                               'Histórico: '.$object->historico); 

            TTransaction::close();                           

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle'); 

            $this->onReload();

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new ContasrecHistoricoCobranca($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for ContasrecHistoricoCobranca
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = clone $this->filter_criteria;

            $filterVar = TSession::getValue("key_formhistoricocobranca");
            $criteria->add(new TFilter('contasrec_id', '=', $filterVar));

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }
            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onClear( $param )
    {
        $this->form->clear(true);

        $object = new stdClass();
        $object->contasrec_id = TSession::getValue('key_formhistoricocobranca');

        TForm::sendData(self::$formName, $object);

    }

    public function onShow($param = null)
    {

        // TSession::setValue('key_formhistoricocobranca', $param['key']);

        $object = new stdClass();
        $object->contasrec_id = TSession::getValue('key_formhistoricocobranca');
        TForm::sendData(self::$formName, $object);

    } 

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  array('onReload')))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }


}

