<?php

class ContasrecHistoricoCobrancaReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'ContasrecHistoricoCobranca';
    private static $primaryKey = 'id';
    private static $formName = 'form_ContasrecHistoricoCobrancaReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Histórico de Cobranças");

        $data_cobranca_inicial = new TDate('data_cobranca_inicial');
        $data_cobranca_final = new TDate('data_cobranca_final');

        $data_cobranca_final->setSize('100%');
        $data_cobranca_inicial->setSize('100%');

        $data_cobranca_final->setMask('dd/mm/yyyy');
        $data_cobranca_inicial->setMask('dd/mm/yyyy');

        $data_cobranca_final->setDatabaseMask('yyyy-mm-dd');
        $data_cobranca_inicial->setDatabaseMask('yyyy-mm-dd');

        $row1 = $this->form->addFields([new TLabel("Data Cobrança Inicial:", null, '14px', null, '100%'),$data_cobranca_inicial],[new TLabel("Data Cobrança Final:", null, '14px', null, '100%'),$data_cobranca_final]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratehtml = $this->form->addAction("Gerar HTML", new TAction([$this, 'onGenerateHtml']), 'far:file-code #ffffff');
        $this->btn_ongeneratehtml = $btn_ongeneratehtml;
        $btn_ongeneratehtml->addStyleClass('btn-primary'); 

        $btn_ongeneratepdf = $this->form->addAction("Gerar PDF", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        $btn_ongeneratexls = $this->form->addAction("Gerar XLS", new TAction([$this, 'onGenerateXls']), 'far:file-excel #00a65a');
        $this->btn_ongeneratexls = $btn_ongeneratexls;

        $btn_ongeneratertf = $this->form->addAction("Gerar RTF", new TAction([$this, 'onGenerateRtf']), 'far:file-alt #324bcc');
        $this->btn_ongeneratertf = $btn_ongeneratertf;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Financeiro","Histórico de Cobranças"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }

    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }

    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }

    public function onGenerateRtf($param = null) 
    {
        $this->onGenerate('rtf');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->data_cobranca_inicial) AND ( (is_scalar($data->data_cobranca_inicial) AND $data->data_cobranca_inicial !== '') OR (is_array($data->data_cobranca_inicial) AND (!empty($data->data_cobranca_inicial)) )) )
        {

            $filters[] = new TFilter('data_cobranca', '>=', $data->data_cobranca_inicial);// create the filter 
        }
        if (isset($data->data_cobranca_final) AND ( (is_scalar($data->data_cobranca_final) AND $data->data_cobranca_final !== '') OR (is_array($data->data_cobranca_final) AND (!empty($data->data_cobranca_final)) )) )
        {

            $filters[] = new TFilter('data_cobranca', '<=', $data->data_cobranca_final);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            $param = [];
            // creates a repository for ContasrecHistoricoCobranca
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            $filterVar = TSession::getValue("userunitid");
            $criteria->add(new TFilter('contasrec_id', 'in', "(SELECT id FROM contasrec WHERE system_unit_id = '{$filterVar}')"));

            $param['order'] = 'contasrec_id,data_cobranca';
            $param['direction'] = 'asc';

            $criteria->setProperties($param);

            if ($filters)
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            if ($objects)
            {
                $widths = array(200,200);
                $reportExtension = 'pdf';
                switch ($format)
                {
                    case 'html':
                        $tr = new TTableWriterHTML($widths);
                        $reportExtension = 'html';
                        break;
                    case 'xls':
                        $tr = new TTableWriterXLS($widths);
                        $reportExtension = 'xls';
                        break;
                    case 'pdf':
                        $tr = new TTableWriterPDF($widths, 'L', 'A4');
                        $reportExtension = 'pdf';
                        break;
                    case 'htmlPdf':
                        $reportExtension = 'pdf';
                        $tr = new BTableWriterHtmlPDF($widths, 'L', 'A4');
                        break;
                    case 'rtf':
                        if (!class_exists('PHPRtfLite_Autoloader'))
                        {
                            PHPRtfLite::registerAutoloader();
                        }
                        $reportExtension = 'rtf';
                        $tr = new TTableWriterRTF($widths, 'L', 'A4');
                        break;
                }

                if (!empty($tr))
                {
                    // create the document styles
                    $tr->addStyle('title', 'Helvetica', '10', 'B',   '#ffffff', '#b9736d');
                    $tr->addStyle('datap', 'Arial', '10', '',    '#000000', '#f0f0f0');
                    $tr->addStyle('datai', 'Arial', '10', '',    '#000000', '#ffffff');
                    $tr->addStyle('header', 'Helvetica', '16', 'B',   '#000000', '#904F49');
                    $tr->addStyle('footer', 'Helvetica', '10', 'B',  '#000000', '#F2BEBC');
                    $tr->addStyle('break', 'Helvetica', '10', 'B',  '#ffffff', '#7e5251');
                    $tr->addStyle('total', 'Helvetica', '10', 'I',  '#000000', '#debbb9');
                    $tr->addStyle('breakTotal', 'Helvetica', '10', 'I',  '#000000', '#ffdfdd');

                    // add titles row
                    $tr->addRow();
                    $tr->addCell("Data Cobrança", 'left', 'title');
                    $tr->addCell("Histórico", 'left', 'title');

                    $grandTotal = [];
                    $breakTotal = [];
                    $breakValue = null;
                    $firstRow = true;

                    // controls the background filling
                    $colour = false;                
                    foreach ($objects as $object)
                    {
                        $style = $colour ? 'datap' : 'datai';

                        if ($object->contasrec_id !== $breakValue)
                        {
                            if (!$firstRow)
                            {
                                $tr->addRow();

                            }
                            $tr->addRow();
                            $tr->addCell($object->render('Id Título: {contasrec->id}  / Cliente: {contasrec->cliente->nome} / Vencimento: {contasrec->data_vencimento_formatada} /Valor: {contasrec->valor_formatado}'), 'left', 'break', 2);
                            $breakTotal = [];
                        }
                        $breakValue = $object->contasrec_id;

                        $firstRow = false;

                        $object->data_cobranca = call_user_func(function($value, $object, $row) 
                        {
                            if(!empty(trim($value)))
                            {
                                try
                                {
                                    $date = new DateTime($value);
                                    return $date->format('d/m/Y');
                                }
                                catch (Exception $e)
                                {
                                    return $value;
                                }
                            }
                        }, $object->data_cobranca, $object, null);

                        $tr->addRow();

                        $tr->addCell($object->data_cobranca, 'left', $style);
                        $tr->addCell($object->historico, 'left', $style);

                        $colour = !$colour;

                    }

                    $tr->addRow();


                    $file = 'report_'.uniqid().".{$reportExtension}";
                    // stores the file
                    if (!file_exists("app/output/{$file}") || is_writable("app/output/{$file}"))
                    {
                        $tr->save("app/output/{$file}");
                    }
                    else
                    {
                        throw new Exception(_t('Permission denied') . ': ' . "app/output/{$file}");
                    }

                    parent::openFile("app/output/{$file}");

                    // shows the success message
                    new TMessage('info', _t('Report generated. Please, enable popups'));
                }
            }
            else
            {
                new TMessage('error', _t('No records found'));
            }

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }


}

