<?php

class ContasrecList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Contasrec';
    private static $primaryKey = 'id';
    private static $formName = 'form_ContasrecList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    private $filtrarVencidos = false;
    private $filtrarVencendoHoje = false;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de contas a receber");
        $this->limit = 50;

        $id = new TEntry('id');
        $numero_documento = new TEntry('numero_documento');
        $contasrec_origem_id = new TDBCombo('contasrec_origem_id', 'kraftsoft', 'ContasrecOrigem', 'id', '{descricao}','descricao asc'  );
        $forma_pagamento_id = new TDBCombo('forma_pagamento_id', 'kraftsoft', 'FormaPagamento', 'id', '{descricao}','descricao asc'  );
        $contasrec_situacao_id = new TDBSelect('contasrec_situacao_id', 'kraftsoft', 'ContasrecSituacao', 'id', '{descricao}','id asc'  );
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{nome}','nome asc'  );
        $data_emissao_inicial = new TDate('data_emissao_inicial');
        $data_emissao_final = new TDate('data_emissao_final');
        $data_vencimento = new TDate('data_vencimento');
        $data_vencimento_final = new TDate('data_vencimento_final');
        $data_pagamento = new TDate('data_pagamento');
        $data_pagamento_final = new TDate('data_pagamento_final');
        $data_competencia_inicial = new TDate('data_competencia_inicial');
        $data_competencia_final = new TDate('data_competencia_final');


        $cliente_id->enableSearch();
        $forma_pagamento_id->enableSearch();
        $contasrec_origem_id->enableSearch();
        $contasrec_situacao_id->enableSearch();

        $data_pagamento->setMask('dd/mm/yyyy');
        $data_vencimento->setMask('dd/mm/yyyy');
        $data_emissao_final->setMask('dd/mm/yyyy');
        $data_emissao_inicial->setMask('dd/mm/yyyy');
        $data_pagamento_final->setMask('dd/mm/yyyy');
        $data_vencimento_final->setMask('dd/mm/yyyy');
        $data_competencia_final->setMask('dd/mm/yyyy');
        $data_competencia_inicial->setMask('dd/mm/yyyy');

        $data_pagamento->setDatabaseMask('yyyy-mm-dd');
        $data_vencimento->setDatabaseMask('yyyy-mm-dd');
        $data_emissao_final->setDatabaseMask('yyyy-mm-dd');
        $data_emissao_inicial->setDatabaseMask('yyyy-mm-dd');
        $data_pagamento_final->setDatabaseMask('yyyy-mm-dd');
        $data_vencimento_final->setDatabaseMask('yyyy-mm-dd');
        $data_competencia_final->setDatabaseMask('yyyy-mm-dd');
        $data_competencia_inicial->setDatabaseMask('yyyy-mm-dd');

        $id->setSize('100%');
        $cliente_id->setSize('100%');
        $data_pagamento->setSize('100%');
        $data_vencimento->setSize('100%');
        $numero_documento->setSize('100%');
        $forma_pagamento_id->setSize('100%');
        $data_emissao_final->setSize('100%');
        $contasrec_origem_id->setSize('100%');
        $data_emissao_inicial->setSize('100%');
        $data_pagamento_final->setSize('100%');
        $data_vencimento_final->setSize('100%');
        $data_competencia_final->setSize('100%');
        $data_competencia_inicial->setSize('100%');
        $contasrec_situacao_id->setSize('100%', 70);

        $row1 = $this->form->addFields([new TLabel("Nº Título (id):", null, '14px', null, '100%'),$id],[new TLabel("Número Documento:", null, '14px', null, '100%'),$numero_documento]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Origem:", null, '14px', null),$contasrec_origem_id],[new TLabel("Forma de Pagamento:", null, '14px', null),$forma_pagamento_id]);
        $row2->layout = [' col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Situação:", null, '14px', null),$contasrec_situacao_id],[new TLabel("Cliente:", null, '14px', null),$cliente_id]);
        $row3->layout = [' col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Data Emissão Inicial:", null, '14px', null, '100%'),$data_emissao_inicial],[new TLabel("Data Emissão Final:", null, '14px', null, '100%'),$data_emissao_final],[new TLabel("Data Vencimento Inicial:", null, '14px', null, '100%'),$data_vencimento],[new TLabel("Data Vencimento Final:", null, '14px', null, '100%'),$data_vencimento_final]);
        $row4->layout = [' col-sm-3',' col-sm-3',' col-sm-3',' col-sm-3'];

        $row5 = $this->form->addFields([new TLabel("Data Pagamento Inicial:", null, '14px', null, '100%'),$data_pagamento],[new TLabel("Data Pagamento Final:", null, '14px', null, '100%'),$data_pagamento_final],[new TLabel("Data Competência Inicial:", null, '14px', null, '100%'),$data_competencia_inicial],[new TLabel("Data Competência Final:", null, '14px', null, '100%'),$data_competencia_final]);
        $row5->layout = [' col-sm-3',' col-sm-3',' col-sm-3',' col-sm-3'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));
        $filterVar = Ativo::Ativo;
        $this->filter_criteria->add(new TFilter('ativo_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_data_emissao_transformed = new TDataGridColumn('data_emissao', "Data Emissão", 'left');
        $column_cliente_nome = new TDataGridColumn('cliente->nome', "Cliente", 'left');
        $column_forma_pagamento_descricao = new TDataGridColumn('forma_pagamento->descricao', "Forma pagamento", 'left');
        $column_numero_documento = new TDataGridColumn('numero_documento', "Nº Documento", 'left');
        $column_parcela = new TDataGridColumn('parcela', "Parcela", 'left');
        $column_data_vencimento_transformed = new TDataGridColumn('data_vencimento', "Vencimento", 'left');
        $column_valor_transformed = new TDataGridColumn('valor', "Valor", 'left');
        $column_valor_pago_transformed = new TDataGridColumn('valor_pago', "Valor Pago", 'left');
        $column_contasrec_origem_descricao = new TDataGridColumn('contasrec_origem->descricao', "Origem", 'left');
        $column_contasrec_situacao_descricao_transformed = new TDataGridColumn('contasrec_situacao->descricao', "Situação", 'left');

        $column_id->setTotalFunction( function($values) { 
            return count((array) $values); 
        }); 

        $column_valor_transformed->setTotalFunction( function($values) { 
            return array_sum((array) $values); 
        }); 

        $column_valor_pago_transformed->setTotalFunction( function($values) { 
            return array_sum((array) $values); 
        }); 

        $column_data_emissao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_data_vencimento_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
                $data_atual = strtotime(date('Y-m-d'));
                $data_vencimento = strtotime($object->data_vencimento);

                // date('d/m/Y', strtotime($this->data_emissao)); 

                if ($data_vencimento < $data_atual
                and $object->contasrec_situacao_id != ContasrecSituacao::Pago
                and $object->contasrec_situacao_id != ContasrecSituacao::Cancelado)
                {
                    return '<font color="red"><b>'.TDate::date2br($object->data_vencimento).'</b></font>';
                }
                else
                {
                    return TDate::date2br($object->data_vencimento);
                }
        });

        $column_valor_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_valor_pago_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_contasrec_situacao_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            // $label = $object->contasrec_situacao->descricao;
            // $cor = $object->contasrec_situacao->cor;

            // $div = new TElement('div');
            // $div->style="text-shadow:none; font-size:14px; font-weight:bold; color:#FFFFFF; width:160px; text-align:center;background-color: $cor;padding:10px";
            // $div->add($label);

            // return $div;    //code here

              return "<span class='label' style='background-color:{$object->contasrec_situacao->cor}'>{$object->contasrec_situacao->descricao}</span>";

        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);
        $order_data_emissao_transformed = new TAction(array($this, 'onReload'));
        $order_data_emissao_transformed->setParameter('order', 'data_emissao');
        $column_data_emissao_transformed->setAction($order_data_emissao_transformed);
        $order_numero_documento = new TAction(array($this, 'onReload'));
        $order_numero_documento->setParameter('order', 'numero_documento');
        $column_numero_documento->setAction($order_numero_documento);
        $order_data_vencimento_transformed = new TAction(array($this, 'onReload'));
        $order_data_vencimento_transformed->setParameter('order', 'data_vencimento');
        $column_data_vencimento_transformed->setAction($order_data_vencimento_transformed);
        $order_valor_transformed = new TAction(array($this, 'onReload'));
        $order_valor_transformed->setParameter('order', 'valor');
        $column_valor_transformed->setAction($order_valor_transformed);
        $order_valor_pago_transformed = new TAction(array($this, 'onReload'));
        $order_valor_pago_transformed->setParameter('order', 'valor_pago');
        $column_valor_pago_transformed->setAction($order_valor_pago_transformed);

        $this->builder_datagrid_check_all = new TCheckButton('builder_datagrid_check_all');
        $this->builder_datagrid_check_all->setIndexValue('on');
        $this->builder_datagrid_check_all->onclick = "Builder.checkAll(this)";
        $this->builder_datagrid_check_all->style = 'cursor:pointer';
        $this->builder_datagrid_check_all->setProperty('class', 'filled-in');
        $this->builder_datagrid_check_all->id = 'builder_datagrid_check_all';

        $label = new TLabel('');
        $label->style = 'margin:0';
        $label->class = 'checklist-label';
        $this->builder_datagrid_check_all->after($label);
        $label->for = 'builder_datagrid_check_all';

        $this->builder_datagrid_check = $this->datagrid->addColumn( new TDataGridColumn('builder_datagrid_check', $this->builder_datagrid_check_all, 'center',  '1%') );

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_data_emissao_transformed);
        $this->datagrid->addColumn($column_cliente_nome);
        $this->datagrid->addColumn($column_forma_pagamento_descricao);
        $this->datagrid->addColumn($column_numero_documento);
        $this->datagrid->addColumn($column_parcela);
        $this->datagrid->addColumn($column_data_vencimento_transformed);
        $this->datagrid->addColumn($column_valor_transformed);
        $this->datagrid->addColumn($column_valor_pago_transformed);
        $this->datagrid->addColumn($column_contasrec_origem_descricao);
        $this->datagrid->addColumn($column_contasrec_situacao_descricao_transformed);

        $action_group = new TDataGridActionGroup("", 'fas:cog');
        $action_group->addHeader('');

        $action_onShow = new TDataGridAction(array('ContasrecFormView', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Abrir");
        $action_onShow->setImage('fas:book-open #03A9F4');
        $action_onShow->setField(self::$primaryKey);

        $action_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_onShow);

        $action_onEdit = new TDataGridAction(array('ContasrecForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('ContasrecList::onExibeEditar');

        $action_group->addAction($action_onEdit);

        $action_onDelete = new TDataGridAction(array('ContasrecList', 'onDelete'));
        $action_onDelete->setUseButton(TRUE);
        $action_onDelete->setButtonClass('btn btn-default');
        $action_onDelete->setLabel("Excluir");
        $action_onDelete->setImage('fas:trash-alt #dd5a43');
        $action_onDelete->setField(self::$primaryKey);
        $action_onDelete->setDisplayCondition('ContasrecList::onExibeExcluir');

        $action_group->addAction($action_onDelete);

        $action_ContasrecBaixasForm_onShow = new TDataGridAction(array('ContasrecBaixasForm', 'onShow'));
        $action_ContasrecBaixasForm_onShow->setUseButton(TRUE);
        $action_ContasrecBaixasForm_onShow->setButtonClass('btn btn-default');
        $action_ContasrecBaixasForm_onShow->setLabel("Baixar Título");
        $action_ContasrecBaixasForm_onShow->setImage('fas:money-check #009688');
        $action_ContasrecBaixasForm_onShow->setField(self::$primaryKey);
        $action_ContasrecBaixasForm_onShow->setDisplayCondition('ContasrecList::onExibeBaixarTitulo');
        $action_ContasrecBaixasForm_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_ContasrecBaixasForm_onShow);

        $action_onCancelarTitulo = new TDataGridAction(array('ContasrecList', 'onCancelarTitulo'));
        $action_onCancelarTitulo->setUseButton(TRUE);
        $action_onCancelarTitulo->setButtonClass('btn btn-default');
        $action_onCancelarTitulo->setLabel("Cancelar Título");
        $action_onCancelarTitulo->setImage('fas:trash-alt #E40606');
        $action_onCancelarTitulo->setField(self::$primaryKey);
        $action_onCancelarTitulo->setDisplayCondition('ContasrecList::onExibeCancelarTitulo');

        $action_group->addAction($action_onCancelarTitulo);

        $action_onImprimirBoleto = new TDataGridAction(array('ContasrecList', 'onImprimirBoleto'));
        $action_onImprimirBoleto->setUseButton(TRUE);
        $action_onImprimirBoleto->setButtonClass('btn btn-default');
        $action_onImprimirBoleto->setLabel("Imprimir Boleto");
        $action_onImprimirBoleto->setImage('fas:file-pdf #673AB7');
        $action_onImprimirBoleto->setField(self::$primaryKey);
        $action_onImprimirBoleto->setDisplayCondition('ContasrecList::onExibeImprimirBoleto');

        $action_group->addAction($action_onImprimirBoleto);

        $action_ContasrecBaixasSimpleList_onShow = new TDataGridAction(array('ContasrecBaixasSimpleList', 'onShow'));
        $action_ContasrecBaixasSimpleList_onShow->setUseButton(TRUE);
        $action_ContasrecBaixasSimpleList_onShow->setButtonClass('btn btn-default');
        $action_ContasrecBaixasSimpleList_onShow->setLabel("Visualizar Baixas");
        $action_ContasrecBaixasSimpleList_onShow->setImage('fas:search-plus #FF5722');
        $action_ContasrecBaixasSimpleList_onShow->setField(self::$primaryKey);

        $action_ContasrecBaixasSimpleList_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_ContasrecBaixasSimpleList_onShow);

        $action_onVerOrigem = new TDataGridAction(array('ContasrecList', 'onVerOrigem'));
        $action_onVerOrigem->setUseButton(TRUE);
        $action_onVerOrigem->setButtonClass('btn btn-default');
        $action_onVerOrigem->setLabel("Ver Origem");
        $action_onVerOrigem->setImage('fas:atom #4CAF50');
        $action_onVerOrigem->setField(self::$primaryKey);

        $action_group->addAction($action_onVerOrigem);

        $action_onHistoricoCobrancas = new TDataGridAction(array('ContasrecList', 'onHistoricoCobrancas'));
        $action_onHistoricoCobrancas->setUseButton(TRUE);
        $action_onHistoricoCobrancas->setButtonClass('btn btn-default');
        $action_onHistoricoCobrancas->setLabel("Histórico de Cobranças");
        $action_onHistoricoCobrancas->setImage('far:calendar-check #FF9800');
        $action_onHistoricoCobrancas->setField(self::$primaryKey);

        $action_onHistoricoCobrancas->setParameter('key', '{id}');

        $action_group->addAction($action_onHistoricoCobrancas);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup("Listagem de contas a receber");
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $button_cadastrar = new TButton('button_button_cadastrar');
        $button_cadastrar->setAction(new TAction(['ContasrecForm', 'onShow']), "Cadastrar");
        $button_cadastrar->addStyleClass('btn-default');
        $button_cadastrar->setImage('fas:plus #69aa46');

        $this->datagrid_form->addField($button_cadastrar);

        $button_cadastrar_em_bloco = new TButton('button_button_cadastrar_em_bloco');
        $button_cadastrar_em_bloco->setAction(new TAction(['ContasrecBlocoForm', 'onShow']), "Cadastrar em Bloco");
        $button_cadastrar_em_bloco->addStyleClass('btn-default');
        $button_cadastrar_em_bloco->setImage('fas:align-justify #FF5722');

        $this->datagrid_form->addField($button_cadastrar_em_bloco);

        $button_baixar_em_bloco = new TButton('button_button_baixar_em_bloco');
        $button_baixar_em_bloco->setAction(new TAction(['ContasrecList', 'onBaixarBloco']), "Baixar em Bloco");
        $button_baixar_em_bloco->addStyleClass('btn-default');
        $button_baixar_em_bloco->setImage('fas:caret-square-down #E91E63');

        $this->datagrid_form->addField($button_baixar_em_bloco);

        $btnShowCurtainFilters = new TButton('button_btnShowCurtainFilters');
        $btnShowCurtainFilters->setAction(new TAction(['ContasrecList', 'onShowCurtainFilters']), "Filtros");
        $btnShowCurtainFilters->addStyleClass('btn-default');
        $btnShowCurtainFilters->setImage('fas:filter #000000');

        $this->datagrid_form->addField($btnShowCurtainFilters);

        $button_limpar_filtros = new TButton('button_button_limpar_filtros');
        $button_limpar_filtros->setAction(new TAction(['ContasrecList', 'onClearFilters']), "Limpar filtros");
        $button_limpar_filtros->addStyleClass('btn-default');
        $button_limpar_filtros->setImage('fas:eraser #f44336');

        $this->datagrid_form->addField($button_limpar_filtros);

        $button_atualizar = new TButton('button_button_atualizar');
        $button_atualizar->setAction(new TAction(['ContasrecList', 'onRefresh']), "Atualizar");
        $button_atualizar->addStyleClass('btn-default');
        $button_atualizar->setImage('fas:sync-alt #03a9f4');

        $this->datagrid_form->addField($button_atualizar);

        $button_vencidos = new TButton('button_button_vencidos');
        $button_vencidos->setAction(new TAction(['ContasrecList', 'onFiltrarVencidos']), "Vencidos");
        $button_vencidos->addStyleClass('btn-default');
        $button_vencidos->setImage('fas:money-bill #F44336');

        $this->datagrid_form->addField($button_vencidos);

        $button_vencendo_hoje = new TButton('button_button_vencendo_hoje');
        $button_vencendo_hoje->setAction(new TAction(['ContasrecList', 'onVencendoHoje']), "Vencendo Hoje");
        $button_vencendo_hoje->addStyleClass('btn-default');
        $button_vencendo_hoje->setImage('fas:money-bill #FFC107');

        $this->datagrid_form->addField($button_vencendo_hoje);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['ContasrecList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['ContasrecList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['ContasrecList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['ContasrecList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_left_actions->add($button_cadastrar);
        $head_left_actions->add($button_cadastrar_em_bloco);
        $head_left_actions->add($button_baixar_em_bloco);
        $head_left_actions->add($btnShowCurtainFilters);
        $head_left_actions->add($button_limpar_filtros);
        $head_left_actions->add($button_atualizar);
        $head_left_actions->add($button_vencidos);
        $head_left_actions->add($button_vencendo_hoje);

        $head_right_actions->add($dropdown_button_exportar);

        $this->btnShowCurtainFilters = $btnShowCurtainFilters;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Financeiro","Contas a receber"]));
        }

        $container->add($panel);

        parent::add($container);

    }

    public static function onExibeEditar($object)
    {
        try 
        {
            if($object->contasrec_situacao_id==ContasrecSituacao::EmAberto
            or $object->contasrec_situacao_id==ContasrecSituacao::PagoParcialmente)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onDelete($param = null) 
    { 
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new Contasrec($key, FALSE); 

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }
    public static function onExibeExcluir($object)
    {
        try 
        {
            // if($object)
            // {
            //     return true;
            // }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeBaixarTitulo($object)
    {
        try 
        {
            if($object->contasrec_situacao_id==ContasrecSituacao::EmAberto
            or $object->contasrec_situacao_id==ContasrecSituacao::PagoParcialmente)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onCancelarTitulo($param = null) 
    {
        try 
        {
          new TQuestion("Deseja cancelar o título?", new TAction([__CLASS__, 'onCancelarTituloYes'], $param), new TAction([__CLASS__, 'onCancelarTituloNo'], $param));
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeCancelarTitulo($object)
    {
        try 
        {
            if($object->contasrec_situacao_id==ContasrecSituacao::EmAberto
            and $object->contasrec_origem_id==ContasrecOrigem::Avulso)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onImprimirBoleto($param = null) 
    {
        try 
        {
            //code here

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeImprimirBoleto($object)
    {
        try 
        {
            $tipodocumento = (isset($object->contasrec_forma_pagamento->tipo_documento_id)) ? $object->contasrec_forma_pagamento->tipo_documento_id  : 0;

            if( $tipodocumento  == TipoDocumento::Boleto)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onVerOrigem($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);   
            $contasrec = Contasrec::find($param['id']);

            if ($contasrec->contasrec_origem_id==ContasrecOrigem::Venda)
            {
                 $pageParam = ['key' => $contasrec->venda_formapgto->venda_id]; // ex.: = ['key' => 10]

                 TApplication::loadPage('VendaFormView', 'OnShow', $pageParam);                
            }
            else
            {
              new TMessage('info', 'Este título é de origem avulsa!');  
            }

            TTransaction::close();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onHistoricoCobrancas($param = null) 
    {
        try 
        {
             $pageParam = ['key' => $param['id']]; // ex.: = ['key' => 10]
             TSession::setValue('key_formhistoricocobranca', $param['id']);   

             TApplication::loadPage('ContasrecHistoricoCobrancaFormList', 'OnShow', $pageParam);      

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('object');
                $object->data  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }
    public function onBaixarBloco($param = null) 
    {
        try 
        {
                $session_checks = TSession::getValue(__CLASS__.'builder_datagrid_check');
                if($session_checks)
                {
                     $pageParam = []; // ex.: = ['key' => 10]

                     TApplication::loadPage('ContasRecBaixaBlocoForm', 'OnShow', $pageParam);      
                }            
                else
                {
                    new TMessage('info', 'Nenhum título selecionado!');    
                }

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onShowCurtainFilters($param = null) 
    {
        try 
        {
            //code here

                        $filter = new self([]);

            $btnClose = new TButton('closeCurtain');
            $btnClose->class = 'btn btn-sm btn-default';
            $btnClose->style = 'margin-right:10px;';
            $btnClose->onClick = "Template.closeRightPanel();";
            $btnClose->setLabel("Fechar");
            $btnClose->setImage('fas:times');

            $filter->form->addHeaderWidget($btnClose);

            $page = new TPage();
            $page->setTargetContainer('adianti_right_panel');
            $page->setProperty('page-name', 'ContasrecListSearch');
            $page->setProperty('page_name', 'ContasrecListSearch');
            $page->adianti_target_container = 'adianti_right_panel';
            $page->target_container = 'adianti_right_panel';
            $page->add($filter->form);
            $page->setIsWrapped(true);
            $page->show();

            $style = new TStyle('right-panel > .container-part[page-name=ContasrecListSearch]');
            $style->width = '70% !important';
            $style->show(true);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onClearFilters($param = null) 
    {
        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }
    public function onRefresh($param = null) 
    {
        $this->onReload([]);
    }
    public function onFiltrarVencidos($param = null) 
    {
        try 
        {
            $this->filtrarVencidos = true;
            $this->onSearch([]);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onVencendoHoje($param = null) 
    {
        try 
        {
            $this->filtrarVencendoHoje = true;
            $this->onSearch([]);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->numero_documento) AND ( (is_scalar($data->numero_documento) AND $data->numero_documento !== '') OR (is_array($data->numero_documento) AND (!empty($data->numero_documento)) )) )
        {

            $filters[] = new TFilter('numero_documento', 'like', "%{$data->numero_documento}%");// create the filter 
        }

        if (isset($data->contasrec_origem_id) AND ( (is_scalar($data->contasrec_origem_id) AND $data->contasrec_origem_id !== '') OR (is_array($data->contasrec_origem_id) AND (!empty($data->contasrec_origem_id)) )) )
        {

            $filters[] = new TFilter('contasrec_origem_id', '=', $data->contasrec_origem_id);// create the filter 
        }

        if (isset($data->forma_pagamento_id) AND ( (is_scalar($data->forma_pagamento_id) AND $data->forma_pagamento_id !== '') OR (is_array($data->forma_pagamento_id) AND (!empty($data->forma_pagamento_id)) )) )
        {

            $filters[] = new TFilter('forma_pagamento_id', '=', $data->forma_pagamento_id);// create the filter 
        }

        if (isset($data->contasrec_situacao_id) AND ( (is_scalar($data->contasrec_situacao_id) AND $data->contasrec_situacao_id !== '') OR (is_array($data->contasrec_situacao_id) AND (!empty($data->contasrec_situacao_id)) )) )
        {

            $filters[] = new TFilter('contasrec_situacao_id', 'in', $data->contasrec_situacao_id);// create the filter 
        }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }

        if (isset($data->data_emissao_inicial) AND ( (is_scalar($data->data_emissao_inicial) AND $data->data_emissao_inicial !== '') OR (is_array($data->data_emissao_inicial) AND (!empty($data->data_emissao_inicial)) )) )
        {

            $filters[] = new TFilter('data_emissao', '>=', $data->data_emissao_inicial);// create the filter 
        }

        if (isset($data->data_emissao_final) AND ( (is_scalar($data->data_emissao_final) AND $data->data_emissao_final !== '') OR (is_array($data->data_emissao_final) AND (!empty($data->data_emissao_final)) )) )
        {

            $filters[] = new TFilter('data_emissao', '<=', $data->data_emissao_final);// create the filter 
        }

        if (isset($data->data_vencimento) AND ( (is_scalar($data->data_vencimento) AND $data->data_vencimento !== '') OR (is_array($data->data_vencimento) AND (!empty($data->data_vencimento)) )) )
        {

            $filters[] = new TFilter('data_vencimento', '>=', $data->data_vencimento);// create the filter 
        }

        if (isset($data->data_vencimento_final) AND ( (is_scalar($data->data_vencimento_final) AND $data->data_vencimento_final !== '') OR (is_array($data->data_vencimento_final) AND (!empty($data->data_vencimento_final)) )) )
        {

            $filters[] = new TFilter('data_vencimento', '<=', $data->data_vencimento_final);// create the filter 
        }

        if (isset($data->data_pagamento) AND ( (is_scalar($data->data_pagamento) AND $data->data_pagamento !== '') OR (is_array($data->data_pagamento) AND (!empty($data->data_pagamento)) )) )
        {

            $filters[] = new TFilter('data_pagamento', '>=', $data->data_pagamento);// create the filter 
        }

        if (isset($data->data_pagamento_final) AND ( (is_scalar($data->data_pagamento_final) AND $data->data_pagamento_final !== '') OR (is_array($data->data_pagamento_final) AND (!empty($data->data_pagamento_final)) )) )
        {

            $filters[] = new TFilter('data_pagamento', '<=', $data->data_pagamento_final);// create the filter 
        }

        if (isset($data->data_competencia_inicial) AND ( (is_scalar($data->data_competencia_inicial) AND $data->data_competencia_inicial !== '') OR (is_array($data->data_competencia_inicial) AND (!empty($data->data_competencia_inicial)) )) )
        {

            $filters[] = new TFilter('data_competencia', '>=', $data->data_competencia_inicial);// create the filter 
        }

        if (isset($data->data_competencia_final) AND ( (is_scalar($data->data_competencia_final) AND $data->data_competencia_final !== '') OR (is_array($data->data_competencia_final) AND (!empty($data->data_competencia_final)) )) )
        {

            $filters[] = new TFilter('data_competencia', '<=', $data->data_competencia_final);// create the filter 
        }

        if ($this->filtrarVencidos)
        {
            $data->contaspag_situacao_id = '';
            $data->data_vencimento_inicial = '';
            $data->data_vencimento_final = '';

            $filters[] = new TFilter('data_vencimento', '<', date('Y-m-d'));
            $filters[] = new TFilter('contasrec_situacao_id', 'in', [ContasrecSituacao::EmAberto,ContasrecSituacao::PagoParcialmente]);
        }
        elseif ($this->filtrarVencendoHoje)
        {
            $data->contaspag_situacao_id = '';
            $data->data_vencimento_inicial = '';
            $data->data_vencimento_final = '';

            $filters[] = new TFilter('data_vencimento', '=', date('Y-m-d'));
            $filters[] = new TFilter('contasrec_situacao_id', 'in', [ContasrecSituacao::EmAberto,ContasrecSituacao::PagoParcialmente]);
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Contasrec
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            $session_checks = TSession::getValue(__CLASS__.'builder_datagrid_check');

            //</blockLine><btnShowCurtainFiltersAutoCode>
            if(!empty($this->btnShowCurtainFilters) && empty($this->btnShowCurtainFiltersAdjusted))
            {
                $this->btnShowCurtainFiltersAdjusted = true;
                $this->btnShowCurtainFilters->style = 'position: relative';
                $countFilters = count($filters ?? []);
                $this->btnShowCurtainFilters->setLabel($this->btnShowCurtainFilters->getLabel(). "<span class='badge badge-success' style='position: absolute'>{$countFilters}<span>");
            }
            //</blockLine></btnShowCurtainFiltersAutoCode>

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {
                    $check = new TCheckGroup('builder_datagrid_check');
                    $check->addItems([$object->id => '']);
                    $check->getButtons()[$object->id]->onclick = 'event.stopPropagation()';

                    if(!$this->datagrid_form->getField('builder_datagrid_check[]'))
                    {
                        $this->datagrid_form->setFields([$check]);
                    }

                    $check->setChangeAction(new TAction([$this, 'builderSelectCheck']));
                    $object->builder_datagrid_check = $check;

                    if(!empty($session_checks[$object->id]))
                    {
                        $object->builder_datagrid_check->setValue([$object->id=>$object->id]);
                    }

                    if ($object->contasrec_situacao_id == ContasrecSituacao::Pago or $object->contasrec_situacao_id == ContasrecSituacao::Cancelado)
                    {
                        unset($object->builder_datagrid_check);
                    }

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function onCancelarTituloYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $contasrec = Contasrec::find($param['id']);     

            if($contasrec) 
            {
                $contasrec->contasrec_situacao_id=ContasrecSituacao::Cancelado;
                $contasrec->store();

                $lanc_contabil = ContabilLancamentos::where('contasrec_id','=',$contasrec->id)->first(); 

                if ($lanc_contabil)
                {
                    $lanc_contabil->delete();                
                }
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');  
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onCancelarTituloNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function builderSelectCheck($param)
    {
        $session_checks = TSession::getValue(__CLASS__.'builder_datagrid_check');

        $valueOn = null;
        if(!empty($param['_field_data_json']))
        {
            $obj = json_decode($param['_field_data_json']);
            if($obj)
            {
                $valueOn = $obj->valueOn;
            }
        }

        $key = empty($param['key']) ? $valueOn : $param['key'];

        if(empty($param['builder_datagrid_check']) && !empty($session_checks[$key]))
        {
            unset($session_checks[$key]);
        }
        elseif(!empty($param['builder_datagrid_check']) && !in_array($key, $param['builder_datagrid_check']) && !empty($session_checks[$key]))
        {
            unset($session_checks[$key]);
        }
        elseif(!empty($param['builder_datagrid_check']) && in_array($key, $param['builder_datagrid_check']))
        {
            $session_checks[$key] = $key;
        }

        TSession::setValue(__CLASS__.'builder_datagrid_check', $session_checks);
    }

}

