<?php

class FormaPagamentoForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'FormaPagamento';
    private static $primaryKey = 'id';
    private static $formName = 'form_FormaPagamentoForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Cadastro de forma de pagamento");


        $id = new TEntry('id');
        $codigo_interno = new TEntry('codigo_interno');
        $tipo_documento_id = new TDBCombo('tipo_documento_id', 'kraftsoft', 'TipoDocumento', 'id', '{descricao}','descricao asc'  );
        $meio_pagamento_tef_id = new TDBCombo('meio_pagamento_tef_id', 'kraftsoft', 'MeioPagamentoTef', 'id', '{descricao}','descricao asc'  );
        $descricao = new TEntry('descricao');
        $notafiscal_tpag_id = new TDBCombo('notafiscal_tpag_id', 'kraftsoft', 'NotafiscalTpag', 'id', '{tpag} -{descricao}','tpag asc'  );
        $taxa = new TNumeric('taxa', '2', ',', '.' );
        $prazo_pagamento = new TEntry('prazo_pagamento');
        $utiliza_pdv = new TRadioGroup('utiliza_pdv');
        $obriga_nf_pdv = new TRadioGroup('obriga_nf_pdv');
        $criado_em = new TDateTime('criado_em');
        $criado_por = new TEntry('criado_por');
        $alterado_em = new TDateTime('alterado_em');
        $alterado_por = new TEntry('alterado_por');

        $codigo_interno->addValidation("Código Interno", new TRequiredValidator()); 
        $tipo_documento_id->addValidation("Tipo Documento", new TRequiredValidator()); 
        $descricao->addValidation("Descricao", new TRequiredValidator()); 

        $notafiscal_tpag_id->enableSearch();
        $utiliza_pdv->addItems(["S"=>"Sim","N"=>"Não"]);
        $obriga_nf_pdv->addItems(["S"=>"Sim","N"=>"Não"]);

        $utiliza_pdv->setLayout('horizontal');
        $obriga_nf_pdv->setLayout('horizontal');

        $utiliza_pdv->setUseButton();
        $obriga_nf_pdv->setUseButton();

        $criado_em->setMask('dd/mm/yyyy hh:ii');
        $alterado_em->setMask('dd/mm/yyyy hh:ii');

        $criado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $alterado_em->setDatabaseMask('yyyy-mm-dd hh:ii');

        $id->setEditable(false);
        $criado_em->setEditable(false);
        $criado_por->setEditable(false);
        $alterado_em->setEditable(false);
        $alterado_por->setEditable(false);

        $id->setSize(100);
        $taxa->setSize('100%');
        $utiliza_pdv->setSize(80);
        $descricao->setSize('100%');
        $obriga_nf_pdv->setSize(80);
        $criado_em->setSize('100%');
        $criado_por->setSize('100%');
        $alterado_em->setSize('100%');
        $alterado_por->setSize('100%');
        $codigo_interno->setSize('100%');
        $prazo_pagamento->setSize('100%');
        $tipo_documento_id->setSize('100%');
        $notafiscal_tpag_id->setSize('100%');
        $meio_pagamento_tef_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Código Interno:", '#FF0000', '14px', null),$codigo_interno]);
        $row1->layout = [' col-sm-6',' col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Tipo Documento:", '#FF0000', '14px', null),$tipo_documento_id],[new TLabel("Meio de pagamento integrado - TEF:", null, '14px', null),$meio_pagamento_tef_id]);
        $row2->layout = [' col-sm-6',' col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Descrição:", '#FF0000', '14px', null),$descricao],[new TLabel("Forma Pagamento NFe/NFc-e (Tpag):", null, '14px', null),$notafiscal_tpag_id]);
        $row3->layout = [' col-sm-6',' col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Taxa %:", null, '14px', null),$taxa],[new TLabel("Prazo Pagamento (Dias):", null, '14px', null),$prazo_pagamento]);
        $row4->layout = [' col-sm-6',' col-sm-6'];

        $row5 = $this->form->addFields([new TLabel("Utilizar no PDV:", null, '14px', null, '100%'),$utiliza_pdv],[new TLabel("Obrigar emissão de NF no PDV:", null, '14px', null, '100%'),$obriga_nf_pdv]);
        $row5->layout = [' col-sm-6',' col-sm-6'];

        $row6 = $this->form->addContent([new TFormSeparator("Histórico de Alterações", '#333', '18', '#eee')]);
        $row7 = $this->form->addFields([new TLabel("Criado em:", null, '14px', null, '100%'),$criado_em],[new TLabel("Criado Por:", null, '14px', null),$criado_por],[new TLabel("Alterado em:", null, '14px', null),$alterado_em],[new TLabel("Alterado Por:", null, '14px', null),$alterado_por]);
        $row7->layout = [' col-sm-2',' col-sm-4',' col-sm-2',' col-sm-4'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onclear = $this->form->addAction("Limpar formulário", new TAction([$this, 'onClear']), 'fas:eraser #dd5a43');
        $this->btn_onclear = $btn_onclear;

        $btn_onshow = $this->form->addAction("Voltar", new TAction(['FormaPagamentoHeaderList', 'onShow']), 'fas:arrow-left #000000');
        $this->btn_onshow = $btn_onshow;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Financeiro","Cadastro de forma de pagamento"]));
        }
        $container->add($this->form);

        parent::add($container);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new FormaPagamento(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);

            $object->store(); // save the object 

            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('FormaPagamentoHeaderList', 'onShow', $loadPageParam); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new FormaPagamento($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

