<?php

class SangriaSuprimentoForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'CaixaLancamentos';
    private static $primaryKey = 'id';
    private static $formName = 'form_SangriaSuprimentoForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Sangria/Suprimento de Caixa");


        $data_hora_lancamento = new TDateTime('data_hora_lancamento');
        $caixa_tipo_lancamento_id = new TRadioGroup('caixa_tipo_lancamento_id');
        $valor = new TNumeric('valor', '2', ',', '.' );
        $observacao = new TText('observacao');
        $caixa_id = new THidden('caixa_id');
        $id = new THidden('id');

        $data_hora_lancamento->addValidation("Data Hora Lancamento", new TRequiredValidator()); 
        $caixa_tipo_lancamento_id->addValidation("Tipo Lançamento", new TRequiredValidator()); 
        $valor->addValidation("Valor", new TRequiredValidator()); 

        $data_hora_lancamento->setEditable(false);
        $data_hora_lancamento->setMask('dd/mm/yyyy hh:ii');
        $data_hora_lancamento->setDatabaseMask('yyyy-mm-dd hh:ii');
        $caixa_tipo_lancamento_id->addItems(["0"=>"Sangria","1"=>"Suprimento"]);
        $caixa_tipo_lancamento_id->setLayout('horizontal');
        $id->setSize(200);
        $valor->setSize('100%');
        $caixa_id->setSize(200);
        $observacao->setSize('100%', 70);
        $data_hora_lancamento->setSize('100%');
        $caixa_tipo_lancamento_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Data Hora Lancamento:", '#ff0000', '14px', null, '100%'),$data_hora_lancamento],[new TLabel("Tipo Lançamento:", '#ff0000', '14px', null, '100%'),$caixa_tipo_lancamento_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Valor:", '#ff0000', '14px', null, '100%'),$valor]);
        $row2->layout = ['col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Observação:", null, '14px', null, '100%'),$observacao],[$caixa_id,$id]);
        $row3->layout = ['col-sm-12','col-sm-6'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new CaixaLancamentos(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            $object->caixa_tipo_lancamento_id = $data->caixa_tipo_lancamento_id == 0 ? CaixaTipoLancamento::Debito : CaixaTipoLancamento::FundoCaixa ;
            $object->system_users_id = TSession::getValue('userid');
            $object->forma_pagamento_id = Funcoes::retornaFormaPagamentoPadraoDinheiroCaixa();
            $object->data_hora_lancamento = date('Y-m-d H:i:s');

            $object->store(); // save the object 

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            new TMessage('info', "Registro salvo", $messageAction); 

                        TScript::create("Template.closeRightPanel();"); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new CaixaLancamentos($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

        $object = new stdClass();
        $object->data_hora_lancamento = date('Y-m-d H:i:s');
        $object->caixa_id = $param['caixa_id'];
        $object->caixa_tipo_lancamento_id = 0;        

        TForm::sendData(self::$formName, $object);
    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

