<?php

class MDFeEncerrarForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = '';
    private static $activeRecord = '';
    private static $primaryKey = '';
    private static $formName = 'form_MDFeEncerrarForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Encerrar MDF-e");


        $id = new TEntry('id');
        $numero = new TEntry('numero');
        $data_hora_emissao = new TDateTime('data_hora_emissao');
        $data_encerramento = new TDate('data_encerramento');
        $municipio_encerramento = new TDBCombo('municipio_encerramento', 'kraftsoft', 'Cidade', 'id', '{nome}  - {estado->sigla}','nome asc'  );

        $data_encerramento->addValidation("Data Encerramento", new TRequiredValidator()); 
        $municipio_encerramento->addValidation("Município Encerramento", new TRequiredValidator()); 

        $data_encerramento->setValue(date('d/m/Y'));
        $municipio_encerramento->enableSearch();
        $data_encerramento->setMask('dd/mm/yyyy');
        $data_hora_emissao->setMask('dd/mm/yyyy hh:ii');

        $data_encerramento->setDatabaseMask('yyyy-mm-dd');
        $data_hora_emissao->setDatabaseMask('yyyy-mm-dd hh:ii');

        $id->setEditable(false);
        $numero->setEditable(false);
        $data_hora_emissao->setEditable(false);

        $id->setSize('100%');
        $numero->setSize('100%');
        $data_hora_emissao->setSize('100%');
        $data_encerramento->setSize('100%');
        $municipio_encerramento->setSize('100%');


        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Número:", null, '14px', null, '100%'),$numero],[new TLabel("Data e hora emissão:", null, '14px', null, '100%'),$data_hora_emissao]);
        $row1->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row2 = $this->form->addFields([new TLabel("Data Encerramento:", '#F44336', '14px', null, '100%'),$data_encerramento],[new TLabel("Município Encerramento:", '#F44336', '14px', null),$municipio_encerramento]);
        $row2->layout = [' col-sm-6','col-sm-6'];

        // create the form actions
        $btn_onencerrar = $this->form->addAction("Confirmar", new TAction([$this, 'onEncerrar']), 'fas:rocket #ffffff');
        $this->btn_onencerrar = $btn_onencerrar;
        $btn_onencerrar->addStyleClass('btn-primary'); 

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

    }

    public function onEncerrar($param = null) 
    {
        try
        {
           // new TQuestion("Confirma o encerramento deste MDF-e?", new TAction([__CLASS__, 'onEncerrarYes'], $param), new TAction([__CLASS__, 'onEncerrarNo'], $param));

            TTransaction::open('kraftsoft');

            $this->form->validate();

            $data = $this->form->getData();

            $mdfe = Mdfe::find($data->id);

            if($mdfe)
            {

                $mdfe->data_encerramento = $data->data_encerramento;
                $mdfe->cidade_encerramento_id = $data->municipio_encerramento;
                $mdfe->cmunencerramento = $mdfe->cidade_encerramento->codigo_ibge;

                $retorno = API_MDFe::Encerra($mdfe);

                if($retorno->codigo == 5001)
                {
                     $pageParam = ['titulo' => 'Retorno da SEFAZ'];

                     TSession::setValue('mens_personalizada', 'Retorno da SEFAZ: '."\n".API_MDFe::formataRetorno($retorno));
                     TApplication::loadPage('MensagemPersonalizada', 'OnShow', $pageParam);                    
                 }
                 else
                 {
                     TToast::show('success', "MDF-e encerrado com sucesso!", 'topRight', 'far:check-circle');

                     TApplication::loadPage('MdfeList', 'onReload');
                 }

            }

            TTransaction::close();                
        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    }

    public function onShow($param = null)
    {               

        $object = new stdClass();
        $object->id = $param['id'];

        TTransaction::open('kraftsoft');

        $mdfe = Mdfe::find($param['id']);

        TTransaction::close();   

        if ($mdfe)
        {
            $object->numero = $mdfe->nMDF;
            $object->data_hora_emissao = $mdfe->dhEmi;
        }

        TForm::sendData(self::$formName, $object);

    } 

    public  function onEncerrarYes($param = null) 
    {
        try 
        {
            TTransaction::open('kraftsoft');

             $this->form->validate();

            //(new TRequiredValidator)->validate('Data Encerramento', $param['data_encerramento']);

            $data = $this->form->getData();

            // if ($data->data_encerramento==null)
            // {
            //   new TMessage('error','Informe a data de encerramento!');  
            // }     

            // if ($data->municipio_encerramento==null)
            // {
            //   new TMessage('error','Informe o município de encerramento!');  
            // }                 

            $mdfe = Mdfe::find($data->id);

            if($mdfe)
            {
                $retorno = API_MDFe::Encerra($mdfe);

                if($retorno->codigo == 5001)
                {
                     $pageParam = ['titulo' => 'Retorno da SEFAZ'];

                     TSession::setValue('mens_personalizada', 'Retorno da SEFAZ: '."\n".API_MDFe::formataRetorno($retorno));
                     TApplication::loadPage('MensagemPersonalizada', 'OnShow', $pageParam);                    
                 }
                 else
                 {
                     TToast::show('success', "MDF-e encerrado com sucesso!", 'topRight', 'far:check-circle');

                     TApplication::loadPage('MdfeList', 'onReload');
                 }

            }

            TTransaction::close();        
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onEncerrarNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

}

