<?php

class MdfeEventoSimpleList extends TPage
{

    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'MdfeEvento';
    private static $primaryKey = 'id';
    private static $formName = 'formList_MdfeEvento';
    private $limit = 20;

    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        $this->limit = 0;

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);

        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(320);

        $column_data_evento_transformed = new TDataGridColumn('data_evento', "Data evento", 'left');
        $column_status = new TDataGridColumn('status', "Status", 'left');
        $column_codigo_status = new TDataGridColumn('codigo_status', "Código", 'left');
        $column_motivo_status = new TDataGridColumn('motivo_status', "Motivo", 'left');
        $column_id_evento = new TDataGridColumn('id_evento', "Evento", 'left');
        $column_numero_sequencial = new TDataGridColumn('numero_sequencial', "Número Sequencial", 'left');
        $column_chave = new TDataGridColumn('chave', "chave", 'left');
        $column_protocolo = new TDataGridColumn('protocolo', "Protocolo", 'left');
        $column_tipo_evento = new TDataGridColumn('tipo_evento', "Tipo evento", 'left');
        $column_data_recebimento_transformed = new TDataGridColumn('data_recebimento', "Data Recebimento", 'left');

        $column_data_evento_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_data_recebimento_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });        

        $this->datagrid->addColumn($column_data_evento_transformed);
        $this->datagrid->addColumn($column_status);
        $this->datagrid->addColumn($column_codigo_status);
        $this->datagrid->addColumn($column_motivo_status);
        $this->datagrid->addColumn($column_id_evento);
        $this->datagrid->addColumn($column_numero_sequencial);
        $this->datagrid->addColumn($column_chave);
        $this->datagrid->addColumn($column_protocolo);
        $this->datagrid->addColumn($column_tipo_evento);
        $this->datagrid->addColumn($column_data_recebimento_transformed);


        // create the datagrid model
        $this->datagrid->createModel();

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Fiscal","Eventos da MDFE"]));
        }
        $container->add($panel);

        parent::add($container);

    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for MdfeEvento
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }
            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            if(isset($param['mdfe_id'])){
                $criteria->add(new TFilter('mdfe_id' , '=', $param['mdfe_id']));    
            }else{
                $criteria->add(new TFilter('mdfe_id' , '=', NULL));
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  array('onReload', 'onSearch')))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

}

