<?php

class MdfeList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Mdfe';
    private static $primaryKey = 'id';
    private static $formName = 'form_MdfeList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("MDF-e");
        $this->limit = 20;

        $nMDF = new TEntry('nMDF');
        $mdfe_tpcar_id = new TDBCombo('mdfe_tpcar_id', 'kraftsoft', 'MdfeTpcar', 'id', '{descricao}','descricao asc'  );
        $UFIni = new TDBCombo('UFIni', 'kraftsoft', 'Estado', 'id', '{nome}','nome asc'  );
        $UFFim = new TDBCombo('UFFim', 'kraftsoft', 'Estado', 'id', '{nome}','nome asc'  );
        $chMDF = new TEntry('chMDF');
        $dtemi_inicial = new TDateTime('dtemi_inicial');
        $dtemi_final = new TDateTime('dtemi_final');
        $dhiniviagem_inicial = new TDateTime('dhiniviagem_inicial');
        $dhiniviagem_final = new TDateTime('dhiniviagem_final');


        $UFIni->enableSearch();
        $UFFim->enableSearch();
        $mdfe_tpcar_id->enableSearch();

        $dtemi_final->setMask('dd/mm/yyyy hh:ii');
        $dtemi_inicial->setMask('dd/mm/yyyy hh:ii');
        $dhiniviagem_final->setMask('dd/mm/yyyy hh:ii');
        $dhiniviagem_inicial->setMask('dd/mm/yyyy hh:ii');

        $dtemi_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $dtemi_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');
        $dhiniviagem_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $dhiniviagem_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');

        $nMDF->setSize('100%');
        $UFIni->setSize('100%');
        $UFFim->setSize('100%');
        $chMDF->setSize('100%');
        $dtemi_final->setSize('100%');
        $mdfe_tpcar_id->setSize('100%');
        $dtemi_inicial->setSize('100%');
        $dhiniviagem_final->setSize('100%');
        $dhiniviagem_inicial->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Número:", null, '14px', null, '100%'),$nMDF],[new TLabel("Tipo de Carga:", null, '14px', null, '100%'),$mdfe_tpcar_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("UF Carregamento:", null, '14px', null),$UFIni],[new TLabel("UF Descarregamento:", null, '14px', null),$UFFim]);
        $row2->layout = [' col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Chave:", null, '14px', null),$chMDF],[]);
        $row3->layout = [' col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Data e hora de emissão Inicial:", null, '14px', null, '100%'),$dtemi_inicial],[new TLabel("Data e hora de emissão Final:", null, '14px', null, '100%'),$dtemi_final],[new TLabel("Data e hora de inicio da viagem Inicial:", null, '14px', null, '100%'),$dhiniviagem_inicial],[new TLabel("Data e hora de inicio da viagem Final:", null, '14px', null, '100%'),$dhiniviagem_final]);
        $row4->layout = [' col-sm-3',' col-sm-3',' col-sm-3',' col-sm-3'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_nMDF = new TDataGridColumn('nMDF', "Número", 'left');
        $column_serie = new TDataGridColumn('serie', "Série", 'left');
        $column_dhEmi_transformed = new TDataGridColumn('dhEmi', "Data e hora de emissão", 'left');
        $column_mdfe_tpprop_descricao = new TDataGridColumn('mdfe_tpprop->descricao', "Tipo proprietário", 'left');
        $column_mdfe_tptransp_descricao = new TDataGridColumn('mdfe_tptransp->descricao', "Tipo de Transporte", 'left');
        $column_mdfe_tpcar_descricao = new TDataGridColumn('mdfe_tpcar->descricao', "Tipo de Carga", 'left');
        $column_status = new TDataGridColumn('status', "Status SEFAZ", 'left');
        $column_mdfe_situacao_descricao_transformed = new TDataGridColumn('mdfe_situacao->descricao', "Situação", 'left');

        $column_dhEmi_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_mdfe_situacao_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            $label = $object->mdfe_situacao->descricao;
            $cor = $object->mdfe_situacao->cor;

            return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

            // $div = new TElement('div');
            // $div->style="text-shadow:none; font-size:14px; font-weight:bold; color:#FFFFFF; width:160px; text-align:center;background-color: $cor;padding:10px";
            // $div->add($label);

            // return $div;    //code here
        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_nMDF);
        $this->datagrid->addColumn($column_serie);
        $this->datagrid->addColumn($column_dhEmi_transformed);
        $this->datagrid->addColumn($column_mdfe_tpprop_descricao);
        $this->datagrid->addColumn($column_mdfe_tptransp_descricao);
        $this->datagrid->addColumn($column_mdfe_tpcar_descricao);
        $this->datagrid->addColumn($column_status);
        $this->datagrid->addColumn($column_mdfe_situacao_descricao_transformed);

        $action_group = new TDataGridActionGroup("", 'fas:cog');
        $action_group->addHeader('');

        $action_onShow = new TDataGridAction(array('MdfeFormView', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Abrir");
        $action_onShow->setImage('fas:folder-open #478fca');
        $action_onShow->setField(self::$primaryKey);

        $action_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_onShow);

        $action_onEdit = new TDataGridAction(array('MdfeForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('fas:edit #2196F3');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('MdfeList::onExibeEditar');
        $action_onEdit->setParameter('key', '{id}');

        $action_group->addAction($action_onEdit);

        $action_onEnviar = new TDataGridAction(array('MdfeList', 'onEnviar'));
        $action_onEnviar->setUseButton(TRUE);
        $action_onEnviar->setButtonClass('btn btn-default');
        $action_onEnviar->setLabel("Enviar");
        $action_onEnviar->setImage('fas:cloud-upload-alt #4CAF50');
        $action_onEnviar->setField(self::$primaryKey);
        $action_onEnviar->setDisplayCondition('MdfeList::onExibeEnviar');

        $action_group->addAction($action_onEnviar);

        $action_onPDF = new TDataGridAction(array('MdfeList', 'onPDF'));
        $action_onPDF->setUseButton(TRUE);
        $action_onPDF->setButtonClass('btn btn-default');
        $action_onPDF->setLabel("PDF");
        $action_onPDF->setImage('far:file-pdf #F44336');
        $action_onPDF->setField(self::$primaryKey);
        $action_onPDF->setDisplayCondition('MdfeList::onExibePDF');

        $action_group->addAction($action_onPDF);

        $action_onEncerrar = new TDataGridAction(array('MdfeList', 'onEncerrar'));
        $action_onEncerrar->setUseButton(TRUE);
        $action_onEncerrar->setButtonClass('btn btn-default');
        $action_onEncerrar->setLabel("Encerrar");
        $action_onEncerrar->setImage('fas:flag-checkered #9C27B0');
        $action_onEncerrar->setField(self::$primaryKey);
        $action_onEncerrar->setDisplayCondition('MdfeList::onExibeEncerrar');

        $action_group->addAction($action_onEncerrar);

        $action_onCancelar = new TDataGridAction(array('MdfeList', 'onCancelar'));
        $action_onCancelar->setUseButton(TRUE);
        $action_onCancelar->setButtonClass('btn btn-default');
        $action_onCancelar->setLabel("Cancelar");
        $action_onCancelar->setImage('fas:window-close #F44336');
        $action_onCancelar->setField(self::$primaryKey);
        $action_onCancelar->setDisplayCondition('MdfeList::onExibeCancelar');

        $action_group->addAction($action_onCancelar);

        $action_onConsultarSefaz = new TDataGridAction(array('MdfeList', 'onConsultarSefaz'));
        $action_onConsultarSefaz->setUseButton(TRUE);
        $action_onConsultarSefaz->setButtonClass('btn btn-default');
        $action_onConsultarSefaz->setLabel("Consultar na SEFAZ");
        $action_onConsultarSefaz->setImage('fas:search #4CAF50');
        $action_onConsultarSefaz->setField(self::$primaryKey);

        $action_group->addAction($action_onConsultarSefaz);

        $action_onDelete = new TDataGridAction(array('MdfeList', 'onDelete'));
        $action_onDelete->setUseButton(TRUE);
        $action_onDelete->setButtonClass('btn btn-default');
        $action_onDelete->setLabel("Excluir");
        $action_onDelete->setImage('fas:trash-alt #dd5a43');
        $action_onDelete->setField(self::$primaryKey);
        $action_onDelete->setDisplayCondition('MdfeList::onExibeExcluir');

        $action_group->addAction($action_onDelete);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $button_cadastrar = new TButton('button_button_cadastrar');
        $button_cadastrar->setAction(new TAction(['MdfeForm', 'onShow']), "Cadastrar");
        $button_cadastrar->addStyleClass('btn-default');
        $button_cadastrar->setImage('fas:plus #69aa46');

        $this->datagrid_form->addField($button_cadastrar);

        $btnShowCurtainFilters = new TButton('button_btnShowCurtainFilters');
        $btnShowCurtainFilters->setAction(new TAction(['MdfeList', 'onShowCurtainFilters']), "Filtros");
        $btnShowCurtainFilters->addStyleClass('btn-default');
        $btnShowCurtainFilters->setImage('fas:filter #000000');

        $this->datagrid_form->addField($btnShowCurtainFilters);

        $button_limpar_filtros = new TButton('button_button_limpar_filtros');
        $button_limpar_filtros->setAction(new TAction(['MdfeList', 'onClearFilters']), "Limpar filtros");
        $button_limpar_filtros->addStyleClass('btn-default');
        $button_limpar_filtros->setImage('fas:eraser #f44336');

        $this->datagrid_form->addField($button_limpar_filtros);

        $button_atualizar = new TButton('button_button_atualizar');
        $button_atualizar->setAction(new TAction(['MdfeList', 'onRefresh']), "Atualizar");
        $button_atualizar->addStyleClass('btn-default');
        $button_atualizar->setImage('fas:sync-alt #03a9f4');

        $this->datagrid_form->addField($button_atualizar);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['MdfeList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['MdfeList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['MdfeList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['MdfeList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_left_actions->add($button_cadastrar);
        $head_left_actions->add($btnShowCurtainFilters);
        $head_left_actions->add($button_limpar_filtros);
        $head_left_actions->add($button_atualizar);

        $head_right_actions->add($dropdown_button_exportar);

        $this->btnShowCurtainFilters = $btnShowCurtainFilters;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Fiscal","MDF-e"]));
        }

        $container->add($panel);

        parent::add($container);

    }

    public static function onExibeEditar($object)
    {
        try 
        {
            if($object->mdfe_situacao_id==MdfeSituacao::EmDigitacao)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onEnviar($param = null) 
    {
        try 
        {
            new TQuestion("Enviar MDF-e?", new TAction([__CLASS__, 'onEnviarYes'], $param), new TAction([__CLASS__, 'onEnviarNo'], $param));
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeEnviar($object)
    {
        try 
        {
            if($object->mdfe_situacao_id==MdfeSituacao::EmDigitacao)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onPDF($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $mdfe = Mdfe::find($param['id']);

            if($mdfe)
            {
                 $arquivo = API_MDFe::PDF($mdfe); 

                 if(file_exists($arquivo))
                 {
                    parent::openFile($arquivo);            
                 }
                 else
                 {
                     throw new Exception('PDF não encontrado!');
                 }

            }

            TTransaction::close();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibePDF($object)
    {
        try 
        {
            if(!is_null($object->chMDF))
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onEncerrar($param = null) 
    {
        try 
        {
            $pageParam = ['id' => $param['id']]; // ex.: = ['key' => 10]
            TApplication::loadPage('MDFeEncerrarForm', 'onShow', $pageParam);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeEncerrar($object)
    {
        try 
        {
            if($object->mdfe_situacao_id==MdfeSituacao::Transmitido)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onCancelar($param = null) 
    {
        try 
        {
            $pageParam = ['id' => $param['id']]; // ex.: = ['key' => 10]
            TApplication::loadPage('MDFeCancelamentoForm', 'onShow', $pageParam);
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeCancelar($object)
    {
        try 
        {
            if($object->mdfe_situacao_id==MdfeSituacao::Transmitido)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onConsultarSefaz($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $mdfe = Mdfe::find($param['id']);

            $retorno = API_MDFe::Consulta($mdfe);

            $pageParam = ['titulo' => 'Retorno da SEFAZ'];

            TSession::setValue('mens_personalizada', 'Retorno da SEFAZ: '."\n".API_MDFe::formataRetorno($retorno));
            TApplication::loadPage('MensagemPersonalizada', 'OnShow', $pageParam);                     

            TTransaction::close();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onDelete($param = null) 
    { 
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new Mdfe($key, FALSE); 

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }
    public static function onExibeExcluir($object)
    {
        try 
        {
            if($object->mdfe_situacao_id==MdfeSituacao::EmDigitacao)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('object');
                $object->data  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }
    public static function onShowCurtainFilters($param = null) 
    {
        try 
        {
            //code here

                        $filter = new self([]);

            $btnClose = new TButton('closeCurtain');
            $btnClose->class = 'btn btn-sm btn-default';
            $btnClose->style = 'margin-right:10px;';
            $btnClose->onClick = "Template.closeRightPanel();";
            $btnClose->setLabel("Fechar");
            $btnClose->setImage('fas:times');

            $filter->form->addHeaderWidget($btnClose);

            $page = new TPage();
            $page->setTargetContainer('adianti_right_panel');
            $page->setProperty('page-name', 'MdfeListSearch');
            $page->setProperty('page_name', 'MdfeListSearch');
            $page->adianti_target_container = 'adianti_right_panel';
            $page->target_container = 'adianti_right_panel';
            $page->add($filter->form);
            $page->setIsWrapped(true);
            $page->show();

            $style = new TStyle('right-panel > .container-part[page-name=MdfeListSearch]');
            $style->width = '70% !important';
            $style->show(true);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onClearFilters($param = null) 
    {
        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }
    public function onRefresh($param = null) 
    {
        $this->onReload([]);
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->nMDF) AND ( (is_scalar($data->nMDF) AND $data->nMDF !== '') OR (is_array($data->nMDF) AND (!empty($data->nMDF)) )) )
        {

            $filters[] = new TFilter('nMDF', 'like', "%{$data->nMDF}%");// create the filter 
        }

        if (isset($data->mdfe_tpcar_id) AND ( (is_scalar($data->mdfe_tpcar_id) AND $data->mdfe_tpcar_id !== '') OR (is_array($data->mdfe_tpcar_id) AND (!empty($data->mdfe_tpcar_id)) )) )
        {

            $filters[] = new TFilter('mdfe_tpcar_id', '=', $data->mdfe_tpcar_id);// create the filter 
        }

        if (isset($data->UFIni) AND ( (is_scalar($data->UFIni) AND $data->UFIni !== '') OR (is_array($data->UFIni) AND (!empty($data->UFIni)) )) )
        {

            $filters[] = new TFilter('UFIni', '=', $data->UFIni);// create the filter 
        }

        if (isset($data->UFFim) AND ( (is_scalar($data->UFFim) AND $data->UFFim !== '') OR (is_array($data->UFFim) AND (!empty($data->UFFim)) )) )
        {

            $filters[] = new TFilter('UFFim', '=', $data->UFFim);// create the filter 
        }

        if (isset($data->chMDF) AND ( (is_scalar($data->chMDF) AND $data->chMDF !== '') OR (is_array($data->chMDF) AND (!empty($data->chMDF)) )) )
        {

            $filters[] = new TFilter('chMDF', '=', $data->chMDF);// create the filter 
        }

        if (isset($data->dtemi_inicial) AND ( (is_scalar($data->dtemi_inicial) AND $data->dtemi_inicial !== '') OR (is_array($data->dtemi_inicial) AND (!empty($data->dtemi_inicial)) )) )
        {

            $filters[] = new TFilter('dhEmi', '>=', $data->dtemi_inicial);// create the filter 
        }

        if (isset($data->dtemi_final) AND ( (is_scalar($data->dtemi_final) AND $data->dtemi_final !== '') OR (is_array($data->dtemi_final) AND (!empty($data->dtemi_final)) )) )
        {

            $filters[] = new TFilter('dhEmi', '<=', $data->dtemi_final);// create the filter 
        }

        if (isset($data->dhiniviagem_inicial) AND ( (is_scalar($data->dhiniviagem_inicial) AND $data->dhiniviagem_inicial !== '') OR (is_array($data->dhiniviagem_inicial) AND (!empty($data->dhiniviagem_inicial)) )) )
        {

            $filters[] = new TFilter('dhIniViagem', '>=', $data->dhiniviagem_inicial);// create the filter 
        }

        if (isset($data->dhiniviagem_final) AND ( (is_scalar($data->dhiniviagem_final) AND $data->dhiniviagem_final !== '') OR (is_array($data->dhiniviagem_final) AND (!empty($data->dhiniviagem_final)) )) )
        {

            $filters[] = new TFilter('dhIniViagem', '<=', $data->dhiniviagem_final);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Mdfe
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            //</blockLine><btnShowCurtainFiltersAutoCode>
            if(!empty($this->btnShowCurtainFilters) && empty($this->btnShowCurtainFiltersAdjusted))
            {
                $this->btnShowCurtainFiltersAdjusted = true;
                $this->btnShowCurtainFilters->style = 'position: relative';
                $countFilters = count($filters ?? []);
                $this->btnShowCurtainFilters->setLabel($this->btnShowCurtainFilters->getLabel(). "<span class='badge badge-success' style='position: absolute'>{$countFilters}<span>");
            }
            //</blockLine></btnShowCurtainFiltersAutoCode>

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function onEnviarYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $mdfe = Mdfe::find($param['id']);

            if($mdfe)
            {
                $retorno =  API_MDFe::Enviar($mdfe); 

                 if ($retorno->codigo != 100)     
                 {
                    //  $pageParam = ['mensagem' => 'Retorno da SEFAZ: '."\n".API_MDFe::formataRetorno($retorno)]; 

                    $pageParam = ['titulo' => 'Retorno da SEFAZ'];

                     TSession::setValue('mens_personalizada', 'Retorno da SEFAZ: '."\n".API_MDFe::formataRetorno($retorno));
                     TApplication::loadPage('MensagemPersonalizada', 'OnShow', $pageParam);
                 }
                 else
                 {
                     TApplication::loadPage(__CLASS__, 'onShow', ['key' => $mdfe->id]);
                     TToast::show('success', "Documento enviado com sucesso!", 'topRight', 'far:check-circle');
                 }
            }            

            TTransaction::close();
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onEnviarNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

}

