<?php

class NotafiscalList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Notafiscal';
    private static $primaryKey = 'id';
    private static $formName = 'form_NotafiscalList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Nota Fiscal");
        $this->limit = 20;

        $numero = new TEntry('numero');
        $id = new TEntry('id');
        $notafiscal_tpamb_id = new TDBCombo('notafiscal_tpamb_id', 'kraftsoft', 'NotafiscalTpamb', 'id', '{descricao}','descricao asc'  );
        $data_emissao = new TDateTime('data_emissao');
        $data_emissao_final = new TDateTime('data_emissao_final');
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{nome}  - {cpf_cnpj}','nome asc'  );
        $notafiscal_situacao_id = new TDBCombo('notafiscal_situacao_id', 'kraftsoft', 'NotafiscalSituacao', 'id', '{descricao}','descricao asc'  );
        $notafiscal_mod_id = new TDBCombo('notafiscal_mod_id', 'kraftsoft', 'NotafiscalMod', 'id', '{descricao}','descricao asc'  );


        $data_emissao->setMask('dd/mm/yyyy hh:ii');
        $data_emissao_final->setMask('dd/mm/yyyy hh:ii');

        $data_emissao->setDatabaseMask('yyyy-mm-dd hh:ii:ss');
        $data_emissao_final->setDatabaseMask('yyyy-mm-dd hh:ii:ss');

        $cliente_id->enableSearch();
        $notafiscal_mod_id->enableSearch();
        $notafiscal_situacao_id->enableSearch();

        $id->setSize('100%');
        $numero->setSize('100%');
        $cliente_id->setSize('100%');
        $data_emissao->setSize('100%');
        $notafiscal_mod_id->setSize('100%');
        $data_emissao_final->setSize('100%');
        $notafiscal_tpamb_id->setSize('100%');
        $notafiscal_situacao_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Número:", null, '14px', null, '100%'),$numero],[new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Tipo Ambiente:", null, '14px', null),$notafiscal_tpamb_id],[new TLabel("Data emissão inicial:", null, '14px', null),$data_emissao],[new TLabel(" Data emissão final:", null, '14px', null, '100%'),$data_emissao_final]);
        $row1->layout = [' col-sm-2','col-sm-2',' col-sm-2','col-sm-3','col-sm-3'];

        $row2 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null),$cliente_id],[new TLabel("Situação:", null, '14px', null),$notafiscal_situacao_id],[new TLabel("Modelo:", null, '14px', null),$notafiscal_mod_id]);
        $row2->layout = [' col-sm-5',' col-sm-5',' col-sm-2'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Cadastrar NF-e", new TAction(['NotafiscalForm', 'onShow']), 'fas:plus #03A9F4');
        $this->btn_onshow = $btn_onshow;

        $btn_onshow = $this->form->addAction("Cadastrar NFC-e", new TAction(['NotafiscalConsumidorForm', 'onShow']), 'fas:plus #4CAF50');
        $this->btn_onshow = $btn_onshow;

        $btn_onstatussefaz = $this->form->addAction("Status Sefaz", new TAction([$this, 'onStatusSefaz']), 'fas:wifi #F6F6F6');
        $this->btn_onstatussefaz = $btn_onstatussefaz;
        $btn_onstatussefaz->addStyleClass('btn-success'); 

        $btn_onshow = $this->form->addHeaderAction("Eventos Cancelamento e Carta Correção", new TAction(['PedidoEventoList', 'onShow']), 'fas:apple-alt #FFFFFF');
        $this->btn_onshow = $btn_onshow;
        $btn_onshow->addStyleClass('btn-info'); 

        $btn_onshow = $this->form->addHeaderAction("Eventos Numeração", new TAction(['InutilizarNumeroList', 'onShow']), 'fas:sort-numeric-up-alt #F8F8F8');
        $this->btn_onshow = $btn_onshow;
        $btn_onshow->addStyleClass('btn-warning'); 

        $btn_onshow = $this->form->addHeaderAction("XML Contabilidade", new TAction(['XmlContadorList', 'onShow']), 'far:save #DADADA');
        $this->btn_onshow = $btn_onshow;
        $btn_onshow->addStyleClass('btn-success'); 

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);
        $this->datagrid->enablePopover("Informações", "  <b>Situação: </b> {mensagem}  
<b> Chave: </b>{chave}  ");

        $column_id = new TDataGridColumn('id', "Código", 'left');
        $column_notafiscal_mod_descricao = new TDataGridColumn('notafiscal_mod->descricao', "Modelo", 'left');
        $column_serie = new TDataGridColumn('serie', "Série", 'left');
        $column_numero = new TDataGridColumn('numero', "Número", 'left');
        $column_data_emissao_transformed = new TDataGridColumn('data_emissao', "Data Emissão", 'left');
        $column_cliente_id_transformed = new TDataGridColumn('cliente_id', "Cliente", 'left');
        $column_valor_total_nota_transformed = new TDataGridColumn('valor_total_nota', "Valor", 'left');
        $column_notafiscal_situacao_id_transformed = new TDataGridColumn('notafiscal_situacao_id', "Situação", 'left');
        $column_movimentacao_origem_descricao = new TDataGridColumn('movimentacao_origem->descricao', "Origem", 'left');
        $column_notafiscal_mod_id = new TDataGridColumn('notafiscal_mod_id', "Nova coluna", 'left');

        $column_data_emissao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_cliente_id_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            $cliente = Cliente::find($value);
            if($cliente){
                return $object->cliente_nome."<br><b>CPF/CNPJ: </b>".Funcoes::mascaraCpfCnpj($cliente->cpf_cnpj);    
            }

        });

        $column_valor_total_nota_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_notafiscal_situacao_id_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
                $label = $object->notafiscal_situacao->descricao;
                $cor = $object->notafiscal_situacao->cor;

                return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

                // $div = new TElement('div');
                // $div->style="text-shadow:none; font-size:14px; font-weight:bold; color:#FFFFFF; width:160px; text-align:center;background-color: $cor;padding:10px";
                // $div->add($label);

                // return $div;    //code here

        });        

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_notafiscal_mod_descricao);
        $this->datagrid->addColumn($column_serie);
        $this->datagrid->addColumn($column_numero);
        $this->datagrid->addColumn($column_data_emissao_transformed);
        $this->datagrid->addColumn($column_cliente_id_transformed);
        $this->datagrid->addColumn($column_valor_total_nota_transformed);
        $this->datagrid->addColumn($column_notafiscal_situacao_id_transformed);
        $this->datagrid->addColumn($column_movimentacao_origem_descricao);
        $this->datagrid->addColumn($column_notafiscal_mod_id);

        $action_group = new TDataGridActionGroup("Opções", 'fas:cog');
        $action_group->addHeader('');

        $action_onAbrir = new TDataGridAction(array('NotafiscalList', 'onAbrir'));
        $action_onAbrir->setUseButton(TRUE);
        $action_onAbrir->setButtonClass('btn btn-default');
        $action_onAbrir->setLabel("Abrir");
        $action_onAbrir->setImage('fas:folder-open #478fca');
        $action_onAbrir->setField(self::$primaryKey);

        $action_onAbrir->setParameter('key', '{id}');

        $action_group->addAction($action_onAbrir);

        $action_onEdit = new TDataGridAction(array('NotafiscalList', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('fas:edit #2196F3');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('NotafiscalList::onMostrarEdit');
        $action_onEdit->setParameter('key', '{id}');

        $action_group->addAction($action_onEdit);

        $action_onEnviarNota = new TDataGridAction(array('NotafiscalList', 'onEnviarNota'));
        $action_onEnviarNota->setUseButton(TRUE);
        $action_onEnviarNota->setButtonClass('btn btn-default');
        $action_onEnviarNota->setLabel(" Enviar");
        $action_onEnviarNota->setImage('fas:cloud-upload-alt #4CAF50');
        $action_onEnviarNota->setField(self::$primaryKey);
        $action_onEnviarNota->setDisplayCondition('NotafiscalList::onMostrarEnviar');

        $action_group->addAction($action_onEnviarNota);

        $action_onSeandEmail = new TDataGridAction(array('NotafiscalList', 'onSeandEmail'));
        $action_onSeandEmail->setUseButton(TRUE);
        $action_onSeandEmail->setButtonClass('btn btn-default');
        $action_onSeandEmail->setLabel("Enviar NF-e para email");
        $action_onSeandEmail->setImage('fas:envelope-open #673AB7');
        $action_onSeandEmail->setField(self::$primaryKey);
        $action_onSeandEmail->setDisplayCondition('NotafiscalList::OnHabilitaSendEmail');

        $action_group->addAction($action_onSeandEmail);

        $action_onConsultaNFE = new TDataGridAction(array('NotafiscalList', 'onConsultaNFE'));
        $action_onConsultaNFE->setUseButton(TRUE);
        $action_onConsultaNFE->setButtonClass('btn btn-default');
        $action_onConsultaNFE->setLabel("Consultar na SEFAZ");
        $action_onConsultaNFE->setImage('fas:search #00BCD4');
        $action_onConsultaNFE->setField(self::$primaryKey);
        $action_onConsultaNFE->setDisplayCondition('NotafiscalList::onExibeConsultar');

        $action_group->addAction($action_onConsultaNFE);

        $action_onBaixarXML = new TDataGridAction(array('NotafiscalList', 'onBaixarXML'));
        $action_onBaixarXML->setUseButton(TRUE);
        $action_onBaixarXML->setButtonClass('btn btn-default');
        $action_onBaixarXML->setLabel("Baixa XML");
        $action_onBaixarXML->setImage('fas:angle-double-down #03A9F4');
        $action_onBaixarXML->setField(self::$primaryKey);
        $action_onBaixarXML->setDisplayCondition('NotafiscalList::onHabilitaXMl');

        $action_group->addAction($action_onBaixarXML);

        $action_onCancelarNFE = new TDataGridAction(array('NotafiscalList', 'onCancelarNFE'));
        $action_onCancelarNFE->setUseButton(TRUE);
        $action_onCancelarNFE->setButtonClass('btn btn-default');
        $action_onCancelarNFE->setLabel("Cancelar");
        $action_onCancelarNFE->setImage('fas:ban #F44336');
        $action_onCancelarNFE->setField(self::$primaryKey);
        $action_onCancelarNFE->setDisplayCondition('NotafiscalList::onMostrarCancelar');

        $action_group->addAction($action_onCancelarNFE);

        $action_onPDF = new TDataGridAction(array('NotafiscalList', 'onPDF'));
        $action_onPDF->setUseButton(TRUE);
        $action_onPDF->setButtonClass('btn btn-default');
        $action_onPDF->setLabel("PDF");
        $action_onPDF->setImage('far:file-pdf #E91E63');
        $action_onPDF->setField(self::$primaryKey);
        $action_onPDF->setDisplayCondition('NotafiscalList::onMostrarPDF');

        $action_group->addAction($action_onPDF);

        $action_onCartaCorrecao = new TDataGridAction(array('NotafiscalList', 'onCartaCorrecao'));
        $action_onCartaCorrecao->setUseButton(TRUE);
        $action_onCartaCorrecao->setButtonClass('btn btn-default');
        $action_onCartaCorrecao->setLabel("Carta de Correção");
        $action_onCartaCorrecao->setImage('far:file-alt #000000');
        $action_onCartaCorrecao->setField(self::$primaryKey);
        $action_onCartaCorrecao->setDisplayCondition('NotafiscalList::onExibeCartaCorrecao');

        $action_group->addAction($action_onCartaCorrecao);

        $action_onPDFCartaCorrecao = new TDataGridAction(array('NotafiscalList', 'onPDFCartaCorrecao'));
        $action_onPDFCartaCorrecao->setUseButton(TRUE);
        $action_onPDFCartaCorrecao->setButtonClass('btn btn-default');
        $action_onPDFCartaCorrecao->setLabel("PDF Carta Correção");
        $action_onPDFCartaCorrecao->setImage('far:file-pdf #2196F3');
        $action_onPDFCartaCorrecao->setField(self::$primaryKey);
        $action_onPDFCartaCorrecao->setDisplayCondition('NotafiscalList::onExibePDFCartaCorrecao');

        $action_group->addAction($action_onPDFCartaCorrecao);

        $action_onClonar = new TDataGridAction(array('NotafiscalList', 'onClonar'));
        $action_onClonar->setUseButton(TRUE);
        $action_onClonar->setButtonClass('btn btn-default');
        $action_onClonar->setLabel("Clonar");
        $action_onClonar->setImage('far:copy #FFE821');
        $action_onClonar->setField(self::$primaryKey);

        $action_group->addAction($action_onClonar);

        $action_onExcluir = new TDataGridAction(array('NotafiscalList', 'onExcluir'));
        $action_onExcluir->setUseButton(TRUE);
        $action_onExcluir->setButtonClass('btn btn-default');
        $action_onExcluir->setLabel("Excluir");
        $action_onExcluir->setImage('fas:trash-alt #E70505');
        $action_onExcluir->setField(self::$primaryKey);
        $action_onExcluir->setDisplayCondition('NotafiscalList::onExibeExcluir');

        $action_group->addAction($action_onExcluir);

        $action_OnInutilizar = new TDataGridAction(array('NotafiscalList', 'OnInutilizar'));
        $action_OnInutilizar->setUseButton(TRUE);
        $action_OnInutilizar->setButtonClass('btn btn-default');
        $action_OnInutilizar->setLabel("Inutilizar numero ");
        $action_OnInutilizar->setImage('fas:backspace #F44336');
        $action_OnInutilizar->setField(self::$primaryKey);
        $action_OnInutilizar->setDisplayCondition('NotafiscalList::onHabilita');

        $action_group->addAction($action_OnInutilizar);

        $action_onShow = new TDataGridAction(array('NotafiscalLogsTimeLine', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Log NFe");
        $action_onShow->setImage('fas:cloud-meatball #9C27B0');
        $action_onShow->setField(self::$primaryKey);
        $action_onShow->setDisplayCondition('NotafiscalList::onOffLog');
        $action_onShow->setParameter('notafiscal_id', '{id}');

        $action_group->addAction($action_onShow);

        $action_OnSeachNota = new TDataGridAction(array('NotafiscalList', 'OnSeachNota'));
        $action_OnSeachNota->setUseButton(TRUE);
        $action_OnSeachNota->setButtonClass('btn btn-default');
        $action_OnSeachNota->setLabel("Consultar na NF-e");
        $action_OnSeachNota->setImage('fas:search-plus #673AB7');
        $action_OnSeachNota->setField(self::$primaryKey);
        $action_OnSeachNota->setDisplayCondition('NotafiscalList::onConsultaOff');

        $action_group->addAction($action_OnSeachNota);

        $action_PedidoEventoForm_onShow = new TDataGridAction(array('PedidoEventoForm', 'onShow'));
        $action_PedidoEventoForm_onShow->setUseButton(TRUE);
        $action_PedidoEventoForm_onShow->setButtonClass('btn btn-default');
        $action_PedidoEventoForm_onShow->setLabel("Carta de Correção");
        $action_PedidoEventoForm_onShow->setImage('fas:newspaper #03A9F4');
        $action_PedidoEventoForm_onShow->setField(self::$primaryKey);
        $action_PedidoEventoForm_onShow->setDisplayCondition('NotafiscalList::onOffcarta');
        $action_PedidoEventoForm_onShow->setParameter('notafiscal_id', '{id}');
        $action_PedidoEventoForm_onShow->setParameter("tipo", "carta");

        $action_group->addAction($action_PedidoEventoForm_onShow);

        $action_PedidoEventoForm_onShow = new TDataGridAction(array('PedidoEventoForm', 'onShow'));
        $action_PedidoEventoForm_onShow->setUseButton(TRUE);
        $action_PedidoEventoForm_onShow->setButtonClass('btn btn-default');
        $action_PedidoEventoForm_onShow->setLabel("Cancelar NF-e");
        $action_PedidoEventoForm_onShow->setImage('fas:bacon #F44336');
        $action_PedidoEventoForm_onShow->setField(self::$primaryKey);

        $action_PedidoEventoForm_onShow->setParameter('notafiscal_id', '{id}');
        $action_PedidoEventoForm_onShow->setParameter("tipo", "cancelado");

        $action_group->addAction($action_PedidoEventoForm_onShow);

        $action_oneciboCarta = new TDataGridAction(array('NotafiscalList', 'oneciboCarta'));
        $action_oneciboCarta->setUseButton(TRUE);
        $action_oneciboCarta->setButtonClass('btn btn-default');
        $action_oneciboCarta->setLabel("ReciboCarta");
        $action_oneciboCarta->setImage('fas:pencil-ruler #2196F3');
        $action_oneciboCarta->setField(self::$primaryKey);
        $action_oneciboCarta->setDisplayCondition('NotafiscalList::DesbilitarCarta');

        $action_group->addAction($action_oneciboCarta);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['NotafiscalList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:table #00b894' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['NotafiscalList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['NotafiscalList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_right_actions->add($dropdown_button_exportar);

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Fiscal","Nota Fiscal"]));
        }
        $container->add($this->form);
        $container->add($panel);

        parent::add($container);

    }

    public function onAbrir($param = null) 
    {
        try 
        {
             $pageParam = ['key' => $param['id']]; // ex.: = ['key' => 10]

              TTransaction::open(self::$database);

              $nota = Notafiscal::find($param['id']);

              TTransaction::close();

              if ($nota->notafiscal_mod_id==NotafiscalMod::NFe)
              {
                TApplication::loadPage('NotafiscalFormView', 'OnShow', $pageParam);
              }
              else
              {
                TApplication::loadPage('NotafiscalConsumidorFormView', 'OnShow', $pageParam);
              }
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onEdit($param = null) 
    {
        try 
        {
             $pageParam = ['key' => $param['id']]; // ex.: = ['key' => 10]

              TTransaction::open(self::$database);

              $nota = Notafiscal::find($param['id']);

              TTransaction::close();

              if ($nota->notafiscal_mod_id==NotafiscalMod::NFe)
              {
                TApplication::loadPage('NotafiscalForm', 'OnEdit', $pageParam);
              }
              else
              {
                TApplication::loadPage('NotafiscalConsumidorForm', 'OnEdit', $pageParam);
              }

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarEdit($object)
    {
        try 
        {
            if($object->notafiscal_situacao_id==NotafiscalSituacao::EmDigitacao)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onEnviarNota($param = null) 
    {
        try 
        {

            new TQuestion("Enviar Nota Fiscal?", new TAction([__CLASS__, 'onEnviarNotaYes'], $param), new TAction([__CLASS__, 'onEnviarNotaNo'], $param));
       $this->onReload();
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarEnviar($object)
    {
        try 
        {

            if(is_null($object->status))
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onSeandEmail($param = null) 
    {
        try 
        {

          $EmialNFe= new SendEmailNFE($param['id']);
           $response=  $EmialNFe->send();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function OnHabilitaSendEmail($object)
    {
        try 
        {
    if($object->notafiscal_situacao_id >=2)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onConsultaNFE($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $nota = Notafiscal::find($param['id']);

            if($nota)
            {
                cloudDFE_NFE::consultaNFE($nota->chave); 
            }

            TTransaction::close();

            $this->onReload($param);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeConsultar($object)
    {
        try 
        {
                   if($object->notafiscal_situacao_id >=2)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onBaixarXML($param = null) 
    {
        try 
        {
                $donloads= new  DownloadsXML($param['key']);

                $baixar= $donloads->dow();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onHabilitaXMl($object)
    {
        try 
        {
            if(isset($object->xml))
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onCancelarNFE($param = null) 
    {
        try 
        {
            $pageParam = ['id' => $param['id']]; // ex.: = ['key' => 10]
            TApplication::loadPage('CancelementoNotaForm', 'onShow', $pageParam);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarCancelar($object)
    {
        try 
        {
            if($object->codigo_sefaz == 100)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onPDF($param = null) 
    {
        try 
        {

        ImprimirNFe::print($param['id']);

            // TTransaction::open(self::$database);

            // $nota = Notafiscal::find($param['id']);

            // if($nota)
            // {

            //      if($nota->notafiscal_mod_id == NotafiscalMod::NFe)
            //      {                
            //          $arquivo = cloudDFE_NFE::pdfNFE($nota->chave); 
            //      }
            //      else
            //      {
            //          $arquivo = cloudDFE_NFE::pdfNFCe($nota->chave); 
            //      }

            //      if(file_exists($arquivo)){
            //             parent::openFile($arquivo);            
            //      }else{
            //          throw new Exception('PDF não encontrado!');
            //      }
            // }

            // TTransaction::close();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarPDF($object)
    {
        try 
        {
            if(!is_null($object->chave))
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onCartaCorrecao($param = null) 
    {
        try 
        {
            $pageParam = ['id' => $param['id']]; // ex.: = ['key' => 10]
            TApplication::loadPage('CartaCorrecaoNotaForm', 'onShow', $pageParam);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeCartaCorrecao($object)
    {
        try 
        {
            if (($object->codigo_sefaz == 100) && ($object->notafiscal_mod_id==NotafiscalMod::NFe))
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onPDFCartaCorrecao($param = null) 
    {
        try 
        {
            //code here

            TTransaction::open(self::$database);

            $nota = Notafiscal::find($param['id']);

            if($nota)
            {
                 $arquivo = cloudDFE_NFE::pdfCCe($nota->chave); 

                 if(file_exists($arquivo)){
                        parent::openFile($arquivo);            
                 }else{
                     throw new Exception('PDF não encontrado.');
                 }

            }

            TTransaction::close();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibePDFCartaCorrecao($object)
    {
        try 
        {
            if($object->sn_carta_correcao == 'S')
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onClonar($param = null) 
    {
        try 
        {
            //code here

            TTransaction::open(self::$database);

            funcaoNFE::DuplicarNota($param['id']);

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show("success", "Nota clonada com sucesso!", "topCenter", "fas:check-circle");

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExcluir($param = null) 
    {
        try 
        {
            new TQuestion("Excluir a Nota Fiscal?", new TAction([__CLASS__, 'onExcluirYes'], $param), new TAction([__CLASS__, 'onExcluirNo'], $param));
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onExibeExcluir($object)
    {
        try 
        {
            if($object->notafiscal_situacao_id==NotafiscalSituacao::EmDigitacao)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function OnInutilizar($param = null) 
    {
        try 
        {

  new TQuestion("Você deseja realmente descontinuar a numeração da NFe? (Selecione 'sim' para confirmar): ", new TAction([__CLASS__, 'onYes'], $param), new TAction([__CLASS__, 'onNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onHabilita($object)
    {
        try 
        {
             if($object->notafiscal_situacao_id <=1){

                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onOffLog($object)
    {
        try 
        {
          if($object->notafiscal_situacao_id >=2)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function OnSeachNota($param = null) 
    {
        try 
        {
    $Consulta= new ConsultarNFe($param['id']);
        $Consulta->Seach();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onConsultaOff($object)
    {
        try 
        {
              if($object->notafiscal_situacao_id >=2)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onOffcarta($object)
    {
        try 
        {
            if($object->notafiscal_situacao_id==2)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function oneciboCarta($param = null) 
    {
        try 
        {
  $recibo= new  ReciboCartaCorrecaoNFe($param['id']);
  $result= $recibo->ReciboCarta();

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function DesbilitarCarta($object)
    {
        try 
        {

        $response= ClassMaster::CartaOFF($object->id);
             if($response==1)
            {
                return true;
            }

            return false;

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('object');
                $object->data  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }
    public function onStatusSefaz($param = null) 
    {
        try 
        {
                $sTATUS= new  ConSusultaStatusSefaz(TSession::getValue('userunitid'));

                $response=   $sTATUS->Cosulta();

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->numero) AND ( (is_scalar($data->numero) AND $data->numero !== '') OR (is_array($data->numero) AND (!empty($data->numero)) )) )
        {

            $filters[] = new TFilter('numero', 'like', "%{$data->numero}%");// create the filter 
        }

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->notafiscal_tpamb_id) AND ( (is_scalar($data->notafiscal_tpamb_id) AND $data->notafiscal_tpamb_id !== '') OR (is_array($data->notafiscal_tpamb_id) AND (!empty($data->notafiscal_tpamb_id)) )) )
        {

            $filters[] = new TFilter('notafiscal_tpamb_id', '=', $data->notafiscal_tpamb_id);// create the filter 
        }

        if (isset($data->data_emissao) AND ( (is_scalar($data->data_emissao) AND $data->data_emissao !== '') OR (is_array($data->data_emissao) AND (!empty($data->data_emissao)) )) )
        {

            $filters[] = new TFilter('data_emissao', '>=', $data->data_emissao);// create the filter 
        }

        if (isset($data->data_emissao_final) AND ( (is_scalar($data->data_emissao_final) AND $data->data_emissao_final !== '') OR (is_array($data->data_emissao_final) AND (!empty($data->data_emissao_final)) )) )
        {

            $filters[] = new TFilter('data_emissao', '<=', $data->data_emissao_final);// create the filter 
        }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }

        if (isset($data->notafiscal_situacao_id) AND ( (is_scalar($data->notafiscal_situacao_id) AND $data->notafiscal_situacao_id !== '') OR (is_array($data->notafiscal_situacao_id) AND (!empty($data->notafiscal_situacao_id)) )) )
        {

            $filters[] = new TFilter('notafiscal_situacao_id', '=', $data->notafiscal_situacao_id);// create the filter 
        }

        if (isset($data->notafiscal_mod_id) AND ( (is_scalar($data->notafiscal_mod_id) AND $data->notafiscal_mod_id !== '') OR (is_array($data->notafiscal_mod_id) AND (!empty($data->notafiscal_mod_id)) )) )
        {

            $filters[] = new TFilter('notafiscal_mod_id', '=', $data->notafiscal_mod_id);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Notafiscal
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function onEnviarNotaYes($param = null) 
    {
        try 
        {
TTransaction::open(self::$database);
$emitente = SystemUnit::find(TSession::getValue('userunitid'));
$nota = Notafiscal::find($param['id']);

if ($nota) {
    if ($nota->notafiscal_mod_id == NotafiscalMod::NFe) {
        switch ($emitente->nfe_api_documentos_id) {
            case 3:
                $xmlResult = new GerarNFeSped($nota->id);
                $xmlResult->create();

                $this->onReload();
                break;
            case 2:
                // aqui segunda alternativa
                break;
            case 1:
                $retorno = cloudDFE_NFE::criarNFE($nota);

                if ($retorno->codigo != 100) {
                    new TMessage('info', 'Retorno da SEFAZ:' . "\n" . cloudDFE_NFE::formataRetorno($retorno));
                } else {
                    TApplication::loadPage('NotafiscalList', 'onShow');
                    TToast::show('success', "Documento enviado com sucesso!", 'topRight', 'far:check-circle');
                }
                break;
        }
    } else {
        switch ($emitente->nfe_api_documentos_id) {
            case 3:
               $xmlResult = new GerarNFCeSped($nota->id);
               $response=  $xmlResult->create();

             // $this->onReload();
                break;
            case 2:
                // aqui segunda alternativa
                break;
            case 1:
                // $retorno = cloudDFE_NFE::criarNfce($nota);

                // if ($retorno->codigo != 100) {
                //     new TMessage('info', 'Retorno da SEFAZ:' . "\n" . cloudDFE_NFE::formataRetorno($retorno));
                // } else {
                //     TApplication::loadPage('NotafiscalList', 'onShow');
                //     TToast::show('success', "Documento enviado com sucesso!", 'topRight', 'far:check-circle');
                // }
                // break;
        }
    }
}
TTransaction::close();

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onEnviarNotaNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onExcluirYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $nota = Notafiscal::find($param['key']);
            $nota->delete();

            TTransaction::close();

            // reload the listing
            $this->onReload( $param );

            TToast::show('success', AdiantiCoreTranslator::translate('Record deleted'), 'topRight', 'far:check-circle');           
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onExcluirNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onYes($param = null) 
    {
        try 
        {
         $inutilizar= new InutilizaSpedNFeOne($param['id']);
         $numero=   $inutilizar->inutone();

          $this->onReload();
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

}

