<?php

class NotafiscalReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Notafiscal';
    private static $primaryKey = 'id';
    private static $formName = 'form_NotafiscalReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Relatório de Notas Fiscais");

        $numero = new TEntry('numero');
        $notafiscal_tpamb_id = new TDBCombo('notafiscal_tpamb_id', 'kraftsoft', 'NotafiscalTpamb', 'id', '{descricao}','descricao asc'  );
        $data_emissao = new TDate('data_emissao');
        $data_emissao_final = new TDate('data_emissao_final');
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{nome}  - {cpf_cnpj}','nome asc'  );
        $notafiscal_mod_id = new TDBCombo('notafiscal_mod_id', 'kraftsoft', 'NotafiscalMod', 'id', '{descricao}','descricao asc'  );

        $data_emissao->setMask('dd/mm/yyyy');
        $data_emissao_final->setMask('dd/mm/yyyy');

        $data_emissao->setDatabaseMask('yyyy-mm-dd');
        $data_emissao_final->setDatabaseMask('yyyy-mm-dd');

        $cliente_id->enableSearch();
        $notafiscal_mod_id->enableSearch();

        $numero->setSize('100%');
        $cliente_id->setSize('100%');
        $data_emissao->setSize('100%');
        $notafiscal_mod_id->setSize('100%');
        $data_emissao_final->setSize('100%');
        $notafiscal_tpamb_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Número:", null, '14px', null, '100%'),$numero],[new TLabel("Tipo Ambiente:", null, '14px', null),$notafiscal_tpamb_id],[new TLabel("Data emissão inicial:", null, '14px', null),$data_emissao],[new TLabel(" Data emissão final:", null, '14px', null, '100%'),$data_emissao_final]);
        $row1->layout = ['col-sm-3',' col-sm-3',' col-sm-3',' col-sm-3'];

        $row2 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null),$cliente_id],[new TLabel("Modelo:", null, '14px', null),$notafiscal_mod_id]);
        $row2->layout = [' col-sm-9',' col-sm-3'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratehtml = $this->form->addAction("Gerar HTML", new TAction([$this, 'onGenerateHtml']), 'far:file-code #ffffff');
        $this->btn_ongeneratehtml = $btn_ongeneratehtml;
        $btn_ongeneratehtml->addStyleClass('btn-primary'); 

        $btn_ongeneratepdf = $this->form->addAction("Gerar PDF", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        $btn_ongeneratexls = $this->form->addAction("Gerar XLS", new TAction([$this, 'onGenerateXls']), 'far:file-excel #00a65a');
        $this->btn_ongeneratexls = $btn_ongeneratexls;

        $btn_ongeneratertf = $this->form->addAction("Gerar RTF", new TAction([$this, 'onGenerateRtf']), 'far:file-alt #324bcc');
        $this->btn_ongeneratertf = $btn_ongeneratertf;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Fiscal","Relatório de Notas Fiscais"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }
    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }
    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }
    public function onGenerateRtf($param = null) 
    {
        $this->onGenerate('rtf');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->numero) AND ( (is_scalar($data->numero) AND $data->numero !== '') OR (is_array($data->numero) AND (!empty($data->numero)) )) )
        {

            $filters[] = new TFilter('numero', 'like', "%{$data->numero}%");// create the filter 
        }
        if (isset($data->notafiscal_tpamb_id) AND ( (is_scalar($data->notafiscal_tpamb_id) AND $data->notafiscal_tpamb_id !== '') OR (is_array($data->notafiscal_tpamb_id) AND (!empty($data->notafiscal_tpamb_id)) )) )
        {

            $filters[] = new TFilter('notafiscal_tpamb_id', '=', $data->notafiscal_tpamb_id);// create the filter 
        }
        if (isset($data->data_emissao) AND ( (is_scalar($data->data_emissao) AND $data->data_emissao !== '') OR (is_array($data->data_emissao) AND (!empty($data->data_emissao)) )) )
        {

            $filters[] = new TFilter('data_emissao', '>=', $data->data_emissao);// create the filter 
        }
        if (isset($data->data_emissao_final) AND ( (is_scalar($data->data_emissao_final) AND $data->data_emissao_final !== '') OR (is_array($data->data_emissao_final) AND (!empty($data->data_emissao_final)) )) )
        {

            $filters[] = new TFilter('data_emissao', '<=', $data->data_emissao_final);// create the filter 
        }
        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }
        if (isset($data->notafiscal_mod_id) AND ( (is_scalar($data->notafiscal_mod_id) AND $data->notafiscal_mod_id !== '') OR (is_array($data->notafiscal_mod_id) AND (!empty($data->notafiscal_mod_id)) )) )
        {

            $filters[] = new TFilter('notafiscal_mod_id', '=', $data->notafiscal_mod_id);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            $param = [];
            // creates a repository for Notafiscal
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            $filterVar = TSession::getValue("userunitid");
            $criteria->add(new TFilter('system_unit_id', '=', $filterVar));

            $param['order'] = 'data_emissao';
            $param['direction'] = 'asc';

            $criteria->setProperties($param);

            if ($filters)
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            $criteria->add(new TFilter('cliente_id', 'in', "(SELECT id FROM cliente WHERE deletado_em is NULL )")); 

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            if ($objects)
            {
                $widths = array(200,200,200,200,200,200);
                $reportExtension = 'pdf';
                switch ($format)
                {
                    case 'html':
                        $tr = new TTableWriterHTML($widths);
                        $reportExtension = 'html';
                        break;
                    case 'xls':
                        $tr = new TTableWriterXLS($widths);
                        $reportExtension = 'xls';
                        break;
                    case 'pdf':
                        $tr = new TTableWriterPDF($widths, 'L', 'A4');
                        $reportExtension = 'pdf';
                        break;
                    case 'htmlPdf':
                        $reportExtension = 'pdf';
                        $tr = new BTableWriterHtmlPDF($widths, 'L', 'A4');
                        break;
                    case 'rtf':
                        if (!class_exists('PHPRtfLite_Autoloader'))
                        {
                            PHPRtfLite::registerAutoloader();
                        }
                        $reportExtension = 'rtf';
                        $tr = new TTableWriterRTF($widths, 'L', 'A4');
                        break;
                }

                if (!empty($tr))
                {
                    // create the document styles
                    $tr->addStyle('title', 'Helvetica', '10', 'B',   '#ffffff', '#96c0e6');
                    $tr->addStyle('datap', 'Arial', '10', '',    '#000000', '#f0f0f0');
                    $tr->addStyle('datai', 'Arial', '10', '',    '#000000', '#ffffff');
                    $tr->addStyle('header', 'Helvetica', '16', 'B',   '#4589c5', '#494D90');
                    $tr->addStyle('footer', 'Helvetica', '10', 'B',  '#4589c5', '#B1B1EA');
                    $tr->addStyle('break', 'Helvetica', '10', 'B',  '#ffffff', '#8da4bc');
                    $tr->addStyle('total', 'Helvetica', '10', 'I',  '#000000', '#b2d1dc');
                    $tr->addStyle('breakTotal', 'Helvetica', '10', 'I',  '#000000', '#b3d5f3');

                    // add titles row
                    $tr->addRow();
                    $tr->addCell("Número", 'left', 'title');
                    $tr->addCell("Data Emissão", 'left', 'title');
                    $tr->addCell("Destinatário", 'left', 'title');
                    $tr->addCell("CPF/CNPJ", 'left', 'title');
                    $tr->addCell("Chave", 'left', 'title');
                    $tr->addCell("Valor", 'left', 'title');

                    $grandTotal = [];
                    $breakTotal = [];
                    $breakValue = null;
                    $firstRow = true;

                    // controls the background filling
                    $colour = false;                
                    foreach ($objects as $object)
                    {
                        $style = $colour ? 'datap' : 'datai';

                        $grandTotal['valor_total_nota'][] = $object->valor_total_nota;
                        $breakTotal['valor_total_nota'][] = $object->valor_total_nota;

                        $firstRow = false;

                        $object->data_emissao = call_user_func(function($value, $object, $row)
                        {
                            if(!empty(trim($value)))
                            {
                                try
                                {
                                    $date = new DateTime($value);
                                    return $date->format('d/m/Y H:i');
                                }
                                catch (Exception $e)
                                {
                                    return $value;
                                }
                            }
                        }, $object->data_emissao, $object, null);

                        $object->valor_total_nota = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->valor_total_nota, $object, null);

                        $tr->addRow();

                        $tr->addCell($object->numero, 'left', $style);
                        $tr->addCell($object->data_emissao, 'left', $style);
                        $tr->addCell($object->cliente_nome, 'left', $style);
                        $tr->addCell($object->cliente_cpfcnpj, 'left', $style);
                        $tr->addCell($object->chave, 'left', $style);
                        $tr->addCell($object->valor_total_nota, 'right', $style);

                        $colour = !$colour;

                    }

                    $tr->addRow();

                    $grandTotal_valor_total_nota = array_sum($grandTotal['valor_total_nota']);

                    $grandTotal_valor_total_nota = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $grandTotal_valor_total_nota); 

                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell($grandTotal_valor_total_nota, 'right', 'total');

                    $file = 'report_'.uniqid().".{$reportExtension}";
                    // stores the file
                    if (!file_exists("app/output/{$file}") || is_writable("app/output/{$file}"))
                    {
                        $tr->save("app/output/{$file}");
                    }
                    else
                    {
                        throw new Exception(_t('Permission denied') . ': ' . "app/output/{$file}");
                    }

                    parent::openFile("app/output/{$file}");

                    // shows the success message
                    new TMessage('info', _t('Report generated. Please, enable popups'));
                }
            }
            else
            {
                new TMessage('error', _t('No records found'));
            }

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }


}

