<?php

class NotasFiscaisCFOP extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = '';
    private static $activeRecord = '';
    private static $primaryKey = '';
    private static $formName = 'form_NotasFiscaisCFOP';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Notas fiscais por CFOP");


        $data_emissao_inicial = new TDate('data_emissao_inicial');
        $data_emissao_final = new TDate('data_emissao_final');
        $tipo_nota = new TRadioGroup('tipo_nota');


        $tipo_nota->addItems(["1"=>"(55) NF-e","2"=>"(65) NFC-e"]);
        $tipo_nota->setLayout('horizontal');
        $tipo_nota->setUseButton();
        $data_emissao_final->setMask('dd/mm/yyyy');
        $data_emissao_inicial->setMask('dd/mm/yyyy');

        $data_emissao_final->setDatabaseMask('yyyy-mm-dd');
        $data_emissao_inicial->setDatabaseMask('yyyy-mm-dd');

        $tipo_nota->setSize(80);
        $data_emissao_final->setSize('100%');
        $data_emissao_inicial->setSize('100%');

        $tipo_nota->setValue('1');
        $data_emissao_final->setValue(date('d/m/Y'));
        $data_emissao_inicial->setValue(date('d/m/Y'));


        $row1 = $this->form->addFields([new TLabel("Data emissão inicial:", null, '14px', null, '100%'),$data_emissao_inicial],[new TLabel("Data emissão final:", null, '14px', null, '100%'),$data_emissao_final]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Tipo Nota:", null, '14px', null, '100%'),$tipo_nota],[]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        // create the form actions
        $btn_ongerarplanilha = $this->form->addAction("Gerar planilha", new TAction([$this, 'onGerarPlanilha']), 'fas:rocket #ffffff');
        $this->btn_ongerarplanilha = $btn_ongerarplanilha;
        $btn_ongerarplanilha->addStyleClass('btn-primary'); 

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Fiscal","Notas fiscais por CFOP"]));
        }
        $container->add($this->form);

        parent::add($container);

    }

    public function onGerarPlanilha($param = null) 
    {
        try
        {
            TTransaction::open('kraftsoft');

            $arquivo = 'app/output/'.uniqid().'.xls';

            $largura_colunas = [100,400,100];

            $planilha = new TTableWriterXLS($largura_colunas);
            // $planilha->addStyle('title', 'Helvetica', '10', 'B', '#ffffff', '#000000');
            // $planilha->addStyle('data',  'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

            $planilha->addRow();
            $planilha->addCell('CFOP','center', 'title');
            $planilha->addCell('VALOR TOTAL NOTA', 'center', 'title');
            $planilha->addCell('VALOR TOTAL PRODUTOS', 'center', 'title');
            $planilha->addCell('BASE ICMS', 'center', 'title');
            $planilha->addCell('VALOR ICMS', 'center', 'title');
            $planilha->addCell('VALOR IPI', 'center', 'title');
            $planilha->addCell('BASE IPI', 'center', 'title');

            // $produtos = Produto::all();
            $conn = TTransaction::get();

            $vssql = 'select i.cfop,
                        sum(n.valor_total_nota) as valor_total_nota,
                        sum(n.valor_total_produtos) as valor_total_produtos,
                        sum(n.icms_basecalc) as base_icms,
                        sum(n.icms_valor) as valor_icms,
                        sum(n.ipi_valor) as valor_ipi,
                        sum(i.ipi_valor_base) as base_ipi
                        from notafiscal n
                inner join notafiscal_itens i on n.id = i.notafiscal_id '.
                'where n.data_emissao between "'.date(implode("-", array_reverse(explode("/", $param['data_emissao_inicial'])))).'"'.
                ' and "'.date(implode("-", array_reverse(explode("/", $param['data_emissao_final'])))).'"'.                                
                ' group by i.cfop';

            // echo 'sql '.$vssql;    

            $listacfops = $conn->query($vssql);            

            foreach ($listacfops as $cfop)
            {
                $planilha->addRow();        
                $planilha->addCell($cfop['cfop'],'center', 'data');
                $planilha->addCell(number_format($cfop['valor_total_nota'], 2, ',', '.'), 'right', 'data');
                $planilha->addCell(number_format($cfop['valor_total_produtos'], 2, ',', '.'), 'right', 'data');
                $planilha->addCell(number_format($cfop['base_icms'], 2, ',', '.'), 'right', 'data');
                $planilha->addCell(number_format($cfop['valor_icms'], 2, ',', '.'), 'right', 'data');
                $planilha->addCell(number_format($cfop['valor_ipi'], 2, ',', '.'), 'right', 'data');
                $planilha->addCell(number_format($cfop['base_ipi'], 2, ',', '.'), 'right', 'data');
            }

            $planilha->save($arquivo);

            TPage::openFile($arquivo);

            TTransaction::close();
        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    }

    public function onShow($param = null)
    {               

    } 

}

