<?php

class OrdemServicoList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'OrdemServico';
    private static $primaryKey = 'id';
    private static $formName = 'form_OrdemServicoList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de ordem de serviço");
        $this->limit = 20;

        $id = new TEntry('id');
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{nome}','nome asc'  );
        $vendedor_id = new TDBCombo('vendedor_id', 'kraftsoft', 'Vendedor', 'id', '{nome}','nome asc'  );
        $data_emissao_inicial = new TDate('data_emissao_inicial');
        $data_emissao_final = new TDate('data_emissao_final');
        $system_unit_id = new TDBCombo('system_unit_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $ordem_servico_situacao_id = new TDBCombo('ordem_servico_situacao_id', 'kraftsoft', 'OrdemServicoSituacao', 'id', '{descricao}','descricao asc'  );


        $data_emissao_final->setMask('dd/mm/yyyy');
        $data_emissao_final->setDatabaseMask('yyyy-mm-dd');
        $cliente_id->enableSearch();
        $vendedor_id->enableSearch();
        $system_unit_id->enableSearch();
        $ordem_servico_situacao_id->enableSearch();

        $id->setSize('100%');
        $cliente_id->setSize('100%');
        $vendedor_id->setSize('100%');
        $system_unit_id->setSize('100%');
        $data_emissao_final->setSize('100%');
        $data_emissao_inicial->setSize('100%');
        $ordem_servico_situacao_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Cliente:", null, '14px', null, '100%'),$cliente_id],[new TLabel("Vendedor:", null, '14px', null, '100%'),$vendedor_id]);
        $row1->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row2 = $this->form->addFields([new TLabel("Data Emissão Inicial:", null, '14px', null, '100%'),$data_emissao_inicial],[new TLabel("Data Emissão Final:", null, '14px', null, '100%'),$data_emissao_final],[new TLabel("Filial de saída:", null, '14px', null, '100%'),$system_unit_id]);
        $row2->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row3 = $this->form->addFields([new TLabel("Situação:", null, '14px', null, '100%'),$ordem_servico_situacao_id]);
        $row3->layout = [' col-sm-4'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Cadastrar", new TAction(['OrdemServicoForm', 'onShow']), 'fas:plus #69aa46');
        $this->btn_onshow = $btn_onshow;

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_numero_os = new TDataGridColumn('numero_os', "Número O.S.", 'left');
        $column_data_emissao_transformed = new TDataGridColumn('data_emissao', "Data Emissão", 'left');
        $column_cliente_nome = new TDataGridColumn('cliente->nome', "Cliente", 'left');
        $column_prazo_entrega_transformed = new TDataGridColumn('prazo_entrega', "Prazo de Entrega", 'left');
        $column_valor_transformed = new TDataGridColumn('valor', "Valor", 'left');
        $column_ordem_servico_situacao_descricao_transformed = new TDataGridColumn('ordem_servico_situacao->descricao', "Situação", 'center');

        $column_data_emissao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_prazo_entrega_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_valor_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_ordem_servico_situacao_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            //code here
            $label = $object->ordem_servico_situacao->descricao;
            $cor = $object->ordem_servico_situacao->cor;

            return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_numero_os);
        $this->datagrid->addColumn($column_data_emissao_transformed);
        $this->datagrid->addColumn($column_cliente_nome);
        $this->datagrid->addColumn($column_prazo_entrega_transformed);
        $this->datagrid->addColumn($column_valor_transformed);
        $this->datagrid->addColumn($column_ordem_servico_situacao_descricao_transformed);

        $action_group = new TDataGridActionGroup("", 'fas:cog');
        $action_group->addHeader('');

        $action_onShow = new TDataGridAction(array('OrdemServicoFormView', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Abrir");
        $action_onShow->setImage('fas:book-open #00BCD4');
        $action_onShow->setField(self::$primaryKey);

        $action_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_onShow);

        $action_onEdit = new TDataGridAction(array('OrdemServicoForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('OrdemServicoList::onMostrarEditar');

        $action_group->addAction($action_onEdit);

        $action_onAceitarOrcamento = new TDataGridAction(array('OrdemServicoList', 'onAceitarOrcamento'));
        $action_onAceitarOrcamento->setUseButton(TRUE);
        $action_onAceitarOrcamento->setButtonClass('btn btn-default');
        $action_onAceitarOrcamento->setLabel("Aceitar Orçamento");
        $action_onAceitarOrcamento->setImage('fas:check #4CAF50');
        $action_onAceitarOrcamento->setField(self::$primaryKey);
        $action_onAceitarOrcamento->setDisplayCondition('OrdemServicoList::onMostrarAceitarOrcamento');

        $action_group->addAction($action_onAceitarOrcamento);

        $action_onRecusarOrcamento = new TDataGridAction(array('OrdemServicoList', 'onRecusarOrcamento'));
        $action_onRecusarOrcamento->setUseButton(TRUE);
        $action_onRecusarOrcamento->setButtonClass('btn btn-default');
        $action_onRecusarOrcamento->setLabel("Recusar Orçamento");
        $action_onRecusarOrcamento->setImage('fas:thumbs-down #F44336');
        $action_onRecusarOrcamento->setField(self::$primaryKey);
        $action_onRecusarOrcamento->setDisplayCondition('OrdemServicoList::onMostrarRecusarOrcamento');

        $action_group->addAction($action_onRecusarOrcamento);

        $action_onGerarVenda = new TDataGridAction(array('OrdemServicoList', 'onGerarVenda'));
        $action_onGerarVenda->setUseButton(TRUE);
        $action_onGerarVenda->setButtonClass('btn btn-default');
        $action_onGerarVenda->setLabel("Gerar Venda");
        $action_onGerarVenda->setImage('fas:dollar-sign #CDDC39');
        $action_onGerarVenda->setField(self::$primaryKey);
        $action_onGerarVenda->setDisplayCondition('OrdemServicoList::onMostrarGerarVenda');

        $action_group->addAction($action_onGerarVenda);

        $action_onGenerate = new TDataGridAction(array('OrdemServicoDocument', 'onGenerate'));
        $action_onGenerate->setUseButton(TRUE);
        $action_onGenerate->setButtonClass('btn btn-default');
        $action_onGenerate->setLabel("Imprimir");
        $action_onGenerate->setImage('fas:print #FF5722');
        $action_onGenerate->setField(self::$primaryKey);

        $action_group->addAction($action_onGenerate);

        $action_OrdemServicoPropostaDocument_onGenerate = new TDataGridAction(array('OrdemServicoPropostaDocument', 'onGenerate'));
        $action_OrdemServicoPropostaDocument_onGenerate->setUseButton(TRUE);
        $action_OrdemServicoPropostaDocument_onGenerate->setButtonClass('btn btn-default');
        $action_OrdemServicoPropostaDocument_onGenerate->setLabel("Imprimir Proposta");
        $action_OrdemServicoPropostaDocument_onGenerate->setImage('fas:print #795548');
        $action_OrdemServicoPropostaDocument_onGenerate->setField(self::$primaryKey);

        $action_group->addAction($action_OrdemServicoPropostaDocument_onGenerate);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['OrdemServicoList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['OrdemServicoList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['OrdemServicoList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['OrdemServicoList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_right_actions->add($dropdown_button_exportar);

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Ordem de Serviço","Ordem de serviço"]));
        }
        $container->add($this->form);
        $container->add($panel);

        parent::add($container);

    }

    public static function onMostrarEditar($object)
    {
        try 
        {
            if($object->ordem_servico_situacao_id==OrdemServicoSituacao::OrcamentoAprovado
            or $object->ordem_servico_situacao_id==OrdemServicoSituacao::OrcamentoRecusado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onAceitarOrcamento($param = null) 
    {
        try 
        {
            //code here
            new TQuestion("Deseja gerar a Ordem de Serviço?", new TAction([__CLASS__, 'onAceitarOrcamentoYes'], $param), new TAction([__CLASS__, 'onAceitarOrcamentoNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarAceitarOrcamento($object)
    {
        try 
        {
            if($object->ordem_servico_situacao_id==OrdemServicoSituacao::OrcamentoAprovado
            or $object->ordem_servico_situacao_id==OrdemServicoSituacao::OrcamentoRecusado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onRecusarOrcamento($param = null) 
    {
        try 
        {
            //code here
            $pageParam = ['id' => $param['key']];
            TApplication::loadPage('RecusarOSForm', 'onShow', $pageParam);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarRecusarOrcamento($object)
    {
        try 
        {
            if($object->ordem_servico_situacao_id==OrdemServicoSituacao::OrcamentoAprovado
            or $object->ordem_servico_situacao_id==OrdemServicoSituacao::OrcamentoRecusado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onGerarVenda($param = null) 
    {
        try 
        {
            //code here

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarGerarVenda($object)
    {
        try 
        {
            if($object->ordem_servico_situacao_id==OrdemServicoSituacao::OrcamentoRecusado)
            {
                return false;
            }
            if($object->ordem_servico_situacao_id==OrdemServicoSituacao::OrcamentoAberto)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('iframe');
                $object->src  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }

        if (isset($data->vendedor_id) AND ( (is_scalar($data->vendedor_id) AND $data->vendedor_id !== '') OR (is_array($data->vendedor_id) AND (!empty($data->vendedor_id)) )) )
        {

            $filters[] = new TFilter('vendedor_id', '=', $data->vendedor_id);// create the filter 
        }

        if (isset($data->data_emissao_inicial) AND ( (is_scalar($data->data_emissao_inicial) AND $data->data_emissao_inicial !== '') OR (is_array($data->data_emissao_inicial) AND (!empty($data->data_emissao_inicial)) )) )
        {

            $filters[] = new TFilter('data_emissao', '>=', $data->data_emissao_inicial);// create the filter 
        }

        if (isset($data->data_emissao_final) AND ( (is_scalar($data->data_emissao_final) AND $data->data_emissao_final !== '') OR (is_array($data->data_emissao_final) AND (!empty($data->data_emissao_final)) )) )
        {

            $filters[] = new TFilter('data_emissao', '<=', $data->data_emissao_final);// create the filter 
        }

        if (isset($data->system_unit_id) AND ( (is_scalar($data->system_unit_id) AND $data->system_unit_id !== '') OR (is_array($data->system_unit_id) AND (!empty($data->system_unit_id)) )) )
        {

            $filters[] = new TFilter('system_unit_id', '=', $data->system_unit_id);// create the filter 
        }

        if (isset($data->ordem_servico_situacao_id) AND ( (is_scalar($data->ordem_servico_situacao_id) AND $data->ordem_servico_situacao_id !== '') OR (is_array($data->ordem_servico_situacao_id) AND (!empty($data->ordem_servico_situacao_id)) )) )
        {

            $filters[] = new TFilter('ordem_servico_situacao_id', '=', $data->ordem_servico_situacao_id);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for OrdemServico
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function onAceitarOrcamentoYes($param = null) 
    {
        try 
        {
            //code here
            TTransaction::open(self::$database);

            $ordem_servico = OrdemServico::find($param['key']);

            if($ordem_servico)
            {
                $ordem_servico->data_abertura_os = date('Y-m-d H:i:s');
                $ordem_servico->ordem_servico_situacao_id = OrdemServicoSituacao::OrcamentoAprovado;
                $ordem_servico->store();

                OrdemServicoHistorico::LancarHistorico(OrdemServicoSituacao::OrcamentoAprovado, $ordem_servico->id, 'Mudança de Situação de O.S: Orcamento Aprovado');
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onShow', ['key' => $ordem_servico->id]);

            TToas::show('success', "O.S. Gerada com sucesso!", 'topRight', 'far:check-circle');

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onAceitarOrcamentoNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

}

