<?php

class OrdemProducaoList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'OrdemProducao';
    private static $primaryKey = 'id';
    private static $formName = 'form_OrdemProducaoList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de ordens de produção");
        $this->limit = 20;

        $data_producao_inicial = new TDate('data_producao_inicial');
        $data_producao_final = new TDate('data_producao_final');
        $ordem_producao_situacao_id = new TDBCombo('ordem_producao_situacao_id', 'kraftsoft', 'OrdemProducaoSituacao', 'id', '{descricao}','descricao asc'  );
        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc'  );


        $data_producao_final->setMask('dd/mm/yyyy');
        $data_producao_inicial->setMask('dd/mm/yyyy');

        $data_producao_final->setDatabaseMask('yyyy-mm-dd');
        $data_producao_inicial->setDatabaseMask('yyyy-mm-dd');

        $produto_id->enableSearch();
        $ordem_producao_situacao_id->enableSearch();

        $produto_id->setSize('100%');
        $data_producao_final->setSize('100%');
        $data_producao_inicial->setSize('100%');
        $ordem_producao_situacao_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Data Produção Inicial:", null, '14px', null, '100%'),$data_producao_inicial],[new TLabel("Data Produção Final:", null, '14px', null, '100%'),$data_producao_final],[new TLabel("Situação:", null, '14px', null),$ordem_producao_situacao_id]);
        $row1->layout = [' col-sm-3',' col-sm-3',' col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Produto:", null, '14px', null),$produto_id]);
        $row2->layout = [' col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Cadastrar", new TAction(['OrdemProducaoForm', 'onShow']), 'fas:plus #69aa46');
        $this->btn_onshow = $btn_onshow;

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_data_producao_transformed = new TDataGridColumn('data_producao', "Data Produção", 'left');
        $column_produto_nome = new TDataGridColumn('produto->nome', "Produto", 'left');
        $column_quantidade_transformed = new TDataGridColumn('quantidade', "Quantidade", 'left');
        $column_ordem_producao_situacao_descricao_transformed = new TDataGridColumn('ordem_producao_situacao->descricao', "Situação", 'left');

        $column_data_producao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_quantidade_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {

                    if(!$value)
                    {
                        $value = 0;
                    }

                    if(is_numeric($value))
                    {
                        return number_format($value, 2, ",", ".");
                    }
                    else
                    {
                        return $value;
                    }

        });

        $column_ordem_producao_situacao_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
                $label = $object->ordem_producao_situacao->descricao;
                $cor = $object->ordem_producao_situacao->cor;

                return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

                // $div = new TElement('div');
                // $div->style="text-shadow:none; font-size:14px; font-weight:bold; color:#FFFFFF; width:160px; text-align:center;background-color: $cor;padding:10px";
                // $div->add($label);

                // return $div;    //code here
        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_data_producao_transformed);
        $this->datagrid->addColumn($column_produto_nome);
        $this->datagrid->addColumn($column_quantidade_transformed);
        $this->datagrid->addColumn($column_ordem_producao_situacao_descricao_transformed);

        $action_group = new TDataGridActionGroup("", 'fas:cog');
        $action_group->addHeader('');

        $action_onShow = new TDataGridAction(array('OrdemProducaoFormView2', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Abrir");
        $action_onShow->setImage('fas:book-open #03A9F4');
        $action_onShow->setField(self::$primaryKey);

        $action_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_onShow);

        $action_onEdit = new TDataGridAction(array('OrdemProducaoForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('OrdemProducaoList::onMostrarEditar');

        $action_group->addAction($action_onEdit);

        $action_onPendente = new TDataGridAction(array('OrdemProducaoList', 'onPendente'));
        $action_onPendente->setUseButton(TRUE);
        $action_onPendente->setButtonClass('btn btn-default');
        $action_onPendente->setLabel("Pendente");
        $action_onPendente->setImage('fas:clock #FFC107');
        $action_onPendente->setField(self::$primaryKey);
        $action_onPendente->setDisplayCondition('OrdemProducaoList::onMostrarPendente');

        $action_group->addAction($action_onPendente);

        $action_onEmAndamento = new TDataGridAction(array('OrdemProducaoList', 'onEmAndamento'));
        $action_onEmAndamento->setUseButton(TRUE);
        $action_onEmAndamento->setButtonClass('btn btn-default');
        $action_onEmAndamento->setLabel("Em Andamento");
        $action_onEmAndamento->setImage('fas:bacon #03A9F4');
        $action_onEmAndamento->setField(self::$primaryKey);
        $action_onEmAndamento->setDisplayCondition('OrdemProducaoList::onMostrarEmAndamento');

        $action_group->addAction($action_onEmAndamento);

        $action_onFinalizada = new TDataGridAction(array('OrdemProducaoList', 'onFinalizada'));
        $action_onFinalizada->setUseButton(TRUE);
        $action_onFinalizada->setButtonClass('btn btn-default');
        $action_onFinalizada->setLabel("Finalizada");
        $action_onFinalizada->setImage('fas:flag-checkered #009688');
        $action_onFinalizada->setField(self::$primaryKey);
        $action_onFinalizada->setDisplayCondition('OrdemProducaoList::onMostrarFinalizada');

        $action_group->addAction($action_onFinalizada);

        $action_onCancelar = new TDataGridAction(array('OrdemProducaoList', 'onCancelar'));
        $action_onCancelar->setUseButton(TRUE);
        $action_onCancelar->setButtonClass('btn btn-default');
        $action_onCancelar->setLabel("Cancelar");
        $action_onCancelar->setImage('fas:window-close #F44336');
        $action_onCancelar->setField(self::$primaryKey);
        $action_onCancelar->setDisplayCondition('OrdemProducaoList::onMostrarCancelar');

        $action_group->addAction($action_onCancelar);

        $action_onGenerate = new TDataGridAction(array('OrdemProducaoDocument', 'onGenerate'));
        $action_onGenerate->setUseButton(TRUE);
        $action_onGenerate->setButtonClass('btn btn-default');
        $action_onGenerate->setLabel("Imprimir");
        $action_onGenerate->setImage('fas:print #FF5722');
        $action_onGenerate->setField(self::$primaryKey);

        $action_group->addAction($action_onGenerate);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['OrdemProducaoList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['OrdemProducaoList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['OrdemProducaoList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['OrdemProducaoList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_right_actions->add($dropdown_button_exportar);

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Ordens de Produção","Ordens de produção"]));
        }
        $container->add($this->form);
        $container->add($panel);

        parent::add($container);

    }

    public static function onMostrarEditar($object)
    {
        try 
        {
            if($object->ordem_producao_situacao_id==OrdemProducaoSituacao::Finalizada
            or $object->ordem_producao_situacao_id==OrdemProducaoSituacao::Cancelada)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onPendente($param = null) 
    {
        try 
        {
            new TQuestion("Confirma ordem de produção pendente?", new TAction([__CLASS__, 'onPendenteYes'], $param), new TAction([__CLASS__, 'onPendenteNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarPendente($object)
    {
        try 
        {
            if($object->ordem_producao_situacao_id==OrdemProducaoSituacao::Finalizada
            or $object->ordem_producao_situacao_id==OrdemProducaoSituacao::Cancelada)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onEmAndamento($param = null) 
    {
        try 
        {
            new TQuestion("Confirma orem de produção em andamento?", new TAction([__CLASS__, 'onEmAndamentoYes'], $param), new TAction([__CLASS__, 'onEmAndamentoNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarEmAndamento($object)
    {
        try 
        {
            if($object->ordem_producao_situacao_id==OrdemProducaoSituacao::Finalizada
            or $object->ordem_producao_situacao_id==OrdemProducaoSituacao::Cancelada)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onFinalizada($param = null) 
    {
        try 
        {
            new TQuestion("Deseja finalizar esta ordem de produção?", new TAction([__CLASS__, 'onFinalizadaYes'], $param), new TAction([__CLASS__, 'onFinalizadaNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarFinalizada($object)
    {
        try 
        {
            if($object->ordem_producao_situacao_id==OrdemProducaoSituacao::Finalizada
            or $object->ordem_producao_situacao_id==OrdemProducaoSituacao::Cancelada)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onCancelar($param = null) 
    {
        try 
        {
            new TQuestion("Deseja cancelar esta ordem de produção?", new TAction([__CLASS__, 'onCancelarYes'], $param), new TAction([__CLASS__, 'onCancelarNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarCancelar($object)
    {
        try 
        {
            if($object->ordem_producao_situacao_id==OrdemProducaoSituacao::Cancelada)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('iframe');
                $object->src  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->data_producao_inicial) AND ( (is_scalar($data->data_producao_inicial) AND $data->data_producao_inicial !== '') OR (is_array($data->data_producao_inicial) AND (!empty($data->data_producao_inicial)) )) )
        {

            $filters[] = new TFilter('data_producao', '>=', $data->data_producao_inicial);// create the filter 
        }

        if (isset($data->data_producao_final) AND ( (is_scalar($data->data_producao_final) AND $data->data_producao_final !== '') OR (is_array($data->data_producao_final) AND (!empty($data->data_producao_final)) )) )
        {

            $filters[] = new TFilter('data_producao', '>=', $data->data_producao_final);// create the filter 
        }

        if (isset($data->ordem_producao_situacao_id) AND ( (is_scalar($data->ordem_producao_situacao_id) AND $data->ordem_producao_situacao_id !== '') OR (is_array($data->ordem_producao_situacao_id) AND (!empty($data->ordem_producao_situacao_id)) )) )
        {

            $filters[] = new TFilter('ordem_producao_situacao_id', '=', $data->ordem_producao_situacao_id);// create the filter 
        }

        if (isset($data->produto_id) AND ( (is_scalar($data->produto_id) AND $data->produto_id !== '') OR (is_array($data->produto_id) AND (!empty($data->produto_id)) )) )
        {

            $filters[] = new TFilter('produto_id', '=', $data->produto_id);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for OrdemProducao
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public  function onPendenteYes($param = null) 
    {
        try 
        {
            //code here
            TTransaction::open(self::$database);

            $op = OrdemProducao::find($param['key']);     

            if($op) 
            {
                $op->ordem_producao_situacao_id=OrdemProducaoSituacao::Pendente;
                $op->store();

                OrdemProducaoSituacaoHistorico::LancarHistorico(OrdemProducaoSituacao::Pendente,
                                                                $op->id,
                                                                'Mudança de Situação de O.P.: Pendente');

            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onShow', ['key' => $op->id]);

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle'); 
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onPendenteNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onEmAndamentoYes($param = null) 
    {
        try 
        {
            //code here
            TTransaction::open(self::$database);

            $op = OrdemProducao::find($param['key']);     

            if($op) 
            {
                $op->ordem_producao_situacao_id=OrdemProducaoSituacao::EmAndamento;
                $op->store();

                OrdemProducaoSituacaoHistorico::LancarHistorico(OrdemProducaoSituacao::EmAndamento,
                                                                $op->id,
                                                                'Mudança de Situação de O.P.: Em Andamento');

            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onShow', ['key' => $op->id]);

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');  
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onEmAndamentoNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onFinalizadaYes($param = null) 
    {
        try 
        {
            //code here
             TTransaction::open(self::$database);

            $op = OrdemProducao::find($param['key']);     

            if($op) 
            {
                //movimentando o estoque...
                OrdemProducao::LancaEstoqueOrdemProducao($op);

                $op->ordem_producao_situacao_id=OrdemProducaoSituacao::Finalizada;
                $op->store();

                OrdemProducaoSituacaoHistorico::LancarHistorico(OrdemProducaoSituacao::Finalizada,
                                                                $op->id,
                                                                'Mudança de Situação de O.P.: Finalizada');

            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onShow', ['key' => $op->id]);

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle'); 
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onFinalizadaNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onCancelarYes($param = null) 
    {
        try 
        {
            //code here
            TTransaction::open(self::$database);

            $op = OrdemProducao::find($param['key']);

            if($op)
            {
                $op->ordem_producao_situacao_id=OrdemProducaoSituacao::Cancelada;
                $op->store();

                OrdemProducaoSituacaoHistorico::LancarHistorico(OrdemProducaoSituacao::Cancelada, $op->id, 'Mudança de Situação de O.P: Cancelada');
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onShow', ['key' => $op->id]);
            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public  function onCancelarNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

}

