<?php

class ProdutoBarCode extends TPage
{
    private static $database = 'kraftsoft';
    private static $activeRecord = 'ProdutoFilial';
    private static $primaryKey = 'id';
    private static $formName = 'formBarcode_ProdutoFilial';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Etiqueta de código de barra de produto - Modelo 1");

        $produto_nome = new TDBCombo('produto_nome', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc'  );
        $estoque_deposito_padrao_id = new TDBCombo('estoque_deposito_padrao_id', 'kraftsoft', 'EstoqueDeposito', 'id', '{system_unit->name} - {descricao}','descricao asc'  );
        $codigo_balanca = new TEntry('codigo_balanca');

        $produto_nome->enableSearch();
        $estoque_deposito_padrao_id->enableSearch();

        $produto_nome->setSize('100%');
        $codigo_balanca->setSize('100%');
        $estoque_deposito_padrao_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Produto:", null, '14px', null, '100%'),$produto_nome]);
        $row1->layout = [' col-sm-12'];

        $row2 = $this->form->addFields([new TLabel("Depósito Padrão:", null, '14px', null, '100%'),$estoque_deposito_padrao_id],[new TLabel("Código Balança:", null, '14px', null, '100%'),$codigo_balanca]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongenerate = $this->form->addAction("Gerar", new TAction([$this, 'onGenerate']), 'fas:cog #ffffff');
        $this->btn_ongenerate = $btn_ongenerate;
        $btn_ongenerate->addStyleClass('btn-primary'); 

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->add(TBreadCrumb::create(["Produto","Etiqueta de código de barra de produto - Modelo 1"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerate($param)
    {
        try 
        {
            TTransaction::open(self::$database);

            $data = $this->form->getData();
            $criteria = new TCriteria();

            if (isset($data->produto_nome) AND ( (is_scalar($data->produto_nome) AND $data->produto_nome !== '') OR (is_array($data->produto_nome) AND (!empty($data->produto_nome)) )) ) 
            {

                $criteria->add(new TFilter('produto_id', '=', $data->produto_nome));
            }
            if (isset($data->estoque_deposito_padrao_id) AND ( (is_scalar($data->estoque_deposito_padrao_id) AND $data->estoque_deposito_padrao_id !== '') OR (is_array($data->estoque_deposito_padrao_id) AND (!empty($data->estoque_deposito_padrao_id)) )) ) 
            {

                $criteria->add(new TFilter('estoque_deposito_padrao_id', '=', $data->estoque_deposito_padrao_id));
            }
            if (isset($data->codigo_balanca) AND ( (is_scalar($data->codigo_balanca) AND $data->codigo_balanca !== '') OR (is_array($data->codigo_balanca) AND (!empty($data->codigo_balanca)) )) ) 
            {

                $criteria->add(new TFilter('codigo_balanca', 'like', "%{$data->codigo_balanca}%"));
            }

            TSession::setValue(__CLASS__.'_filter_data', $data);

            $properties = [];

            $properties['leftMargin']    = 10; // Left margin
            $properties['topMargin']     = 10; // Top margin
            $properties['labelWidth']    = 60; // Label width in mm
            $properties['labelHeight']   = 27; // Label height in mm
            $properties['spaceBetween']  = 4;  // Space between labels
            $properties['rowsPerPage']   = 10;  // Label rows per page
            $properties['colsPerPage']   = 3;  // Label cols per page
            $properties['fontSize']      = 8; // Text font size
            $properties['barcodeHeight'] = 12; // Barcode Height
            $properties['imageMargin']   = 0;
            $properties['barcodeMethod'] = 'EAN13';

            $label  = "<strong>
{produto->nome_curto}
#barcode#
                       R$ {preco_venda}
</strong>";

            $bcgen = new AdiantiBarcodeDocumentGenerator('p', 'A4');
            $bcgen->setProperties($properties);
            $bcgen->setLabelTemplate($label);

            $class = self::$activeRecord;

            $bcgen = new AdiantiBarcodeDocumentGenerator('p', 'A3');

            $objects = $class::getObjects($criteria);

            if ($objects)
            {
                foreach ($objects as $object)
                {

                    $bcgen->addObject($object);
                }

                $filename = 'tmp/barcode_'.uniqid().'.pdf';

                $bcgen->setBarcodeContent('{produto->codigo_EAN}');
                $bcgen->generate();
                $bcgen->save($filename);

                parent::openFile($filename);
                new TMessage('info', _t('Barcodes successfully generated'));
            }
            else
            {
                new TMessage('info', _t('No records found'));   
            }

            TTransaction::close();

            $this->form->setData($data);

        } 
        catch (Exception $e) 
        {
            $this->form->setData($data);

            // shows the exception error message
            new TMessage('error', $e->getMessage());

            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }


}

