<?php

class ProdutoCardList extends TPage
{
    private $form; // form
    private $cardView; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Produto';
    private static $primaryKey = 'id';
    private static $formName = 'form_ProdutoCardList';
    private $showMethods = ['onReload', 'onSearch'];

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();
        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Produtos (Consulta por Cards)");

        $criteria_produto_grupo_id = new TCriteria();

        $filterVar = TipoRecurso::Produto;
        $criteria_produto_grupo_id->add(new TFilter('tipo_recurso_id', '=', $filterVar)); 

        $id = new TEntry('id');
        $nome = new TEntry('nome');
        $referencia = new TEntry('referencia');
        $codigo_EAN = new TEntry('codigo_EAN');
        $produto_grupo_id = new TDBCombo('produto_grupo_id', 'kraftsoft', 'ProdutoGrupo', 'id', '{descricao}','descricao asc' , $criteria_produto_grupo_id );
        $produtogrupo_subgrupo_id = new TCombo('produtogrupo_subgrupo_id');
        $filial_id = new TDBCombo('filial_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );

        $produto_grupo_id->setChangeAction(new TAction([$this,'onChangeproduto_grupo_id']));

        $filial_id->enableSearch();
        $id->setSize('100%');
        $nome->setSize('100%');
        $filial_id->setSize('100%');
        $referencia->setSize('100%');
        $codigo_EAN->setSize('100%');
        $produto_grupo_id->setSize('100%');
        $produtogrupo_subgrupo_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null),$id],[new TLabel("Nome:", null, '14px', null, '100%'),$nome]);
        $row1->layout = [' col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Referência:", null, '14px', null),$referencia],[new TLabel("Código EAN:", null, '14px', null),$codigo_EAN]);
        $row2->layout = [' col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Grupo:", null, '14px', null),$produto_grupo_id],[new TLabel("Sub-Grupo:", null, '14px', null),$produtogrupo_subgrupo_id]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        $row4 = $this->form->addFields([new TLabel("Filial:", null, '14px', null),$filial_id],[]);
        $row4->layout = [' col-sm-6','col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );
        $this->fireEvents( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        $btn_onshow = $this->form->addAction("Cadastrar", new TAction(['ProdutoForm', 'onShow']), 'fas:plus #69aa46');
        $this->btn_onshow = $btn_onshow;

        $this->cardView = new TCardView;

        $this->cardView->setContentHeight(170);
        $this->cardView->setTitleTemplate('#{id} - {nome}');
        $this->cardView->setItemTemplate("<div class=\"media\">
  <img style=\"width: 200px\"; src=\"{imagem}\" class=\"mr-3\">
  <div class=\"media-body\">
<br>
    <h5 class=\"mt-0\">Grupo</h5>
<p>{produto_grupo->descricao} </p>
      </div>
</div> ");

        $this->cardView->setItemDatabase(self::$database);

        $this->filter_criteria = new TCriteria;

        $action_ProdutoForm_onEdit = new TAction(['ProdutoForm', 'onEdit'], ['key'=> '{id}']);

        $this->cardView->addAction($action_ProdutoForm_onEdit, "Editar", 'far:edit #478fca', null, "Editar", false); 

        $action_ProdutoCardList_onDelete = new TAction(['ProdutoCardList', 'onDelete'], ['key'=> '{id}']);

        $this->cardView->addAction($action_ProdutoCardList_onDelete, "Excluir", 'fas:trash-alt #dd5a43', null, "Excluir", false); 

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));

        $panel = new TPanelGroup;
        $panel->add($this->cardView);

        $panel->addFooter($this->pageNavigation);

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->add(TBreadCrumb::create(["Produto","Produtos (Consulta por Cards)"]));
        $container->add($this->form);
        $container->add($panel);

        parent::add($container);

    }

    public static function onChangeproduto_grupo_id($param)
    {
        try
        {

            if (isset($param['produto_grupo_id']) && $param['produto_grupo_id'])
            { 
                $criteria = TCriteria::create(['produto_grupo_id' => $param['produto_grupo_id']]);
                TDBCombo::reloadFromModel(self::$formName, 'produtogrupo_subgrupo_id', 'kraftsoft', 'ProdutogrupoSubgrupo', 'id', '{descricao}', 'descricao asc', $criteria, TRUE); 
            } 
            else 
            { 
                TCombo::clearField(self::$formName, 'produtogrupo_subgrupo_id'); 
            }  

        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    } 

    public function onDelete($param = null) 
    { 
        if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new Produto($key, FALSE); 

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        }
    }

    public function fireEvents( $object )
    {
        $obj = new stdClass;
        if(is_object($object) && get_class($object) == 'stdClass')
        {
            if(isset($object->produto_grupo_id))
            {
                $value = $object->produto_grupo_id;

                $obj->produto_grupo_id = $value;
            }
            if(isset($object->produtogrupo_subgrupo_id))
            {
                $value = $object->produtogrupo_subgrupo_id;

                $obj->produtogrupo_subgrupo_id = $value;
            }
        }
        elseif(is_object($object))
        {
            if(isset($object->produto_grupo_id))
            {
                $value = $object->produto_grupo_id;

                $obj->produto_grupo_id = $value;
            }
            if(isset($object->produtogrupo_subgrupo_id))
            {
                $value = $object->produtogrupo_subgrupo_id;

                $obj->produtogrupo_subgrupo_id = $value;
            }
        }
        TForm::sendData(self::$formName, $obj);
    }  

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        // get the search form data
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->nome) AND ( (is_scalar($data->nome) AND $data->nome !== '') OR (is_array($data->nome) AND (!empty($data->nome)) )) )
        {

            $filters[] = new TFilter('nome', 'like', "%{$data->nome}%");// create the filter 
        }

        if (isset($data->referencia) AND ( (is_scalar($data->referencia) AND $data->referencia !== '') OR (is_array($data->referencia) AND (!empty($data->referencia)) )) )
        {

            $filters[] = new TFilter('referencia', 'like', "%{$data->referencia}%");// create the filter 
        }

        if (isset($data->codigo_EAN) AND ( (is_scalar($data->codigo_EAN) AND $data->codigo_EAN !== '') OR (is_array($data->codigo_EAN) AND (!empty($data->codigo_EAN)) )) )
        {

            $filters[] = new TFilter('codigo_EAN', '=', $data->codigo_EAN);// create the filter 
        }

        if (isset($data->produto_grupo_id) AND ( (is_scalar($data->produto_grupo_id) AND $data->produto_grupo_id !== '') OR (is_array($data->produto_grupo_id) AND (!empty($data->produto_grupo_id)) )) )
        {

            $filters[] = new TFilter('produto_grupo_id', '=', $data->produto_grupo_id);// create the filter 
        }

        if (isset($data->produtogrupo_subgrupo_id) AND ( (is_scalar($data->produtogrupo_subgrupo_id) AND $data->produtogrupo_subgrupo_id !== '') OR (is_array($data->produtogrupo_subgrupo_id) AND (!empty($data->produtogrupo_subgrupo_id)) )) )
        {

            $filters[] = new TFilter('produtogrupo_subgrupo_id', '=', $data->produtogrupo_subgrupo_id);// create the filter 
        }

        if (isset($data->filial_id) AND ( (is_scalar($data->filial_id) AND $data->filial_id !== '') OR (is_array($data->filial_id) AND (!empty($data->filial_id)) )) )
        {

            $filters[] = new TFilter('id', 'in', "(SELECT produto_id FROM produto_filial WHERE system_unit_id = '{$data->filial_id}')");// create the filter 
        }

        $this->fireEvents($data);

        $param = array();
        $param['offset']     = 0;
        $param['first_page'] = 1;

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload($param);
    }

    public function onReload($param = NULL)
    {
        try
        {

            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Produto
            $repository = new TRepository(self::$activeRecord);
            $limit = 20;

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->cardView->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $this->cardView->addItem($object);

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

}

