<?php

class ProdutoFilialHistoricoReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'ProdutoFilialHistorico';
    private static $primaryKey = 'id';
    private static $formName = 'form_ProdutoFilialHistoricoReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Relatório de histórico de preços por produto");

        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc'  );
        $system_unit_id = new TDBCombo('system_unit_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $criado_em_ini = new TDateTime('criado_em_ini');
        $criado_em_fim = new TDateTime('criado_em_fim');
        $criado_por = new TEntry('criado_por');

        $produto_id->enableSearch();
        $system_unit_id->enableSearch();

        $criado_em_ini->setMask('dd/mm/yyyy hh:ii');
        $criado_em_fim->setMask('dd/mm/yyyy hh:ii');

        $criado_em_ini->setDatabaseMask('yyyy-mm-dd hh:ii');
        $criado_em_fim->setDatabaseMask('yyyy-mm-dd hh:ii');

        $produto_id->setSize('100%');
        $criado_por->setSize('100%');
        $criado_em_ini->setSize('100%');
        $criado_em_fim->setSize('100%');
        $system_unit_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Produto:", null, '14px', null, '100%'),$produto_id],[new TLabel("Filial:", null, '14px', null, '100%'),$system_unit_id]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Data Inicial:", null, '14px', null, '100%'),$criado_em_ini],[new TLabel("Data Final:", null, '14px', null, '100%'),$criado_em_fim],[new TLabel("Usuário:", null, '14px', null, '100%'),$criado_por]);
        $row2->layout = [' col-sm-3',' col-sm-3','col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratehtml = $this->form->addAction("Gerar HTML", new TAction([$this, 'onGenerateHtml']), 'far:file-code #ffffff');
        $this->btn_ongeneratehtml = $btn_ongeneratehtml;
        $btn_ongeneratehtml->addStyleClass('btn-primary'); 

        $btn_ongeneratepdf = $this->form->addAction("Gerar PDF", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        $btn_ongeneratexls = $this->form->addAction("Gerar XLS", new TAction([$this, 'onGenerateXls']), 'far:file-excel #00a65a');
        $this->btn_ongeneratexls = $btn_ongeneratexls;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Produto","Relatório de histórico de preços por produto"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }
    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }
    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->produto_id) AND ( (is_scalar($data->produto_id) AND $data->produto_id !== '') OR (is_array($data->produto_id) AND (!empty($data->produto_id)) )) )
        {

            $filters[] = new TFilter('produto_id', '=', $data->produto_id);// create the filter 
        }
        if (isset($data->system_unit_id) AND ( (is_scalar($data->system_unit_id) AND $data->system_unit_id !== '') OR (is_array($data->system_unit_id) AND (!empty($data->system_unit_id)) )) )
        {

            $filters[] = new TFilter('produto_filial_id', 'in', "(SELECT id FROM produto_filial WHERE system_unit_id = '{$data->system_unit_id}')");// create the filter 
        }
        if (isset($data->criado_em_ini) AND ( (is_scalar($data->criado_em_ini) AND $data->criado_em_ini !== '') OR (is_array($data->criado_em_ini) AND (!empty($data->criado_em_ini)) )) )
        {

            $filters[] = new TFilter('criado_em', '>=', $data->criado_em_ini);// create the filter 
        }
        if (isset($data->criado_em_fim) AND ( (is_scalar($data->criado_em_fim) AND $data->criado_em_fim !== '') OR (is_array($data->criado_em_fim) AND (!empty($data->criado_em_fim)) )) )
        {

            $filters[] = new TFilter('criado_em', '<=', $data->criado_em_fim);// create the filter 
        }
        if (isset($data->criado_por) AND ( (is_scalar($data->criado_por) AND $data->criado_por !== '') OR (is_array($data->criado_por) AND (!empty($data->criado_por)) )) )
        {

            $filters[] = new TFilter('criado_por', 'like', "%{$data->criado_por}%");// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            $param = [];
            // creates a repository for ProdutoFilialHistorico
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            $param['order'] = 'criado_em';
            $param['direction'] = 'asc';

            $criteria->setProperties($param);

            if ($filters)
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            if ($objects)
            {
                $widths = array(200,200,200,200,200,200,200,200,200);
                $reportExtension = 'pdf';
                switch ($format)
                {
                    case 'html':
                        $tr = new TTableWriterHTML($widths);
                        $reportExtension = 'html';
                        break;
                    case 'xls':
                        $tr = new TTableWriterXLS($widths);
                        $reportExtension = 'xls';
                        break;
                    case 'pdf':
                        $tr = new TTableWriterPDF($widths, 'L', 'A4');
                        $reportExtension = 'pdf';
                        break;
                    case 'htmlPdf':
                        $reportExtension = 'pdf';
                        $tr = new BTableWriterHtmlPDF($widths, 'L', 'A4');
                        break;
                    case 'rtf':
                        if (!class_exists('PHPRtfLite_Autoloader'))
                        {
                            PHPRtfLite::registerAutoloader();
                        }
                        $reportExtension = 'rtf';
                        $tr = new TTableWriterRTF($widths, 'L', 'A4');
                        break;
                }

                if (!empty($tr))
                {
                    // create the document styles
                    $tr->addStyle('title', 'Helvetica', '10', 'B',   '#ffffff', '#96c0e6');
                    $tr->addStyle('datap', 'Arial', '10', '',    '#000000', '#f0f0f0');
                    $tr->addStyle('datai', 'Arial', '10', '',    '#000000', '#ffffff');
                    $tr->addStyle('header', 'Helvetica', '16', 'B',   '#4589c5', '#494D90');
                    $tr->addStyle('footer', 'Helvetica', '10', 'B',  '#4589c5', '#B1B1EA');
                    $tr->addStyle('break', 'Helvetica', '10', 'B',  '#ffffff', '#8da4bc');
                    $tr->addStyle('total', 'Helvetica', '10', 'I',  '#000000', '#b2d1dc');
                    $tr->addStyle('breakTotal', 'Helvetica', '10', 'I',  '#000000', '#b3d5f3');

                    // Quando o relatório a ser gerado for PDF
                    if ($format == 'pdf')
                    {
                        // Adiciona o cabeçalho
                        $tr->setHeaderCallback(
                            function($tr)
                            {
                                $pdf = $tr->getNativeWriter();

                                // Define a fonte/ estilos
                                $pdf->SetFont('Arial','B',15); 

                                // Define o posicionamento do texto
                                $pdf->Cell(100); 

                                // Texto do cabeçalho
                                $pdf->Cell(100,10, utf8_decode('Relatório de histórico de preços por produto') ,0,0,'C'); 
                                $pdf->Ln(10); 

                            }
                        );

                        // Adiciona o footer do relatório
                        $tr->setFooterCallback(
                            function($tr)
                            {
                                $pdf = $tr->getNativeWriter();

                                // Necessário para obter o número total de páginas
                                $pdf->AliasNbPages();

                                // Posiciona o footer no final da página
                                $pdf->SetY(-40);

                                // Define o estilho do footer
                                $pdf->SetFont('Arial'   ,'B',12); 
                                $pdf->Cell(110); 

                                // Obtém o número da página atual
                                $numero = $pdf->PageNo();

                                // Footer
                                $pdf->Cell(0,10, utf8_decode("Página: {$numero}/{nb}") ,0,0,'R');

                                // Line break 
                                $pdf->Ln(20); 
                            }
                        );
                    }
                    else // Para os demais formatos
                    {
                         $tr->setHeaderCallback(
                            function($tr)
                            {
                                $tr->addRow();
                                $tr->addCell('Relatório de histórico de preços por produto', 'center', 'header', 100);
                            }
                        );

                        $tr->setFooterCallback(
                            function($tr)
                            {
                                $tr->addRow();
                                $tr->addCell(date('d/m/Y h:i:s'), 'center', 'footer', 100);
                            }
                        );
                    }

                    // add titles row
                    $tr->addRow();
                    $tr->addCell("Filial", 'left', 'title');
                    $tr->addCell("Id", 'left', 'title');
                    $tr->addCell("Produto", 'left', 'title');
                    $tr->addCell("Data", 'left', 'title');
                    $tr->addCell("Usuário", 'left', 'title');
                    $tr->addCell("Preço venda antigo", 'left', 'title');
                    $tr->addCell("Preço venda novo", 'left', 'title');
                    $tr->addCell("Valor custo antigo", 'left', 'title');
                    $tr->addCell("Valor custo novo", 'left', 'title');

                    $grandTotal = [];
                    $breakTotal = [];
                    $breakValue = null;
                    $firstRow = true;

                    // controls the background filling
                    $colour = false;                
                    foreach ($objects as $object)
                    {
                        $style = $colour ? 'datap' : 'datai';

                        $firstRow = false;

                        $object->criado_em = call_user_func(function($value, $object, $row)
                        {
                            if(!empty(trim($value)))
                            {
                                try
                                {
                                    $date = new DateTime($value);
                                    return $date->format('d/m/Y H:i');
                                }
                                catch (Exception $e)
                                {
                                    return $value;
                                }
                            }
                        }, $object->criado_em, $object, null);

                        $object->preco_venda_antigo = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->preco_venda_antigo, $object, null);

                        $object->preco_venda_novo = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->preco_venda_novo, $object, null);

                        $object->valor_custo_antigo = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->valor_custo_antigo, $object, null);

                        $object->valor_custo_novo = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->valor_custo_novo, $object, null);

                        $tr->addRow();

                        $tr->addCell($object->produto_filial->system_unit->name, 'left', $style);
                        $tr->addCell($object->produto_id, 'left', $style);
                        $tr->addCell($object->produto->nome, 'left', $style);
                        $tr->addCell($object->criado_em, 'left', $style);
                        $tr->addCell($object->criado_por, 'left', $style);
                        $tr->addCell($object->preco_venda_antigo, 'left', $style);
                        $tr->addCell($object->preco_venda_novo, 'left', $style);
                        $tr->addCell($object->valor_custo_antigo, 'left', $style);
                        $tr->addCell($object->valor_custo_novo, 'left', $style);

                        $colour = !$colour;

                    }

                    $file = 'report_'.uniqid().".{$reportExtension}";
                    // stores the file
                    if (!file_exists("app/output/{$file}") || is_writable("app/output/{$file}"))
                    {
                        $tr->save("app/output/{$file}");
                    }
                    else
                    {
                        throw new Exception(_t('Permission denied') . ': ' . "app/output/{$file}");
                    }

                    parent::openFile("app/output/{$file}");

                    // shows the success message
                    new TMessage('info', _t('Report generated. Please, enable popups'));
                }
            }
            else
            {
                new TMessage('error', _t('No records found'));
            }

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }


}

