<?php

class UnidadeMedidaForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = 'kraftsoft';
    private static $activeRecord = 'UnidadeMedida';
    private static $primaryKey = 'id';
    private static $formName = 'form_UnidadeMedidaForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param )
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Cadastro de unidade de medida");


        $id = new TEntry('id');
        $nome = new TEntry('nome');
        $sigla = new TEntry('sigla');
        $criado_em = new TDateTime('criado_em');
        $criado_por = new TEntry('criado_por');
        $alterado_em = new TDateTime('alterado_em');
        $alterado_por = new TEntry('alterado_por');

        $nome->addValidation("Nome", new TRequiredValidator()); 
        $sigla->addValidation("Sigla", new TRequiredValidator()); 

        $criado_em->setMask('dd/mm/yyyy hh:ii');
        $alterado_em->setMask('dd/mm/yyyy hh:ii');

        $criado_em->setDatabaseMask('yyyy-mm-dd hh:ii');
        $alterado_em->setDatabaseMask('yyyy-mm-dd hh:ii');

        $id->setEditable(false);
        $criado_em->setEditable(false);
        $criado_por->setEditable(false);
        $alterado_em->setEditable(false);
        $alterado_por->setEditable(false);

        $id->setSize(100);
        $nome->setSize('100%');
        $sigla->setSize('100%');
        $criado_em->setSize('100%');
        $criado_por->setSize('100%');
        $alterado_em->setSize('100%');
        $alterado_por->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[]);
        $row1->layout = ['col-sm-6','col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Nome:", '#ff0000', '14px', null, '100%'),$nome],[new TLabel("Sigla:", '#ff0000', '14px', null, '100%'),$sigla]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addContent([new TFormSeparator("Histórico de Atualizações", '#333', '18', '#eee')]);
        $row4 = $this->form->addFields([new TLabel("Criado em:", null, '14px', null, '100%'),$criado_em],[new TLabel("Criado Por:", null, '14px', null),$criado_por],[new TLabel("Alterado em:", null, '14px', null, '100%'),$alterado_em],[new TLabel("Alterado Por:", null, '14px', null),$alterado_por]);
        $row4->layout = [' col-sm-2',' col-sm-4',' col-sm-2',' col-sm-4'];

        // create the form actions
        $btn_onsave = $this->form->addAction("Salvar", new TAction([$this, 'onSave']), 'fas:save #ffffff');
        $this->btn_onsave = $btn_onsave;
        $btn_onsave->addStyleClass('btn-primary'); 

        $btn_onclear = $this->form->addAction("Limpar formulário", new TAction([$this, 'onClear']), 'fas:eraser #dd5a43');
        $this->btn_onclear = $btn_onclear;

        $btn_onshow = $this->form->addAction("Voltar", new TAction(['UnidadeMedidaHeaderList', 'onShow']), 'fas:arrow-left #000000');
        $this->btn_onshow = $btn_onshow;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Produto","Cadastro de unidade de medida"]));
        }
        $container->add($this->form);

        parent::add($container);

    }

    public function onSave($param = null) 
    {
        try
        {
            TTransaction::open(self::$database); // open a transaction

            $messageAction = null;

            $this->form->validate(); // validate form data

            $object = new UnidadeMedida(); // create an empty object 

            $data = $this->form->getData(); // get form data as array
            $object->fromArray( (array) $data); // load the object with data

            //salvando o nome do usuario
            Funcoes::SalvarUsuarioCadastro($object);

            $object->store(); // save the object 

            $loadPageParam = [];

            if(!empty($param['target_container']))
            {
                $loadPageParam['target_container'] = $param['target_container'];
            }

            // get the generated {PRIMARY_KEY}
            $data->id = $object->id; 

            $this->form->setData($data); // fill form data
            TTransaction::close(); // close the transaction

            TTransaction::open(self::$database);

            //verificação de evento..
            Funcoes::VerificaNotificacaoEvento(TSession::getValue('userid'),
                                              UnidadeMedida::Evento_UnidadeMedidaIncluidaOuAterada,
                                              $object,
                                              $data->nome);        

            TTransaction::close();                                               

            TToast::show('success', "Registro salvo", 'topRight', 'far:check-circle');
            TApplication::loadPage('UnidadeMedidaHeaderList', 'onShow', $loadPageParam); 

        }
        catch (Exception $e) // in case of exception
        {
            //</catchAutoCode> 

            new TMessage('error', $e->getMessage()); // shows the exception error message
            $this->form->setData( $this->form->getData() ); // keep form data
            TTransaction::rollback(); // undo all pending operations
        }
    }

    public function onEdit( $param )
    {
        try
        {
            if (isset($param['key']))
            {
                $key = $param['key'];  // get the parameter $key
                TTransaction::open(self::$database); // open a transaction

                $object = new UnidadeMedida($key); // instantiates the Active Record 

                $this->form->setData($object); // fill the form 

                TTransaction::close(); // close the transaction 
            }
            else
            {
                $this->form->clear();
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }

    /**
     * Clear form data
     * @param $param Request
     */
    public function onClear( $param )
    {
        $this->form->clear(true);

    }

    public function onShow($param = null)
    {

    } 

    public static function getFormName()
    {
        return self::$formName;
    }

}

