<?php

class ClienteVendaList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Venda';
    private static $primaryKey = 'id';
    private static $formName = 'form_VendaList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de vendas");
        $this->limit = 50;

        $id = new TEntry('id');
        $orcamento_id = new TEntry('orcamento_id');
        $data_venda_inicial = new TDateTime('data_venda_inicial');
        $data_venda_final = new TDateTime('data_venda_final');
        $filial_saida_id = new TDBCombo('filial_saida_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $vendedor_id = new TDBCombo('vendedor_id', 'kraftsoft', 'Vendedor', 'id', '{nome}','nome asc'  );
        $venda_situacao_id = new TDBCombo('venda_situacao_id', 'kraftsoft', 'VendaSituacao', 'id', '{descricao}','ordem asc'  );


        $vendedor_id->enableSearch();
        $data_venda_final->setMask('dd/mm/yyyy hh:ii');
        $data_venda_inicial->setMask('dd/mm/yyyy hh:ii');

        $data_venda_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_venda_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');

        $id->setSize('100%');
        $vendedor_id->setSize('100%');
        $orcamento_id->setSize('100%');
        $filial_saida_id->setSize('100%');
        $data_venda_final->setSize('100%');
        $venda_situacao_id->setSize('100%');
        $data_venda_inicial->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null),$id],[new TLabel("Id Orçamento:", null, '14px', null),$orcamento_id],[]);
        $row1->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row2 = $this->form->addFields([new TLabel("Data Inicial:", null, '14px', null, '100%'),$data_venda_inicial],[new TLabel("Data Final:", null, '14px', null, '100%'),$data_venda_final],[new TLabel("Filial de Saída:", null, '14px', null, '100%'),$filial_saida_id]);
        $row2->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row3 = $this->form->addFields([new TLabel("Vendedor:", null, '14px', null),$vendedor_id],[new TLabel("Situação:", null, '14px', null),$venda_situacao_id]);
        $row3->layout = ['col-sm-4','col-sm-4'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        if(!empty($param["key"] ?? ""))
        {
            TSession::setValue(__CLASS__.'load_filter_cliente_id', $param["key"] ?? "");
        }
        $filterVar = TSession::getValue(__CLASS__.'load_filter_cliente_id');
        $this->filter_criteria->add(new TFilter('cliente_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_data_hora_venda_transformed = new TDataGridColumn('data_hora_venda', "Data Hora", 'left');
        $column_system_unit_name = new TDataGridColumn('system_unit->name', "Filial Venda", 'left');
        $column_orcamento_id = new TDataGridColumn('orcamento_id', "Id Orçamento", 'left');
        $column_vendedor_nome = new TDataGridColumn('vendedor->nome', "Vendedor", 'left');
        $column_filial_saida_name = new TDataGridColumn('filial_saida->name', "Filial de Saída", 'left');
        $column_venda_tipo_pagamento_descricao = new TDataGridColumn('venda_tipo_pagamento->descricao', "Tipo Pagamento", 'left');
        $column_valor_total_transformed = new TDataGridColumn('valor_total', "Valor Total", 'left');
        $column_venda_situacao_descricao_transformed = new TDataGridColumn('venda_situacao->descricao', "Situação", 'left');

        $column_data_hora_venda_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_valor_total_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_venda_situacao_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            //code here
            $label = $object->venda_situacao->descricao;
            $cor = $object->venda_situacao->cor;

            return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

            // $div = new TElement('div');
            // $div->style="text-shadow:none; font-size:14px; font-weight:bold; color:#FFFFFF; width:160px; text-align:center;background-color: $cor;padding:10px";
            // $div->add($label);

            // return $div;    //code here

        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_data_hora_venda_transformed);
        $this->datagrid->addColumn($column_system_unit_name);
        $this->datagrid->addColumn($column_orcamento_id);
        $this->datagrid->addColumn($column_vendedor_nome);
        $this->datagrid->addColumn($column_filial_saida_name);
        $this->datagrid->addColumn($column_venda_tipo_pagamento_descricao);
        $this->datagrid->addColumn($column_valor_total_transformed);
        $this->datagrid->addColumn($column_venda_situacao_descricao_transformed);

        $action_group = new TDataGridActionGroup("", 'fas:cog');
        $action_group->addHeader('');

        $action_onShow = new TDataGridAction(array('VendaFormView', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Abrir");
        $action_onShow->setImage('fas:book-open #00BCD4');
        $action_onShow->setField(self::$primaryKey);

        $action_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_onShow);

        $action_onEdit = new TDataGridAction(array('VendaForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #2196F3');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('ClienteVendaList::onMostrarEditar');

        $action_group->addAction($action_onEdit);

        $action_onEnviarFaturamento = new TDataGridAction(array('ClienteVendaList', 'onEnviarFaturamento'));
        $action_onEnviarFaturamento->setUseButton(TRUE);
        $action_onEnviarFaturamento->setButtonClass('btn btn-default');
        $action_onEnviarFaturamento->setLabel("Enviar p/ Faturamento");
        $action_onEnviarFaturamento->setImage('fas:code-branch #9C27B0');
        $action_onEnviarFaturamento->setField(self::$primaryKey);
        $action_onEnviarFaturamento->setDisplayCondition('ClienteVendaList::onMostrarEnviarFaturamento');

        $action_group->addAction($action_onEnviarFaturamento);

        $action_onFaturamento = new TDataGridAction(array('ClienteVendaList', 'onFaturamento'));
        $action_onFaturamento->setUseButton(TRUE);
        $action_onFaturamento->setButtonClass('btn btn-default');
        $action_onFaturamento->setLabel("Faturamento");
        $action_onFaturamento->setImage('fas:money-check-alt #FF5722');
        $action_onFaturamento->setField(self::$primaryKey);
        $action_onFaturamento->setDisplayCondition('ClienteVendaList::onMostrarFaturamento');

        $action_group->addAction($action_onFaturamento);

        $action_onGerarNfe = new TDataGridAction(array('ClienteVendaList', 'onGerarNfe'));
        $action_onGerarNfe->setUseButton(TRUE);
        $action_onGerarNfe->setButtonClass('btn btn-default');
        $action_onGerarNfe->setLabel("Gerar Nf-e");
        $action_onGerarNfe->setImage('fas:file-invoice-dollar #4CAF50');
        $action_onGerarNfe->setField(self::$primaryKey);
        $action_onGerarNfe->setDisplayCondition('ClienteVendaList::onMostrarGerarNFe');

        $action_group->addAction($action_onGerarNfe);

        $action_onGerarNFCe = new TDataGridAction(array('ClienteVendaList', 'onGerarNFCe'));
        $action_onGerarNFCe->setUseButton(TRUE);
        $action_onGerarNFCe->setButtonClass('btn btn-default');
        $action_onGerarNFCe->setLabel("Gerar NFC-e");
        $action_onGerarNFCe->setImage('fas:file-invoice-dollar #CDDC39');
        $action_onGerarNFCe->setField(self::$primaryKey);
        $action_onGerarNFCe->setDisplayCondition('ClienteVendaList::onMostrarGerarNFCe');

        $action_group->addAction($action_onGerarNFCe);

        $action_onEspelhoPDF = new TDataGridAction(array('ClienteVendaList', 'onEspelhoPDF'));
        $action_onEspelhoPDF->setUseButton(TRUE);
        $action_onEspelhoPDF->setButtonClass('btn btn-default');
        $action_onEspelhoPDF->setLabel("Espelho PDF");
        $action_onEspelhoPDF->setImage('far:file-pdf #DD0909');
        $action_onEspelhoPDF->setField(self::$primaryKey);

        $action_group->addAction($action_onEspelhoPDF);

        $action_onCancelar = new TDataGridAction(array('ClienteVendaList', 'onCancelar'));
        $action_onCancelar->setUseButton(TRUE);
        $action_onCancelar->setButtonClass('btn btn-default');
        $action_onCancelar->setLabel("Cancelar");
        $action_onCancelar->setImage('fas:window-close #CE0D0D');
        $action_onCancelar->setField(self::$primaryKey);
        $action_onCancelar->setDisplayCondition('ClienteVendaList::onMostrarCancelar');

        $action_group->addAction($action_onCancelar);

        $action_VendaSituacaoHistoricoTimeLine_onShow = new TDataGridAction(array('VendaSituacaoHistoricoTimeLine', 'onShow'));
        $action_VendaSituacaoHistoricoTimeLine_onShow->setUseButton(TRUE);
        $action_VendaSituacaoHistoricoTimeLine_onShow->setButtonClass('btn btn-default');
        $action_VendaSituacaoHistoricoTimeLine_onShow->setLabel("Linha do tempo das situações desta venda");
        $action_VendaSituacaoHistoricoTimeLine_onShow->setImage('fas:clock #FF9800');
        $action_VendaSituacaoHistoricoTimeLine_onShow->setField(self::$primaryKey);

        $action_VendaSituacaoHistoricoTimeLine_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_VendaSituacaoHistoricoTimeLine_onShow);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $btnShowCurtainFilters = new TButton('button_btnShowCurtainFilters');
        $btnShowCurtainFilters->setAction(new TAction(['ClienteVendaList', 'onShowCurtainFilters']), "Filtros");
        $btnShowCurtainFilters->addStyleClass('btn-default');
        $btnShowCurtainFilters->setImage('fas:filter #000000');

        $this->datagrid_form->addField($btnShowCurtainFilters);

        $button_limpar_filtros = new TButton('button_button_limpar_filtros');
        $button_limpar_filtros->setAction(new TAction(['ClienteVendaList', 'onClearFilters']), "Limpar filtros");
        $button_limpar_filtros->addStyleClass('btn-default');
        $button_limpar_filtros->setImage('fas:eraser #f44336');

        $this->datagrid_form->addField($button_limpar_filtros);

        $button_atualizar = new TButton('button_button_atualizar');
        $button_atualizar->setAction(new TAction(['ClienteVendaList', 'onRefresh']), "Atualizar");
        $button_atualizar->addStyleClass('btn-default');
        $button_atualizar->setImage('fas:sync-alt #03a9f4');

        $this->datagrid_form->addField($button_atualizar);

        $head_left_actions->add($btnShowCurtainFilters);
        $head_left_actions->add($button_limpar_filtros);
        $head_left_actions->add($button_atualizar);

        $this->btnShowCurtainFilters = $btnShowCurtainFilters;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Vendas","Vendas Cliente"]));
        }

        $container->add($panel);

        parent::add($container);

    }

    public static function onMostrarEditar($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::Finalizada 
            or $object->venda_situacao_id==VendaSituacao::Cancelada
            or $object->venda_situacao_id==VendaSituacao::AguardandoExpedicao)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onEnviarFaturamento($param = null) 
    {
        try 
        {
          new TQuestion("Deseja enviar venda para faturamento?", new TAction([__CLASS__, 'onEnviarFaturamentoYes'], $param), new TAction([__CLASS__, 'onEnviarFaturamentoNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarEnviarFaturamento($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::Finalizada 
            or $object->venda_situacao_id==VendaSituacao::Cancelada
            or $object->venda_situacao_id==VendaSituacao::AguardandoExpedicao)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onFaturamento($param = null) 
    {
        try 
        {
           // new TQuestion("Deseja realizar o faturamento?", new TAction([__CLASS__, 'onFaturamentoYes'], $param), new TAction([__CLASS__, 'onFaturamentoNo'], $param));

          $pageParam = ['venda_id' => $param['id']]; // ex.: = ['key' => 10]  
          TApplication::loadPage('VendaFaturamentoSimpleList', 'onReload', $pageParam);             
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarFaturamento($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::Finalizada 
            or $object->venda_situacao_id==VendaSituacao::Cancelada
            or $object->venda_situacao_id==VendaSituacao::AguardandoExpedicao)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onGerarNfe($param = null) 
    {
        try 
        {
            new TQuestion("Deseja gerar a NF-e?", new TAction([__CLASS__, 'onGerarNFeYes'], $param), new TAction([__CLASS__, 'onGerarNFeNo'], $param));
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarGerarNFe($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::AguardandoFaturamento)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onGerarNFCe($param = null) 
    {
        try 
        {
            new TQuestion("Deseja gerar a NFC-e?", new TAction([__CLASS__, 'onGerarNFCeYes'], $param), new TAction([__CLASS__, 'onGerarNFCeNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarGerarNFCe($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::AguardandoFaturamento)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onEspelhoPDF($param = null) 
    {
        try 
        {
            $documento = Venda::Espelho_Venda($param['id']);

            if(file_exists($documento)){
                parent::openFile($documento);   
            }else{
                new TMessage('error', "Arquivo não encontrado!");
            }

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onCancelar($param = null) 
    { 

     new TQuestion("Deseja cancelar a venda?", new TAction([__CLASS__, 'onCancelarYes'], $param), new TAction([__CLASS__, 'onCancelarNo'], $param));

     /*   if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new Venda($key, FALSE); 

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        } */
    }
    public static function onMostrarCancelar($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::Finalizada 
            or $object->venda_situacao_id==VendaSituacao::Cancelada
            or $object->venda_situacao_id==VendaSituacao::AguardandoExpedicao)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onShowCurtainFilters($param = null) 
    {
        try 
        {
            //code here

                        $filter = new self([]);

            $btnClose = new TButton('closeCurtain');
            $btnClose->class = 'btn btn-sm btn-default';
            $btnClose->style = 'margin-right:10px;';
            $btnClose->onClick = "Template.closeRightPanel();";
            $btnClose->setLabel("Fechar");
            $btnClose->setImage('fas:times');

            $filter->form->addHeaderWidget($btnClose);

            $page = new TPage();
            $page->setTargetContainer('adianti_right_panel');
            $page->setProperty('page-name', 'ClienteVendaListSearch');
            $page->setProperty('page_name', 'ClienteVendaListSearch');
            $page->adianti_target_container = 'adianti_right_panel';
            $page->target_container = 'adianti_right_panel';
            $page->add($filter->form);
            $page->setIsWrapped(true);
            $page->show();

            $style = new TStyle('right-panel > .container-part[page-name=ClienteVendaListSearch]');
            $style->width = '70% !important';
            $style->show(true);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onClearFilters($param = null) 
    {
        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }
    public function onRefresh($param = null) 
    {
        $this->onReload([]);
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->orcamento_id) AND ( (is_scalar($data->orcamento_id) AND $data->orcamento_id !== '') OR (is_array($data->orcamento_id) AND (!empty($data->orcamento_id)) )) )
        {

            $filters[] = new TFilter('orcamento_id', '=', $data->orcamento_id);// create the filter 
        }

        if (isset($data->data_venda_inicial) AND ( (is_scalar($data->data_venda_inicial) AND $data->data_venda_inicial !== '') OR (is_array($data->data_venda_inicial) AND (!empty($data->data_venda_inicial)) )) )
        {

            $filters[] = new TFilter('data_hora_venda', '>=', $data->data_venda_inicial);// create the filter 
        }

        if (isset($data->data_venda_final) AND ( (is_scalar($data->data_venda_final) AND $data->data_venda_final !== '') OR (is_array($data->data_venda_final) AND (!empty($data->data_venda_final)) )) )
        {

            $filters[] = new TFilter('data_hora_venda', '<=', $data->data_venda_final);// create the filter 
        }

        if (isset($data->filial_saida_id) AND ( (is_scalar($data->filial_saida_id) AND $data->filial_saida_id !== '') OR (is_array($data->filial_saida_id) AND (!empty($data->filial_saida_id)) )) )
        {

            $filters[] = new TFilter('filial_saida_id', '=', $data->filial_saida_id);// create the filter 
        }

        if (isset($data->vendedor_id) AND ( (is_scalar($data->vendedor_id) AND $data->vendedor_id !== '') OR (is_array($data->vendedor_id) AND (!empty($data->vendedor_id)) )) )
        {

            $filters[] = new TFilter('vendedor_id', '=', $data->vendedor_id);// create the filter 
        }

        if (isset($data->venda_situacao_id) AND ( (is_scalar($data->venda_situacao_id) AND $data->venda_situacao_id !== '') OR (is_array($data->venda_situacao_id) AND (!empty($data->venda_situacao_id)) )) )
        {

            $filters[] = new TFilter('venda_situacao_id', '=', $data->venda_situacao_id);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Venda
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            //</blockLine><btnShowCurtainFiltersAutoCode>
            if(!empty($this->btnShowCurtainFilters) && empty($this->btnShowCurtainFiltersAdjusted))
            {
                $this->btnShowCurtainFiltersAdjusted = true;
                $this->btnShowCurtainFilters->style = 'position: relative';
                $countFilters = count($filters ?? []);
                $this->btnShowCurtainFilters->setLabel($this->btnShowCurtainFilters->getLabel(). "<span class='badge badge-success' style='position: absolute'>{$countFilters}<span>");
            }
            //</blockLine></btnShowCurtainFiltersAutoCode>

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function onEnviarFaturamentoYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $venda = Venda::find($param['id']);     

            if($venda) 
            {
                $venda->venda_situacao_id=VendaSituacao::AguardandoFaturamento;
                $venda->store();

                VendaSituacaoHistorico::LancarHistorico(VendaSituacao::AguardandoFaturamento,
                                                        $venda->id,
                                                        'Mudança de Situação de venda: Aguardando Faturamento');                
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');  
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onEnviarFaturamentoNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onFaturamentoYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $venda = Venda::find($param['id']);     

            if($venda) 
            {
                $venda->venda_situacao_id=VendaSituacao::Finalizada;
                $venda->store();
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');    
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onFaturamentoNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onGerarNFeYes($param = null) 
    {
        try 
        {
            TTransaction::open('kraftsoft');    

            Venda::ValidaCamposNFe($param['id']);

            $nfe_id = funcaoNFE::geraNotaVenda($param['id'], NotafiscalMod::NFe);

            if(is_numeric($nfe_id))
            {
                $pageParam = ['key' => $nfe_id]; 
                TApplication::loadPage('NotafiscalFormView', 'onShow', $pageParam);

            }
            else
            {
                throw new Exception($nfe_id);
            }

            TTransaction::close();       
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onGerarNFeNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onGerarNFCeYes($param = null) 
    {
        try 
        {
            TTransaction::open('kraftsoft');    

            Venda::ValidaCamposNFe($param['id']);

            $nfe_id = funcaoNFE::geraNotaVenda($param['id'], NotafiscalMod::NFCe);

            if(is_numeric($nfe_id))
            {
                $pageParam = ['key' => $nfe_id]; 
                TApplication::loadPage('NotafiscalConsumidorFormView', 'onShow', $pageParam);

            }
            else
            {
                throw new Exception($nfe_id);
            }

            TTransaction::close();          

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onGerarNFCeNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onCancelarYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $venda = Venda::find($param['id']);     

            if($venda) 
            {
                $venda->venda_situacao_id=VendaSituacao::Cancelada;
                $venda->store();

                //historico de situação de venda..
                VendaSituacaoHistorico::LancarHistorico(VendaSituacao::Cancelada,
                                                        $venda->id,
                                                        'Mudança de Situação de venda: Cancelada');                           
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');  
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onCancelarNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

}

