<?php

class ComissaoVendedorPorVendaReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Venda';
    private static $primaryKey = 'id';
    private static $formName = 'form_ComissaoVendedorPorVendaReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Relatório de comissão vendedor por venda");

        $id = new TEntry('id');
        $orcamento_id = new TEntry('orcamento_id');
        $system_unit_id = new TDBCombo('system_unit_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $data_venda_inicial = new TDateTime('data_venda_inicial');
        $data_venda_final = new TDateTime('data_venda_final');
        $filial_saida_id = new TDBCombo('filial_saida_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '-{id} -{nome} -{cpf_cnpj}','nome asc'  );
        $vendedor_id = new TDBCombo('vendedor_id', 'kraftsoft', 'Vendedor', 'id', '{nome}','nome asc'  );
        $venda_situacao_id = new TDBCombo('venda_situacao_id', 'kraftsoft', 'VendaSituacao', 'id', '{descricao}','ordem asc'  );

        $system_unit_id->setValue(TSession::getValue("userunitid"));
        $data_venda_final->setMask('dd/mm/yyyy hh:ii');
        $data_venda_inicial->setMask('dd/mm/yyyy hh:ii');

        $data_venda_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_venda_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');

        $cliente_id->enableSearch();
        $vendedor_id->enableSearch();
        $system_unit_id->enableSearch();
        $filial_saida_id->enableSearch();
        $venda_situacao_id->enableSearch();

        $id->setSize('100%');
        $cliente_id->setSize('100%');
        $vendedor_id->setSize('100%');
        $orcamento_id->setSize('100%');
        $system_unit_id->setSize('100%');
        $filial_saida_id->setSize('100%');
        $data_venda_final->setSize('100%');
        $venda_situacao_id->setSize('100%');
        $data_venda_inicial->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Id Orçamento:", null, '14px', null, '100%'),$orcamento_id],[new TLabel("Filial Venda:", null, '14px', null, '100%'),$system_unit_id]);
        $row1->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row2 = $this->form->addFields([new TLabel("Data Inicial:", null, '14px', null, '100%'),$data_venda_inicial],[new TLabel("Data Final:", null, '14px', null, '100%'),$data_venda_final],[new TLabel("Filial de saida", null, '14px', null, '100%'),$filial_saida_id]);
        $row2->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row3 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null, '100%'),$cliente_id],[new TLabel("Vendedor:", null, '14px', null, '100%'),$vendedor_id],[new TLabel("Situação:", null, '14px', null, '100%'),$venda_situacao_id]);
        $row3->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratehtml = $this->form->addAction("Gerar HTML", new TAction([$this, 'onGenerateHtml']), 'far:file-code #ffffff');
        $this->btn_ongeneratehtml = $btn_ongeneratehtml;
        $btn_ongeneratehtml->addStyleClass('btn-primary'); 

        $btn_ongeneratepdf = $this->form->addAction("Gerar PDF", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        $btn_ongeneratexls = $this->form->addAction("Gerar XLS", new TAction([$this, 'onGenerateXls']), 'far:file-excel #00a65a');
        $this->btn_ongeneratexls = $btn_ongeneratexls;

        $btn_ongeneratertf = $this->form->addAction("Gerar RTF", new TAction([$this, 'onGenerateRtf']), 'far:file-alt #324bcc');
        $this->btn_ongeneratertf = $btn_ongeneratertf;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Vendas","Relatório de comissão vendedor por venda"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }

    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }

    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }

    public function onGenerateRtf($param = null) 
    {
        $this->onGenerate('rtf');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }
        if (isset($data->orcamento_id) AND ( (is_scalar($data->orcamento_id) AND $data->orcamento_id !== '') OR (is_array($data->orcamento_id) AND (!empty($data->orcamento_id)) )) )
        {

            $filters[] = new TFilter('orcamento_id', '=', $data->orcamento_id);// create the filter 
        }
        if (isset($data->system_unit_id) AND ( (is_scalar($data->system_unit_id) AND $data->system_unit_id !== '') OR (is_array($data->system_unit_id) AND (!empty($data->system_unit_id)) )) )
        {

            $filters[] = new TFilter('filial_saida_id', '=', $data->system_unit_id);// create the filter 
        }
        if (isset($data->data_venda_inicial) AND ( (is_scalar($data->data_venda_inicial) AND $data->data_venda_inicial !== '') OR (is_array($data->data_venda_inicial) AND (!empty($data->data_venda_inicial)) )) )
        {

            $filters[] = new TFilter('data_hora_venda', '>=', $data->data_venda_inicial);// create the filter 
        }
        if (isset($data->data_venda_final) AND ( (is_scalar($data->data_venda_final) AND $data->data_venda_final !== '') OR (is_array($data->data_venda_final) AND (!empty($data->data_venda_final)) )) )
        {

            $filters[] = new TFilter('data_hora_venda', '<=', $data->data_venda_final);// create the filter 
        }
        if (isset($data->filial_saida_id) AND ( (is_scalar($data->filial_saida_id) AND $data->filial_saida_id !== '') OR (is_array($data->filial_saida_id) AND (!empty($data->filial_saida_id)) )) )
        {

            $filters[] = new TFilter('filial_saida_id', '=', $data->filial_saida_id);// create the filter 
        }
        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }
        if (isset($data->vendedor_id) AND ( (is_scalar($data->vendedor_id) AND $data->vendedor_id !== '') OR (is_array($data->vendedor_id) AND (!empty($data->vendedor_id)) )) )
        {

            $filters[] = new TFilter('vendedor_id', '=', $data->vendedor_id);// create the filter 
        }
        if (isset($data->venda_situacao_id) AND ( (is_scalar($data->venda_situacao_id) AND $data->venda_situacao_id !== '') OR (is_array($data->venda_situacao_id) AND (!empty($data->venda_situacao_id)) )) )
        {

            $filters[] = new TFilter('venda_situacao_id', '=', $data->venda_situacao_id);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            $param = [];
            // creates a repository for Venda
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            $param['order'] = 'vendedor_id';
            $param['direction'] = 'asc';

            $criteria->setProperties($param);

            if ($filters)
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            if ($objects)
            {
                $widths = array(200,200,200,200,200,200,200,200,200);
                $reportExtension = 'pdf';
                switch ($format)
                {
                    case 'html':
                        $tr = new TTableWriterHTML($widths);
                        $reportExtension = 'html';
                        break;
                    case 'xls':
                        $tr = new TTableWriterXLS($widths);
                        $reportExtension = 'xls';
                        break;
                    case 'pdf':
                        $tr = new TTableWriterPDF($widths, 'L', 'A4');
                        $reportExtension = 'pdf';
                        break;
                    case 'htmlPdf':
                        $reportExtension = 'pdf';
                        $tr = new BTableWriterHtmlPDF($widths, 'L', 'A4');
                        break;
                    case 'rtf':
                        if (!class_exists('PHPRtfLite_Autoloader'))
                        {
                            PHPRtfLite::registerAutoloader();
                        }
                        $reportExtension = 'rtf';
                        $tr = new TTableWriterRTF($widths, 'L', 'A4');
                        break;
                }

                if (!empty($tr))
                {
                    // create the document styles
                    $tr->addStyle('title', 'Helvetica', '10', 'B',   '#ffffff', '#96c0e6');
                    $tr->addStyle('datap', 'Arial', '10', '',    '#000000', '#f0f0f0');
                    $tr->addStyle('datai', 'Arial', '10', '',    '#000000', '#ffffff');
                    $tr->addStyle('header', 'Helvetica', '16', 'B',   '#4589c5', '#494D90');
                    $tr->addStyle('footer', 'Helvetica', '10', 'B',  '#4589c5', '#B1B1EA');
                    $tr->addStyle('break', 'Helvetica', '10', 'B',  '#ffffff', '#8da4bc');
                    $tr->addStyle('total', 'Helvetica', '10', 'I',  '#000000', '#b2d1dc');
                    $tr->addStyle('breakTotal', 'Helvetica', '10', 'I',  '#000000', '#b3d5f3');

                    // add titles row
                    $tr->addRow();
                    $tr->addCell("Id", 'left', 'title');
                    $tr->addCell("Data Hora", 'left', 'title');
                    $tr->addCell("Filial venda", 'left', 'title');
                    $tr->addCell("Cliente", 'center', 'title');
                    $tr->addCell("Valor Total", 'left', 'title');
                    $tr->addCell("Valor Comissão", 'left', 'title');
                    $tr->addCell("Comissão a vista", 'left', 'title');
                    $tr->addCell("Comissão a prazo", 'left', 'title');
                    $tr->addCell("Situação", 'center', 'title');

                    $grandTotal = [];
                    $breakTotal = [];
                    $breakValue = null;
                    $firstRow = true;

                    // controls the background filling
                    $colour = false;                
                    foreach ($objects as $object)
                    {
                        $style = $colour ? 'datap' : 'datai';

                        if ($object->vendedor_id !== $breakValue)
                        {
                            if (!$firstRow)
                            {
                                $tr->addRow();

                                $breakTotal_valor_total = array_sum($breakTotal['valor_total']);
                                $breakTotal_valor_comissao = array_sum($breakTotal['valor_comissao']);
                                $breakTotal_comissao_avista = array_sum($breakTotal['comissao_avista']);
                                $breakTotal_comissao_aprazo = array_sum($breakTotal['comissao_aprazo']);

                                $breakTotal_valor_total = call_user_func(function($value)
                                {
                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return "R$ " . number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }
                                }, $breakTotal_valor_total); 

                                $breakTotal_valor_comissao = call_user_func(function($value)
                                {
                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return "R$ " . number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }
                                }, $breakTotal_valor_comissao); 

                                $breakTotal_comissao_avista = call_user_func(function($value)
                                {
                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return "R$ " . number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }
                                }, $breakTotal_comissao_avista); 

                                $breakTotal_comissao_aprazo = call_user_func(function($value)
                                {
                                    if(!$value)
                                    {
                                        $value = 0;
                                    }

                                    if(is_numeric($value))
                                    {
                                        return "R$ " . number_format($value, 2, ",", ".");
                                    }
                                    else
                                    {
                                        return $value;
                                    }
                                }, $breakTotal_comissao_aprazo); 

                                $tr->addCell('', 'center', 'breakTotal');
                                $tr->addCell('', 'center', 'breakTotal');
                                $tr->addCell('', 'center', 'breakTotal');
                                $tr->addCell('', 'center', 'breakTotal');
                                $tr->addCell($breakTotal_valor_total, 'left', 'breakTotal');
                                $tr->addCell($breakTotal_valor_comissao, 'left', 'breakTotal');
                                $tr->addCell($breakTotal_comissao_avista, 'left', 'breakTotal');
                                $tr->addCell($breakTotal_comissao_aprazo, 'left', 'breakTotal');
                                $tr->addCell('', 'center', 'breakTotal');
                            }
                            $tr->addRow();
                            $tr->addCell($object->render('{vendedor->id}'), 'left', 'break', 9);
                            $breakTotal = [];
                        }
                        $breakValue = $object->vendedor_id;

                        $grandTotal['valor_total'][] = $object->valor_total;
                        $breakTotal['valor_total'][] = $object->valor_total;
                        $grandTotal['valor_comissao'][] = $object->valor_comissao;
                        $breakTotal['valor_comissao'][] = $object->valor_comissao;
                        $grandTotal['comissao_avista'][] = $object->comissao_avista;
                        $breakTotal['comissao_avista'][] = $object->comissao_avista;
                        $grandTotal['comissao_aprazo'][] = $object->comissao_aprazo;
                        $breakTotal['comissao_aprazo'][] = $object->comissao_aprazo;

                        $firstRow = false;

                        $object->data_hora_venda = call_user_func(function($value, $object, $row)
                        {
                            if(!empty(trim($value)))
                            {
                                try
                                {
                                    $date = new DateTime($value);
                                    return $date->format('d/m/Y H:i');
                                }
                                catch (Exception $e)
                                {
                                    return $value;
                                }
                            }
                        }, $object->data_hora_venda, $object, null);

                        $object->valor_total = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->valor_total, $object, null);

                        $object->valor_comissao = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->valor_comissao, $object, null);

                        $object->comissao_avista = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->comissao_avista, $object, null);

                        $object->comissao_aprazo = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->comissao_aprazo, $object, null);

                        $tr->addRow();

                        $tr->addCell($object->id, 'left', $style);
                        $tr->addCell($object->data_hora_venda, 'left', $style);
                        $tr->addCell($object->system_unit->name, 'left', $style);
                        $tr->addCell($object->cliente->nome, 'left', $style);
                        $tr->addCell($object->valor_total, 'left', $style);
                        $tr->addCell($object->valor_comissao, 'left', $style);
                        $tr->addCell($object->comissao_avista, 'left', $style);
                        $tr->addCell($object->comissao_aprazo, 'left', $style);
                        $tr->addCell($object->venda_situacao->descricao, 'left', $style);

                        $colour = !$colour;

                    }

                    $tr->addRow();

                    $breakTotal_valor_total = array_sum($breakTotal['valor_total']);
                    $breakTotal_valor_comissao = array_sum($breakTotal['valor_comissao']);
                    $breakTotal_comissao_avista = array_sum($breakTotal['comissao_avista']);
                    $breakTotal_comissao_aprazo = array_sum($breakTotal['comissao_aprazo']);

                    $breakTotal_valor_total = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $breakTotal_valor_total); 

                    $breakTotal_valor_comissao = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $breakTotal_valor_comissao); 

                    $breakTotal_comissao_avista = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $breakTotal_comissao_avista); 

                    $breakTotal_comissao_aprazo = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $breakTotal_comissao_aprazo); 

                    $tr->addCell('', 'center', 'breakTotal');
                    $tr->addCell('', 'center', 'breakTotal');
                    $tr->addCell('', 'center', 'breakTotal');
                    $tr->addCell('', 'center', 'breakTotal');
                    $tr->addCell($breakTotal_valor_total, 'left', 'breakTotal');
                    $tr->addCell($breakTotal_valor_comissao, 'left', 'breakTotal');
                    $tr->addCell($breakTotal_comissao_avista, 'left', 'breakTotal');
                    $tr->addCell($breakTotal_comissao_aprazo, 'left', 'breakTotal');
                    $tr->addCell('', 'center', 'breakTotal');

                    $tr->addRow();

                    $grandTotal_valor_total = array_sum($grandTotal['valor_total']);
                    $grandTotal_valor_comissao = array_sum($grandTotal['valor_comissao']);
                    $grandTotal_comissao_avista = array_sum($grandTotal['comissao_avista']);
                    $grandTotal_comissao_aprazo = array_sum($grandTotal['comissao_aprazo']);

                    $grandTotal_valor_total = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $grandTotal_valor_total); 

                    $grandTotal_valor_comissao = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $grandTotal_valor_comissao); 

                    $grandTotal_comissao_avista = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $grandTotal_comissao_avista); 

                    $grandTotal_comissao_aprazo = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $grandTotal_comissao_aprazo); 

                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell($grandTotal_valor_total, 'left', 'total');
                    $tr->addCell($grandTotal_valor_comissao, 'left', 'total');
                    $tr->addCell($grandTotal_comissao_avista, 'left', 'total');
                    $tr->addCell($grandTotal_comissao_aprazo, 'left', 'total');
                    $tr->addCell('', 'center', 'total');

                    $file = 'report_'.uniqid().".{$reportExtension}";
                    // stores the file
                    if (!file_exists("app/output/{$file}") || is_writable("app/output/{$file}"))
                    {
                        $tr->save("app/output/{$file}");
                    }
                    else
                    {
                        throw new Exception(_t('Permission denied') . ': ' . "app/output/{$file}");
                    }

                    parent::openFile("app/output/{$file}");

                    // shows the success message
                    new TMessage('info', _t('Report generated. Please, enable popups'));
                }
            }
            else
            {
                new TMessage('error', _t('No records found'));
            }

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }


}

