<?php

class OrcamentoEmailForm extends TPage
{
    protected $form;
    private $formFields = [];
    private static $database = '';
    private static $activeRecord = '';
    private static $primaryKey = '';
    private static $formName = 'form_OrcamentoEmailForm';

    /**
     * Form constructor
     * @param $param Request
     */
    public function __construct( $param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);
        // define the form title
        $this->form->setFormTitle("Enviar e-mail de orçamento");

        $criteria_lista_orcamentos = new TCriteria();

        $filterVar = self::getOrcamentos_id($param);
        $criteria_lista_orcamentos->add(new TFilter('id', 'in', $filterVar)); 

        $lista_orcamentos = new TCheckList('lista_orcamentos');
        $email_template_id = new TDBCombo('email_template_id', 'kraftsoft', 'EmailTemplate', 'id', '{titulo}','titulo asc'  );
        $mensagem = new THtmlEditor('mensagem');
        $orcamento_id = new THidden('orcamento_id');

        $email_template_id->setChangeAction(new TAction([$this,'onChangeTemplateEmail']));

        $lista_orcamentos->addValidation("Orçamento", new TRequiredValidator()); 
        $email_template_id->addValidation("Template de E-mail", new TRequiredValidator()); 

        $email_template_id->enableSearch();
        $orcamento_id->setValue($param["orcamento_id"] ?? "");
        $lista_orcamentos->setValue(self::getOrcamentos_id($param));

        $orcamento_id->setSize(200);
        $mensagem->setSize('100%', 200);
        $email_template_id->setSize('100%');

        $lista_orcamentos->setIdColumn('id');

        $column_lista_orcamentos_id = $lista_orcamentos->addColumn('id', "Id", 'center' , '17%');
        $column_lista_orcamentos_data_orcamento_transformed = $lista_orcamentos->addColumn('data_orcamento', "Data", 'center' , '17%');
        $column_lista_orcamentos_cliente_nome = $lista_orcamentos->addColumn('cliente->nome', "Cliente", 'center' , '17%');
        $column_lista_orcamentos_vendedor_nome = $lista_orcamentos->addColumn('vendedor->nome', "Vendedor", 'center' , '17%');
        $column_lista_orcamentos_valor_total_transformed = $lista_orcamentos->addColumn('valor_total', "Valor Total", 'center' , '17%');
        $column_lista_orcamentos_orcamento_situacao_descricao = $lista_orcamentos->addColumn('orcamento_situacao->descricao', "Situação", 'center' , '17%');

        $column_lista_orcamentos_data_orcamento_transformed->setTransformer(function($value, $object, $row) 
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_lista_orcamentos_valor_total_transformed->setTransformer(function($value, $object, $row) 
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });        

        $lista_orcamentos->setHeight(250);
        $lista_orcamentos->makeScrollable();

        $lista_orcamentos->fillWith('kraftsoft', 'Orcamento', 'id', 'id asc' , $criteria_lista_orcamentos);


        $row1 = $this->form->addFields([new TLabel("Orçamentos:", '#F44336', '14px', null, '100%'),$lista_orcamentos]);
        $row1->layout = [' col-sm-12'];

        $row2 = $this->form->addFields([new TLabel("Template de E-mail:", '#F44336', '14px', null),$email_template_id]);
        $row2->layout = [' col-sm-12'];

        $row3 = $this->form->addFields([$mensagem]);
        $row3->layout = [' col-sm-12'];

        $row4 = $this->form->addFields([$orcamento_id]);
        $row4->layout = [' col-sm-12'];

        // create the form actions
        $btn_onenviaremail = $this->form->addAction("Enviar", new TAction([$this, 'onEnviarEmail']), 'fas:rocket #ffffff');
        $this->btn_onenviaremail = $btn_onenviaremail;
        $btn_onenviaremail->addStyleClass('btn-primary'); 

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $this->form->addHeaderWidget($btnClose);

        parent::add($this->form);

        $style = new TStyle('right-panel > .container-part[page-name=OrcamentoEmailForm]');
        $style->width = '60% !important';   
        $style->show(true);

    }

    public static function onChangeTemplateEmail($param = null) 
    {
        try 
        {
            if(!empty($param['key']))
            {
                TTransaction::open('kraftsoft');

                $EmailTemplate = new EmailTemplate($param['key']);

                TTransaction::close();

                $obj = new stdClass();
                $obj->mensagem = $EmailTemplate->mensagem;

                TForm::sendData(self::$formName, $obj);

            }

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public function onEnviarEmail($param = null) 
    {
        try
        {
           $this->form->validate();
           $data = $this->form->getData();
           $mensagem = $data->mensagem;

           if($data->lista_orcamentos)
           {
                TTransaction::open('kraftsoft');
                $EmailTemplate = new EmailTemplate($data->email_template_id);               

                foreach($data->lista_orcamentos as $orcamento_id)
                {
                    $orcamento = new Orcamento($orcamento_id);

                    $mensagem = str_replace('{nome}', $orcamento->cliente->nome, $mensagem);
                    $EmailTemplate->titulo = str_replace('{nome}', $orcamento->cliente->nome, $EmailTemplate->titulo);

                    if($orcamento->cliente->email)
                    {

                        $html = new AdiantiHTMLDocumentParser('app/documents/OrcamentoDocumentTemplate.html');
                        $html->setMaster($orcamento);

                        $criteria_OrcamentoItem_orcamento_id = new TCriteria();
                        $criteria_OrcamentoItem_orcamento_id->add(new TFilter('orcamento_id', '=', $orcamento->id));

                        //<onBeforeGetObjects>

                        $orcamento->data_orcamento = TDate::date2br($orcamento->data_orcamento);

                        //</onBeforeGetObjects>

                        $objectsOrcamentoItem_orcamento_id = OrcamentoItem::getObjects($criteria_OrcamentoItem_orcamento_id);
                        $html->setDetail('OrcamentoItem.orcamento_id', $objectsOrcamentoItem_orcamento_id);

                        $pageSize = 'A4';
                        $document = 'tmp/'.uniqid().'.pdf'; 

                        //<onBeforeProcess>

                        if(!$orcamento->desconto)
                        {
                            $orcamento->desconto = '0.00';
                        }

                        //</onBeforeProcess>

                        $html->process();
                        $html->saveAsPDF($document, $pageSize, 'portrait');

                        //carregando as preferencias de email...
                        $preferences = SystemPreference::getAllPreferences();
                        $host        = $preferences['smtp_host'];
                        $porta       = $preferences['smtp_port'];
                        $usuario     = $preferences['smtp_user'];
                        $senha       = $preferences['smtp_pass'];
                        $email       = $preferences['mail_from'];    
                        $auth        = $preferences['smtp_auth'];

                        $mail = new TMail;
                        $mail->setFrom($email, $email);
                        $mail->setSubject($EmailTemplate->titulo);
                        $mail->setHtmlBody($mensagem);
                        $mail->addAddress($orcamento->cliente->email, $orcamento->cliente->nome);
                        $mail->SetUseSmtp();
                        $mail->SMTPSecure = 'ssl';
                        $mail->SMTPAuth = $auth;
                        $mail->SetSmtpHost($host, $porta);
                        $mail->SetSmtpUser($usuario, $senha);
                        // $mail->addCC('email da pessoa');
                        // $mail->addBCC('email da pessoa');
                        $mail->addAttach($document); // para enviar arquivos
                        $mail->send();                        

                        //teste envio 2..
                        // MailService::send($orcamento->cliente->email, $EmailTemplate->titulo, $mensagem, 'html');
                    }
                }

                TTransaction::close();
           }      

           $this->form->setData($data);
           new TMessage('info', 'Email(s) enviado(s)!');

           if (!$data->lista_orcamentos)
           {
               TSession::setValue('OrcamentoListbuilder_datagrid_check', null);

               TApplication::loadPage('OrcamentoList', 'onShow');
           }

           //fecha a cortina lateral...
           TScript::create("Template.closeRightPanel();");           

        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    }

    public function onShow($param = null)
    {               

    } 

    public static function getOrcamentos_id($param)
    {
        if(!empty($param['orcamento_id']))
        {
            return [ $param['orcamento_id'] ];
        }
        else if(TSession::getValue('OrcamentoListbuilder_datagrid_check'))
        {
            return TSession::getValue('OrcamentoListbuilder_datagrid_check');                     
        }

        return [-1];  
    }

}

