<?php

class OrcamentoList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Orcamento';
    private static $primaryKey = 'id';
    private static $formName = 'form_OrcamentoList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de orçamentos");
        $this->limit = 20;

        $id = new TEntry('id');
        $system_unit_id = new TDBCombo('system_unit_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $orcamento_situacao_id = new TDBCombo('orcamento_situacao_id', 'kraftsoft', 'OrcamentoSituacao', 'id', '{descricao}','descricao asc'  );
        $data_orcamento_inicial = new TDateTime('data_orcamento_inicial');
        $data_orcamento_final = new TDateTime('data_orcamento_final');
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{nome}','nome asc'  );
        $vendedor_id = new TDBCombo('vendedor_id', 'kraftsoft', 'Vendedor', 'id', '{nome}','nome asc'  );


        $data_orcamento_final->setMask('dd/mm/yyyy hh:ii');
        $data_orcamento_inicial->setMask('dd/mm/yyyy hh:ii');

        $data_orcamento_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_orcamento_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');

        $cliente_id->enableSearch();
        $vendedor_id->enableSearch();
        $system_unit_id->enableSearch();
        $orcamento_situacao_id->enableSearch();

        $id->setSize('100%');
        $cliente_id->setSize('100%');
        $vendedor_id->setSize('100%');
        $system_unit_id->setSize('100%');
        $data_orcamento_final->setSize('100%');
        $orcamento_situacao_id->setSize('100%');
        $data_orcamento_inicial->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null, '100%'),$id],[new TLabel("Filial de Saída:", null, '14px', null),$system_unit_id],[new TLabel("Situacao:", null, '14px', null, '100%'),$orcamento_situacao_id]);
        $row1->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row2 = $this->form->addFields([new TLabel("Data Inicial:", null, '14px', null, '100%'),$data_orcamento_inicial],[new TLabel("Data Final:", null, '14px', null, '100%'),$data_orcamento_final]);
        $row2->layout = [' col-sm-4',' col-sm-4'];

        $row3 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null, '100%'),$cliente_id],[new TLabel("Vendedor:", null, '14px', null, '100%'),$vendedor_id],[]);
        $row3->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_data_orcamento_transformed = new TDataGridColumn('data_orcamento', "Data", 'left');
        $column_cliente_nome = new TDataGridColumn('cliente->nome', "Cliente", 'left');
        $column_vendedor_nome = new TDataGridColumn('vendedor->nome', "Vendedor", 'left');
        $column_system_unit_name = new TDataGridColumn('system_unit->name', "Filial de Saída", 'left');
        $column_valor_total_transformed = new TDataGridColumn('valor_total', "Valor Total", 'left');
        $column_orcamento_situacao_descricao_transformed = new TDataGridColumn('orcamento_situacao->descricao', "Situação", 'left');

        $column_data_orcamento_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_valor_total_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_orcamento_situacao_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            //code here
            $label = $object->orcamento_situacao->descricao;
            $cor = $object->orcamento_situacao->cor;

            return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

            // $div = new TElement('div');
            // $div->style="text-shadow:none; font-size:14px; font-weight:bold; color:#FFFFFF; width:160px; text-align:center;background-color: $cor;padding:10px";
            // $div->add($label);

            // return $div;    //code here

        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->builder_datagrid_check_all = new TCheckButton('builder_datagrid_check_all');
        $this->builder_datagrid_check_all->setIndexValue('on');
        $this->builder_datagrid_check_all->onclick = "Builder.checkAll(this)";
        $this->builder_datagrid_check_all->style = 'cursor:pointer';
        $this->builder_datagrid_check_all->setProperty('class', 'filled-in');
        $this->builder_datagrid_check_all->id = 'builder_datagrid_check_all';

        $label = new TLabel('');
        $label->style = 'margin:0';
        $label->class = 'checklist-label';
        $this->builder_datagrid_check_all->after($label);
        $label->for = 'builder_datagrid_check_all';

        $this->builder_datagrid_check = $this->datagrid->addColumn( new TDataGridColumn('builder_datagrid_check', $this->builder_datagrid_check_all, 'center',  '1%') );

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_data_orcamento_transformed);
        $this->datagrid->addColumn($column_cliente_nome);
        $this->datagrid->addColumn($column_vendedor_nome);
        $this->datagrid->addColumn($column_system_unit_name);
        $this->datagrid->addColumn($column_valor_total_transformed);
        $this->datagrid->addColumn($column_orcamento_situacao_descricao_transformed);

        $action_group = new TDataGridActionGroup("", 'fas:cog');
        $action_group->addHeader('');

        $action_onShow = new TDataGridAction(array('OrcamentoFormView', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Abrir");
        $action_onShow->setImage('fas:book-open #00BCD4');
        $action_onShow->setField(self::$primaryKey);

        $action_group->addAction($action_onShow);

        $action_onEdit = new TDataGridAction(array('OrcamentoForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #478fca');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('OrcamentoList::onMostrarEditar');

        $action_group->addAction($action_onEdit);

        $action_onGerarVenda = new TDataGridAction(array('OrcamentoList', 'onGerarVenda'));
        $action_onGerarVenda->setUseButton(TRUE);
        $action_onGerarVenda->setButtonClass('btn btn-default');
        $action_onGerarVenda->setLabel("Aprovar");
        $action_onGerarVenda->setImage('fas:thumbs-up #4CAF50');
        $action_onGerarVenda->setField(self::$primaryKey);
        $action_onGerarVenda->setDisplayCondition('OrcamentoList::onMostrarAprovado');

        $action_group->addAction($action_onGerarVenda);

        $action_onReprovar = new TDataGridAction(array('OrcamentoList', 'onReprovar'));
        $action_onReprovar->setUseButton(TRUE);
        $action_onReprovar->setButtonClass('btn btn-default');
        $action_onReprovar->setLabel("Reprovar");
        $action_onReprovar->setImage('fas:thumbs-down #FF5722');
        $action_onReprovar->setField(self::$primaryKey);
        $action_onReprovar->setDisplayCondition('OrcamentoList::onMostrarReprovado');

        $action_group->addAction($action_onReprovar);

        $action_onCancelar = new TDataGridAction(array('OrcamentoList', 'onCancelar'));
        $action_onCancelar->setUseButton(TRUE);
        $action_onCancelar->setButtonClass('btn btn-default');
        $action_onCancelar->setLabel("Cancelar");
        $action_onCancelar->setImage('fas:window-close #CE0D0D');
        $action_onCancelar->setField(self::$primaryKey);
        $action_onCancelar->setDisplayCondition('OrcamentoList::onMostrarCancelado');

        $action_group->addAction($action_onCancelar);

        $action_onGenerate = new TDataGridAction(array('OrcamentoDocument', 'onGenerate'));
        $action_onGenerate->setUseButton(TRUE);
        $action_onGenerate->setButtonClass('btn btn-default');
        $action_onGenerate->setLabel("Imprimir");
        $action_onGenerate->setImage('fas:print #FF5722');
        $action_onGenerate->setField(self::$primaryKey);

        $action_group->addAction($action_onGenerate);

        $action_onClonar = new TDataGridAction(array('OrcamentoList', 'onClonar'));
        $action_onClonar->setUseButton(TRUE);
        $action_onClonar->setButtonClass('btn btn-default');
        $action_onClonar->setLabel("Clonar");
        $action_onClonar->setImage('fas:copy #FFE821');
        $action_onClonar->setField(self::$primaryKey);

        $action_group->addAction($action_onClonar);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup("Listagem de orçamentos");
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $button_cadastrar = new TButton('button_button_cadastrar');
        $button_cadastrar->setAction(new TAction(['OrcamentoForm', 'onShow']), "Cadastrar");
        $button_cadastrar->addStyleClass('btn-default');
        $button_cadastrar->setImage('fas:plus #69aa46');

        $this->datagrid_form->addField($button_cadastrar);

        $btnShowCurtainFilters = new TButton('button_btnShowCurtainFilters');
        $btnShowCurtainFilters->setAction(new TAction(['OrcamentoList', 'onShowCurtainFilters']), "Filtros");
        $btnShowCurtainFilters->addStyleClass('btn-default');
        $btnShowCurtainFilters->setImage('fas:filter #000000');

        $this->datagrid_form->addField($btnShowCurtainFilters);

        $button_limpar_filtros = new TButton('button_button_limpar_filtros');
        $button_limpar_filtros->setAction(new TAction(['OrcamentoList', 'onClearFilters']), "Limpar filtros");
        $button_limpar_filtros->addStyleClass('btn-default');
        $button_limpar_filtros->setImage('fas:eraser #f44336');

        $this->datagrid_form->addField($button_limpar_filtros);

        $button_atualizar = new TButton('button_button_atualizar');
        $button_atualizar->setAction(new TAction(['OrcamentoList', 'onRefresh']), "Atualizar");
        $button_atualizar->addStyleClass('btn-default');
        $button_atualizar->setImage('fas:sync-alt #03a9f4');

        $this->datagrid_form->addField($button_atualizar);

        $button_enviar_e_mails = new TButton('button_button_enviar_e_mails');
        $button_enviar_e_mails->setAction(new TAction(['OrcamentoEmailForm', 'onShow']), "Enviar E-mails");
        $button_enviar_e_mails->addStyleClass('btn-default');
        $button_enviar_e_mails->setImage('fas:envelope #E91E63');

        $this->datagrid_form->addField($button_enviar_e_mails);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['OrcamentoList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['OrcamentoList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['OrcamentoList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['OrcamentoList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_left_actions->add($button_cadastrar);
        $head_left_actions->add($btnShowCurtainFilters);
        $head_left_actions->add($button_limpar_filtros);
        $head_left_actions->add($button_atualizar);
        $head_left_actions->add($button_enviar_e_mails);

        $head_right_actions->add($dropdown_button_exportar);

        $this->btnShowCurtainFilters = $btnShowCurtainFilters;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Vendas","Orçamentos"]));
        }

        $container->add($panel);

        parent::add($container);

    }

    public static function onMostrarEditar($object)
    {
        try 
        {
            if($object->orcamento_situacao_id==OrcamentoSituacao::Aprovado
            or $object->orcamento_situacao_id==OrcamentoSituacao::Cancelado
            or $object->orcamento_situacao_id==OrcamentoSituacao::Reprovado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onGerarVenda($param = null) 
    {
        try 
        {
            //code here
            new TQuestion("Deseja aprovar o orçamento e gerar a venda?", new TAction([__CLASS__, 'onGerarVendaYes'], $param), new TAction([__CLASS__, 'onGerarVendaNo']));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarAprovado($object)
    {
        try 
        {
            if($object->orcamento_situacao_id==OrcamentoSituacao::Aprovado
            or $object->orcamento_situacao_id==OrcamentoSituacao::Cancelado
            or $object->orcamento_situacao_id==OrcamentoSituacao::Reprovado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onReprovar($param = null) 
    {
        try 
        {
            //code here
            $pageParam = ['id' => $param['key']];
            TApplication::loadPage('ReprovarOrcamentoForm', 'onShow', $pageParam);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarReprovado($object)
    {
        try 
        {
            if($object->orcamento_situacao_id==OrcamentoSituacao::Aprovado
            or $object->orcamento_situacao_id==OrcamentoSituacao::Cancelado
            or $object->orcamento_situacao_id==OrcamentoSituacao::Reprovado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onCancelar($param = null) 
    {
        try 
        {
            //code here
            new TQuestion("Deseja cancelar o orçamento?", new TAction([__CLASS__, 'onCancelarYes'], $param), new TAction([__CLASS__, 'onCancelarNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarCancelado($object)
    {
        try 
        {
            if($object->orcamento_situacao_id==OrcamentoSituacao::Aprovado
            or $object->orcamento_situacao_id==OrcamentoSituacao::Reprovado
            or $object->orcamento_situacao_id==OrcamentoSituacao::Cancelado)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onClonar($param = null) 
    {
        try 
        {
            //code here
            new TQuestion("Deseja clonar Orçamento?", new TAction([__CLASS__, 'onClonarYes'], $param), new TAction([__CLASS__, 'onClonarNo'], $param));
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $transformer = $column->getTransformer();
                            if ($transformer)
                            {
                                $value = strip_tags(call_user_func($transformer, $value, $object, null));
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('iframe');
                $object->src  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }
    public static function onShowCurtainFilters($param = null) 
    {
        try 
        {
            //code here

                        $filter = new self([]);

            $btnClose = new TButton('closeCurtain');
            $btnClose->class = 'btn btn-sm btn-default';
            $btnClose->style = 'margin-right:10px;';
            $btnClose->onClick = "Template.closeRightPanel();";
            $btnClose->setLabel("Fechar");
            $btnClose->setImage('fas:times');

            $filter->form->addHeaderWidget($btnClose);

            $page = new TPage();
            $page->setTargetContainer('adianti_right_panel');
            $page->setProperty('page-name', 'OrcamentoListSearch');
            $page->setProperty('page_name', 'OrcamentoListSearch');
            $page->adianti_target_container = 'adianti_right_panel';
            $page->target_container = 'adianti_right_panel';
            $page->add($filter->form);
            $page->setIsWrapped(true);
            $page->show();

            $style = new TStyle('right-panel > .container-part[page-name=OrcamentoListSearch]');
            $style->width = '70% !important';
            $style->show(true);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onClearFilters($param = null) 
    {
        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }
    public function onRefresh($param = null) 
    {
        $this->onReload([]);
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->system_unit_id) AND ( (is_scalar($data->system_unit_id) AND $data->system_unit_id !== '') OR (is_array($data->system_unit_id) AND (!empty($data->system_unit_id)) )) )
        {

            $filters[] = new TFilter('system_unit_id', '=', $data->system_unit_id);// create the filter 
        }

        if (isset($data->orcamento_situacao_id) AND ( (is_scalar($data->orcamento_situacao_id) AND $data->orcamento_situacao_id !== '') OR (is_array($data->orcamento_situacao_id) AND (!empty($data->orcamento_situacao_id)) )) )
        {

            $filters[] = new TFilter('orcamento_situacao_id', '=', $data->orcamento_situacao_id);// create the filter 
        }

        if (isset($data->data_orcamento_inicial) AND ( (is_scalar($data->data_orcamento_inicial) AND $data->data_orcamento_inicial !== '') OR (is_array($data->data_orcamento_inicial) AND (!empty($data->data_orcamento_inicial)) )) )
        {

            $filters[] = new TFilter('data_orcamento', '>=', $data->data_orcamento_inicial);// create the filter 
        }

        if (isset($data->data_orcamento_final) AND ( (is_scalar($data->data_orcamento_final) AND $data->data_orcamento_final !== '') OR (is_array($data->data_orcamento_final) AND (!empty($data->data_orcamento_final)) )) )
        {

            $filters[] = new TFilter('data_orcamento', '<=', $data->data_orcamento_final);// create the filter 
        }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }

        if (isset($data->vendedor_id) AND ( (is_scalar($data->vendedor_id) AND $data->vendedor_id !== '') OR (is_array($data->vendedor_id) AND (!empty($data->vendedor_id)) )) )
        {

            $filters[] = new TFilter('vendedor_id', '=', $data->vendedor_id);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Orcamento
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            $session_checks = TSession::getValue(__CLASS__.'builder_datagrid_check');

            //</blockLine><btnShowCurtainFiltersAutoCode>
            if(!empty($this->btnShowCurtainFilters) && empty($this->btnShowCurtainFiltersAdjusted))
            {
                $this->btnShowCurtainFiltersAdjusted = true;
                $this->btnShowCurtainFilters->style = 'position: relative';
                $countFilters = count($filters ?? []);
                $this->btnShowCurtainFilters->setLabel($this->btnShowCurtainFilters->getLabel(). "<span class='badge badge-success' style='position: absolute'>{$countFilters}<span>");
            }
            //</blockLine></btnShowCurtainFiltersAutoCode>

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {
                    $check = new TCheckGroup('builder_datagrid_check');
                    $check->addItems([$object->id => '']);
                    $check->getButtons()[$object->id]->onclick = 'event.stopPropagation()';

                    if(!$this->datagrid_form->getField('builder_datagrid_check[]'))
                    {
                        $this->datagrid_form->setFields([$check]);
                    }

                    $check->setChangeAction(new TAction([$this, 'builderSelectCheck']));
                    $object->builder_datagrid_check = $check;

                    if(!empty($session_checks[$object->id]))
                    {
                        $object->builder_datagrid_check->setValue([$object->id=>$object->id]);
                    }

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function onGerarVendaYes($param = null) 
    {
        try 
        {
            //code here
            TTransaction::open(self::$database);

            $orcamento = Orcamento::find($param['id']);

            if($orcamento)
            {
                Orcamento::GerarVenda($orcamento);
                $orcamento->store();

                OrcamentoSituacaoHistorico::LancarHistorico(OrcamentoSituacao::Aprovado, $orcamento->id, 'Mudança de Situação do Orçamento: Aprovado');
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToas::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onGerarVendaNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onCancelarYes($param = null) 
    {
        try 
        {
            //code here
            TTransaction::open(self::$database);

            $orcamento = Orcamento::find($param['id']);

            if($orcamento)
            {
                $orcamento->orcamento_situacao_id = OrcamentoSituacao::Cancelado;
                $orcamento->store();

                OrcamentoSituacaoHistorico::LancarHistorico(OrcamentoSituacao::Cancelado, $orcamento->id, 'Mudança de Situação do orçamento: Cancelado');
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');
            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onCancelarNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onClonarYes($param = null) 
    {
        try 
        {
            //code here
            TTransaction::open(self::$database);

            Orcamento::ClonarOrcamento($param['id']);

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show("success", "Orçamento clonado com sucesso!", "topCenter", "fas:check-circle");
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onClonarNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function builderSelectCheck($param)
    {
        $session_checks = TSession::getValue(__CLASS__.'builder_datagrid_check');

        $valueOn = null;
        if(!empty($param['_field_data_json']))
        {
            $obj = json_decode($param['_field_data_json']);
            if($obj)
            {
                $valueOn = $obj->valueOn;
            }
        }

        $key = empty($param['key']) ? $valueOn : $param['key'];

        if(empty($param['builder_datagrid_check']) && !empty($session_checks[$key]))
        {
            unset($session_checks[$key]);
        }
        elseif(!empty($param['builder_datagrid_check']) && !in_array($key, $param['builder_datagrid_check']) && !empty($session_checks[$key]))
        {
            unset($session_checks[$key]);
        }
        elseif(!empty($param['builder_datagrid_check']) && in_array($key, $param['builder_datagrid_check']))
        {
            $session_checks[$key] = $key;
        }

        TSession::setValue(__CLASS__.'builder_datagrid_check', $session_checks);
    }

}

