<?php

class ProdutoSeekWindow extends TWindow
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Produto';
    private static $primaryKey = 'id';
    private static $formName = 'form_ProdutoSeekWindow';
    private $showMethods = ['onReload', 'onSearch'];
    private $limit = 20;

    use BuilderSeekWindowTrait;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();
        parent::setSize(0.8, null);
        parent::setTitle("Busca de Produtos");
        parent::setProperty('class', 'window_modal');

        $param['_seek_window_id'] = $this->id;
        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        $this->limit = 20;

        // define the form title
        $this->form->setFormTitle("Busca de Produtos");

        $id = new TEntry('id');
        $referencia = new TEntry('referencia');
        $nome = new TEntry('nome');
        $codigo_EAN = new TEntry('codigo_EAN');
        $produto_grupo_id = new TDBCombo('produto_grupo_id', 'kraftsoft', 'ProdutoGrupo', 'id', '{descricao}','descricao asc'  );
        $produtogrupo_subgrupo_id = new TCombo('produtogrupo_subgrupo_id');

        $produto_grupo_id->setChangeAction(new TAction([$this,'onChangeproduto_grupo_id']));

        $produto_grupo_id->enableSearch();
        $produtogrupo_subgrupo_id->enableSearch();

        $id->setSize('100%');
        $nome->setSize('100%');
        $referencia->setSize('100%');
        $codigo_EAN->setSize('100%');
        $produto_grupo_id->setSize('100%');
        $produtogrupo_subgrupo_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null),$id],[new TLabel("Referência:", null, '14px', null),$referencia]);
        $row1->layout = [' col-sm-3',' col-sm-3'];

        $row2 = $this->form->addFields([new TLabel("Nome:", null, '14px', null, '100%'),$nome],[new TLabel("Código de barras (EAN):", null, '14px', null),$codigo_EAN]);
        $row2->layout = ['col-sm-6','col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Grupo de produto:", null, '14px', null),$produto_grupo_id],[new TLabel("Sub-Grupo de produto:", null, '14px', null),$produtogrupo_subgrupo_id]);
        $row3->layout = ['col-sm-6','col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );
        $this->fireEvents( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        $this->setSeekParameters($btn_onsearch->getAction(), $param);

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = $this->getSeekFiltersCriteria($param);

        $filterVar = Ativo::Ativo;
        $this->filter_criteria->add(new TFilter('habilitado_id', '=', $filterVar));
        $filterVar = TipoRecurso::Produto;
        $this->filter_criteria->add(new TFilter('tipo_recurso_id', '=', $filterVar));

        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(320);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_referencia = new TDataGridColumn('referencia', "Referência", 'left');
        $column_nome = new TDataGridColumn('nome', "Nome", 'left');
        $column_unidade_medida_nome = new TDataGridColumn('unidade_medida->nome', "Unidade", 'left');
        $column_produto_grupo_descricao = new TDataGridColumn('produto_grupo->descricao', "Grupo", 'left');
        $column_alterado_em_transformed = new TDataGridColumn('alterado_em', "Valor", 'right');
        $column_alterado_por_transformed = new TDataGridColumn('alterado_por', "Quant. Disponível", 'right');
        $column_criado_em_transformed = new TDataGridColumn('criado_em', "Quant. Reservada", 'right');

        $column_alterado_em_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_alterado_por_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {

                    if(!$value)
                    {
                        $value = 0;
                    }

                    if(is_numeric($value))
                    {
                        return number_format($value, 2, ",", ".");
                    }
                    else
                    {
                        return $value;
                    }

        });

        $column_criado_em_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {

                    if(!$value)
                    {
                        $value = 0;
                    }

                    if(is_numeric($value))
                    {
                        return number_format($value, 2, ",", ".");
                    }
                    else
                    {
                        return $value;
                    }

        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $this->setSeekParameters($order_id, $param);
        $column_id->setAction($order_id);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_referencia);
        $this->datagrid->addColumn($column_nome);
        $this->datagrid->addColumn($column_unidade_medida_nome);
        $this->datagrid->addColumn($column_produto_grupo_descricao);
        $this->datagrid->addColumn($column_alterado_em_transformed);
        $this->datagrid->addColumn($column_alterado_por_transformed);
        $this->datagrid->addColumn($column_criado_em_transformed);

        $action_onSelect = new TDataGridAction(array('ProdutoSeekWindow', 'onSelect'));
        $action_onSelect->setUseButton(true);
        $action_onSelect->setButtonClass('btn btn-default btn-sm');
        $action_onSelect->setLabel("Selecionar");
        $action_onSelect->setImage('far:hand-pointer #44bd32');
        $action_onSelect->setField(self::$primaryKey);
        $this->setSeekParameters($action_onSelect, $param);

        $this->datagrid->addAction($action_onSelect);

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $navigationAction = new TAction(array($this, 'onReload'));
        $this->setSeekParameters($navigationAction, $param);
        $this->pageNavigation->setAction($navigationAction);
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        parent::add($this->form);
        parent::add($panel);

    }

    public static function onChangeproduto_grupo_id($param)
    {
        try
        {

            if (isset($param['produto_grupo_id']) && $param['produto_grupo_id'])
            { 
                $criteria = TCriteria::create(['produto_grupo_id' => $param['produto_grupo_id']]);
                TDBCombo::reloadFromModel(self::$formName, 'produtogrupo_subgrupo_id', 'kraftsoft', 'ProdutogrupoSubgrupo', 'id', '{descricao}', 'descricao asc', $criteria, TRUE); 
            } 
            else 
            { 
                TCombo::clearField(self::$formName, 'produtogrupo_subgrupo_id'); 
            }  

        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
        }
    } 

    public static function onSelect($param = null) 
    { 
        try 
        {   
            $seekFields = self::getSeekFields($param);
            $formData = new stdClass();

            if(!empty($param['key']))
            {
                TTransaction::open(self::$database);

                $repository = new TRepository(self::$activeRecord);

                $criteria = self::getSeekFiltersCriteria($param);

                $filterVar = Ativo::Ativo;
                $criteria->add(new TFilter('habilitado_id', '=', $filterVar));

                $filterVar = TipoRecurso::Produto;
                $criteria->add(new TFilter('tipo_recurso_id', '=', $filterVar));

                $criteria->add(new TFilter('id', '=', $param['key']));
                $objects = $repository->load($criteria);

                if($objects)
                {
                    foreach ($objects as $object) 
                    {
                        //$object = $objects[];
                        if($seekFields)
                        {
                            foreach ($seekFields as $seek_field) 
                            {
    
                                $formData->{"{$seek_field['name']}"} = $object->render("{$seek_field['column']}");
                            }
                        }
                    }    
                }
                elseif($seekFields)
                {
                    foreach ($seekFields as $seek_field) 
                    {
                        $formData->{"{$seek_field['name']}"} = '';
                    }   
                }
                TTransaction::close();
            }
            else
            {
                if($seekFields)
                {
                    foreach ($seekFields as $seek_field) 
                    {
                        $formData->{"{$seek_field['name']}"} = '';
                    }   
                }
            }

            TForm::sendData($param['_form_name'], $formData);

            if(!empty($param['_seek_window_id']))
            {
                TWindow::closeWindow($param['_seek_window_id']);
            }
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public function fireEvents( $object )
    {
        $obj = new stdClass;
        if(is_object($object) && get_class($object) == 'stdClass')
        {
            if(isset($object->produto_grupo_id))
            {
                $value = $object->produto_grupo_id;

                $obj->produto_grupo_id = $value;
            }
            if(isset($object->produtogrupo_subgrupo_id))
            {
                $value = $object->produtogrupo_subgrupo_id;

                $obj->produtogrupo_subgrupo_id = $value;
            }
        }
        elseif(is_object($object))
        {
            if(isset($object->produto_grupo_id))
            {
                $value = $object->produto_grupo_id;

                $obj->produto_grupo_id = $value;
            }
            if(isset($object->produtogrupo_subgrupo_id))
            {
                $value = $object->produtogrupo_subgrupo_id;

                $obj->produtogrupo_subgrupo_id = $value;
            }
        }
        TForm::sendData(self::$formName, $obj);
    }  

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        // get the search form data
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->referencia) AND ( (is_scalar($data->referencia) AND $data->referencia !== '') OR (is_array($data->referencia) AND (!empty($data->referencia)) )) )
        {

            $filters[] = new TFilter('referencia', 'like', "%{$data->referencia}%");// create the filter 
        }

        if (isset($data->nome) AND ( (is_scalar($data->nome) AND $data->nome !== '') OR (is_array($data->nome) AND (!empty($data->nome)) )) )
        {

            $filters[] = new TFilter('nome', 'like', "%{$data->nome}%");// create the filter 
        }

        if (isset($data->codigo_EAN) AND ( (is_scalar($data->codigo_EAN) AND $data->codigo_EAN !== '') OR (is_array($data->codigo_EAN) AND (!empty($data->codigo_EAN)) )) )
        {

            $filters[] = new TFilter('codigo_EAN', '=', $data->codigo_EAN);// create the filter 
        }

        if (isset($data->produto_grupo_id) AND ( (is_scalar($data->produto_grupo_id) AND $data->produto_grupo_id !== '') OR (is_array($data->produto_grupo_id) AND (!empty($data->produto_grupo_id)) )) )
        {

            $filters[] = new TFilter('produto_grupo_id', '=', $data->produto_grupo_id);// create the filter 
        }

        if (isset($data->produtogrupo_subgrupo_id) AND ( (is_scalar($data->produtogrupo_subgrupo_id) AND $data->produtogrupo_subgrupo_id !== '') OR (is_array($data->produtogrupo_subgrupo_id) AND (!empty($data->produtogrupo_subgrupo_id)) )) )
        {

            $filters[] = new TFilter('produtogrupo_subgrupo_id', '=', $data->produtogrupo_subgrupo_id);// create the filter 
        }

        $this->fireEvents($data);

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        if (isset($param['static']) && ($param['static'] == '1') )
        {
            $class = get_class($this);
            AdiantiCoreApplication::loadPage($class, 'onReload', ['offset' => 0, 'first_page' => 1]);
        }
        else
        {
            $this->onReload(['offset' => 0, 'first_page' => 1]);
        }
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Produto
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }
            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {
                    // add the object inside the datagrid

                    //preço de venda    
                    $preco_venda = '0,00';                             
                    $prod_filial = ProdutoFilial::where('produto_id', '=', $object->id)
                                                 ->where('system_unit_id', '=', TSession::getValue('userunitid'))->first();

                    if ($prod_filial)                             
                    {
                       $preco_venda = $prod_filial->preco_venda;
                    }

                    $object->alterado_em = $preco_venda;

                    //estoque..
                    $qtd_disponivel = 0;
                    $qtd_reservada = 0;

                    $qtd_disponivel = (double) EstoqueSaldos::where('produto_id', '=', $object->id)
                                                              ->where('system_unit_id', '=', TSession::getValue('userunitid'))
                                                              ->sumby('quant_disponivel');

                    /*
                    $qtd_disponivel = (double) EstoqueLancamentos::where('produto_id', '=', $object->id)
                                                                ->where('system_unit_id', '=', TSession::getValue('userunitid'))
                                                                ->sumby('quantidade');
                    */

                    $qtd_reservada = (double) EstoqueSaldos::where('produto_id', '=', $object->id)
                                                              ->where('system_unit_id', '=', TSession::getValue('userunitid'))
                                                              ->sumby('quant_reservada');     

                     //passando os valores 
                     $object->alterado_por = $qtd_disponivel;
                     $object->criado_em = $qtd_reservada;

                    $this->datagrid->addItem($object);

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

}

