<?php

class VendaFaturamentoSimpleList extends TPage
{

    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'VendaFormapgto';
    private static $primaryKey = 'id';
    private static $formName = 'formList_VendaFormapgto';
    private $limit = 20;

    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        $this->limit = 0;

        $this->form = new BootstrapFormBuilder(self::$formName);

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(320);

        $column_forma_pagamento_descricao = new TDataGridColumn('forma_pagamento->descricao', "Forma de Pagamento", 'left');
        $column_valor_transformed = new TDataGridColumn('valor', "Valor", 'left');

        $column_valor_transformed->setTotalFunction( function($values) { 
            return array_sum((array) $values); 
        }); 

        $column_valor_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });        

        $this->datagrid->addColumn($column_forma_pagamento_descricao);
        $this->datagrid->addColumn($column_valor_transformed);


        // create the datagrid model
        $this->datagrid->createModel();

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        parent::setTargetContainer('adianti_right_panel');

        $btnClose = new TButton('closeCurtain');
        $btnClose->class = 'btn btn-sm btn-default';
        $btnClose->style = 'margin-right:10px;';
        $btnClose->onClick = "Template.closeRightPanel();";
        $btnClose->setLabel("Fechar");
        $btnClose->setImage('fas:times');

        $form = new BootstrapFormBuilder(self::$formName);
        $form->setFormTitle('&nbsp;');
        $form->addContent([$panel]);
        $form->addHeaderWidget($btnClose);

        $form->setFormTitle('Faturamento de Venda');

        $button = TButton::create('acao_confirmar', [$this, 'confirmar'], 'Confirmar Faturamento', 'fa:check green');

        //trazer apenas os usuários com caixas abertos (operador)
        $criteria = new TCriteria;
        $criteria->add(new TFilter('caixa_situacao_id', '=', CaixaSituacao::Aberto));

        $dbcombo = new TDBCombo('caixa_id', 'kraftsoft', 'Caixa', 'id', 'Caixa id: {id} - Operador: {system_users_operador->name}','id asc', $criteria);

        $tlabel = new TLabel('Caixa Faturamento: ');

        $this->form->setFields([$button,$dbcombo]);

        $panel->add($tlabel);
        $panel->add($dbcombo);
        $panel->add($button);    

        parent::add($form);

        $style = new TStyle('right-panel > .container-part[page-name=VendaFaturamentoSimpleList]');
        $style->width = '50% !important';   
        $style->show(true);

    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for VendaFormapgto
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }
            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            TSession::setvalue('venda_id',$param['venda_id']); 
            $criteria->add(new TFilter('venda_id', '=', $param['venda_id']));

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  array('onReload', 'onSearch')))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function onConfirmarYes($param = null) 
    {
        try 
        {
            //var_dump($param);

            if ($param['caixa_id'] =='')
            {
               throw new Exception('Selecione um caixa aberto!');                 
            }

            TTransaction::open(self::$database);

            $venda = Venda::find(TSession::getValue('venda_id'));     
            $caixa = Caixa::find($param['caixa_id']);

            if($venda) 
            {

                $config_filial = ConfiguracoesFilial::where('system_unit_id', '=', TSession::getValue('userunitid'))->first();

                if ($config_filial)
                {
                   if ($config_filial->venda_apenas_com_estoque_id == Resposta::Sim) 
                   {
                     Venda::ValidaVendaSemEstoque($venda->id);                                                
                   }
                }                

                //tirando da reserva caso o estoque tenha sido reservado
                $lancaestoque = EstoqueLancamentos::where('venda_id', '=', $venda->id)
                                                  ->where('estoque_origem_id', '=', EstoqueOrigem::ReservaSaldo)->first();    

                if ($lancaestoque)
                {
                    VendaItem::LancamentoEstoque(EstoqueTipoMovimentacao::SaidaReserva,
                                                 EstoqueOrigem::RetornoReserva, 
                                                 $venda,
                                                 'Ret. reserva venda Id: '.$venda->id);           
                }

                //lançando o estoque...
                VendaItem::LancamentoEstoque(EstoqueTipoMovimentacao::Saida,
                                             EstoqueOrigem::Vendas, 
                                             $venda,
                                             'Saida venda Id: '.$venda->id);                                         

                $venda->venda_situacao_id=VendaSituacao::Finalizada;
                $venda->store();

                //historico de situação de venda..
                VendaSituacaoHistorico::LancarHistorico(VendaSituacao::Finalizada,
                                        $venda->id,
                                        'Mudança de Situação de venda: Finalizada');

                Venda::AtualizaResumoVendasCliente($venda->cliente_id);                        

                $pagamentos = VendaFormapgto::where('venda_id', '=', $venda->id)->load();

                foreach($pagamentos as $pag)
                {
                    //$caixaId,$formapgtoId,$caixatipolancamentoID,$valor,$obs,$origemId,$systemuserID
                    $obs = 'Pgto Venda Nº '.$venda->id;

                    Caixa::lanca_caixa($caixa->id,
                                       $pag->forma_pagamento_id,
                                       CaixaTipoLancamento::Credito,
                                       $pag->valor,
                                       $obs,
                                       $venda->id,
                                       TSession::getValue('userid'));

                    //mudando o contas a receber caso tenha para ativo..                   
                    $lista_contasrec = Contasrec::where('venda_formapgto_id', '=', $pag->id)->load();

                    foreach ($lista_contasrec as $crec) 
                    {            
                      $crec->ativo_id = Ativo::Ativo;  
                      $crec->store();
                    }                                            

                }                
            }

            TTransaction::close();

            //evento de estoque minimo...            
            TTransaction::open(self::$database);

            VendaItem::VerificaProdutosAbaixoDoMinimo($venda);

            TTransaction::close();

        //   $pageParam = ['key' => $venda->id];
        //   TApplication::loadPage('VendaFormView', 'OnShow', $pageParam);

          $pageParam = [];
          TApplication::loadPage('VendaList', 'OnShow', $pageParam);

           TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle'); 
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onConfirmarNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function confirmar($param)
    {
        new TQuestion("Deseja confirmar o faturamento?", new TAction([__CLASS__, 'onConfirmarYes'], $param), new TAction([__CLASS__, 'onConfirmarNo'], $param));
    }

}

