<?php

class VendaItemProdutoSimpleList extends TPage
{

    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'VendaItem';
    private static $primaryKey = 'id';
    private static $formName = 'formList_VendaItem';
    private $limit = 20;

    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        $this->limit = 20;

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);

        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(320);

        $column_venda_id = new TDataGridColumn('venda_id', "Id Venda", 'left');
        $column_venda_cliente_system_unit_name = new TDataGridColumn('venda->cliente->system_unit->name', "Filial", 'left');
        $column_venda_data_hora_venda_transformed = new TDataGridColumn('venda->data_hora_venda', "Data/Hora", 'left');
        $column_venda_cliente_nome = new TDataGridColumn('venda->cliente->nome', "Cliente", 'left');
        $column_venda_cliente_vendedor_nome = new TDataGridColumn('venda->cliente->vendedor->nome', "Vendedor", 'left');
        $column_quantidade_transformed = new TDataGridColumn('quantidade', "Quantidade", 'left');
        $column_valor_unitario_transformed = new TDataGridColumn('valor_unitario', "Valor Unitário", 'left');
        $column_venda_valor_total_transformed = new TDataGridColumn('venda->valor_total', "Total Venda", 'left');

        $column_venda_data_hora_venda_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_quantidade_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {

                    if(!$value)
                    {
                        $value = 0;
                    }

                    if(is_numeric($value))
                    {
                        return number_format($value, 2, ",", ".");
                    }
                    else
                    {
                        return $value;
                    }

        });

        $column_valor_unitario_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_venda_valor_total_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });        

        $this->datagrid->addColumn($column_venda_id);
        $this->datagrid->addColumn($column_venda_cliente_system_unit_name);
        $this->datagrid->addColumn($column_venda_data_hora_venda_transformed);
        $this->datagrid->addColumn($column_venda_cliente_nome);
        $this->datagrid->addColumn($column_venda_cliente_vendedor_nome);
        $this->datagrid->addColumn($column_quantidade_transformed);
        $this->datagrid->addColumn($column_valor_unitario_transformed);
        $this->datagrid->addColumn($column_venda_valor_total_transformed);


        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup();
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Vendas","Vendas de produto"]));
        }
        $container->add($panel);

        parent::add($container);

    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for VendaItem
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            if(!empty($param["produto_id"] ?? ""))
        {
            TSession::setValue(__CLASS__.'load_filter_produto_id', $param["produto_id"] ?? "");
        }
        $filterVar = TSession::getValue(__CLASS__.'load_filter_produto_id');
            $criteria->add(new TFilter('produto_id', '=', $filterVar));
            $filterVar = VendaSituacao::Finalizada;
            $criteria->add(new TFilter('venda_id', 'in', "(SELECT id FROM venda WHERE venda_situacao_id = '{$filterVar}')"));

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }
            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  array('onReload', 'onSearch')))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

}

