<?php

class VendaItemReport extends TPage
{
    private $form; // form
    private $loaded;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'VendaItem';
    private static $primaryKey = 'id';
    private static $formName = 'form_VendaItemReport';

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Relatório De Lucratividade");

        $criteria_produto_id = new TCriteria();

        $filterVar = TipoRecurso::Produto;
        $criteria_produto_id->add(new TFilter('tipo_recurso_id', '=', $filterVar)); 

        $venda_id = new TEntry('venda_id');
        $venda_venda_situacao_id = new TDBCombo('venda_venda_situacao_id', 'kraftsoft', 'VendaSituacao', 'id', '{descricao}','ordem asc'  );
        $produto_id = new TDBCombo('produto_id', 'kraftsoft', 'Produto', 'id', '{nome}','nome asc' , $criteria_produto_id );
        $venda_system_unit_id = new TDBCombo('venda_system_unit_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $data_venda_inicial = new TDate('data_venda_inicial');
        $data_venda_final = new TDate('data_venda_final');

        $venda_system_unit_id->setValue(TSession::getValue('userunitid'));
        $data_venda_final->setMask('dd/mm/yyyy');
        $data_venda_inicial->setMask('dd/mm/yyyy');

        $data_venda_final->setDatabaseMask('yyyy-mm-dd');
        $data_venda_inicial->setDatabaseMask('yyyy-mm-dd');

        $produto_id->enableSearch();
        $venda_system_unit_id->enableSearch();
        $venda_venda_situacao_id->enableSearch();

        $venda_id->setSize('100%');
        $produto_id->setSize('100%');
        $data_venda_final->setSize('100%');
        $data_venda_inicial->setSize('100%');
        $venda_system_unit_id->setSize('100%');
        $venda_venda_situacao_id->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Venda Id:", null, '14px', null),$venda_id],[new TLabel("Venda Situação:", null, '14px', null),$venda_venda_situacao_id]);
        $row1->layout = ['col-sm-6',' col-sm-6'];

        $row2 = $this->form->addFields([new TLabel("Produto:", null, '14px', null, '100%'),$produto_id],[new TLabel("Filial:", null, '14px', null),$venda_system_unit_id]);
        $row2->layout = [' col-sm-6',' col-sm-6'];

        $row3 = $this->form->addFields([new TLabel("Data Venda Inicial:", null, '14px', null),$data_venda_inicial],[new TLabel("Data Venda Final:", null, '14px', null),$data_venda_final]);
        $row3->layout = [' col-sm-6','col-sm-6'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_ongeneratehtml = $this->form->addAction("Gerar HTML", new TAction([$this, 'onGenerateHtml']), 'far:file-code #ffffff');
        $this->btn_ongeneratehtml = $btn_ongeneratehtml;
        $btn_ongeneratehtml->addStyleClass('btn-primary'); 

        $btn_ongeneratepdf = $this->form->addAction("Gerar PDF", new TAction([$this, 'onGeneratePdf']), 'far:file-pdf #d44734');
        $this->btn_ongeneratepdf = $btn_ongeneratepdf;

        $btn_ongeneratexls = $this->form->addAction("Gerar XLS", new TAction([$this, 'onGenerateXls']), 'far:file-excel #00a65a');
        $this->btn_ongeneratexls = $btn_ongeneratexls;

        $btn_ongeneratertf = $this->form->addAction("Gerar RTF", new TAction([$this, 'onGenerateRtf']), 'far:file-alt #324bcc');
        $this->btn_ongeneratertf = $btn_ongeneratertf;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        $container->class = 'form-container';
        $container->add(TBreadCrumb::create(["Vendas","Relatório De Lucratividade"]));
        $container->add($this->form);

        parent::add($container);

    }

    public function onGenerateHtml($param = null) 
    {
        $this->onGenerate('html');
    }
    public function onGeneratePdf($param = null) 
    {
        $this->onGenerate('pdf');
    }
    public function onGenerateXls($param = null) 
    {
        $this->onGenerate('xls');
    }
    public function onGenerateRtf($param = null) 
    {
        $this->onGenerate('rtf');
    }

    /**
     * Register the filter in the session
     */
    public function getFilters()
    {
        // get the search form data
        $data = $this->form->getData();

        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->venda_id) AND ( (is_scalar($data->venda_id) AND $data->venda_id !== '') OR (is_array($data->venda_id) AND (!empty($data->venda_id)) )) )
        {

            $filters[] = new TFilter('venda_id', '=', $data->venda_id);// create the filter 
        }
        if (isset($data->venda_venda_situacao_id) AND ( (is_scalar($data->venda_venda_situacao_id) AND $data->venda_venda_situacao_id !== '') OR (is_array($data->venda_venda_situacao_id) AND (!empty($data->venda_venda_situacao_id)) )) )
        {

            $filters[] = new TFilter('venda_id', 'in', "(SELECT id FROM venda WHERE venda_situacao_id = '{$data->venda_venda_situacao_id}')");// create the filter 
        }
        if (isset($data->produto_id) AND ( (is_scalar($data->produto_id) AND $data->produto_id !== '') OR (is_array($data->produto_id) AND (!empty($data->produto_id)) )) )
        {

            $filters[] = new TFilter('produto_id', '=', $data->produto_id);// create the filter 
        }
        if (isset($data->venda_system_unit_id) AND ( (is_scalar($data->venda_system_unit_id) AND $data->venda_system_unit_id !== '') OR (is_array($data->venda_system_unit_id) AND (!empty($data->venda_system_unit_id)) )) )
        {

            $filters[] = new TFilter('venda_id', 'in', "(SELECT id FROM venda WHERE filial_saida_id = '{$data->venda_system_unit_id}')");// create the filter 
        }
        if (isset($data->data_venda_inicial) AND ( (is_scalar($data->data_venda_inicial) AND $data->data_venda_inicial !== '') OR (is_array($data->data_venda_inicial) AND (!empty($data->data_venda_inicial)) )) )
        {

            $filters[] = new TFilter('venda_id', 'in', "(SELECT id FROM venda WHERE data_hora_venda >= '{$data->data_venda_inicial}')");// create the filter 
        }
        if (isset($data->data_venda_final) AND ( (is_scalar($data->data_venda_final) AND $data->data_venda_final !== '') OR (is_array($data->data_venda_final) AND (!empty($data->data_venda_final)) )) )
        {

            $filters[] = new TFilter('venda_id', 'in', "(SELECT id FROM venda WHERE data_hora_venda <= '{$data->data_venda_final}')");// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);

        return $filters;
    }

    public function onGenerate($format)
    {
        try
        {
            $filters = $this->getFilters();
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);
            $param = [];
            // creates a repository for VendaItem
            $repository = new TRepository(self::$activeRecord);
            // creates a criteria
            $criteria = new TCriteria;

            $criteria->setProperties($param);

            if ($filters)
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            $criteria->add(new TFilter('produto_id', 'in', "(SELECT id FROM produto WHERE deletado_em is NULL )"));

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            if ($objects)
            {
                $widths = array(200,200,200,200,200,200,200,200,200);
                $reportExtension = 'pdf';
                switch ($format)
                {
                    case 'html':
                        $tr = new TTableWriterHTML($widths);
                        $reportExtension = 'html';
                        break;
                    case 'xls':
                        $tr = new TTableWriterXLS($widths);
                        $reportExtension = 'xls';
                        break;
                    case 'pdf':
                        $tr = new TTableWriterPDF($widths, 'L', 'A4');
                        $reportExtension = 'pdf';
                        break;
                    case 'htmlPdf':
                        $reportExtension = 'pdf';
                        $tr = new BTableWriterHtmlPDF($widths, 'L', 'A4');
                        break;
                    case 'rtf':
                        if (!class_exists('PHPRtfLite_Autoloader'))
                        {
                            PHPRtfLite::registerAutoloader();
                        }
                        $reportExtension = 'rtf';
                        $tr = new TTableWriterRTF($widths, 'L', 'A4');
                        break;
                }

                if (!empty($tr))
                {
                    // create the document styles
                    $tr->addStyle('title', 'Helvetica', '10', 'B',   '#ffffff', '#96c0e6');
                    $tr->addStyle('datap', 'Arial', '10', '',    '#000000', '#f0f0f0');
                    $tr->addStyle('datai', 'Arial', '10', '',    '#000000', '#ffffff');
                    $tr->addStyle('header', 'Helvetica', '16', 'B',   '#4589c5', '#494D90');
                    $tr->addStyle('footer', 'Helvetica', '10', 'B',  '#4589c5', '#B1B1EA');
                    $tr->addStyle('break', 'Helvetica', '10', 'B',  '#ffffff', '#8da4bc');
                    $tr->addStyle('total', 'Helvetica', '10', 'I',  '#000000', '#b2d1dc');
                    $tr->addStyle('breakTotal', 'Helvetica', '10', 'I',  '#000000', '#b3d5f3');

                    // add titles row
                    $tr->addRow();
                    $tr->addCell("Venda Id", 'left', 'title');
                    $tr->addCell("Data da Venda", 'left', 'title');
                    $tr->addCell("Produto", 'left', 'title');
                    $tr->addCell("Valor Unitário ", 'left', 'title');
                    $tr->addCell("Custo", 'left', 'title');
                    $tr->addCell("Desconto", 'left', 'title');
                    $tr->addCell("Frete", 'left', 'title');
                    $tr->addCell("Saldo Lucro", 'left', 'title');
                    $tr->addCell("Venda Situação", 'left', 'title');

                    $grandTotal = [];
                    $breakTotal = [];
                    $breakValue = null;
                    $firstRow = true;

                    // controls the background filling
                    $colour = false;                
                    foreach ($objects as $object)
                    {
                        $style = $colour ? 'datap' : 'datai';

                        $column_calculated_1 = $object->evaluate('=( {valor_unitario} - {valor_custo} - {venda->desconto} + {venda->frete_total} )');

                        $grandTotal['column_calculated_1'][] = $column_calculated_1;
                        $breakTotal['column_calculated_1'][] = $column_calculated_1;

                        $firstRow = false;

                        $object->venda->data_hora_venda = call_user_func(function($value, $object, $row)
                        {
                            if(!empty(trim($value)))
                            {
                                try
                                {
                                    $date = new DateTime($value);
                                    return $date->format('d/m/Y H:i');
                                }
                                catch (Exception $e)
                                {
                                    return $value;
                                }
                            }
                        }, $object->venda->data_hora_venda, $object, null);

                        $object->valor_unitario = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->valor_unitario, $object, null);

                        $object->valor_custo = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->valor_custo, $object, null);

                        $object->venda->desconto = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->venda->desconto, $object, null);

                        $object->venda->frete_total = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $object->venda->frete_total, $object, null);

                        $column_calculated_1 = call_user_func(function($value, $object, $row) 
                        {
                            if(!$value)
                            {
                                $value = 0;
                            }

                            if(is_numeric($value))
                            {
                                return "R$ " . number_format($value, 2, ",", ".");
                            }
                            else
                            {
                                return $value;
                            }
                        }, $column_calculated_1, $object, null);

                        $tr->addRow();

                        $tr->addCell($object->venda_id, 'left', $style);
                        $tr->addCell($object->venda->data_hora_venda, 'left', $style);
                        $tr->addCell($object->produto->nome, 'left', $style);
                        $tr->addCell($object->valor_unitario, 'left', $style);
                        $tr->addCell($object->valor_custo, 'left', $style);
                        $tr->addCell($object->venda->desconto, 'left', $style);
                        $tr->addCell($object->venda->frete_total, 'left', $style);
                        $tr->addCell($column_calculated_1, 'left', $style);
                        $tr->addCell($object->venda->venda_situacao->descricao, 'left', $style);

                        $colour = !$colour;

                    }

                    $tr->addRow();

                    $grandTotal_column_calculated_1 = array_sum($grandTotal['column_calculated_1']);

                    $grandTotal_column_calculated_1 = call_user_func(function($value)
                    {
                        if(!$value)
                        {
                            $value = 0;
                        }

                        if(is_numeric($value))
                        {
                            return "R$ " . number_format($value, 2, ",", ".");
                        }
                        else
                        {
                            return $value;
                        }
                    }, $grandTotal_column_calculated_1); 

                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell('', 'center', 'total');
                    $tr->addCell($grandTotal_column_calculated_1, 'left', 'total');
                    $tr->addCell('', 'center', 'total');

                    $file = 'report_'.uniqid().".{$reportExtension}";
                    // stores the file
                    if (!file_exists("app/output/{$file}") || is_writable("app/output/{$file}"))
                    {
                        $tr->save("app/output/{$file}");
                    }
                    else
                    {
                        throw new Exception(_t('Permission denied') . ': ' . "app/output/{$file}");
                    }

                    parent::openFile("app/output/{$file}");

                    // shows the success message
                    new TMessage('info', _t('Report generated. Please, enable popups'));
                }
            }
            else
            {
                new TMessage('error', _t('No records found'));
            }

            // close the transaction
            TTransaction::close();
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }


}

