<?php

class VendaList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $loaded;
    private $filter_criteria;
    private static $database = 'kraftsoft';
    private static $activeRecord = 'Venda';
    private static $primaryKey = 'id';
    private static $formName = 'form_VendaList';
    private $showMethods = ['onReload', 'onSearch', 'onRefresh', 'onClearFilters'];
    private $limit = 20;

    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct($param = null)
    {
        parent::__construct();

        if(!empty($param['target_container']))
        {
            $this->adianti_target_container = $param['target_container'];
        }

        // creates the form
        $this->form = new BootstrapFormBuilder(self::$formName);

        // define the form title
        $this->form->setFormTitle("Listagem de vendas");
        $this->limit = 50;

        $id = new TEntry('id');
        $orcamento_id = new TEntry('orcamento_id');
        $data_venda_inicial = new TDateTime('data_venda_inicial');
        $data_venda_final = new TDateTime('data_venda_final');
        $filial_saida_id = new TDBCombo('filial_saida_id', 'kraftsoft', 'SystemUnit', 'id', '{name}','name asc'  );
        $cliente_id = new TDBCombo('cliente_id', 'kraftsoft', 'Cliente', 'id', '{id} -{nome} -{cpf_cnpj}','nome asc'  );
        $vendedor_id = new TDBCombo('vendedor_id', 'kraftsoft', 'Vendedor', 'id', '{nome}','nome asc'  );
        $venda_situacao_id = new TDBCombo('venda_situacao_id', 'kraftsoft', 'VendaSituacao', 'id', '{descricao}','ordem asc'  );


        $data_venda_final->setMask('dd/mm/yyyy hh:ii');
        $data_venda_inicial->setMask('dd/mm/yyyy hh:ii');

        $data_venda_final->setDatabaseMask('yyyy-mm-dd hh:ii');
        $data_venda_inicial->setDatabaseMask('yyyy-mm-dd hh:ii');

        $cliente_id->enableSearch();
        $vendedor_id->enableSearch();

        $id->setSize('95%');
        $cliente_id->setSize('100%');
        $vendedor_id->setSize('100%');
        $orcamento_id->setSize('100%');
        $filial_saida_id->setSize('100%');
        $data_venda_final->setSize('100%');
        $venda_situacao_id->setSize('100%');
        $data_venda_inicial->setSize('100%');

        $row1 = $this->form->addFields([new TLabel("Id:", null, '14px', null),$id],[new TLabel("Id Orçamento:", null, '14px', null),$orcamento_id],[]);
        $row1->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row2 = $this->form->addFields([new TLabel("Data Inicial:", null, '14px', null, '100%'),$data_venda_inicial],[new TLabel("Data Final:", null, '14px', null, '100%'),$data_venda_final],[new TLabel("Filial de Saída:", null, '14px', null, '100%'),$filial_saida_id]);
        $row2->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        $row3 = $this->form->addFields([new TLabel("Cliente:", null, '14px', null, '100%'),$cliente_id],[new TLabel("Vendedor:", null, '14px', null, '100%'),$vendedor_id],[new TLabel("Situação:", null, '14px', null, '100%'),$venda_situacao_id]);
        $row3->layout = [' col-sm-4',' col-sm-4',' col-sm-4'];

        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__.'_filter_data') );

        $btn_onsearch = $this->form->addAction("Buscar", new TAction([$this, 'onSearch']), 'fas:search #ffffff');
        $this->btn_onsearch = $btn_onsearch;
        $btn_onsearch->addStyleClass('btn-primary'); 

        // creates a Datagrid
        $this->datagrid = new TDataGrid;
        $this->datagrid->disableHtmlConversion();

        $this->datagrid_form = new TForm('datagrid_'.self::$formName);
        $this->datagrid_form->onsubmit = 'return false';

        $this->datagrid = new BootstrapDatagridWrapper($this->datagrid);
        $this->filter_criteria = new TCriteria;

        $filterVar = TSession::getValue("userunitid");
        $this->filter_criteria->add(new TFilter('system_unit_id', '=', $filterVar));

        $this->datagrid->disableDefaultClick();
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->setHeight(250);

        $column_id = new TDataGridColumn('id', "Id", 'center' , '70px');
        $column_data_hora_venda_transformed = new TDataGridColumn('data_hora_venda', "Data Hora", 'left');
        $column_orcamento_id = new TDataGridColumn('orcamento_id', "Id Orçamento", 'left');
        $column_cliente_nome = new TDataGridColumn('cliente->nome', "Cliente", 'left');
        $column_vendedor_nome = new TDataGridColumn('vendedor->nome', "Vendedor", 'left');
        $column_filial_saida_name = new TDataGridColumn('filial_saida->name', "Filial de Saída", 'left');
        $column_venda_tipo_pagamento_descricao = new TDataGridColumn('venda_tipo_pagamento->descricao', "Tipo Pagamento", 'left');
        $column_valor_total_transformed = new TDataGridColumn('valor_total', "Valor Total", 'left');
        $column_venda_situacao_descricao_transformed = new TDataGridColumn('venda_situacao->descricao', "Situação", 'left');
        $column_movimentacao_origem_descricao = new TDataGridColumn('movimentacao_origem->descricao', "Origem", 'left');

        $column_data_hora_venda_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!empty(trim($value)))
            {
                try
                {
                    $date = new DateTime($value);
                    return $date->format('d/m/Y H:i');
                }
                catch (Exception $e)
                {
                    return $value;
                }
            }
        });

        $column_valor_total_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            if(!$value)
            {
                $value = 0;
            }

            if(is_numeric($value))
            {
                return "R$ " . number_format($value, 2, ",", ".");
            }
            else
            {
                return $value;
            }
        });

        $column_venda_situacao_descricao_transformed->setTransformer(function($value, $object, $row, $cell = null, $last_row = null)
        {
            //code here
            $label = $object->venda_situacao->descricao;
            $cor = $object->venda_situacao->cor;

            return "<span class='label' style='background-color:{$cor}'>{$label}</span>";

            // $div = new TElement('div');
            // $div->style="text-shadow:none; font-size:14px; font-weight:bold; color:#FFFFFF; width:160px; text-align:center;background-color: $cor;padding:10px";
            // $div->add($label);

            // return $div;    //code here

        });        

        $order_id = new TAction(array($this, 'onReload'));
        $order_id->setParameter('order', 'id');
        $column_id->setAction($order_id);

        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_data_hora_venda_transformed);
        $this->datagrid->addColumn($column_orcamento_id);
        $this->datagrid->addColumn($column_cliente_nome);
        $this->datagrid->addColumn($column_vendedor_nome);
        $this->datagrid->addColumn($column_filial_saida_name);
        $this->datagrid->addColumn($column_venda_tipo_pagamento_descricao);
        $this->datagrid->addColumn($column_valor_total_transformed);
        $this->datagrid->addColumn($column_venda_situacao_descricao_transformed);
        $this->datagrid->addColumn($column_movimentacao_origem_descricao);

        $action_group = new TDataGridActionGroup("", 'fas:cog');
        $action_group->addHeader('');

        $action_onShow = new TDataGridAction(array('VendaFormView', 'onShow'));
        $action_onShow->setUseButton(TRUE);
        $action_onShow->setButtonClass('btn btn-default');
        $action_onShow->setLabel("Abrir");
        $action_onShow->setImage('fas:book-open #00BCD4');
        $action_onShow->setField(self::$primaryKey);

        $action_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_onShow);

        $action_onEdit = new TDataGridAction(array('VendaForm', 'onEdit'));
        $action_onEdit->setUseButton(TRUE);
        $action_onEdit->setButtonClass('btn btn-default');
        $action_onEdit->setLabel("Editar");
        $action_onEdit->setImage('far:edit #2196F3');
        $action_onEdit->setField(self::$primaryKey);
        $action_onEdit->setDisplayCondition('VendaList::onMostrarEditar');

        $action_group->addAction($action_onEdit);

        $action_onEnviarFaturamento = new TDataGridAction(array('VendaList', 'onEnviarFaturamento'));
        $action_onEnviarFaturamento->setUseButton(TRUE);
        $action_onEnviarFaturamento->setButtonClass('btn btn-default');
        $action_onEnviarFaturamento->setLabel("Enviar p/ Faturamento");
        $action_onEnviarFaturamento->setImage('fas:code-branch #9C27B0');
        $action_onEnviarFaturamento->setField(self::$primaryKey);
        $action_onEnviarFaturamento->setDisplayCondition('VendaList::onMostrarEnviarFaturamento');

        $action_group->addAction($action_onEnviarFaturamento);

        $action_onFaturamento = new TDataGridAction(array('VendaList', 'onFaturamento'));
        $action_onFaturamento->setUseButton(TRUE);
        $action_onFaturamento->setButtonClass('btn btn-default');
        $action_onFaturamento->setLabel("Faturamento");
        $action_onFaturamento->setImage('fas:money-check-alt #FF5722');
        $action_onFaturamento->setField(self::$primaryKey);
        $action_onFaturamento->setDisplayCondition('VendaList::onMostrarFaturamento');

        $action_group->addAction($action_onFaturamento);

        $action_onGerarNfe = new TDataGridAction(array('VendaList', 'onGerarNfe'));
        $action_onGerarNfe->setUseButton(TRUE);
        $action_onGerarNfe->setButtonClass('btn btn-default');
        $action_onGerarNfe->setLabel("Gerar Nf-e");
        $action_onGerarNfe->setImage('fas:file-invoice-dollar #4CAF50');
        $action_onGerarNfe->setField(self::$primaryKey);
        $action_onGerarNfe->setDisplayCondition('VendaList::onMostrarGerarNFe');

        $action_group->addAction($action_onGerarNfe);

        $action_onGerarNFCe = new TDataGridAction(array('VendaList', 'onGerarNFCe'));
        $action_onGerarNFCe->setUseButton(TRUE);
        $action_onGerarNFCe->setButtonClass('btn btn-default');
        $action_onGerarNFCe->setLabel("Gerar NFC-e");
        $action_onGerarNFCe->setImage('fas:file-invoice-dollar #CDDC39');
        $action_onGerarNFCe->setField(self::$primaryKey);
        $action_onGerarNFCe->setDisplayCondition('VendaList::onMostrarGerarNFCe');

        $action_group->addAction($action_onGerarNFCe);

        $action_onEspelhoPDF = new TDataGridAction(array('VendaList', 'onEspelhoPDF'));
        $action_onEspelhoPDF->setUseButton(TRUE);
        $action_onEspelhoPDF->setButtonClass('btn btn-default');
        $action_onEspelhoPDF->setLabel("Espelho PDF");
        $action_onEspelhoPDF->setImage('far:file-pdf #DD0909');
        $action_onEspelhoPDF->setField(self::$primaryKey);

        $action_group->addAction($action_onEspelhoPDF);

        $action_onCancelar = new TDataGridAction(array('VendaList', 'onCancelar'));
        $action_onCancelar->setUseButton(TRUE);
        $action_onCancelar->setButtonClass('btn btn-default');
        $action_onCancelar->setLabel("Cancelar");
        $action_onCancelar->setImage('fas:window-close #CE0D0D');
        $action_onCancelar->setField(self::$primaryKey);
        $action_onCancelar->setDisplayCondition('VendaList::onMostrarCancelar');

        $action_group->addAction($action_onCancelar);

        $action_VendaSituacaoHistoricoTimeLine_onShow = new TDataGridAction(array('VendaSituacaoHistoricoTimeLine', 'onShow'));
        $action_VendaSituacaoHistoricoTimeLine_onShow->setUseButton(TRUE);
        $action_VendaSituacaoHistoricoTimeLine_onShow->setButtonClass('btn btn-default');
        $action_VendaSituacaoHistoricoTimeLine_onShow->setLabel("Linha do tempo das situações desta venda");
        $action_VendaSituacaoHistoricoTimeLine_onShow->setImage('fas:clock #FF9800');
        $action_VendaSituacaoHistoricoTimeLine_onShow->setField(self::$primaryKey);

        $action_VendaSituacaoHistoricoTimeLine_onShow->setParameter('key', '{id}');

        $action_group->addAction($action_VendaSituacaoHistoricoTimeLine_onShow);

        $action_onEstornarVenda = new TDataGridAction(array('VendaList', 'onEstornarVenda'));
        $action_onEstornarVenda->setUseButton(TRUE);
        $action_onEstornarVenda->setButtonClass('btn btn-default');
        $action_onEstornarVenda->setLabel("Estonar Venda");
        $action_onEstornarVenda->setImage('fas:ban #E91E63');
        $action_onEstornarVenda->setField(self::$primaryKey);
        $action_onEstornarVenda->setDisplayCondition('VendaList::onMostrarEstonarVenda');

        $action_group->addAction($action_onEstornarVenda);

        $action_onGenerate = new TDataGridAction(array('CupomNaoFiscalDocument', 'onGenerate'));
        $action_onGenerate->setUseButton(TRUE);
        $action_onGenerate->setButtonClass('btn btn-default');
        $action_onGenerate->setLabel("Cupom não fiscal");
        $action_onGenerate->setImage('fas:print #795548');
        $action_onGenerate->setField(self::$primaryKey);

        $action_group->addAction($action_onGenerate);

        $action_VendaEtiquetaDocument_onGenerate = new TDataGridAction(array('VendaEtiquetaDocument', 'onGenerate'));
        $action_VendaEtiquetaDocument_onGenerate->setUseButton(TRUE);
        $action_VendaEtiquetaDocument_onGenerate->setButtonClass('btn btn-default');
        $action_VendaEtiquetaDocument_onGenerate->setLabel("Etiqueta de Envio");
        $action_VendaEtiquetaDocument_onGenerate->setImage('fas:mail-bulk #4CAF50');
        $action_VendaEtiquetaDocument_onGenerate->setField(self::$primaryKey);

        $action_group->addAction($action_VendaEtiquetaDocument_onGenerate);

        $action_RelatorioDeSeparacaoDocument_onGenerate = new TDataGridAction(array('RelatorioDeSeparacaoDocument', 'onGenerate'));
        $action_RelatorioDeSeparacaoDocument_onGenerate->setUseButton(TRUE);
        $action_RelatorioDeSeparacaoDocument_onGenerate->setButtonClass('btn btn-default');
        $action_RelatorioDeSeparacaoDocument_onGenerate->setLabel("Guia de separação");
        $action_RelatorioDeSeparacaoDocument_onGenerate->setImage('fas:box-open #9E9E9E');
        $action_RelatorioDeSeparacaoDocument_onGenerate->setField(self::$primaryKey);

        $action_group->addAction($action_RelatorioDeSeparacaoDocument_onGenerate);

        $action_VendaDevolucaoForm_onEdit = new TDataGridAction(array('VendaDevolucaoForm', 'onEdit'));
        $action_VendaDevolucaoForm_onEdit->setUseButton(TRUE);
        $action_VendaDevolucaoForm_onEdit->setButtonClass('btn btn-default');
        $action_VendaDevolucaoForm_onEdit->setLabel("Devolução / Troca");
        $action_VendaDevolucaoForm_onEdit->setImage('fas:exchange-alt #00BCD4');
        $action_VendaDevolucaoForm_onEdit->setField(self::$primaryKey);

        $action_group->addAction($action_VendaDevolucaoForm_onEdit);

        $action_onClonar = new TDataGridAction(array('VendaList', 'onClonar'));
        $action_onClonar->setUseButton(TRUE);
        $action_onClonar->setButtonClass('btn btn-default');
        $action_onClonar->setLabel("Clonar");
        $action_onClonar->setImage('fas:copy #FFE821');
        $action_onClonar->setField(self::$primaryKey);

        $action_group->addAction($action_onClonar);

        $this->datagrid->addActionGroup($action_group);    

        // create the datagrid model
        $this->datagrid->createModel();

        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->enableCounters();
        $this->pageNavigation->setAction(new TAction(array($this, 'onReload')));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());

        $panel = new TPanelGroup("Listagem de vendas");
        $panel->datagrid = 'datagrid-container';
        $this->datagridPanel = $panel;
        $this->datagrid_form->add($this->datagrid);
        $panel->add($this->datagrid_form);

        $panel->getBody()->class .= ' table-responsive';

        $panel->addFooter($this->pageNavigation);

        $headerActions = new TElement('div');
        $headerActions->class = ' datagrid-header-actions ';
        $headerActions->style = 'justify-content: space-between;';

        $head_left_actions = new TElement('div');
        $head_left_actions->class = ' datagrid-header-actions-left-actions ';

        $head_right_actions = new TElement('div');
        $head_right_actions->class = ' datagrid-header-actions-left-actions ';

        $headerActions->add($head_left_actions);
        $headerActions->add($head_right_actions);

        $panel->getBody()->insert(0, $headerActions);

        $button_cadastrar = new TButton('button_button_cadastrar');
        $button_cadastrar->setAction(new TAction(['VendaForm', 'onShow']), "Cadastrar");
        $button_cadastrar->addStyleClass('btn-default');
        $button_cadastrar->setImage('fas:plus #69aa46');

        $this->datagrid_form->addField($button_cadastrar);

        $btnShowCurtainFilters = new TButton('button_btnShowCurtainFilters');
        $btnShowCurtainFilters->setAction(new TAction(['VendaList', 'onShowCurtainFilters']), "Filtros");
        $btnShowCurtainFilters->addStyleClass('btn-default');
        $btnShowCurtainFilters->setImage('fas:filter #000000');

        $this->datagrid_form->addField($btnShowCurtainFilters);

        $button_limpar_filtros = new TButton('button_button_limpar_filtros');
        $button_limpar_filtros->setAction(new TAction(['VendaList', 'onClearFilters']), "Limpar filtros");
        $button_limpar_filtros->addStyleClass('btn-default');
        $button_limpar_filtros->setImage('fas:eraser #f44336');

        $this->datagrid_form->addField($button_limpar_filtros);

        $button_atualizar = new TButton('button_button_atualizar');
        $button_atualizar->setAction(new TAction(['VendaList', 'onRefresh']), "Atualizar");
        $button_atualizar->addStyleClass('btn-default');
        $button_atualizar->setImage('fas:sync-alt #03a9f4');

        $this->datagrid_form->addField($button_atualizar);

        $dropdown_button_exportar = new TDropDown("Exportar", 'fas:file-export #2d3436');
        $dropdown_button_exportar->setPullSide('right');
        $dropdown_button_exportar->setButtonClass('btn btn-default waves-effect dropdown-toggle');
        $dropdown_button_exportar->addPostAction( "CSV", new TAction(['VendaList', 'onExportCsv'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-csv #00b894' );
        $dropdown_button_exportar->addPostAction( "XLS", new TAction(['VendaList', 'onExportXls'],['static' => 1]), 'datagrid_'.self::$formName, 'fas:file-excel #4CAF50' );
        $dropdown_button_exportar->addPostAction( "PDF", new TAction(['VendaList', 'onExportPdf'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-pdf #e74c3c' );
        $dropdown_button_exportar->addPostAction( "XML", new TAction(['VendaList', 'onExportXml'],['static' => 1]), 'datagrid_'.self::$formName, 'far:file-code #95a5a6' );

        $head_left_actions->add($button_cadastrar);
        $head_left_actions->add($btnShowCurtainFilters);
        $head_left_actions->add($button_limpar_filtros);
        $head_left_actions->add($button_atualizar);

        $head_right_actions->add($dropdown_button_exportar);

        $this->btnShowCurtainFilters = $btnShowCurtainFilters;

        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        if(empty($param['target_container']))
        {
            $container->add(TBreadCrumb::create(["Vendas","Vendas"]));
        }

        $container->add($panel);

        parent::add($container);

    }

    public static function onMostrarEditar($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::Finalizada 
            or $object->venda_situacao_id==VendaSituacao::Cancelada
            or $object->venda_situacao_id==VendaSituacao::AguardandoExpedicao)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onEnviarFaturamento($param = null) 
    {
        try 
        {
          new TQuestion("Deseja enviar venda para faturamento?", new TAction([__CLASS__, 'onEnviarFaturamentoYes'], $param), new TAction([__CLASS__, 'onEnviarFaturamentoNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarEnviarFaturamento($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::Finalizada 
            or $object->venda_situacao_id==VendaSituacao::Cancelada
            or $object->venda_situacao_id==VendaSituacao::AguardandoExpedicao
            or $object->venda_situacao_id==VendaSituacao::AguardandoFaturamento)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onFaturamento($param = null) 
    {
        try 
        {
           // new TQuestion("Deseja realizar o faturamento?", new TAction([__CLASS__, 'onFaturamentoYes'], $param), new TAction([__CLASS__, 'onFaturamentoNo'], $param));

          $pageParam = ['venda_id' => $param['id']]; // ex.: = ['key' => 10]  
          TApplication::loadPage('VendaFaturamentoSimpleList', 'onReload', $pageParam);             
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarFaturamento($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::Finalizada 
            or $object->venda_situacao_id==VendaSituacao::Cancelada
            or $object->venda_situacao_id==VendaSituacao::AguardandoExpedicao)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onGerarNfe($param = null) 
    {
        try 
        {
            new TQuestion("Deseja gerar a NF-e?", new TAction([__CLASS__, 'onGerarNFeYes'], $param), new TAction([__CLASS__, 'onGerarNFeNo'], $param));
            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarGerarNFe($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::AguardandoFaturamento)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onGerarNFCe($param = null) 
    {
        try 
        {
            new TQuestion("Deseja gerar a NFC-e?", new TAction([__CLASS__, 'onGerarNFCeYes'], $param), new TAction([__CLASS__, 'onGerarNFCeNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarGerarNFCe($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::AguardandoFaturamento)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onEspelhoPDF($param = null) 
    {
        try 
        {
            $documento = Venda::Espelho_Venda($param['id']);

            if(file_exists($documento)){
                parent::openFile($documento);   
            }else{
                new TMessage('error', "Arquivo não encontrado!");
            }

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onCancelar($param = null) 
    { 

     new TQuestion("Deseja cancelar a venda?", new TAction([__CLASS__, 'onCancelarYes'], $param), new TAction([__CLASS__, 'onCancelarNo'], $param));

     /*   if(isset($param['delete']) && $param['delete'] == 1)
        {
            try
            {
                // get the paramseter $key
                $key = $param['key'];
                // open a transaction with database
                TTransaction::open(self::$database);

                // instantiates object
                $object = new Venda($key, FALSE); 

                // deletes the object from the database
                $object->delete();

                // close the transaction
                TTransaction::close();

                // reload the listing
                $this->onReload( $param );
                // shows the success message
                new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'));
            }
            catch (Exception $e) // in case of exception
            {
                // shows the exception error message
                new TMessage('error', $e->getMessage());
                // undo all pending operations
                TTransaction::rollback();
            }
        }
        else
        {
            // define the delete action
            $action = new TAction(array($this, 'onDelete'));
            $action->setParameters($param); // pass the key paramseter ahead
            $action->setParameter('delete', 1);
            // shows a dialog to the user
            new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);   
        } */
    }
    public static function onMostrarCancelar($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::Finalizada 
            or $object->venda_situacao_id==VendaSituacao::Cancelada
            or $object->venda_situacao_id==VendaSituacao::AguardandoExpedicao)
            {
                return false;
            }

            return true;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onEstornarVenda($param = null) 
    {
        try 
        {
            //code here
            new TQuestion("Deseja estornar a venda?", new TAction([__CLASS__, 'onEstornarVendaYes'], $param), new TAction([__CLASS__, 'onEstornarVendaNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public static function onMostrarEstonarVenda($object)
    {
        try 
        {
            if($object->venda_situacao_id==VendaSituacao::Finalizada)
            {
                return true;
            }

            return false;
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onClonar($param = null) 
    {
        try 
        {
            //code here
            new TQuestion("Deseja clonar a Venda?", new TAction([__CLASS__, 'onClonarYes'], $param), new TAction([__CLASS__, 'onClonarNo'], $param));

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onExportCsv($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.csv';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    $handler = fopen($output, 'w');
                    TTransaction::open(self::$database);

                    foreach ($objects as $object)
                    {
                        $row = [];
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();

                            if (isset($object->$column_name))
                            {
                                $row[] = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $row[] = $object->render($column_name);
                            }
                        }

                        fputcsv($handler, $row);
                    }

                    fclose($handler);
                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXls($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xls';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $widths = [];
                $titles = [];

                foreach ($this->datagrid->getColumns() as $column)
                {
                    $titles[] = $column->getLabel();
                    $width    = 100;

                    if (is_null($column->getWidth()))
                    {
                        $width = 100;
                    }
                    else if (strpos($column->getWidth(), '%') !== false)
                    {
                        $width = ((int) $column->getWidth()) * 5;
                    }
                    else if (is_numeric($column->getWidth()))
                    {
                        $width = $column->getWidth();
                    }

                    $widths[] = $width;
                }

                $table = new \TTableWriterXLS($widths);
                $table->addStyle('title',  'Helvetica', '10', 'B', '#ffffff', '#617FC3');
                $table->addStyle('data',   'Helvetica', '10', '',  '#000000', '#FFFFFF', 'LR');

                $table->addRow();

                foreach ($titles as $title)
                {
                    $table->addCell($title, 'center', 'title');
                }

                $this->limit = 0;
                $objects = $this->onReload();

                TTransaction::open(self::$database);
                if ($objects)
                {
                    foreach ($objects as $object)
                    {
                        $table->addRow();
                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $value = '';
                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                            }

                            $table->addCell($value, 'center', 'data');
                        }
                    }
                }
                $table->save($output);
                TTransaction::close();

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportPdf($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.pdf';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $this->datagrid->prepareForPrinting();
                $this->onReload();

                $html = clone $this->datagrid;
                $contents = file_get_contents('app/resources/styles-print.html') . $html->getContents();

                $dompdf = new \Dompdf\Dompdf;
                $dompdf->loadHtml($contents);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();

                file_put_contents($output, $dompdf->output());

                $window = TWindow::create('PDF', 0.8, 0.8);
                $object = new TElement('object');
                $object->data  = $output;
                $object->type  = 'application/pdf';
                $object->style = "width: 100%; height:calc(100% - 10px)";

                $window->add($object);
                $window->show();
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
        }
    }
    public function onExportXml($param = null) 
    {
        try
        {
            $output = 'app/output/'.uniqid().'.xml';

            if ( (!file_exists($output) && is_writable(dirname($output))) OR is_writable($output))
            {
                $this->limit = 0;
                $objects = $this->onReload();

                if ($objects)
                {
                    TTransaction::open(self::$database);

                    $dom = new DOMDocument('1.0', 'UTF-8');
                    $dom->{'formatOutput'} = true;
                    $dataset = $dom->appendChild( $dom->createElement('dataset') );

                    foreach ($objects as $object)
                    {
                        $row = $dataset->appendChild( $dom->createElement( self::$activeRecord ) );

                        foreach ($this->datagrid->getColumns() as $column)
                        {
                            $column_name = $column->getName();
                            $column_name_raw = str_replace(['(','{','->', '-','>','}',')', ' '], ['','','_','','','','','_'], $column_name);

                            if (isset($object->$column_name))
                            {
                                $value = is_scalar($object->$column_name) ? $object->$column_name : '';
                                $row->appendChild($dom->createElement($column_name_raw, $value)); 
                            }
                            else if (method_exists($object, 'render'))
                            {
                                $column_name = (strpos($column_name, '{') === FALSE) ? ( '{' . $column_name . '}') : $column_name;
                                $value = $object->render($column_name);
                                $row->appendChild($dom->createElement($column_name_raw, $value));
                            }
                        }
                    }

                    $dom->save($output);

                    TTransaction::close();
                }
                else
                {
                    throw new Exception(_t('No records found'));
                }

                TPage::openFile($output);
            }
            else
            {
                throw new Exception(_t('Permission denied') . ': ' . $output);
            }
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }
    public static function onShowCurtainFilters($param = null) 
    {
        try 
        {
            //code here

                        $filter = new self([]);

            $btnClose = new TButton('closeCurtain');
            $btnClose->class = 'btn btn-sm btn-default';
            $btnClose->style = 'margin-right:10px;';
            $btnClose->onClick = "Template.closeRightPanel();";
            $btnClose->setLabel("Fechar");
            $btnClose->setImage('fas:times');

            $filter->form->addHeaderWidget($btnClose);

            $page = new TPage();
            $page->setTargetContainer('adianti_right_panel');
            $page->setProperty('page-name', 'VendaListSearch');
            $page->setProperty('page_name', 'VendaListSearch');
            $page->adianti_target_container = 'adianti_right_panel';
            $page->target_container = 'adianti_right_panel';
            $page->add($filter->form);
            $page->setIsWrapped(true);
            $page->show();

            $style = new TStyle('right-panel > .container-part[page-name=VendaListSearch]');
            $style->width = '70% !important';
            $style->show(true);

            //</autoCode>
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }
    public function onClearFilters($param = null) 
    {
        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }
    public function onRefresh($param = null) 
    {
        $this->onReload([]);
    }

    /**
     * Register the filter in the session
     */
    public function onSearch($param = null)
    {
        $data = $this->form->getData();
        $filters = [];

        TSession::setValue(__CLASS__.'_filter_data', NULL);
        TSession::setValue(__CLASS__.'_filters', NULL);

        if (isset($data->id) AND ( (is_scalar($data->id) AND $data->id !== '') OR (is_array($data->id) AND (!empty($data->id)) )) )
        {

            $filters[] = new TFilter('id', '=', $data->id);// create the filter 
        }

        if (isset($data->orcamento_id) AND ( (is_scalar($data->orcamento_id) AND $data->orcamento_id !== '') OR (is_array($data->orcamento_id) AND (!empty($data->orcamento_id)) )) )
        {

            $filters[] = new TFilter('orcamento_id', '=', $data->orcamento_id);// create the filter 
        }

        if (isset($data->data_venda_inicial) AND ( (is_scalar($data->data_venda_inicial) AND $data->data_venda_inicial !== '') OR (is_array($data->data_venda_inicial) AND (!empty($data->data_venda_inicial)) )) )
        {

            $filters[] = new TFilter('data_hora_venda', '>=', $data->data_venda_inicial);// create the filter 
        }

        if (isset($data->data_venda_final) AND ( (is_scalar($data->data_venda_final) AND $data->data_venda_final !== '') OR (is_array($data->data_venda_final) AND (!empty($data->data_venda_final)) )) )
        {

            $filters[] = new TFilter('data_hora_venda', '<=', $data->data_venda_final);// create the filter 
        }

        if (isset($data->filial_saida_id) AND ( (is_scalar($data->filial_saida_id) AND $data->filial_saida_id !== '') OR (is_array($data->filial_saida_id) AND (!empty($data->filial_saida_id)) )) )
        {

            $filters[] = new TFilter('filial_saida_id', '=', $data->filial_saida_id);// create the filter 
        }

        if (isset($data->cliente_id) AND ( (is_scalar($data->cliente_id) AND $data->cliente_id !== '') OR (is_array($data->cliente_id) AND (!empty($data->cliente_id)) )) )
        {

            $filters[] = new TFilter('cliente_id', '=', $data->cliente_id);// create the filter 
        }

        if (isset($data->vendedor_id) AND ( (is_scalar($data->vendedor_id) AND $data->vendedor_id !== '') OR (is_array($data->vendedor_id) AND (!empty($data->vendedor_id)) )) )
        {

            $filters[] = new TFilter('vendedor_id', '=', $data->vendedor_id);// create the filter 
        }

        if (isset($data->venda_situacao_id) AND ( (is_scalar($data->venda_situacao_id) AND $data->venda_situacao_id !== '') OR (is_array($data->venda_situacao_id) AND (!empty($data->venda_situacao_id)) )) )
        {

            $filters[] = new TFilter('venda_situacao_id', '=', $data->venda_situacao_id);// create the filter 
        }

        // fill the form with data again
        $this->form->setData($data);

        // keep the search data in the session
        TSession::setValue(__CLASS__.'_filter_data', $data);
        TSession::setValue(__CLASS__.'_filters', $filters);

        $this->onReload(['offset' => 0, 'first_page' => 1]);
    }

    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'kraftsoft'
            TTransaction::open(self::$database);

            // creates a repository for Venda
            $repository = new TRepository(self::$activeRecord);

            $criteria = clone $this->filter_criteria;

            if (empty($param['order']))
            {
                $param['order'] = 'id';    
            }

            if (empty($param['direction']))
            {
                $param['direction'] = 'desc';
            }

            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $this->limit);

            if($filters = TSession::getValue(__CLASS__.'_filters'))
            {
                foreach ($filters as $filter) 
                {
                    $criteria->add($filter);       
                }
            }

            //</blockLine><btnShowCurtainFiltersAutoCode>
            if(!empty($this->btnShowCurtainFilters) && empty($this->btnShowCurtainFiltersAdjusted))
            {
                $this->btnShowCurtainFiltersAdjusted = true;
                $this->btnShowCurtainFilters->style = 'position: relative';
                $countFilters = count($filters ?? []);
                $this->btnShowCurtainFilters->setLabel($this->btnShowCurtainFilters->getLabel(). "<span class='badge badge-success' style='position: absolute'>{$countFilters}<span>");
            }
            //</blockLine></btnShowCurtainFiltersAutoCode>

            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);

            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {

                    $row = $this->datagrid->addItem($object);
                    $row->id = "row_{$object->id}";

                }
            }

            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);

            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($this->limit); // limit

            // close the transaction
            TTransaction::close();
            $this->loaded = true;

            return $objects;
        }
        catch (Exception $e) // in case of exception
        {
            // shows the exception error message
            new TMessage('error', $e->getMessage());
            // undo all pending operations
            TTransaction::rollback();
        }
    }

    public function onShow($param = null)
    {

    }

    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  $this->showMethods))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }

    public static function onEnviarFaturamentoYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $venda = Venda::find($param['id']);     

            if($venda) 
            {
                $venda->venda_situacao_id=VendaSituacao::AguardandoFaturamento;
                $venda->store();

                VendaSituacaoHistorico::LancarHistorico(VendaSituacao::AguardandoFaturamento,
                                                        $venda->id,
                                                        'Mudança de Situação de venda: Aguardando Faturamento');                
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');  
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onEnviarFaturamentoNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onFaturamentoYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $venda = Venda::find($param['id']);     

            if($venda) 
            {
                $venda->venda_situacao_id=VendaSituacao::Finalizada;
                $venda->store();
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');    
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onFaturamentoNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onGerarNFeYes($param = null) 
    {
        try 
        {
            TTransaction::open('kraftsoft');    

            Venda::ValidaCamposNFe($param['id']);

            $nfe_id = funcaoNFE::geraNotaVenda($param['id'], NotafiscalMod::NFe);

            if(is_numeric($nfe_id))
            {
                $pageParam = ['key' => $nfe_id]; 
                TApplication::loadPage('NotafiscalFormView', 'onShow', $pageParam);

            }
            else
            {
                throw new Exception($nfe_id);
            }

            TTransaction::close();       
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onGerarNFeNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onGerarNFCeYes($param = null) 
    {
        try 
        {
            TTransaction::open('kraftsoft');    

            Venda::ValidaCamposNFe($param['id']);

            $nfe_id = funcaoNFE::geraNotaVenda($param['id'], NotafiscalMod::NFCe);

            if(is_numeric($nfe_id))
            {
                $pageParam = ['key' => $nfe_id]; 
                TApplication::loadPage('NotafiscalConsumidorFormView', 'onShow', $pageParam);

            }
            else
            {
                throw new Exception($nfe_id);
            }

            TTransaction::close();          

        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onGerarNFCeNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onCancelarYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $venda = Venda::find($param['id']);     

            if($venda) 
            {
                $venda->venda_situacao_id=VendaSituacao::Cancelada;
                $venda->store();

                //historico de situação de venda..
                VendaSituacaoHistorico::LancarHistorico(VendaSituacao::Cancelada,
                                                        $venda->id,
                                                        'Mudança de Situação de venda: Cancelada');                           
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');  
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onCancelarNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onEstornarVendaYes($param = null) 
    {
        try 
        {
            TTransaction::open(self::$database);

            $venda = Venda::find($param['id']);     

            if($venda) 
            {
                Venda::EstornarVenda($venda);                  
            }

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show('success', "Operação realizada com sucesso!", 'topRight', 'far:check-circle');  
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onEstornarVendaNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onClonarYes($param = null) 
    {
        try 
        {
            //code here
            TTransaction::open(self::$database);

            Venda::ClonarVenda($param['id']);

            TTransaction::close();

            TApplication::loadPage(__CLASS__, 'onReload');

            TToast::show("success", "Venda clonada com sucesso!", "topCenter", "fas:check-circle");
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

    public static function onClonarNo($param = null) 
    {
        try 
        {
            //code here
        }
        catch (Exception $e) 
        {
            new TMessage('error', $e->getMessage());    
        }
    }

}

