<?php
/**
 * Classe para geração de arquivos XML de notas fiscais.
 *
 * Autor: Rubens dos Santos
 * Email: salvadorbba@gmail.com
 * Celular: (12) 99675-8056
 */
class GerarArquivosContadorXML
{
    private $key;
    private $objeto;

    /**
     * Construtor da classe GerarArquivosContadorXML.
     *
     * @param string $key - A chave para buscar os parâmetros no banco de dados.
     * @param object $objeto - O objeto relacionado à geração dos arquivos XML.
     */
    public function __construct($key)
    {
        $this->key = $key;
    }

    /**
     * Função para criar e obter arquivos XML de notas fiscais autorizadas e canceladas.
     *
     * @return string - Uma mensagem indicando o resultado da operação.
     */
    public function create()
    {

        try {
            TTransaction::open('kraftsoft');
            $this->objeto = XmlContador::find($this->key);

            $dataInicio = $this->objeto->data_inicio;
            $dataFim = $this->objeto->data_fim;

            function sanitizeDate($date)
            {
                // Substitua caracteres especiais por um traço
                $sanitizedDate = preg_replace('/[^a-zA-Z0-9]/', '-', $date);
                return $sanitizedDate;
            }

            // Exemplo de uso:
            $dataInicio1 = substr($dataInicio, 0, 10);
            $dataFim1 = substr($dataFim, 0, 10);

            $dataInicio1 = sanitizeDate($dataInicio1);
            $dataFim1 = sanitizeDate($dataFim1);

            $dataInicio = substr($dataInicio, 0, 10);
            $dataFim = substr($dataFim, 0, 10);

            $dataInicio .= ' 00:00:00';
            $dataFim .= ' 23:59:59';

            $basePasta = 'app/xml/' . $dataInicio1 . '_' . $dataFim1 . '/';
            if (!is_dir($basePasta)) {
                mkdir($basePasta, 0777, true);
            }

            $criteria = new TCriteria;
            $criteria->add(new TFilter('system_unit_id', '=', TSession::getValue("userunitid")));
            $criteria->add(new TFilter('data_emissao', '>=', $dataInicio));
            $criteria->add(new TFilter('data_emissao', '<=', $dataFim));
            $criteria->add(new TFilter('chave', '!=',''));
            $repository = new TRepository('Notafiscal');
            $Notas = $repository->load($criteria);

            foreach ($Notas as $nota) {
             

                $chave = $nota->chave;
                $xmlBase64 = $nota->xml;
                $xmlCancelamentoBase64 = $nota->xml_cancelamento;

                $xmlContent = base64_decode($xmlBase64);
                $xmlCancelamentoContent = base64_decode($xmlCancelamentoBase64);

             

                // Determine a pasta com base no estado da nota
                $estadoPasta = ($nota->status == 'Autorizado') ? 'autorizados' : 'cancelados';
                $pastaDestino = $basePasta . $estadoPasta . '/';

                if (!is_dir($pastaDestino)) {
                    mkdir($pastaDestino, 0777, true);
                }

                // Salve o XML na pasta com base no estado
                $nomeArquivoXML = $pastaDestino . $chave . '.xml';
                file_put_contents($nomeArquivoXML, $xmlContent);
            }

            $data_pasta = 'de_' . MasterClass::cleanPasta($dataInicio1) . '_ate_' . MasterClass::cleanPasta($dataFim1);

            $zipFileName = 'app/xml/' . $data_pasta . '.zip';

            $zip = new ZipArchive();

            if ($zip->open($zipFileName, ZipArchive::CREATE | ZipArchive::OVERWRITE) === true) {
                $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($basePasta));
                foreach ($files as $file) {
                    if (!$file->isDir()) {
                        $localPath = substr($file->getPath(), strlen($basePasta));
                        $zip->addFile($file->getPathname(), $localPath . '/' . $file->getFilename());
                    }
                }
                $zip->close();

                $objeto = XmlContador::find($this->key);
                if ($objeto) {
                    $objeto->situacao = 2;
                    $objeto->caminho_download = $zipFileName;
                    $objeto->store();
                }

                TTransaction::close();

                TToast::show("success", "Arquivo XML gerado, compactado com sucesso.", "center", "");

                self::rrmdir($basePasta);
            } else {
                TToast::show("warning", "Erro ao criar o arquivo ZIP.", "center", "");
            }
        } catch (Exception $e) {
            // Lide com erros aqui
        }
    }

    private static function rrmdir($dir)
    {
        if (is_dir($dir)) {
            $objects = scandir($dir);
            foreach ($objects as $object) {
                if ($object != "." && $object != "..") {
                    if (is_dir($dir . "/" . $object)) {
                        self::rrmdir($dir . "/" . $object);
                    } else {
                        unlink($dir . "/" . $object);
                    }
                }
            }
            rmdir($dir);
        }
    }
}
