<?php

use NFePHP\Common\Certificate;
use NFePHP\NFe\Common\Standardize;
use NFePHP\NFe\Complements;
use NFePHP\NFe\Tools;

/**
 * Classe Gera  cancela nfe
 *
 * Autor: Rubens dos Santos
 * Email: salvadorbba@gmail.com
 * Celular: (12) 99675-8056
 */
class CancelarNFe
{
    private $key;
    private $DocumentoFiscal;
    private $emitente;
    private $tools;
    private $nota;
    private $cliente;
    private $QTDEventos;
    private $pedido;

    private $PedidoEvento;
    private static $database = 'kraftsoft';

    /**
     * Construtor da classe GeraNFe.
     *
     * @param string $key - A chave para buscar os parâmetros no banco de dados.
     */
    public function __construct($key)
    {
        $this->key = $key;

    }

    /**
     * Função para criar e obter um token de autenticação do Sicredi.
     *
     * @return string - O token de acesso obtido.
     */
    public function solicitar()
    {

        TTransaction::open(self::$database);
        $this->emitente = SystemUnit::find(TSession::getValue('userunitid'));

        $this->pedido = PedidoEvento::find($this->key);

        $this->DocumentoFiscal = Notafiscal::find($this->pedido->notafiscal_id);

        TTransaction::close();

        $config = [
            'atualizacao' => date('Y-m-d H:i:s'),
            'tpAmb' => (int) $this->emitente->nfe_tpamb_id,
            'razaosocial' => $this->emitente->razao_social,
            'cnpj' => $this->emitente->cnpj, // PRECISA SER VÁLIDO
            'ie' => $this->emitente->inscricao_estadual, // PRECISA SER VÁLIDO
            'siglaUF' => $this->emitente->end_uf,
            'schemes' => 'PL_009_V4',
            'versao' => '4.00',
        ];

        $configJson = json_encode($config);
        $this->tools = new Tools(
            $configJson,
            Certificate::readPfx(file_get_contents($this->emitente->certificadoA1), $this->emitente->senha_certificado)

        );
        if ($this->DocumentoFiscal->notafiscal_mod_id == 1) {
            $this->tools->model('55');} else {

            $this->tools->model('65');}
        $chave = $this->pedido->chave;
        $xJust = $this->pedido->descricao;
        $nProt = $this->pedido->protocolo;
        $xml = $this->DocumentoFiscal->xml;
        $xml = base64_decode($xml);
        $response = $this->tools->sefazCancela($chave, $xJust, $nProt);

        $stdCl = new Standardize($response);
        //nesse caso $std irá conter uma representação em stdClass do XML
        $std = $stdCl->toStd();

        $retorno = $std->retEvento->infEvento;

        $resposnse_cce = $std->retEvento->infEvento;
        //verifique se o evento foi processado
        if ($std->cStat != 128) {
            //houve alguma falha e o evento não foi processado
            //TRATAR
        } else {
            $cStat = $std->retEvento->infEvento->cStat;
            if ($cStat == '101' || $cStat == '135' || $cStat == '155') {

                $xml_canc = Complements::cancelRegister($xml, $response);

            }TTransaction::open(self::$database);
            if (isset($resposnse_cce->nProt)) {

                $objeto_pedido = PedidoEvento::find($this->pedido->id);
                if ($objeto_pedido) {
                    $objeto_pedido->protocolo_resposta = $resposnse_cce->nProt ?? null;
                    $objeto_pedido->xevento = $resposnse_cce->xMotivo ?? null;
                    $objeto_pedido->status = 2;
                    $objeto_pedido->store();
                }

                $objeto = Notafiscal::find($this->pedido->notafiscal_id);
                if ($objeto) {
                    $objeto->xml_cancelamento = $xml_canc;
                    $objeto->protocolo_cancelamento = $resposnse_cce->nProt;
                    $objeto->datahora_cancelamento = $resposnse_cce->dhRegEvento;
                    $objeto->notafiscal_situacao_id = 3;
                    $objeto->status = 'Cancelada';
                    $objeto->store();
                }

                $eventos = new EventosDocumentos();
                $eventos->tpamb = $resposnse_cce->tpAmb;
                $eventos->veraplic = $resposnse_cce->verAplic;
                $eventos->cstat = $resposnse_cce->cStat;
                $eventos->xmotivo = $resposnse_cce->xMotivo;
                $eventos->chnfe = $resposnse_cce->chNFe;
                $eventos->xevento = $resposnse_cce->xEvento;
                $eventos->tpevento = $resposnse_cce->tpEvento;
                $eventos->nseqevento = $resposnse_cce->nSeqEvento;
                $eventos->dhregevento = $resposnse_cce->dhRegEvento;
                $eventos->nprot = $resposnse_cce->nProt;
                $eventos->xml = $xml_canc;

                $eventos->system_users_id = TSession::getValue('userunitid');
                $eventos->system_unit_id = TSession::getValue('userid');
                $eventos->origem_event = 'cancelamento';

                $eventos->store();

            } else {

                $objeto_pedido = PedidoEvento::find($this->pedido->id);
                if ($objeto_pedido) {

                    $objeto_pedido->status = 4;
                    $objeto_pedido->store();
                }

            }

            TTransaction::close();
            // Código gerado pelo snippet: "Exibir mensagem"
            new TMessage('info', "Soliciação enviada com sucesso!<br>Protocolo: " . $resposnse_cce->nProt . '<br>Retorno:' . $resposnse_cce->xMotivo);
            // -----

        }

    }
}
