<?php

use NFePHP\Common\Certificate;
use NFePHP\NFe\Common\Standardize;
use NFePHP\NFe\Make;
use NFePHP\NFe\Tools;
use NFePHP\NFe\Complements;
class GerarNFCeSped
{

    private $key;
    private $vendas;
    private $emitente;
    private $tools;
    private $DocumentoFiscal;
    private $carrinho_itens;
    private $cliente;
    private $NaturezaOp;
    private $endereco;
    private $cidade;
    private $estado;
    private $duplicata;
    private $frete;

    private static $database = 'kraftsoft';

    public function __construct($key)
    {
        $this->key = $key;
    }

    /**
     * Classe para gerar NFC-e para obtenção do XML
     *
     * Autor: Rubens dos Santos
     * Email: salvadorbba@gmail.com
     * Celular: (12) 99675-8056
     *
     * nfephp-org/sped-da:dev-master
     * nfephp-org/sped-mail:^1.0
     * nfephp-org/sped-nfe:^5.0
     * https://github.com/nfephp-org
     */
    public function create()
    {

        try {

            TTransaction::open(self::$database);
            $this->emitente = SystemUnit::find(TSession::getValue('userunitid'));
            $codigo_ibge_uf = substr($this->emitente->end_mun_codibge, 0, 2);
            $this->DocumentoFiscal = Notafiscal::find($this->key);
            $this->cliente = Cliente::find($this->DocumentoFiscal->cliente_id);
            $this->endereco = ClienteEndereco::where('cliente_id', '=', $this->DocumentoFiscal->cliente_id)->first();
            $this->cidade = Cidade::find($this->endereco->cidade_id);
            $this->estado = Estado::find($this->cidade->estado_id);
            $this->NaturezaOp = NaturezaOperacao::find($this->DocumentoFiscal->natureza_operacao_id);
            $this->carrinho_itens = NotafiscalItens::where('notafiscal_id', '=', $this->DocumentoFiscal->id)->load();
            $volumes = NotafiscalFreteVolumes::where('notafiscal_id', '=', $this->key)->load();
            $this->duplicata = NotafiscalDup::where('notafiscal_id', '=', $this->key)->load();

            $this->frete = NotafiscalModfrete::find($this->DocumentoFiscal->notafiscal_modfrete_id);

            TTransaction::close();

            $config = [
                'atualizacao' => date('Y-m-d H:i:s'),
                'tpAmb' => (int) $this->emitente->nfe_tpamb_id,
                'razaosocial' => $this->emitente->razao_social,
                'cnpj' => $this->emitente->cnpj, // PRECISA SER VÁLIDO
                'ie' => $this->emitente->inscricao_estadual, // PRECISA SER VÁLIDO
                'siglaUF' => $this->emitente->end_uf,
                'schemes' => 'PL_009_V4',
                'versao' => '4.00',
                "CSC" => $this->emitente->nfce_csc_homologacao,
                "CSCid" => '0002',
            ];

            $configJson = json_encode($config);
            $this->tools = new Tools(
                $configJson,
                Certificate::readPfx(file_get_contents($this->emitente->certificadoA1), $this->emitente->senha_certificado)

                //falta implementar do cloud nfe ****
            );

            #######conteudo da sped-nfe##########
            $nfe = $this->tools->model('65');
            $nfe = new Make();
            $stdInNFe = new stdClass();
            $stdInNFe->versao = '4.00'; //versão do layout (string)
            $stdInNFe->pk_nItem = null; //deixe essa variavel sempre como NULL
            $nfe->taginfNFe($stdInNFe);

            $stdIde = new stdClass();
            $stdIde->cUF = $codigo_ibge_uf;
            //   $stdIde->cNF = $nNFGerado;
            $stdIde->cNF = rand(11111111, 99999999);
            $stdIde->natOp = "VENDA AO CONSUMIDOR";
            $stdIde->mod = 65; //Modelo do Documento Fiscal

            // $stdIde->serie = 500;
            // $stdIde->nNF = 19; //Código Numérico que compõe a Chave de Acesso

            $stdIde->serie = $this->DocumentoFiscal->serie;
            $stdIde->nNF = $this->DocumentoFiscal->numero; //Código Numérico que compõe a Chave de Acesso
            $stdIde->dhEmi = date('Y-m-d') . 'T' . date('H:i:s') . '-03:00';
            $stdIde->dhSaiEnt = date('Y-m-d') . 'T' . date('H:i:s') . '-03:00';
            $stdIde->tpNF = 1; /// 1 = saida 0 = entrada

            $stdIde->idDest = '1';

            $stdIde->cMunFG = $this->emitente->end_mun_codibge; //Código do Município dO ibge
            $stdIde->tpImp = 4;
            $stdIde->tpEmis = 1; //Número do Documento Fiscal
            $stdIde->cDV = 7; //Dígito Verificador da Chave de Acesso
            $stdIde->tpAmb = $this->emitente->nfe_tpamb_id;
            $stdIde->finNFe = 1; //Se NF-e complementar (finNFe=2):– Não informado NF referenciada (NF modelo 1 ou NF-e)
            $stdIde->indFinal = 1;
            $stdIde->indPres = 1;
            $stdIde->procEmi = 0;
            $stdIde->verProc = "1.0.0"; //Identificador da versão do processo de emissão (informar a versão do aplicativo emissor de NF-e).
            $nfe->tagide($stdIde);

            ///////////// dados emitente ///////////////////////
            $stdEmit = new stdClass();
            $stdEmit->xNome = $this->emitente->razao_social;
            $stdEmit->xFant = $this->emitente->name;
            $stdEmit->IE = $this->emitente->inscricao_estadual;
            $stdEmit->IEST = null;
            $stdEmit->IM = $this->emitente->im ?? null;
            $stdEmit->CNAE = $this->emitente->cnae;
            $stdEmit->CRT = 2; /// revisa
            $stdEmit->CNPJ = $this->emitente->cnpj; //indicar apenas um CNPJ ou CPF
            $nfe->tagemit($stdEmit);

            $stdEnderEmit = new stdClass();
            $stdEnderEmit->xLgr = $this->emitente->end_logradouro;
            $stdEnderEmit->nro = $this->emitente->end_numero;
            $stdEnderEmit->xCpl = $this->emitente->end_complemento;
            $stdEnderEmit->xBairro = $this->emitente->end_bairro;
            $stdEnderEmit->cMun = $this->emitente->end_mun_codibge;
            $stdEnderEmit->xMun = $this->emitente->end_municipio;
            $stdEnderEmit->UF = $this->emitente->end_uf;
            $stdEnderEmit->CEP = MasterClass::TrataDoc($this->emitente->end_cep);
            $stdEnderEmit->cPais = '1058';
            $stdEnderEmit->xPais = 'Brasil';
            $stdEnderEmit->fone = MasterClass::TrataDoc($this->emitente->fone);
            $nfe->tagenderEmit($stdEnderEmit);

            if ($this->cliente->cpf_cnpj) {
                $stdDest = new stdClass();

                $stdDest->indIEDest = 9;

                $stdDest->ISUF = "";
                $stdDest->IM = "";

                $stdDest->xNome = $this->cliente->nome;

                // se não tiver Inscrição estadual
                $stdDest->IE = null;

                $documento = MasterClass::TrataDoc($this->cliente->cpf_cnpj);

                if (strlen($documento) == 11) {
                    $stdDest->CPF = MasterClass::TrataDoc($this->cliente->cpf_cnpj);
                    $stdDest->CNPJ = null;

                } else {

                    $stdDest->CNPJ = MasterClass::TrataDoc($this->cliente->cpf_cnpj);
                    $stdDest->CPF = null;
                    $stdDest->IE = $this->cliente->rg ?? null;

                }
                $nfe->tagdest($stdDest);
            }

            $item = 0;
            $soma = 0;
            $vdesc = 0;
            $vFrete = 0;
            $vseg = 0;
            $voutro = 0;
            $vCOFINS = 0;
            $vPIS = 0;
            $ICMSvBC = 0;
            $ICMSvICMS = 0;

            foreach ($this->carrinho_itens as $itens_nfe):
                TTransaction::open(self::$database);
                $ItemOrig = NotafiscalItemOrig::find($itens_nfe->notafiscal_item_orig_id);

                $ItemModbc = NotafiscalItemModbc::find($itens_nfe->notafiscal_item_modbc_id);

                TTransaction::close();
                $item++;
                $stdProdutos = new stdClass();
                $stdProdutos->item = $item;
                $stdProdutos->cEAN = !empty($stdProdutos->cEAN) ? $stdProdutos->cEAN : 'SEM GTIN';
                $stdProdutos->cEANTrib = !empty($stdProdutos->cEANTrib) ? $stdProdutos->cEANTrib : 'SEM GTIN';
                $stdProdutos->cProd = $itens_nfe->produto_id;
                $stdProdutos->xProd = $itens_nfe->xprod;
                $stdProdutos->NCM = $itens_nfe->ncm;
                $stdProdutos->CFOP = "5102";
                $stdProdutos->uCom = $itens_nfe->ucom;
                $stdProdutos->qCom = $itens_nfe->qcom;
                $stdProdutos->vUnCom = $itens_nfe->vuncom;
                $stdProdutos->vProd = $itens_nfe->vprod;
                $stdProdutos->uTrib = $itens_nfe->utrib;
                $stdProdutos->qTrib = $itens_nfe->qtrib;
                $stdProdutos->vUnTrib = $itens_nfe->vuntrib;
                $stdProdutos->indTot = 1;
                $nfe->tagprod($stdProdutos);

                $stdICMS = new stdClass();
                $stdICMS->item = $item; //item da NFe
                $stdICMS->orig = 0;
                $stdICMS->CST = "00";
                $stdICMS->modBC = 1;
                $stdICMS->vBC = 0.00;
                $stdICMS->pICMS = 0.00;
                $stdICMS->vICMS = 0.00;
                $ICMS = $nfe->tagICMS($stdICMS);

                $stdPIS = new stdClass();
                $stdPIS->item = $item; //item da NFe
                $stdPIS->CST = '99';
                $stdPIS->vBC = 0.00;
                $stdPIS->pPIS = 0.00;
                $stdPIS->vPIS = 0.00;
                $nfe->tagPIS($stdPIS);

                $stdCOFINS = new stdClass();
                $stdCOFINS->item = $item; //item da NFe
                $stdCOFINS->CST = '99';
                $stdCOFINS->vBC = 0.00;
                $stdCOFINS->pCOFINS = 0.00;
                $stdCOFINS->vCOFINS = 0.00;
                $nfe->tagCOFINS($stdCOFINS);

                $stdtagimposto = new stdClass();
                $stdtagimposto->item = $item;

                $stdtagimposto->vTotTrib = doubleval($itens_nfe->vtottrib);
                $nfe->tagimposto($stdtagimposto);

                $soma += $stdProdutos->vProd;
                $vdesc += $itens_nfe->vdesc;
                $vfrete += $itens_nfe->vfrete;
                $vseg += $itens_nfe->vseg;
                $voutro += $itens_nfe->voutro;
                $vCOFINS += $std->vCOFINS;
                $vPIS += $stdtagPIS->vPIS;
                $ICMSvBC += $stdtagICMS->vBC;
                $ICMSvICMS += $stdtagICMS->vICMS;
            endforeach;
            $vTotTrib = $vCOFINS + $vPIS + $ICMSvBC + $ICMSvICMS;
            $total_acumulado = $soma + $vfrete + $vseg + $voutro + $vCOFINS + $vPIS + $ICMSvBC + $ICMSvICMS;
            $desconto = $vdesc;
            $total_com_desconto = $total_acumulado - $desconto;

            $stdICMSTot = new stdClass();
            $stdICMSTot->vBC = 0.00;
            $stdICMSTot->vICMS = 0.00;
            $stdICMSTot->vProd = doubleval($soma);
            $stdICMSTot->vPIS = 0.00;
            $stdICMSTot->vCOFINS = 0.00;
            $stdICMSTot->vNF = doubleval($total_com_desconto);
            $stdICMSTot->vTotTrib = doubleval($vTotTrib);

            $stdtransp = new stdClass();
            $stdtransp->modFrete = 9;
            $nfe->tagtransp($stdtransp);

            $stdtroco = new stdClass();
            $stdtroco->vTroco = 0.00;
            $troco = $nfe->tagpag($stdtroco);

            $stdPag = new stdClass();
            $stdPag->tPag = "01";
            $stdPag->vPag = $stdICMSTot->vNF;
            //  $std->indPag =0;
            $pags = $nfe->tagdetPag($stdPag);

            // Novas formas de pagamento para o campo tPag:
            //     01 - Dinheiro
            //     02 - Cheque
            //     03 - Cartão de Crédito
            //     04 - Cartão de Débito
            //     05 - Crédito Loja
            //     10 - Vale Alimentação
            //     11 - Vale Refeição
            //     12 - Vale Presente
            //     13 - Vale Combustível
            //     14 - Duplicata Mercantil
            //     15 - Boleto Bancário
            //     90 - Sem Pagamento
            //     99 - Outros
            $stdtaginfRespTec = new stdClass();
            $stdtaginfRespTec->CNPJ = '11788740000150'; //CNPJ da pessoa jurídica responsável pelo sistema utilizado na emissão do documento fiscal eletrônico
            $stdtaginfRespTec->xContato = 'Rubens dos Santos'; //Nome da pessoa a ser contatada
            $stdtaginfRespTec->email = 'salvadorbba@gmail.com'; //E-mail da pessoa jurídica a ser contatada
            $stdtaginfRespTec->fone = MasterClass::TrataDoc('(71)99675-8056'); //Telefone da pessoa jurídica/física a ser contatada
            $nfe->taginfRespTec($stdtaginfRespTec);
            ErroNFeGeracao::view($nfe->dom->errors, $this->key);

            $XML = $nfe->getXML();
            $Chave = $nfe->getChave();
            $erros = $nfe->getErrors();
            $modelo = $nfe->getModelo();
            $padrao = 'xml_inicial';
            $xml_inicial = ClassMaster::getXML($this->key, $XML, $Chave, $padrao);

            if ($xml_inicial == 1) {
                TToast::show("show", "XML NFC-e Gerado com sucesso " . $padrao, "topLeft", "");
                $response_assinado = $this->tools->signNFe($XML);

                $st = new Standardize();
                $std = $st->toStd($response_assinado);
                $padrao = 'xml_assinado';
                $xml_assinado = ClassMaster::getXML($this->key, $response_assinado, $Chave, $padrao);

                if ($xml_assinado == 1) {
                    TToast::show("show", "XML NFC-e Assinado com sucesso " . $padrao, "topLeft", "");
                }

                try {
                    $idLote = str_pad(100, 15, '0', STR_PAD_LEFT); // Identificador do lote
                    $protocolo_busca = $this->tools->sefazEnviaLote([$response_assinado], $idLote, 1);
                    $st = new Standardize();
                    $std_prod = $st->toStd($protocolo_busca);

                    $padrao = 'protolo_xml';
                    $protolo_xml = ClassMaster::getXML($this->key, $protocolo_busca, $Chave, $padrao);
           

               
                            $autorizado = Complements::toAuthorize($response_assinado, $protocolo_busca);
                            
                            // $st = new Standardize();
                            // $response = $st->toStd($autorizado);
                         
                            
                            $padrao = 'xml';
                            $xml = ClassMaster::getXML($this->key, $autorizado, $Chave, $padrao, $std_prod->protNFe->infProt->nProt);
                            $xml_autorizado = ClassMaster::getXML($this->key, $autorizado, $Chave, 'xml_autorizado', $std_prod->protNFe->infProt->nProt);

                            if ($xml_autorizado == 1) {

                                $pageParam = []; // ex.: = ['key' => 10]

                                TApplication::loadPage('NotafiscalList', 'onShow', $pageParam);

                            }

                       
 
                    if ($std_prod->cStat != 103) {

                        $std_prod->protNFe->infProt->cStat;

                        new TMessage('info', 'Codigo: ' . $std_prod->protNFe->infProt->cStat . '<br> <b>Mensagem: </b>' . $std_prod->protNFe->infProt->xMotivo . '<hr>');

                        exit("[$std_prod->$std_prod] $std_prod->xMotivo");
                    }

                    $recibo = $std_prod->infRec->nRec; // Vamos usar a variável $recibo para consultar o status da nota
                } catch (Exception $e) {
                    //aqui você trata possiveis exceptions do envio

                    exit($e->getMessage());
                }

            }
        } catch (\Exception $e) {
            echo 'Erro na criação da instância da classe Tools: ' . $e->getMessage();
            // Outras ações de tratamento de erro, se necessário
        }
    }
}
