<?php

use NFePHP\Common\Certificate;
use NFePHP\NFe\Common\Standardize;
use NFePHP\NFe\Complements;
use NFePHP\NFe\Make;
use NFePHP\NFe\Tools;

class GerarNFeSped
{

    private $key;
    private $vendas;
    private $emitente;
    private $tools;
    private $DocumentoFiscal;
    private $carrinho_itens;
    private $cliente;
    private $NaturezaOp;
    private $endereco;
    private $cidade;
    private $estado;
    private $duplicata;
    private $frete;

    private static $database = 'kraftsoft';

    public function __construct($key)
    {
        $this->key = $key;
    }

    /**
     * Classe para gerar NF-e para obtenção do XML
     *
     * Autor: Rubens dos Santos
     * Email: salvadorbba@gmail.com
     * Celular: (12) 99675-8056
     *
     * nfephp-org/sped-da:dev-master
     * nfephp-org/sped-mail:^1.0
     * nfephp-org/sped-nfe:^5.0
     * https://github.com/nfephp-org
     */
    public function create()
    {

        try {

            TTransaction::open(self::$database);
            $this->emitente = SystemUnit::find(TSession::getValue('userunitid'));
            $codigo_ibge_uf = substr($this->emitente->end_mun_codibge, 0, 2);
            $this->DocumentoFiscal = Notafiscal::find($this->key);

            $this->cliente = Cliente::find($this->DocumentoFiscal->cliente_id);
            $this->endereco = ClienteEndereco::where('cliente_id', '=', $this->DocumentoFiscal->cliente_id)->first();
            $this->cidade = Cidade::find($this->endereco->cidade_id);
            $this->estado = Estado::find($this->cidade->estado_id);
            $this->NaturezaOp = NaturezaOperacao::find($this->DocumentoFiscal->natureza_operacao_id);
            $this->carrinho_itens = NotafiscalItens::where('notafiscal_id', '=', $this->DocumentoFiscal->id)->load();
            $volumes = NotafiscalFreteVolumes::where('notafiscal_id', '=', $this->key)->load();
            $this->duplicata = NotafiscalDup::where('notafiscal_id', '=', $this->key)->load();

            $this->frete = NotafiscalModfrete::find($this->DocumentoFiscal->notafiscal_modfrete_id);

            TTransaction::close();

            $config = [
                'atualizacao' => date('Y-m-d H:i:s'),
                'tpAmb' => (int) $this->emitente->nfe_tpamb_id,
                'razaosocial' => $this->emitente->razao_social,
                'cnpj' => $this->emitente->cnpj, // PRECISA SER VÁLIDO
                'ie' => $this->emitente->inscricao_estadual, // PRECISA SER VÁLIDO
                'siglaUF' => $this->emitente->end_uf,
                'schemes' => 'PL_009_V4',
                'versao' => '4.00',
            ];

            $configJson = json_encode($config);
            $this->tools = new Tools(
                $configJson,
                Certificate::readPfx(file_get_contents($this->emitente->certificadoA1), $this->emitente->senha_certificado)

                //falta implementar do cloud nfe ****
            );

            $nfe = new Make();

            $std = new stdClass();
            $std->versao = '4.00';
            $std->Id = null;
            $std->pk_nItem = '';
            $nfe->taginfNFe($std);
            $stdIde = new stdClass();
            $stdIde->cUF = $codigo_ibge_uf;

            $stdIde->cNF = rand(11111111, 99999999);
            $stdIde->natOp = $this->NaturezaOp->descricao;
            $stdIde->mod = 55; //Modelo do Documento Fiscal

            // $stdIde->serie = 500;
            // $stdIde->nNF = 19; //Código Numérico que compõe a Chave de Acesso

            $stdIde->serie = $this->DocumentoFiscal->serie;
            $stdIde->nNF = $this->DocumentoFiscal->numero; //Código Numérico que compõe a Chave de Acesso
            $stdIde->dhEmi = date('Y-m-d') . 'T' . date('H:i:s') . '-03:00';
            $stdIde->dhSaiEnt = date('Y-m-d') . 'T' . date('H:i:s') . '-03:00';
            $stdIde->tpNF = 1; /// 1 = saida 0 = entrada

            $codigo_ibge_uf = substr($this->emitente->end_mun_codibge, 0, 2);

            if ($codigo_ibge_uf == $this->endereco->ibge_estado):
                $stdIde->idDest = '1';
            elseif ($codigo_ibge_uf != $this->endereco->ibge_estado):
                $stdIde->idDest = '2';
            else:
                $stdIde->idDest = '3';
            endif;

            $stdIde->cMunFG = $this->emitente->end_mun_codibge; //Código do Município dO ibge
            $stdIde->tpImp = 1;
            $stdIde->tpEmis = 1; //Número do Documento Fiscal
            $stdIde->cDV = 7; //Dígito Verificador da Chave de Acesso
            $stdIde->tpAmb = $this->emitente->nfe_tpamb_id;
            $stdIde->finNFe = 1; //Se NF-e complementar (finNFe=2):– Não informado NF referenciada (NF modelo 1 ou NF-e)
            $stdIde->indFinal = 1;
            $stdIde->indPres = 0;
            $stdIde->procEmi = 0;
            $stdIde->verProc = "kraftsoft 4.0"; //Identificador da versão do processo de emissão (informar a versão do aplicativo emissor de NF-e).
            $tagide = $nfe->tagide($stdIde);

            ///////////// dados emitente ///////////////////////
            $stdEmit = new stdClass();
            $stdEmit->xNome = $this->emitente->razao_social;
            $stdEmit->xFant = $this->emitente->name;
            $stdEmit->IE = $this->emitente->inscricao_estadual ?? null;
            $stdEmit->IEST = null;
            $stdEmit->IM = $this->emitente->im ?? null;
            $stdEmit->CNAE = $this->emitente->cnae;
            $stdEmit->CRT = 2; /// revisa
            $stdEmit->CNPJ = MasterClass::TrataDoc($this->emitente->cnpj); //indicar apenas um CNPJ ou CPF
            $tagemit = $nfe->tagemit($stdEmit);

            $stdEnderEmit = new stdClass();
            $stdEnderEmit->xLgr = $this->emitente->end_logradouro;
            $stdEnderEmit->nro = $this->emitente->end_numero;
            $stdEnderEmit->xCpl = $this->emitente->end_complemento;
            $stdEnderEmit->xBairro = $this->emitente->end_bairro;
            $stdEnderEmit->cMun = $this->emitente->end_mun_codibge;
            $stdEnderEmit->xMun = $this->emitente->end_municipio;
            $stdEnderEmit->UF = $this->emitente->end_uf;
            $stdEnderEmit->CEP = MasterClass::TrataDoc($this->emitente->end_cep);
            $stdEnderEmit->cPais = '1058';
            $stdEnderEmit->xPais = 'Brasil';
            $stdEnderEmit->fone = MasterClass::TrataDoc($this->emitente->fone);
            $tagenderEmit = $nfe->tagenderEmit($stdEnderEmit);
            ///////////// dados emitente ///////////////////////

            ///////////// dados cliente ///////////////////////
            $stdDest = new stdClass();
            $stdDest->xNome = $this->cliente->nome;

            if (isset($this->cliente->rg)) {
                $stdDest->indIEDest = 1; // se não tiver Inscrição estadual
                $stdDest->IE = $this->cliente->rg;
            } else {

                $stdDest->indIEDest = 9; // se não tiver Inscrição estadual
                $stdDest->IE = null;
            }

            $stdDest->indIEDest = 9;
            $documento = MasterClass::TrataDoc($this->cliente->cpf_cnpj);

            if (strlen($documento) == 11) {
                $stdDest->CPF = MasterClass::TrataDoc($this->cliente->cpf_cnpj);
                $stdDest->CNPJ = null;

            } else {

                $stdDest->CNPJ = MasterClass::TrataDoc($this->cliente->cpf_cnpj);
                $stdDest->CPF = null;
                $stdDest->IE = $this->cliente->rg ?? null;

            }
            $stdDest->ISUF = '' ?? null;
            $stdDest->IM = $this->cliente->im ?? null;
            $stdDest->email = $this->cliente->email ?? null;

            $tagdest = $nfe->tagdest($stdDest);

            $stdEndereDest = new stdClass();
            $stdEndereDest->xLgr = $this->endereco->logradouro;
            $stdEndereDest->nro = $this->endereco->numero ?? null;
            $stdEndereDest->xCpl = $this->endereco->complemento ?? null;
            $stdEndereDest->xBairro = $this->endereco->bairro;
            $stdEndereDest->cMun = $this->cidade->codigo_ibge;
            $stdEndereDest->xMun = $this->cidade->nome;
            $stdEndereDest->UF = $this->estado->sigla;
            $stdEndereDest->CEP = MasterClass::TrataDoc($this->endereco->cep);
            $stdEndereDest->cPais = '1058';
            $stdEndereDest->xPais = 'Brasil';
            $stdEndereDest->fone = MasterClass::TrataDoc($this->endereco->telef) ?? null;
            $stdEndereDest->email = $this->cliente->email ?? null;
            $nfe->tagenderDest($stdEndereDest);
            ///////////// dados cliente ///////////////////////

            $stdtagtransp = new stdClass();
            $stdtagtransp->modFrete = $this->frete->valor;
            $nfe->tagtransp($stdtagtransp);

            $item = 0;
            $soma = 0;
            $vdesc = 0;
            $vFrete = 0;
            $vseg = 0;
            $voutro = 0;
            $vCOFINS = 0;
            $vPIS = 0;
            $ICMSvBC = 0;
            $ICMSvICMS = 0;

            foreach ($this->carrinho_itens as $itens_nfe):
                TTransaction::open(self::$database);
                $ItemOrig = NotafiscalItemOrig::find($itens_nfe->notafiscal_item_orig_id);

                $ItemModbc = NotafiscalItemModbc::find($itens_nfe->notafiscal_item_modbc_id);

                TTransaction::close();
                $item++;
                $stdProdutos = new stdClass();
                $stdProdutos->item = $item;
                $stdProdutos->cEAN = !empty($stdProdutos->cEAN) ? $stdProdutos->cEAN : 'SEM GTIN';
                $stdProdutos->cEANTrib = !empty($stdProdutos->cEANTrib) ? $stdProdutos->cEANTrib : 'SEM GTIN';
                $stdProdutos->cProd = $itens_nfe->produto_id;
                $stdProdutos->xProd = $itens_nfe->xprod;
                $stdProdutos->NCM = $itens_nfe->ncm;
                $stdProdutos->CFOP = $itens_nfe->cfop;
                $stdProdutos->uCom = $itens_nfe->ucom;
                $stdProdutos->qCom = $itens_nfe->qcom;
                $stdProdutos->vUnCom = $itens_nfe->vuncom;
                $stdProdutos->vProd = $itens_nfe->vprod;
                $stdProdutos->uTrib = $itens_nfe->utrib;
                $stdProdutos->qTrib = $itens_nfe->qtrib;
                $stdProdutos->vUnTrib = $itens_nfe->vuntrib;
                $stdProdutos->indTot = 1;
                $nfe->tagprod($stdProdutos);

                $stdtagimposto = new stdClass();
                $stdtagimposto->item = $item;
                $stdtagimposto->vTotTrib = $itens_nfe->vtottrib;
                $nfe->tagimposto($stdtagimposto);

                $stdtagICMS = new stdClass();
                $stdtagICMS->item = $item;
                $stdtagICMS->orig = $ItemOrig->codigo;
                $stdtagICMS->CST = ClassMaster::interpretarCodigo($itens_nfe->icms_cst);
                $stdtagICMS->modBC = $ItemOrig->codigo;
                $stdtagICMS->vBC = doubleval($itens_nfe->icms_basecalc);
                $stdtagICMS->pICMS = doubleval($itens_nfe->icms_pcredsn);
                $stdtagICMS->vICMS = doubleval($itens_nfe->icms_valor);
                $nfe->tagICMS($stdtagICMS);

                $stdtagPIS = new stdClass();
                $stdtagPIS->item = $item;
                $stdtagPIS->CST = $itens_nfe->pis_cst;
                $stdtagPIS->vBC = doubleval($itens_nfe->pis_valor_base);
                $stdtagPIS->pPIS = doubleval($itens_nfe->pis_aliquota);
                $stdtagPIS->vPIS = doubleval($itens_nfe->pis_valor_base);
                $nfe->tagPIS($stdtagPIS);

                $std = new stdClass();
                $std->item = $item;
                $std->CST = $itens_nfe->cofins_cst;
                $std->vBC = doubleval($itens_nfe->cofins_valor_base);
                $std->pCOFINS = doubleval($itens_nfe->cofins_aliquota);
                $std->vCOFINS = doubleval($itens_nfe->cofins_valor);
                $nfe->tagCOFINS($std);

                $stdtagimposto = new stdClass();
                $stdtagimposto->item = $item;

                $stdtagimposto->vTotTrib = doubleval($itens_nfe->vtottrib);
                $nfe->tagimposto($stdtagimposto);

                // ICMS Interestadual
                $stdicms_interestadual = new stdClass();
                $stdicms_interestadual->item = $item;
                $stdicms_interestadual->vBCUFDest = doubleval($itens_nfe->icms_vbcfcpufdest);
                $vBC = $stdicms_interestadual->vBCUFDest * $stdicms_interestadual->pICMSInter;
                $stdicms_interestadual->pFCPUFDest = doubleval($itens_nfe->icms_pfcpufdest);
                $stdicms_interestadual->pICMSUFDest = doubleval($itens_nfe->icms_picmsufdest);
                $stdicms_interestadual->pICMSInter = doubleval($itens_nfe->icms_picmsinter);
                $stdicms_interestadual->pICMSInterPart = doubleval($itens_nfe->icms_picmsinterpart);
                $valorIntermediario = $vBC * $stdicms_interestadual->pICMSInterPart;
                $stdicms_interestadual->vFCPUFDest = doubleval($itens_nfe->icms_vfcpufdest);
                $stdicms_interestadual->vICMSUFDest = $vBCUFDest * $pICMSUFDest - $valorIntermediario;
                $stdicms_interestadual->vICMSUFRemet = $itens_nfe->icms_vicmsufremet ?? 0;
                $nfe->tagICMSUFDest($stdicms_interestadual);

                $soma += $stdProdutos->vProd;
                $vdesc += $itens_nfe->vdesc;
                $vfrete += $itens_nfe->vfrete;
                $vseg += $itens_nfe->vseg;
                $voutro += $itens_nfe->voutro;
                $vCOFINS += $std->vCOFINS;
                $vPIS += $stdtagPIS->vPIS;
                $ICMSvBC += $stdtagICMS->vBC;
                $ICMSvICMS += $stdtagICMS->vICMS;
            endforeach;

            $vTotTrib = $vCOFINS + $vPIS + $ICMSvBC + $ICMSvICMS;
            $total_acumulado = $soma + $vfrete + $vseg + $voutro + $vCOFINS + $vPIS + $ICMSvBC + $ICMSvICMS;
            $desconto = $vdesc;
            $total_com_desconto = $total_acumulado - $desconto;

            $stdICMSTot = new stdClass();
            $stdICMSTot->vBC = $ICMSvBC;
            $stdICMSTot->vICMS = $ICMSvICMS;
            $stdICMSTot->vProd = $soma;
            $stdICMSTot->vFrete = doubleval($vfrete);
            $stdICMSTot->vSeg = doubleval($vseg);
            $stdICMSTot->vDesc = $vdesc;
            $stdICMSTot->vPIS = doubleval($vPIS);
            $stdICMSTot->vCOFINS = doubleval($vCOFINS);
            $stdICMSTot->vOutro = doubleval($voutro);
            $stdICMSTot->vNF = doubleval($total_com_desconto);
            $stdICMSTot->vTotTrib = doubleval($vTotTrib);
            $nfe->tagICMSTot($stdICMSTot);

            $stdtagpag = new stdClass();
            $stdtagpag->vTroco = 0;
            $nfe->tagpag($stdtagpag);

            $stdtaginfRespTec = new stdClass();
            $stdtaginfRespTec->CNPJ = '11788740000150'; //CNPJ da pessoa jurídica responsável pelo sistema utilizado na emissão do documento fiscal eletrônico
            $stdtaginfRespTec->xContato = 'Rubens dos Santos'; //Nome da pessoa a ser contatada
            $stdtaginfRespTec->email = 'salvadorbba@gmail.com'; //E-mail da pessoa jurídica a ser contatada
            $stdtaginfRespTec->fone = MasterClass::TrataDoc('(71)99675-8056'); //Telefone da pessoa jurídica/física a ser contatada
            $nfe->taginfRespTec($stdtaginfRespTec);

            $stdTronsportadora = new stdClass();

            $stdTronsportadora = new stdClass();

            switch ($this->frete->valor) {
                case '0':
                case '1':
                case '2':
                    $stdTronsportadora->xNome = $this->DocumentoFiscal->transp_nome;
                    $stdTronsportadora->IE = $this->DocumentoFiscal->transp_inscest;
                    $stdTronsportadora->xEnder = $this->DocumentoFiscal->transp_endereco;
                    $stdTronsportadora->xMun = $this->DocumentoFiscal->transp_cidade;
                    $stdTronsportadora->UF = $this->DocumentoFiscal->transp_uf;
                    $stdTronsportadora->CNPJ = (strlen($this->DocumentoFiscal->transp_cnpj) == '14') ? $this->DocumentoFiscal->transp_cnpj : null;
                    $stdTronsportadora->CPF = (strlen($this->DocumentoFiscal->transp_cnpj) != '14') ? $this->DocumentoFiscal->transp_cnpj : null;
                    break;
                case '3':
                    $stdTronsportadora->xNome = $this->emitente->razao_social;
                    $stdTronsportadora->IE = $this->emitente->inscricao_estadual ?? null;
                    $stdTronsportadora->xEnder = $this->emitente->end_logradouro . ',' . $this->emitente->end_numero . ' ' . $this->emitente->end_bairro;
                    $stdTronsportadora->xMun = $this->emitente->end_municipio;
                    $stdTronsportadora->UF = $this->emitente->end_uf;
                    $stdTronsportadora->CNPJ = (strlen($this->emitente->cnpj) == '14') ? $this->emitente->cnpj : null;
                    $stdTronsportadora->CPF = null;
                    break;
                case '4':
                    $stdTronsportadora->xNome = $this->cliente->nome;
                    $stdTronsportadora->IE = $this->cliente->rg;
                    $stdTronsportadora->xEnder = $this->endereco->logradouro . ' ' . $this->endereco->numero;
                    $stdTronsportadora->xMun = $this->cidade->nome;
                    $stdTronsportadora->UF = $this->estado->sigla;
                    $stdTronsportadora->CNPJ = (strlen($this->cliente->cpf_cnpj) == '14') ? $this->cliente->cpf_cnpj : null;
                    $stdTronsportadora->CPF = (strlen($this->cliente->cpf_cnpj) != '14') ? $this->cliente->cpf_cnpj : null;
                    break;
                case '9':
                    // Sem Ocorrência de Transporte
                    // Pode retornar ou fazer outra ação, dependendo da lógica desejada.
                    return;
            }
            
            if(isset($volumes )){
            $itens = 0;
            foreach ($volumes as $itens_colume) {
                $itens++;
                if (isset($ListaProd->nlacre)) {
                    $stdVolume = new stdClass();
                    $stdVolume->item = $itens; //indicativo do numero do volume
                    $stdVolume->qVol = $itens_colume->qVol;
                    $stdVolume->esp = $itens_colume->esp;
                    $stdVolume->marca = $itens_colume->marca;
                    $stdVolume->nVol = $itens_colume->nVol;
                    $stdVolume->pesoL = $itens_colume->pesol;
                    $stdVolume->pesoB = $itens_colume->pesob;
                    $nfe->tagvol($stdVolume);

                }

            }
}


 
            if ($codigo_ibge_uf  == $this->endereco->ibge_estado) {
                if (isset($this->DocumentoFiscal->transp_placaveiculo)) {
                    $stdveiculo = new stdClass();
                    $stdveiculo->placa = ($this->DocumentoFiscal->transp_placaveiculo);
                    $stdveiculo->UF = $this->DocumentoFiscal->transp_veiculouf;
                    $stdveiculo->RNTC = $this->DocumentoFiscal->transp_veiculoRNTC;

                    $nfe->tagveicTransp($stdveiculo);
                }
            }

            if (isset($this->DocumentoFiscal->entrega_cpfcnpj)) {
                $stdentrega = new stdClass();

                if (strlen($this->DocumentoFiscal->entrega_cpfcnpj) == '11') {
                    $stdentrega->CNPJ = null;
                    $stdentrega->CPF = $this->DocumentoFiscal->entrega_cpfcnpj;
                    $stdentrega->IE = null;
                } else {
                    $stdentrega->CNPJ = $this->DocumentoFiscal->entrega_cpfcnpj;
                    $stdentrega->CPF = null;
                    $stdentrega->IE = null;

                }

                $stdentrega->xNome = $this->DocumentoFiscal->entrega_xnome;
                $stdentrega->xLgr = $this->DocumentoFiscal->entrega_xlgr;
                $stdentrega->nro = $this->DocumentoFiscal->entrega_nro;
                $stdentrega->xCpl = $this->DocumentoFiscal->entrega_xcpl;
                $stdentrega->xBairro = $this->DocumentoFiscal->entrega_xbairro;
                $stdentrega->cMun = $this->DocumentoFiscal->entrega_cmun;
                $stdentrega->xMun = $this->DocumentoFiscal->entrega_xmun;
                $stdentrega->UF = $this->DocumentoFiscal->entrega_uf;
                $stdentrega->CEP = $this->DocumentoFiscal->cep_entrega;
                $stdentrega->cPais = '1058';
                $stdentrega->xPais = 'Brasil';
                $stdentrega->fone = null;
                $stdentrega->email = null;
                $nfe->tagentrega($stdentrega);
            }

            $stdautXML = new stdClass();

            //só pode haver um ou CNPJ ou CPF, se um deles é especificado o outro deverá ser null
            $stdautXML->CNPJ = '10572014000133';
            $stdautXML->CPF = null;

            $tagautXML = $nfe->tagautXML($stdautXML);

            $stdinfAdic = new stdClass();
            $stdinfAdic->infAdFisco = 'informacoes para o fisco';
            $stdinfAdic->infCpl = "Valor aproximado tributos R$: 9,43 (4,20%) Fonte: IBPT";
            $nfe->taginfAdic($stdinfAdic);

            if (!empty($this->duplicata)) {
                foreach ($this->duplicata as $dup) {
                    $cont++;
                    $std->nDup = $cont;
                    $std->dVenc = $dup->dvenc;
                    $std->vDup = doubleval($dup->vdup);
                    $nfe->tagdup($std);
                }
            }
            $stdtagdetPag = new stdClass();
            $stdtagdetPag->indPag = 0;
            $stdtagdetPag->tPag = "01";
            $stdtagdetPag->vPag = $soma;
            $stdtagdetPag->indPag = 0;
            $nfe->tagdetPag($stdtagdetPag);

            ErroNFeGeracao::view($nfe->dom->errors, $this->key);

            $xml = $nfe->monta();
            $XML = $nfe->getXML();
            $erros = $nfe->getErrors();
            $Chave = $nfe->getChave();

            if (isset($Chave)) {
                $padrao = 'xml_inicial';
                $xml_inicial = ClassMaster::getXML($this->key, $XML, $Chave, $padrao);
                if ($xml_inicial == 1) {
                    //     TToast::show("show", "XML Gerado " . $padrao, "topLeft", "");
                    $padrao = 'xml_assinado';
                    $xmlAssinado = $this->tools->signNFe($XML);

                    $st = new Standardize();
                    $std = $st->toStd($xmlAssinado);

                    $xml_assinado = ClassMaster::getXML($this->key, $xmlAssinado, $Chave, $padrao);

                    if ($xml_assinado == 1) {

                        //  TToast::show("show", "XML Assinado " . $padrao, "topLeft", "");

                    } else {

                        //      TToast::show("info", "erro ao Assinar " . $padrao, "topLeft", "");
                    }

                    try {
                        $idLote = str_pad(100, 15, '0', STR_PAD_LEFT); // Identificador do lote
                        $resp = $this->tools->sefazEnviaLote([$xmlAssinado], $idLote);
                        $st = new Standardize();
                        $std = $st->toStd($resp);
                        $stds = $st->toStd($resp);

                        sleep(5);

                        if ($std->cStat != 103) {

                            $std->protNFe->infProt->cStat;

                            new TMessage('info', 'Codigo: ' . $std->protNFe->infProt->cStat . '<br> <b>Mensagem: </b>' . $std->protNFe->infProt->xMotivo . '<hr>');

                            exit("[$std->cStat] $std->xMotivo");
                        }

                        $recibo = $std->infRec->nRec; // Vamos usar a variável $recibo para consultar o status da nota
                    } catch (Exception $e) {
                        //aqui você trata possiveis exceptions do envio

                        exit($e->getMessage());
                    }

                    ######################protocolar########################

                    try {

                         $protocolo_busca = $this->tools->sefazConsultaRecibo($recibo);
                        $st = new Standardize();
                        $std_prod = $st->toStd($protocolo_busca);
              
                        $padrao = 'protolo_xml';
                        $protolo_xml = ClassMaster::getXML($this->key, $protocolo_busca , $Chave, $padrao);

                        if ($std->protNFe->infProt->cStat == 539) {

                            new TMessage('info', 'Rejeição: Duplicidade de NF-e, com diferenca na Chave de Acesso');

                            exit;
                        }

                        if ($std->protNFe->infProt->cStat >= 225) {

                            TToast::show("show", 'Codigo: ' . $std_prod->protNFe->infProt->cStat . '<br> <b>Mensagem: </b>' . $std_prod->protNFe->infProt->xMotivo . '<hr>', "center", "");

                            exit;
                        }
                    } catch (Exception $e) {
                        //aqui você trata possíveis exceptions da consulta
                        exit($e->getMessage());
                    }
 

                    try {
                        $autorizado = Complements::toAuthorize($xmlAssinado, $protocolo_busca);
                        $padrao = 'xml';
                        $xml = ClassMaster::getXML($this->key, $autorizado, $Chave, $padrao, $std_prod->protNFe->infProt->nProt);
                        $xml_autorizado = ClassMaster::getXML($this->key, $autorizado, $Chave, 'xml_autorizado', $std_prod->protNFe->infProt->nProt);

                        if ($xml_autorizado == 1) {

                            $pageParam = []; // ex.: = ['key' => 10]

                            TApplication::loadPage('NotafiscalList', 'onShow', $pageParam);

                        }

                    } catch (\Exception $e) {
                        echo "Erro: " . $e->getMessage();
                    }

                    if ($std_prod->cStat != 103) {

                        $std_prod->protNFe->infProt->cStat;

                        new TMessage('info', 'Codigo: ' . $std_prod->protNFe->infProt->cStat . '<br> <b>Mensagem: </b>' . $std_prod->protNFe->infProt->xMotivo . '<hr>');

                        exit("[$std_prod->$std_prod] $std_prod->xMotivo");
                    }

                    ######################autorizar########################

                }

            }

        } catch (\Exception $e) {

            new TMessage('info', $e->getMessage());

        }

    }

}
