<?php

use NFePHP\Common\Certificate;
use NFePHP\NFe\Common\Standardize;
use NFePHP\NFe\Tools;

/**
 * Classe Gera  carata de correção obtenção do XML
 *
 * Autor: Rubens dos Santos
 * Email: salvadorbba@gmail.com
 * Celular: (12) 99675-8056
 */
class InutilizaSpedNFe
{
    private $key;
    private $vendas;
    private $emitente;
    private $tools;
    private $nota;
    private $cliente;
    private $inutilizar;

    private $PedidoEvento;
    private static $database = 'kraftsoft';

    /**
     * Construtor da classe InutilizaSpedNFe
     *
     * @param string $key - A chave para buscar os parâmetros no banco de dados.
     */
    public function __construct($key)
    {
        $this->key = $key;
    }

    /**
     * Função para criar e obter um token de autenticação do Sicredi.
     *
     * @return string - O token de acesso obtido.
     */
    public function inut()
    {

        try {

            // Abrir uma transação de banco de dados no contexto 'agendamentos'.
            TTransaction::open(self::$database);
            $this->inutilizar = InutilizarNumero::find($this->key);

            $this->emitente = SystemUnit::find(TSession::getValue('userunitid'));

            $nNFIni = $this->inutilizar->ninicio;
            $nNFFin = $this->inutilizar->nfinal;

            TTransaction::close();

            $config = [
                'atualizacao' => date('Y-m-d H:i:s'),
                'tpAmb' => (int) $this->emitente->nfe_tpamb_id,
                'razaosocial' => $this->emitente->razao_social,
                'cnpj' => $this->emitente->cnpj, // PRECISA SER VÁLIDO
                'ie' => $this->emitente->inscricao_estadual, // PRECISA SER VÁLIDO
                'siglaUF' => $this->emitente->end_uf,
                'schemes' => 'PL_009_V4',
                'versao' => '4.00',
            ];

            $configJson = json_encode($config);
            $this->tools = new Tools(
                $configJson,
                Certificate::readPfx(file_get_contents($this->emitente->certificadoA1), $this->emitente->senha_certificado)
                
                //falta implementar do cloud nfe ****
            );

            $nSerie = $this->inutilizar->serie;
            $nIni = $this->inutilizar->ninicio;
            $nFin = $this->inutilizar->nfinal;
            $xJust = $this->inutilizar->xjust;

            ///// inutilizar junto a sefaz
            $response = $this->tools->sefazInutiliza($nSerie, $nIni, $nFin, $xJust);
            $stdCl = new Standardize($response);
            $std = $stdCl->toStd();

            TTransaction::open(self::$database);
            if ($std->infInut->cStat == 102) {
                $objeto = InutilizarNumero::find($this->key);
                if ($objeto) {
                    $objeto->cstat = $std->infInut->cStat;
                    $objeto->protocolo = $std->infInut->nProt;
                    $objeto->xml_retorno = $response;
                    $objeto->xmotivo = $std->infInut->xMotivo;
                    $objeto->tipo = 1;
                    $objeto->store();
                }

                sleep(1);

                $Numero_nota = NotafiscalSeries::where('notafiscal_mod_id', '=', 1)->
                    where('system_unit_id', '=', TSession::getValue('userunitid'))->first();

                if ($Numero_nota) {
                    $Numero_nota->notafiscal_tpamb_id = $this->emitente->nfe_tpamb_id;
                    $Numero_nota->alterado_por = TSession::getValue('userid');
                    $Numero_nota->alterado_em = date('Y-m-d H:i:s');
                    $Numero_nota->sequencial = $std->infInut->nNFFin;
                    $Numero_nota->store();
                }

                TTransaction::close();
                $message = "Código: " . $std->infInut->cStat . "<br>" .
                "Motivo: " . $std->infInut->xMotivo . "<br>" .
                "UF: " . $std->infInut->cUF . "<br>" .
                "Ano: " . $std->infInut->ano . "<br>" .
                "CNPJ: " . $std->infInut->CNPJ . "<br>" .
                "Modelo: " . $std->infInut->mod . "<br>" .
                "Série: " . $std->infInut->serie . "<br>" .
                "Número Inicial: " . $std->infInut->nNFIni . "<br>" .
                "Número Final: " . $std->infInut->nNFFin . "<br>" .
                "Data de Recebimento: " . $std->infInut->dhRecbto . "<br>" .
                "Número de Protocolo: " . $std->infInut->nProt;

            } else {

                $message = "Código: " . $std->infInut->cStat . "<br>" .
                "Motivo: " . $std->infInut->xMotivo . "<br>" .
                "UF: " . $std->infInut->cUF . "<br>" .
                "Data de Recebimento: " . $std->infInut->dhRecbto . "<br>";

            }

            new TMessage('info', $message);
        } catch (\Exception $e) {

            return $e->getMessage();
        }
    }
}
