<?php

use NFePHP\Common\Certificate;
use NFePHP\NFe\Make;
use NFePHP\NFe\Tools;

class GerarNFeSped
{

    private $key;
    private $vendas;
    private $emitente;
    private $tools;
    private $DocumentoFiscal;
    private $carrinho_itens;
    private $cliente;
    private $NaturezaOp;
    private $endereco;
    private $cidade;
    private $estado;

    private static $database = 'kraftsoft';

    public function __construct($key)
    {
        $this->key = $key;
    }

    /**
     * Classe para gerar NF-e para obtenção do XML
     *
     * Autor: Rubens dos Santos
     * Email: salvadorbba@gmail.com
     * Celular: (12) 99675-8056
     *
     * nfephp-org/sped-da:dev-master
     * nfephp-org/sped-mail:^1.0
     * nfephp-org/sped-nfe:^5.0
     * https://github.com/nfephp-org
     */
    public function create()
    {

        try {

            TTransaction::open(self::$database);
            $this->emitente = SystemUnit::find(TSession::getValue('userunitid'));
            $codigo_ibge_uf = substr($this->emitente->end_mun_codibge, 0, 2);
            $this->DocumentoFiscal = Notafiscal::find($this->key);
            $this->cliente = Cliente::find($this->DocumentoFiscal->cliente_id);
            $this->endereco = ClienteEndereco::where('cliente_id', '=', $this->DocumentoFiscal->cliente_id)->first();
            $this->cidade = Cidade::find($this->endereco->cidade_id);
            $this->estado = Estado::find($this->cidade->estado_id);
            $this->NaturezaOp = NaturezaOperacao::find($this->DocumentoFiscal->natureza_operacao_id);
            $this->carrinho_itens = NotafiscalItens::where('notafiscal_id', '=', $this->DocumentoFiscal->id)->load();

            $volumes = NotafiscalFreteVolumes::where('notafiscal_id', '=', $this->key)->load();

            TTransaction::close();
 
          $config = [
                'atualizacao' => date('Y-m-d H:i:s'),
                'tpAmb' => (int) $this->emitente->nfe_tpamb_id,
                'razaosocial' => $this->emitente->razao_social,
                'cnpj' => $this->emitente->cnpj, // PRECISA SER VÁLIDO
                'ie' => $this->emitente->inscricao_estadual, // PRECISA SER VÁLIDO
                'siglaUF' => $this->emitente->end_uf,
                'schemes' => 'PL_009_V4',
                'versao' => '4.00',
            ];

            $configJson = json_encode($config);
            $this->tools = new Tools(
                $configJson,
                Certificate::readPfx(file_get_contents($this->emitente->certificadoA1), $this->emitente->senha_certificado)

                //falta implementar do cloud nfe ****
            );

            $nfe = new Make();

            $std = new stdClass();
            $std->versao = '4.00';
            $std->Id = null;
            $std->pk_nItem = '';
            $nfe->taginfNFe($std);
            $stdIde = new stdClass();
            $stdIde->cUF = $codigo_ibge_uf;

            $stdIde->cNF = rand(11111111, 99999999);
            $stdIde->natOp = $this->NaturezaOp->descricao;
            $stdIde->mod = 55; //Modelo do Documento Fiscal

            $stdIde->serie = $this->DocumentoFiscal->serie;
            $stdIde->nNF = $this->DocumentoFiscal->numero; //Código Numérico que compõe a Chave de Acesso
            $stdIde->dhEmi = date('Y-m-d') . 'T' . date('H:i:s') . '-03:00';
            $stdIde->dhSaiEnt = date('Y-m-d') . 'T' . date('H:i:s') . '-03:00';
            $stdIde->tpNF = 1; /// 1 = saida 0 = entrada

            $codigo_ibge_uf = substr($this->emitente->end_mun_codibge, 0, 2);

            if ($this->cliente->cuf == $this->endereco->ibge_estado):
                $stdIde->idDest = '1';
            elseif ($this->cliente->cuf != $this->endereco->ibge_estado):
                $stdIde->idDest = '2';
            else:
                $stdIde->idDest = '3';
            endif;

            $stdIde->cMunFG = $this->emitente->end_mun_codibge; //Código do Município dO ibge
            $stdIde->tpImp = 1;
            $stdIde->tpEmis = 1; //Número do Documento Fiscal
            $stdIde->cDV = 7; //Dígito Verificador da Chave de Acesso
            $stdIde->tpAmb = $this->emitente->nfe_tpamb_id;
            $stdIde->finNFe = 1; //Se NF-e complementar (finNFe=2):– Não informado NF referenciada (NF modelo 1 ou NF-e)
            $stdIde->indFinal = 1;
            $stdIde->indPres = 0;
            $stdIde->procEmi = 0;
            $stdIde->verProc = "kraftsoft 4.0"; //Identificador da versão do processo de emissão (informar a versão do aplicativo emissor de NF-e).
            $tagide = $nfe->tagide($stdIde);

            ///////////// dados emitente ///////////////////////
            $stdEmit = new stdClass();
            $stdEmit->xNome = $this->emitente->razao_social;
            $stdEmit->xFant = $this->emitente->name;
            $stdEmit->IE = $this->emitente->inscricao_estadual ?? null;
            $stdEmit->IEST = null;
            $stdEmit->IM = $this->emitente->im ?? null;
            $stdEmit->CNAE = $this->emitente->cnae;
            $stdEmit->CRT = $this->emitente->notafiscal_crt_id;
            $stdEmit->CNPJ = MasterClass::TrataDoc($this->emitente->cnpj); //indicar apenas um CNPJ ou CPF
            $tagemit = $nfe->tagemit($stdEmit);

            $stdEnderEmit = new stdClass();
            $stdEnderEmit->xLgr = $this->emitente->end_logradouro;
            $stdEnderEmit->nro = $this->emitente->end_numero;
            $stdEnderEmit->xCpl = $this->emitente->end_complemento;
            $stdEnderEmit->xBairro = $this->emitente->end_bairro;
            $stdEnderEmit->cMun = $this->emitente->end_mun_codibge;
            $stdEnderEmit->xMun = $this->emitente->end_municipio;
            $stdEnderEmit->UF = $this->emitente->end_uf;
            $stdEnderEmit->CEP = MasterClass::TrataDoc($this->emitente->end_cep);
            $stdEnderEmit->cPais = '1058';
            $stdEnderEmit->xPais = 'Brasil';
            $stdEnderEmit->fone = MasterClass::TrataDoc($this->emitente->fone);
            $tagenderEmit = $nfe->tagenderEmit($stdEnderEmit);
            ///////////// dados emitente ///////////////////////

            ///////////// dados cliente ///////////////////////
            $stdDest = new stdClass();
            $stdDest->xNome = $this->cliente->nome;

            if (isset($this->cliente->rg)) {
                $stdDest->indIEDest = 1; // se não tiver Inscrição estadual
                $stdDest->IE = $this->cliente->rg;
            } else {

                $stdDest->indIEDest = 9; // se não tiver Inscrição estadual
                $stdDest->IE = null;
            }

            $stdDest->indIEDest = 9;
            $documento = MasterClass::TrataDoc($this->cliente->cpf_cnpj);

            if (strlen($documento) == 11) {
                $stdDest->CPF = MasterClass::TrataDoc($this->cliente->cpf_cnpj);
                $stdDest->CNPJ = null;

            } else {

                $stdDest->CNPJ = MasterClass::TrataDoc($this->cliente->cpf_cnpj);
                $stdDest->CPF = null;
                $stdDest->IE = $this->cliente->rg ?? null;

            }
            $stdDest->ISUF = '' ?? null;
            $stdDest->IM = $this->cliente->im ?? null;
            $stdDest->email = $this->cliente->email ?? null;

            $tagdest = $nfe->tagdest($stdDest);
 
            $stdEndereDest = new stdClass();
            $stdEndereDest->xLgr = $this->endereco->logradouro;
            $stdEndereDest->nro = $this->endereco->numero ?? null;
            $stdEndereDest->xCpl = $this->endereco->complemento ?? null;
            $stdEndereDest->xBairro = $this->endereco->bairro;
            $stdEndereDest->cMun = $this->cidade->codigo_ibge;
            $stdEndereDest->xMun = $this->cidade->nome;
            $stdEndereDest->UF = $this->estado->sigla;
            $stdEndereDest->CEP = MasterClass::TrataDoc($this->endereco->cep);
            $stdEndereDest->cPais = '1058';
            $stdEndereDest->xPais = 'Brasil';
            $stdEndereDest->fone = MasterClass::TrataDoc($this->endereco->telef) ?? null;
            $stdEndereDest->email = $this->cliente->email ?? null;
            $nfe->tagenderDest($stdEndereDest);
            ///////////// dados cliente ///////////////////////

            $stdtagtransp = new stdClass();
            $stdtagtransp->modFrete = $this->DocumentoFiscal->notafiscal_modfrete_id;
            $nfe->tagtransp($stdtagtransp);
            
            
            
            

            $item = 0;
            $soma = 0;
            foreach ($this->carrinho_itens as $itens_nfe):

                $item++;
                $stdProdutos = new stdClass();
                $stdProdutos->item = $item;
                $stdProdutos->cEAN = !empty($stdProdutos->cEAN) ? $stdProdutos->cEAN : 'SEM GTIN';
                $stdProdutos->cEANTrib = !empty($stdProdutos->cEANTrib) ? $stdProdutos->cEANTrib : 'SEM GTIN';
                $stdProdutos->cProd = $itens_nfe->produto_id;
                $stdProdutos->xProd = $itens_nfe->xprod;
                $stdProdutos->NCM = $itens_nfe->ncm;
                $stdProdutos->CFOP = $itens_nfe->cfop;
                $stdProdutos->uCom = $itens_nfe->ucom;
                $stdProdutos->qCom = $itens_nfe->qcom;
                $stdProdutos->vUnCom = $itens_nfe->vuncom;
                $stdProdutos->vProd = $itens_nfe->vprod;
                $stdProdutos->uTrib = $itens_nfe->utrib;
                $stdProdutos->qTrib = $itens_nfe->qtrib;
                $stdProdutos->vUnTrib = $itens_nfe->vuntrib;
                $stdProdutos->indTot =   1 ;
                $nfe->tagprod($stdProdutos);

 

                $soma += $stdProdutos->vUnCom * $stdProdutos->qCom;
    $soma += $stdProdutos->vProd;

                if (count($volumes) > 0) {

                    $std = new stdClass();

                    foreach ($volumes as $vol) {
                        $stdtagvol = new stdClass();
                        $stdtagvol->item = $item;
                        $stdtagvol->qVol = strval($vol->qVol);
                        $stdtagvol->esp = $vol->esp;
                        $stdtagvol->marca = $vol->marca;
                        $stdtagvol->nVol = $vol->nVol;
                        $stdtagvol->pesoL = $vol->pesol;
                        $stdtagvol->pesoB = $vol->pesob;
                        $nfe->tagvol($stdtagvol);
                    }
                }

                if (count($volumes) > 0) {
                    foreach ($volumes as $vol) {
                        $stdtaglacres = new stdClass();
                        $stdtaglacres->item = $item; //indicativo do numero do volume
                        $stdtaglacres->nLacre = $nota->transp_numeracao;

                        $nfe->taglacres($std);
                    }
                }

                TTransaction::open(self::$database);
                $origem = NotafiscalItemOrig::find($itens_nfe->notafiscal_item_orig_id);
                TTransaction::close();
                $stdtagimposto = new stdClass();
                $stdtagimposto->item = $item;

                $stdtagimposto->vTotTrib = doubleval($itens_nfe->vtottrib);
                $nfe->tagimposto($stdtagimposto);
                
                
                
                
                
                 
              $stdICMSItem = new stdClass();
                $stdICMSItem->item = $item;
                $stdICMSItem->orig = strval($origem->codigo);
                $stdICMSItem->CST = $itens_nfe->icms_cst;

                // Adapte conforme necessário para cada CST
                if ($itens_nfe->icms_cst == '00') {
                    $stdICMSItem->modBC = $itens_nfe->icms_modcst;
                    $stdICMSItem->vBC = doubleval($itens_nfe->icms_basecalc) ?? null;
                    $stdICMSItem->pICMS = doubleval($itens_nfe->icms_aliquota) ?? null;
                    $stdICMSItem->vICMS = doubleval($itens_nfe->icms_valor) ?? null;
                }
                // Adicione as demais condições conforme necessário

                // Adicione a tag ICMS ao XML
                $nfe->tagICMS($stdICMSItem);

                // ICMS Interestadual
                $stdicms_interestadual = new stdClass();
                $stdicms_interestadual->item = $item;
                $stdicms_interestadual->vBCUFDest = !is_null($itens_nfe->icms_vbcfcpufdest) ? doubleval($itens_nfe->icms_vbcfcpufdest) : 0;
                $stdicms_interestadual->pFCPUFDest = doubleval($itens_nfe->icms_pfcpufdest);
                $stdicms_interestadual->pICMSUFDest = doubleval($itens_nfe->icms_picmsufdest);
                $stdicms_interestadual->pICMSInter = doubleval($itens_nfe->icms_picmsinter);
                $stdicms_interestadual->pICMSInterPart = doubleval($itens_nfe->icms_picmsinterpart);
                $stdicms_interestadual->vFCPUFDest = doubleval($itens_nfe->icms_vfcpufdest);
                $stdicms_interestadual->vICMSUFDest = doubleval($itens_nfe->icms_vicmsufdest);
                $stdicms_interestadual->vICMSUFRemet = $item->icms_vicmsufremet ?? 0;
                $nfe->tagICMSUFDest($stdicms_interestadual);

                // ... (restante do código)
         
            endforeach;

            $stdtagICMSTot = new stdClass();
            // Soma total dos produtos
            $stdtagICMSTot->vProd = $soma;
            // Valores adicionais
            $stdtagICMSTot->vFrete = doubleval($item->vfrete);
            $stdtagICMSTot->vSeg = doubleval($item->vseg);
            $stdtagICMSTot->vDesc = doubleval($item->vdesc);
            $stdtagICMSTot->vOutro = doubleval($item->voutro);

            // Calcular o valor total da NF-e
            $totalNF = $soma +
            doubleval($item->vfrete) +
            doubleval($item->vseg) -
            doubleval($item->vdesc) +
            doubleval($item->voutro);
            $stdtagICMSTot->vNF = $totalNF;
            $nfe->tagICMSTot($stdtagICMSTot);

            $stdtaginfRespTec = new stdClass();
            $stdtaginfRespTec->CNPJ = '11788740000150'; //CNPJ da pessoa jurídica responsável pelo sistema utilizado na emissão do documento fiscal eletrônico
            $stdtaginfRespTec->xContato = 'Rubens dos Santos'; //Nome da pessoa a ser contatada
            $stdtaginfRespTec->email = 'salvadorbba@gmail.com'; //E-mail da pessoa jurídica a ser contatada
            $stdtaginfRespTec->fone = MasterClass::TrataDoc('(71)99675-8056'); //Telefone da pessoa jurídica/física a ser contatada
            $nfe->taginfRespTec($stdtaginfRespTec);


                $stdautXML = new stdClass();

            //só pode haver um ou CNPJ ou CPF, se um deles é especificado o outro deverá ser null
            $stdautXML->CNPJ = '10572014000133';
            $stdautXML->CPF = null;
 
            $tagautXML = $nfe->tagautXML($stdautXML);

            $stdinfAdic = new stdClass();
            $stdinfAdic->infAdFisco = 'informacoes para o fisco';
            $stdinfAdic->infCpl = 'Valor aproximado tributos R$: 9,43 (4,20%) Fonte: IBPT';
            $nfe->taginfAdic($std);
            
            
            
            
            $std = new  stdClass();
            $std->nFat = '002';
            $std->vOrig = 100;
            $std->vLiq = 100;
            $nfe->tagfat($std);
            
            $std = new  stdClass();
            $std->nDup = '001';
            $std->dVenc = date('Y-m-d');
            $std->vDup = 11.03;
            $nfe->tagdup($std);
            
            $std = new \stdClass();
            $std->vTroco = 0;
            $nfe->tagpag($std);
            
              ErroNFeGeracao::view($nfe->dom->errors,$this->key);
            return  $nfe;
            exit;
             $xml = $nfe->monta();
             $XML = $nfe->getXML();
             $erros = $nfe->getErrors();
             $Chave = $nfe->getChave();

if(isset($Chave)){
    $padrao='xml_inicial';
    $xml_inicial=  ClassMaster::getXML($this->key,  $XML, $Chave, $padrao);
      if( $xml_inicial==1){
       
           TToast::show("show", "XML Gerado ".  $padrao, "topLeft", "");
            $padrao='xml_assinado';
            $xmlAssinado = $this->tools->signNFe($XML);
            
                $st = new Standardize();
                $std = $st->toStd($xmlAssinado);
        
            return  $std;
            
            $xml_assinado=  ClassMaster::getXML($this->key,  $xmlAssinado, $Chave, $padrao);
           
           
               if( $xml_assinado==1){
                   
                       TToast::show("show", "XML Assinado ".  $padrao, "topLeft", "");
                   
                   
               }
           
   }
   
   
   
   
 
}





        } catch (\Exception $e) {
            
            
         
        new TMessage('info', $e->getMessage());
     


 
  
        }

    }

}
<?php
 
class CalculadoraAliquotaICMS
{
    public static function calcularAliquota($estadoOrigem, $estadoDestino)
    {
        $regiaoOrigem = self::obterRegiao($estadoOrigem);
        $regiaoDestino = self::obterRegiao($estadoDestino);

        $aliquotaInterestadual = self::obterAliquotaInterestadual($regiaoOrigem);
        $aliquotaInterna = self::obterAliquotaInterna($regiaoDestino);

        return [
            'interestadual' => $aliquotaInterestadual,
            'interna' => $aliquotaInterna,
        ];
    }

    private static function obterRegiao($estado)
    {
        $regioes = [
            'Norte' => ['AC', 'AP', 'AM', 'PA', 'RO', 'RR', 'TO'],
            'Nordeste' => ['AL', 'BA', 'CE', 'MA', 'PB', 'PE', 'PI', 'RN', 'SE'],
            'Centro-Oeste' => ['DF', 'GO', 'MT', 'MS'],
            'Sudeste' => ['ES', 'MG', 'RJ', 'SP'],
            'Sul' => ['PR', 'RS', 'SC'],
        ];

        foreach ($regioes as $regiao => $estados) {
            if (in_array($estado, $estados)) {
                return $regiao;
            }
        }

        return 'Outro';
    }

    private static function obterAliquotaInterestadual($regiaoOrigem)
    {
        switch ($regiaoOrigem) {
            case 'Norte': return 12;
            case 'Nordeste': return 7;
            case 'Centro-Oeste': return 7;
            case 'Sudeste': return 12;
            case 'Sul': return 7;
            default: return 4; // Alíquota padrão para regiões não listadas
        }
    }

    private static function obterAliquotaInterna($regiaoDestino)
    {
        switch ($regiaoDestino) {
            case 'Norte': return 17;
            case 'Nordeste': return 12;
            case 'Centro-Oeste': return 12;
            case 'Sudeste': return 18;
            case 'Sul': return 12;
            default: return 12; // Alíquota padrão para regiões não listadas
        }
    }
}


 