<?php

class SystemUsers extends TRecord
{
    const TABLENAME  = 'system_users';
    const PRIMARYKEY = 'id';
    const IDPOLICY   =  'max'; // {max, serial}

    private $system_unit;
    private $frontpage;

    private $unit;
    private $system_user_groups = array();
    private $system_user_programs = array();
    private $system_user_units = array();
            

    /**
     * Constructor method
     */
    public function __construct($id = NULL, $callObjectLoad = TRUE)
    {
        parent::__construct($id, $callObjectLoad);
        parent::addAttribute('name');
        parent::addAttribute('login');
        parent::addAttribute('password');
        parent::addAttribute('email');
        parent::addAttribute('frontpage_id');
        parent::addAttribute('system_unit_id');
        parent::addAttribute('active');
        parent::addAttribute('accepted_term_policy');
    
    }

    /**
     * Method set_system_unit
     * Sample of usage: $var->system_unit = $object;
     * @param $object Instance of SystemUnit
     */
    public function set_system_unit(SystemUnit $object)
    {
        $this->system_unit = $object;
        $this->system_unit_id = $object->id;
    }

    /**
     * Method get_system_unit
     * Sample of usage: $var->system_unit->attribute;
     * @returns SystemUnit instance
     */
    public function get_system_unit()
    {
    
        // loads the associated object
        if (empty($this->system_unit))
            $this->system_unit = new SystemUnit($this->system_unit_id);
    
        // returns the associated object
        return $this->system_unit;
    }
    /**
     * Method set_system_program
     * Sample of usage: $var->system_program = $object;
     * @param $object Instance of SystemProgram
     */
    public function set_frontpage(SystemProgram $object)
    {
        $this->frontpage = $object;
        $this->frontpage_id = $object->id;
    }

    /**
     * Method get_frontpage
     * Sample of usage: $var->frontpage->attribute;
     * @returns SystemProgram instance
     */
    public function get_frontpage()
    {
    
        // loads the associated object
        if (empty($this->frontpage))
            $this->frontpage = new SystemProgram($this->frontpage_id);
    
        // returns the associated object
        return $this->frontpage;
    }

    /**
     * Method getCaixas
     */
    public function getCaixasBySystemUsersAberturas()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_abertura_id', '=', $this->id));
        return Caixa::getObjects( $criteria );
    }
    /**
     * Method getCaixas
     */
    public function getCaixasBySystemUsersOperadors()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_operador_id', '=', $this->id));
        return Caixa::getObjects( $criteria );
    }
    /**
     * Method getCaixas
     */
    public function getCaixasBySystemUsersFechamentos()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_fechamento_id', '=', $this->id));
        return Caixa::getObjects( $criteria );
    }
    /**
     * Method getCaixaLancamentoss
     */
    public function getCaixaLancamentoss()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_id', '=', $this->id));
        return CaixaLancamentos::getObjects( $criteria );
    }
    /**
     * Method getEventosSystemusers
     */
    public function getEventosSystemusers()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_id', '=', $this->id));
        return EventosSystemuser::getObjects( $criteria );
    }
    /**
     * Method getInutilizarNumeros
     */
    public function getInutilizarNumeros()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_id', '=', $this->id));
        return InutilizarNumero::getObjects( $criteria );
    }
    /**
     * Method getMdfeLogss
     */
    public function getMdfeLogss()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_id', '=', $this->id));
        return MdfeLogs::getObjects( $criteria );
    }
    /**
     * Method getNotafiscalLogss
     */
    public function getNotafiscalLogss()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_id', '=', $this->id));
        return NotafiscalLogs::getObjects( $criteria );
    }
    /**
     * Method getOrdemServicos
     */
    public function getOrdemServicos()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('usuario_aprovacao_id', '=', $this->id));
        return OrdemServico::getObjects( $criteria );
    }
    /**
     * Method getPainelIndicadorUsuarios
     */
    public function getPainelIndicadorUsuarios()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_id', '=', $this->id));
        return PainelIndicadorUsuario::getObjects( $criteria );
    }
    /**
     * Method getPedidoEventos
     */
    public function getPedidoEventos()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_id', '=', $this->id));
        return PedidoEvento::getObjects( $criteria );
    }
    /**
     * Method getSystemUserProgramPermissoess
     */
    public function getSystemUserProgramPermissoess()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_users_id', '=', $this->id));
        return SystemUserProgramPermissoes::getObjects( $criteria );
    }
    /**
     * Method getXmlContadors
     */
    public function getXmlContadors()
    {
        $criteria = new TCriteria;
        $criteria->add(new TFilter('system_user_id', '=', $this->id));
        return XmlContador::getObjects( $criteria );
    }

    public function set_caixa_caixa_situacao_to_string($caixa_caixa_situacao_to_string)
    {
        if(is_array($caixa_caixa_situacao_to_string))
        {
            $values = CaixaSituacao::where('id', 'in', $caixa_caixa_situacao_to_string)->getIndexedArray('descricao', 'descricao');
            $this->caixa_caixa_situacao_to_string = implode(', ', $values);
        }
        else
        {
            $this->caixa_caixa_situacao_to_string = $caixa_caixa_situacao_to_string;
        }

        $this->vdata['caixa_caixa_situacao_to_string'] = $this->caixa_caixa_situacao_to_string;
    }

    public function get_caixa_caixa_situacao_to_string()
    {
        if(!empty($this->caixa_caixa_situacao_to_string))
        {
            return $this->caixa_caixa_situacao_to_string;
        }
    
        $values = Caixa::where('system_users_fechamento_id', '=', $this->id)->getIndexedArray('caixa_situacao_id','{caixa_situacao->descricao}');
        return implode(', ', $values);
    }

    public function set_caixa_system_unit_to_string($caixa_system_unit_to_string)
    {
        if(is_array($caixa_system_unit_to_string))
        {
            $values = SystemUnit::where('id', 'in', $caixa_system_unit_to_string)->getIndexedArray('name', 'name');
            $this->caixa_system_unit_to_string = implode(', ', $values);
        }
        else
        {
            $this->caixa_system_unit_to_string = $caixa_system_unit_to_string;
        }

        $this->vdata['caixa_system_unit_to_string'] = $this->caixa_system_unit_to_string;
    }

    public function get_caixa_system_unit_to_string()
    {
        if(!empty($this->caixa_system_unit_to_string))
        {
            return $this->caixa_system_unit_to_string;
        }
    
        $values = Caixa::where('system_users_fechamento_id', '=', $this->id)->getIndexedArray('system_unit_id','{system_unit->name}');
        return implode(', ', $values);
    }

    public function set_caixa_system_users_abertura_to_string($caixa_system_users_abertura_to_string)
    {
        if(is_array($caixa_system_users_abertura_to_string))
        {
            $values = SystemUsers::where('id', 'in', $caixa_system_users_abertura_to_string)->getIndexedArray('name', 'name');
            $this->caixa_system_users_abertura_to_string = implode(', ', $values);
        }
        else
        {
            $this->caixa_system_users_abertura_to_string = $caixa_system_users_abertura_to_string;
        }

        $this->vdata['caixa_system_users_abertura_to_string'] = $this->caixa_system_users_abertura_to_string;
    }

    public function get_caixa_system_users_abertura_to_string()
    {
        if(!empty($this->caixa_system_users_abertura_to_string))
        {
            return $this->caixa_system_users_abertura_to_string;
        }
    
        $values = Caixa::where('system_users_fechamento_id', '=', $this->id)->getIndexedArray('system_users_abertura_id','{system_users_abertura->name}');
        return implode(', ', $values);
    }

    public function set_caixa_system_users_operador_to_string($caixa_system_users_operador_to_string)
    {
        if(is_array($caixa_system_users_operador_to_string))
        {
            $values = SystemUsers::where('id', 'in', $caixa_system_users_operador_to_string)->getIndexedArray('name', 'name');
            $this->caixa_system_users_operador_to_string = implode(', ', $values);
        }
        else
        {
            $this->caixa_system_users_operador_to_string = $caixa_system_users_operador_to_string;
        }

        $this->vdata['caixa_system_users_operador_to_string'] = $this->caixa_system_users_operador_to_string;
    }

    public function get_caixa_system_users_operador_to_string()
    {
        if(!empty($this->caixa_system_users_operador_to_string))
        {
            return $this->caixa_system_users_operador_to_string;
        }
    
        $values = Caixa::where('system_users_fechamento_id', '=', $this->id)->getIndexedArray('system_users_operador_id','{system_users_operador->name}');
        return implode(', ', $values);
    }

    public function set_caixa_system_users_fechamento_to_string($caixa_system_users_fechamento_to_string)
    {
        if(is_array($caixa_system_users_fechamento_to_string))
        {
            $values = SystemUsers::where('id', 'in', $caixa_system_users_fechamento_to_string)->getIndexedArray('name', 'name');
            $this->caixa_system_users_fechamento_to_string = implode(', ', $values);
        }
        else
        {
            $this->caixa_system_users_fechamento_to_string = $caixa_system_users_fechamento_to_string;
        }

        $this->vdata['caixa_system_users_fechamento_to_string'] = $this->caixa_system_users_fechamento_to_string;
    }

    public function get_caixa_system_users_fechamento_to_string()
    {
        if(!empty($this->caixa_system_users_fechamento_to_string))
        {
            return $this->caixa_system_users_fechamento_to_string;
        }
    
        $values = Caixa::where('system_users_fechamento_id', '=', $this->id)->getIndexedArray('system_users_fechamento_id','{system_users_fechamento->name}');
        return implode(', ', $values);
    }

    public function set_caixa_lancamentos_caixa_to_string($caixa_lancamentos_caixa_to_string)
    {
        if(is_array($caixa_lancamentos_caixa_to_string))
        {
            $values = Caixa::where('id', 'in', $caixa_lancamentos_caixa_to_string)->getIndexedArray('id', 'id');
            $this->caixa_lancamentos_caixa_to_string = implode(', ', $values);
        }
        else
        {
            $this->caixa_lancamentos_caixa_to_string = $caixa_lancamentos_caixa_to_string;
        }

        $this->vdata['caixa_lancamentos_caixa_to_string'] = $this->caixa_lancamentos_caixa_to_string;
    }

    public function get_caixa_lancamentos_caixa_to_string()
    {
        if(!empty($this->caixa_lancamentos_caixa_to_string))
        {
            return $this->caixa_lancamentos_caixa_to_string;
        }
    
        $values = CaixaLancamentos::where('system_users_id', '=', $this->id)->getIndexedArray('caixa_id','{caixa->id}');
        return implode(', ', $values);
    }

    public function set_caixa_lancamentos_forma_pagamento_to_string($caixa_lancamentos_forma_pagamento_to_string)
    {
        if(is_array($caixa_lancamentos_forma_pagamento_to_string))
        {
            $values = FormaPagamento::where('id', 'in', $caixa_lancamentos_forma_pagamento_to_string)->getIndexedArray('descricao', 'descricao');
            $this->caixa_lancamentos_forma_pagamento_to_string = implode(', ', $values);
        }
        else
        {
            $this->caixa_lancamentos_forma_pagamento_to_string = $caixa_lancamentos_forma_pagamento_to_string;
        }

        $this->vdata['caixa_lancamentos_forma_pagamento_to_string'] = $this->caixa_lancamentos_forma_pagamento_to_string;
    }

    public function get_caixa_lancamentos_forma_pagamento_to_string()
    {
        if(!empty($this->caixa_lancamentos_forma_pagamento_to_string))
        {
            return $this->caixa_lancamentos_forma_pagamento_to_string;
        }
    
        $values = CaixaLancamentos::where('system_users_id', '=', $this->id)->getIndexedArray('forma_pagamento_id','{forma_pagamento->descricao}');
        return implode(', ', $values);
    }

    public function set_caixa_lancamentos_caixa_tipo_lancamento_to_string($caixa_lancamentos_caixa_tipo_lancamento_to_string)
    {
        if(is_array($caixa_lancamentos_caixa_tipo_lancamento_to_string))
        {
            $values = CaixaTipoLancamento::where('id', 'in', $caixa_lancamentos_caixa_tipo_lancamento_to_string)->getIndexedArray('descricao', 'descricao');
            $this->caixa_lancamentos_caixa_tipo_lancamento_to_string = implode(', ', $values);
        }
        else
        {
            $this->caixa_lancamentos_caixa_tipo_lancamento_to_string = $caixa_lancamentos_caixa_tipo_lancamento_to_string;
        }

        $this->vdata['caixa_lancamentos_caixa_tipo_lancamento_to_string'] = $this->caixa_lancamentos_caixa_tipo_lancamento_to_string;
    }

    public function get_caixa_lancamentos_caixa_tipo_lancamento_to_string()
    {
        if(!empty($this->caixa_lancamentos_caixa_tipo_lancamento_to_string))
        {
            return $this->caixa_lancamentos_caixa_tipo_lancamento_to_string;
        }
    
        $values = CaixaLancamentos::where('system_users_id', '=', $this->id)->getIndexedArray('caixa_tipo_lancamento_id','{caixa_tipo_lancamento->descricao}');
        return implode(', ', $values);
    }

    public function set_caixa_lancamentos_system_users_to_string($caixa_lancamentos_system_users_to_string)
    {
        if(is_array($caixa_lancamentos_system_users_to_string))
        {
            $values = SystemUsers::where('id', 'in', $caixa_lancamentos_system_users_to_string)->getIndexedArray('name', 'name');
            $this->caixa_lancamentos_system_users_to_string = implode(', ', $values);
        }
        else
        {
            $this->caixa_lancamentos_system_users_to_string = $caixa_lancamentos_system_users_to_string;
        }

        $this->vdata['caixa_lancamentos_system_users_to_string'] = $this->caixa_lancamentos_system_users_to_string;
    }

    public function get_caixa_lancamentos_system_users_to_string()
    {
        if(!empty($this->caixa_lancamentos_system_users_to_string))
        {
            return $this->caixa_lancamentos_system_users_to_string;
        }
    
        $values = CaixaLancamentos::where('system_users_id', '=', $this->id)->getIndexedArray('system_users_id','{system_users->name}');
        return implode(', ', $values);
    }

    public function set_eventos_systemuser_eventos_to_string($eventos_systemuser_eventos_to_string)
    {
        if(is_array($eventos_systemuser_eventos_to_string))
        {
            $values = Eventos::where('id', 'in', $eventos_systemuser_eventos_to_string)->getIndexedArray('descricao', 'descricao');
            $this->eventos_systemuser_eventos_to_string = implode(', ', $values);
        }
        else
        {
            $this->eventos_systemuser_eventos_to_string = $eventos_systemuser_eventos_to_string;
        }

        $this->vdata['eventos_systemuser_eventos_to_string'] = $this->eventos_systemuser_eventos_to_string;
    }

    public function get_eventos_systemuser_eventos_to_string()
    {
        if(!empty($this->eventos_systemuser_eventos_to_string))
        {
            return $this->eventos_systemuser_eventos_to_string;
        }
    
        $values = EventosSystemuser::where('system_users_id', '=', $this->id)->getIndexedArray('eventos_id','{eventos->descricao}');
        return implode(', ', $values);
    }

    public function set_eventos_systemuser_system_users_to_string($eventos_systemuser_system_users_to_string)
    {
        if(is_array($eventos_systemuser_system_users_to_string))
        {
            $values = SystemUsers::where('id', 'in', $eventos_systemuser_system_users_to_string)->getIndexedArray('name', 'name');
            $this->eventos_systemuser_system_users_to_string = implode(', ', $values);
        }
        else
        {
            $this->eventos_systemuser_system_users_to_string = $eventos_systemuser_system_users_to_string;
        }

        $this->vdata['eventos_systemuser_system_users_to_string'] = $this->eventos_systemuser_system_users_to_string;
    }

    public function get_eventos_systemuser_system_users_to_string()
    {
        if(!empty($this->eventos_systemuser_system_users_to_string))
        {
            return $this->eventos_systemuser_system_users_to_string;
        }
    
        $values = EventosSystemuser::where('system_users_id', '=', $this->id)->getIndexedArray('system_users_id','{system_users->name}');
        return implode(', ', $values);
    }

    public function set_eventos_systemuser_system_unit_to_string($eventos_systemuser_system_unit_to_string)
    {
        if(is_array($eventos_systemuser_system_unit_to_string))
        {
            $values = SystemUnit::where('id', 'in', $eventos_systemuser_system_unit_to_string)->getIndexedArray('name', 'name');
            $this->eventos_systemuser_system_unit_to_string = implode(', ', $values);
        }
        else
        {
            $this->eventos_systemuser_system_unit_to_string = $eventos_systemuser_system_unit_to_string;
        }

        $this->vdata['eventos_systemuser_system_unit_to_string'] = $this->eventos_systemuser_system_unit_to_string;
    }

    public function get_eventos_systemuser_system_unit_to_string()
    {
        if(!empty($this->eventos_systemuser_system_unit_to_string))
        {
            return $this->eventos_systemuser_system_unit_to_string;
        }
    
        $values = EventosSystemuser::where('system_users_id', '=', $this->id)->getIndexedArray('system_unit_id','{system_unit->name}');
        return implode(', ', $values);
    }

    public function set_inutilizar_numero_system_unit_to_string($inutilizar_numero_system_unit_to_string)
    {
        if(is_array($inutilizar_numero_system_unit_to_string))
        {
            $values = SystemUnit::where('id', 'in', $inutilizar_numero_system_unit_to_string)->getIndexedArray('name', 'name');
            $this->inutilizar_numero_system_unit_to_string = implode(', ', $values);
        }
        else
        {
            $this->inutilizar_numero_system_unit_to_string = $inutilizar_numero_system_unit_to_string;
        }

        $this->vdata['inutilizar_numero_system_unit_to_string'] = $this->inutilizar_numero_system_unit_to_string;
    }

    public function get_inutilizar_numero_system_unit_to_string()
    {
        if(!empty($this->inutilizar_numero_system_unit_to_string))
        {
            return $this->inutilizar_numero_system_unit_to_string;
        }
    
        $values = InutilizarNumero::where('system_users_id', '=', $this->id)->getIndexedArray('system_unit_id','{system_unit->name}');
        return implode(', ', $values);
    }

    public function set_inutilizar_numero_system_users_to_string($inutilizar_numero_system_users_to_string)
    {
        if(is_array($inutilizar_numero_system_users_to_string))
        {
            $values = SystemUsers::where('id', 'in', $inutilizar_numero_system_users_to_string)->getIndexedArray('name', 'name');
            $this->inutilizar_numero_system_users_to_string = implode(', ', $values);
        }
        else
        {
            $this->inutilizar_numero_system_users_to_string = $inutilizar_numero_system_users_to_string;
        }

        $this->vdata['inutilizar_numero_system_users_to_string'] = $this->inutilizar_numero_system_users_to_string;
    }

    public function get_inutilizar_numero_system_users_to_string()
    {
        if(!empty($this->inutilizar_numero_system_users_to_string))
        {
            return $this->inutilizar_numero_system_users_to_string;
        }
    
        $values = InutilizarNumero::where('system_users_id', '=', $this->id)->getIndexedArray('system_users_id','{system_users->name}');
        return implode(', ', $values);
    }

    public function set_mdfe_logs_mdfe_to_string($mdfe_logs_mdfe_to_string)
    {
        if(is_array($mdfe_logs_mdfe_to_string))
        {
            $values = Mdfe::where('id', 'in', $mdfe_logs_mdfe_to_string)->getIndexedArray('id', 'id');
            $this->mdfe_logs_mdfe_to_string = implode(', ', $values);
        }
        else
        {
            $this->mdfe_logs_mdfe_to_string = $mdfe_logs_mdfe_to_string;
        }

        $this->vdata['mdfe_logs_mdfe_to_string'] = $this->mdfe_logs_mdfe_to_string;
    }

    public function get_mdfe_logs_mdfe_to_string()
    {
        if(!empty($this->mdfe_logs_mdfe_to_string))
        {
            return $this->mdfe_logs_mdfe_to_string;
        }
    
        $values = MdfeLogs::where('system_users_id', '=', $this->id)->getIndexedArray('mdfe_id','{mdfe->id}');
        return implode(', ', $values);
    }

    public function set_mdfe_logs_system_users_to_string($mdfe_logs_system_users_to_string)
    {
        if(is_array($mdfe_logs_system_users_to_string))
        {
            $values = SystemUsers::where('id', 'in', $mdfe_logs_system_users_to_string)->getIndexedArray('name', 'name');
            $this->mdfe_logs_system_users_to_string = implode(', ', $values);
        }
        else
        {
            $this->mdfe_logs_system_users_to_string = $mdfe_logs_system_users_to_string;
        }

        $this->vdata['mdfe_logs_system_users_to_string'] = $this->mdfe_logs_system_users_to_string;
    }

    public function get_mdfe_logs_system_users_to_string()
    {
        if(!empty($this->mdfe_logs_system_users_to_string))
        {
            return $this->mdfe_logs_system_users_to_string;
        }
    
        $values = MdfeLogs::where('system_users_id', '=', $this->id)->getIndexedArray('system_users_id','{system_users->name}');
        return implode(', ', $values);
    }

    public function set_notafiscal_logs_notafiscal_to_string($notafiscal_logs_notafiscal_to_string)
    {
        if(is_array($notafiscal_logs_notafiscal_to_string))
        {
            $values = Notafiscal::where('id', 'in', $notafiscal_logs_notafiscal_to_string)->getIndexedArray('id', 'id');
            $this->notafiscal_logs_notafiscal_to_string = implode(', ', $values);
        }
        else
        {
            $this->notafiscal_logs_notafiscal_to_string = $notafiscal_logs_notafiscal_to_string;
        }

        $this->vdata['notafiscal_logs_notafiscal_to_string'] = $this->notafiscal_logs_notafiscal_to_string;
    }

    public function get_notafiscal_logs_notafiscal_to_string()
    {
        if(!empty($this->notafiscal_logs_notafiscal_to_string))
        {
            return $this->notafiscal_logs_notafiscal_to_string;
        }
    
        $values = NotafiscalLogs::where('system_users_id', '=', $this->id)->getIndexedArray('notafiscal_id','{notafiscal->id}');
        return implode(', ', $values);
    }

    public function set_notafiscal_logs_system_users_to_string($notafiscal_logs_system_users_to_string)
    {
        if(is_array($notafiscal_logs_system_users_to_string))
        {
            $values = SystemUsers::where('id', 'in', $notafiscal_logs_system_users_to_string)->getIndexedArray('name', 'name');
            $this->notafiscal_logs_system_users_to_string = implode(', ', $values);
        }
        else
        {
            $this->notafiscal_logs_system_users_to_string = $notafiscal_logs_system_users_to_string;
        }

        $this->vdata['notafiscal_logs_system_users_to_string'] = $this->notafiscal_logs_system_users_to_string;
    }

    public function get_notafiscal_logs_system_users_to_string()
    {
        if(!empty($this->notafiscal_logs_system_users_to_string))
        {
            return $this->notafiscal_logs_system_users_to_string;
        }
    
        $values = NotafiscalLogs::where('system_users_id', '=', $this->id)->getIndexedArray('system_users_id','{system_users->name}');
        return implode(', ', $values);
    }

    public function set_ordem_servico_system_unit_to_string($ordem_servico_system_unit_to_string)
    {
        if(is_array($ordem_servico_system_unit_to_string))
        {
            $values = SystemUnit::where('id', 'in', $ordem_servico_system_unit_to_string)->getIndexedArray('name', 'name');
            $this->ordem_servico_system_unit_to_string = implode(', ', $values);
        }
        else
        {
            $this->ordem_servico_system_unit_to_string = $ordem_servico_system_unit_to_string;
        }

        $this->vdata['ordem_servico_system_unit_to_string'] = $this->ordem_servico_system_unit_to_string;
    }

    public function get_ordem_servico_system_unit_to_string()
    {
        if(!empty($this->ordem_servico_system_unit_to_string))
        {
            return $this->ordem_servico_system_unit_to_string;
        }
    
        $values = OrdemServico::where('usuario_aprovacao_id', '=', $this->id)->getIndexedArray('system_unit_id','{system_unit->name}');
        return implode(', ', $values);
    }

    public function set_ordem_servico_cliente_to_string($ordem_servico_cliente_to_string)
    {
        if(is_array($ordem_servico_cliente_to_string))
        {
            $values = Cliente::where('id', 'in', $ordem_servico_cliente_to_string)->getIndexedArray('nome', 'nome');
            $this->ordem_servico_cliente_to_string = implode(', ', $values);
        }
        else
        {
            $this->ordem_servico_cliente_to_string = $ordem_servico_cliente_to_string;
        }

        $this->vdata['ordem_servico_cliente_to_string'] = $this->ordem_servico_cliente_to_string;
    }

    public function get_ordem_servico_cliente_to_string()
    {
        if(!empty($this->ordem_servico_cliente_to_string))
        {
            return $this->ordem_servico_cliente_to_string;
        }
    
        $values = OrdemServico::where('usuario_aprovacao_id', '=', $this->id)->getIndexedArray('cliente_id','{cliente->nome}');
        return implode(', ', $values);
    }

    public function set_ordem_servico_cliente_contato_to_string($ordem_servico_cliente_contato_to_string)
    {
        if(is_array($ordem_servico_cliente_contato_to_string))
        {
            $values = ClienteContato::where('id', 'in', $ordem_servico_cliente_contato_to_string)->getIndexedArray('nome', 'nome');
            $this->ordem_servico_cliente_contato_to_string = implode(', ', $values);
        }
        else
        {
            $this->ordem_servico_cliente_contato_to_string = $ordem_servico_cliente_contato_to_string;
        }

        $this->vdata['ordem_servico_cliente_contato_to_string'] = $this->ordem_servico_cliente_contato_to_string;
    }

    public function get_ordem_servico_cliente_contato_to_string()
    {
        if(!empty($this->ordem_servico_cliente_contato_to_string))
        {
            return $this->ordem_servico_cliente_contato_to_string;
        }
    
        $values = OrdemServico::where('usuario_aprovacao_id', '=', $this->id)->getIndexedArray('cliente_contato_id','{cliente_contato->nome}');
        return implode(', ', $values);
    }

    public function set_ordem_servico_vendedor_to_string($ordem_servico_vendedor_to_string)
    {
        if(is_array($ordem_servico_vendedor_to_string))
        {
            $values = Vendedor::where('id', 'in', $ordem_servico_vendedor_to_string)->getIndexedArray('nome', 'nome');
            $this->ordem_servico_vendedor_to_string = implode(', ', $values);
        }
        else
        {
            $this->ordem_servico_vendedor_to_string = $ordem_servico_vendedor_to_string;
        }

        $this->vdata['ordem_servico_vendedor_to_string'] = $this->ordem_servico_vendedor_to_string;
    }

    public function get_ordem_servico_vendedor_to_string()
    {
        if(!empty($this->ordem_servico_vendedor_to_string))
        {
            return $this->ordem_servico_vendedor_to_string;
        }
    
        $values = OrdemServico::where('usuario_aprovacao_id', '=', $this->id)->getIndexedArray('vendedor_id','{vendedor->nome}');
        return implode(', ', $values);
    }

    public function set_ordem_servico_ordem_servico_situacao_to_string($ordem_servico_ordem_servico_situacao_to_string)
    {
        if(is_array($ordem_servico_ordem_servico_situacao_to_string))
        {
            $values = OrdemServicoSituacao::where('id', 'in', $ordem_servico_ordem_servico_situacao_to_string)->getIndexedArray('descricao', 'descricao');
            $this->ordem_servico_ordem_servico_situacao_to_string = implode(', ', $values);
        }
        else
        {
            $this->ordem_servico_ordem_servico_situacao_to_string = $ordem_servico_ordem_servico_situacao_to_string;
        }

        $this->vdata['ordem_servico_ordem_servico_situacao_to_string'] = $this->ordem_servico_ordem_servico_situacao_to_string;
    }

    public function get_ordem_servico_ordem_servico_situacao_to_string()
    {
        if(!empty($this->ordem_servico_ordem_servico_situacao_to_string))
        {
            return $this->ordem_servico_ordem_servico_situacao_to_string;
        }
    
        $values = OrdemServico::where('usuario_aprovacao_id', '=', $this->id)->getIndexedArray('ordem_servico_situacao_id','{ordem_servico_situacao->descricao}');
        return implode(', ', $values);
    }

    public function set_ordem_servico_ordem_servico_producao_etapa_to_string($ordem_servico_ordem_servico_producao_etapa_to_string)
    {
        if(is_array($ordem_servico_ordem_servico_producao_etapa_to_string))
        {
            $values = OrdemServicoProducaoEtapa::where('id', 'in', $ordem_servico_ordem_servico_producao_etapa_to_string)->getIndexedArray('descricao', 'descricao');
            $this->ordem_servico_ordem_servico_producao_etapa_to_string = implode(', ', $values);
        }
        else
        {
            $this->ordem_servico_ordem_servico_producao_etapa_to_string = $ordem_servico_ordem_servico_producao_etapa_to_string;
        }

        $this->vdata['ordem_servico_ordem_servico_producao_etapa_to_string'] = $this->ordem_servico_ordem_servico_producao_etapa_to_string;
    }

    public function get_ordem_servico_ordem_servico_producao_etapa_to_string()
    {
        if(!empty($this->ordem_servico_ordem_servico_producao_etapa_to_string))
        {
            return $this->ordem_servico_ordem_servico_producao_etapa_to_string;
        }
    
        $values = OrdemServico::where('usuario_aprovacao_id', '=', $this->id)->getIndexedArray('ordem_servico_producao_etapa_id','{ordem_servico_producao_etapa->descricao}');
        return implode(', ', $values);
    }

    public function set_ordem_servico_condicao_pagamento_to_string($ordem_servico_condicao_pagamento_to_string)
    {
        if(is_array($ordem_servico_condicao_pagamento_to_string))
        {
            $values = CondicaoPagamento::where('id', 'in', $ordem_servico_condicao_pagamento_to_string)->getIndexedArray('descricao', 'descricao');
            $this->ordem_servico_condicao_pagamento_to_string = implode(', ', $values);
        }
        else
        {
            $this->ordem_servico_condicao_pagamento_to_string = $ordem_servico_condicao_pagamento_to_string;
        }

        $this->vdata['ordem_servico_condicao_pagamento_to_string'] = $this->ordem_servico_condicao_pagamento_to_string;
    }

    public function get_ordem_servico_condicao_pagamento_to_string()
    {
        if(!empty($this->ordem_servico_condicao_pagamento_to_string))
        {
            return $this->ordem_servico_condicao_pagamento_to_string;
        }
    
        $values = OrdemServico::where('usuario_aprovacao_id', '=', $this->id)->getIndexedArray('condicao_pagamento_id','{condicao_pagamento->descricao}');
        return implode(', ', $values);
    }

    public function set_ordem_servico_cliente_endereco_to_string($ordem_servico_cliente_endereco_to_string)
    {
        if(is_array($ordem_servico_cliente_endereco_to_string))
        {
            $values = ClienteEndereco::where('id', 'in', $ordem_servico_cliente_endereco_to_string)->getIndexedArray('logradouro', 'logradouro');
            $this->ordem_servico_cliente_endereco_to_string = implode(', ', $values);
        }
        else
        {
            $this->ordem_servico_cliente_endereco_to_string = $ordem_servico_cliente_endereco_to_string;
        }

        $this->vdata['ordem_servico_cliente_endereco_to_string'] = $this->ordem_servico_cliente_endereco_to_string;
    }

    public function get_ordem_servico_cliente_endereco_to_string()
    {
        if(!empty($this->ordem_servico_cliente_endereco_to_string))
        {
            return $this->ordem_servico_cliente_endereco_to_string;
        }
    
        $values = OrdemServico::where('usuario_aprovacao_id', '=', $this->id)->getIndexedArray('cliente_endereco_id','{cliente_endereco->logradouro}');
        return implode(', ', $values);
    }

    public function set_ordem_servico_usuario_aprovacao_to_string($ordem_servico_usuario_aprovacao_to_string)
    {
        if(is_array($ordem_servico_usuario_aprovacao_to_string))
        {
            $values = SystemUsers::where('id', 'in', $ordem_servico_usuario_aprovacao_to_string)->getIndexedArray('name', 'name');
            $this->ordem_servico_usuario_aprovacao_to_string = implode(', ', $values);
        }
        else
        {
            $this->ordem_servico_usuario_aprovacao_to_string = $ordem_servico_usuario_aprovacao_to_string;
        }

        $this->vdata['ordem_servico_usuario_aprovacao_to_string'] = $this->ordem_servico_usuario_aprovacao_to_string;
    }

    public function get_ordem_servico_usuario_aprovacao_to_string()
    {
        if(!empty($this->ordem_servico_usuario_aprovacao_to_string))
        {
            return $this->ordem_servico_usuario_aprovacao_to_string;
        }
    
        $values = OrdemServico::where('usuario_aprovacao_id', '=', $this->id)->getIndexedArray('usuario_aprovacao_id','{usuario_aprovacao->name}');
        return implode(', ', $values);
    }

    public function set_painel_indicador_usuario_painel_indicadores_to_string($painel_indicador_usuario_painel_indicadores_to_string)
    {
        if(is_array($painel_indicador_usuario_painel_indicadores_to_string))
        {
            $values = PainelIndicadores::where('id', 'in', $painel_indicador_usuario_painel_indicadores_to_string)->getIndexedArray('descricao', 'descricao');
            $this->painel_indicador_usuario_painel_indicadores_to_string = implode(', ', $values);
        }
        else
        {
            $this->painel_indicador_usuario_painel_indicadores_to_string = $painel_indicador_usuario_painel_indicadores_to_string;
        }

        $this->vdata['painel_indicador_usuario_painel_indicadores_to_string'] = $this->painel_indicador_usuario_painel_indicadores_to_string;
    }

    public function get_painel_indicador_usuario_painel_indicadores_to_string()
    {
        if(!empty($this->painel_indicador_usuario_painel_indicadores_to_string))
        {
            return $this->painel_indicador_usuario_painel_indicadores_to_string;
        }
    
        $values = PainelIndicadorUsuario::where('system_users_id', '=', $this->id)->getIndexedArray('painel_indicadores_id','{painel_indicadores->descricao}');
        return implode(', ', $values);
    }

    public function set_painel_indicador_usuario_indicador_personalizado_to_string($painel_indicador_usuario_indicador_personalizado_to_string)
    {
        if(is_array($painel_indicador_usuario_indicador_personalizado_to_string))
        {
            $values = IndicadorPersonalizado::where('id', 'in', $painel_indicador_usuario_indicador_personalizado_to_string)->getIndexedArray('descricao', 'descricao');
            $this->painel_indicador_usuario_indicador_personalizado_to_string = implode(', ', $values);
        }
        else
        {
            $this->painel_indicador_usuario_indicador_personalizado_to_string = $painel_indicador_usuario_indicador_personalizado_to_string;
        }

        $this->vdata['painel_indicador_usuario_indicador_personalizado_to_string'] = $this->painel_indicador_usuario_indicador_personalizado_to_string;
    }

    public function get_painel_indicador_usuario_indicador_personalizado_to_string()
    {
        if(!empty($this->painel_indicador_usuario_indicador_personalizado_to_string))
        {
            return $this->painel_indicador_usuario_indicador_personalizado_to_string;
        }
    
        $values = PainelIndicadorUsuario::where('system_users_id', '=', $this->id)->getIndexedArray('indicador_personalizado_id','{indicador_personalizado->descricao}');
        return implode(', ', $values);
    }

    public function set_painel_indicador_usuario_system_users_to_string($painel_indicador_usuario_system_users_to_string)
    {
        if(is_array($painel_indicador_usuario_system_users_to_string))
        {
            $values = SystemUsers::where('id', 'in', $painel_indicador_usuario_system_users_to_string)->getIndexedArray('name', 'name');
            $this->painel_indicador_usuario_system_users_to_string = implode(', ', $values);
        }
        else
        {
            $this->painel_indicador_usuario_system_users_to_string = $painel_indicador_usuario_system_users_to_string;
        }

        $this->vdata['painel_indicador_usuario_system_users_to_string'] = $this->painel_indicador_usuario_system_users_to_string;
    }

    public function get_painel_indicador_usuario_system_users_to_string()
    {
        if(!empty($this->painel_indicador_usuario_system_users_to_string))
        {
            return $this->painel_indicador_usuario_system_users_to_string;
        }
    
        $values = PainelIndicadorUsuario::where('system_users_id', '=', $this->id)->getIndexedArray('system_users_id','{system_users->name}');
        return implode(', ', $values);
    }

    public function set_pedido_evento_notafiscal_to_string($pedido_evento_notafiscal_to_string)
    {
        if(is_array($pedido_evento_notafiscal_to_string))
        {
            $values = Notafiscal::where('id', 'in', $pedido_evento_notafiscal_to_string)->getIndexedArray('id', 'id');
            $this->pedido_evento_notafiscal_to_string = implode(', ', $values);
        }
        else
        {
            $this->pedido_evento_notafiscal_to_string = $pedido_evento_notafiscal_to_string;
        }

        $this->vdata['pedido_evento_notafiscal_to_string'] = $this->pedido_evento_notafiscal_to_string;
    }

    public function get_pedido_evento_notafiscal_to_string()
    {
        if(!empty($this->pedido_evento_notafiscal_to_string))
        {
            return $this->pedido_evento_notafiscal_to_string;
        }
    
        $values = PedidoEvento::where('system_users_id', '=', $this->id)->getIndexedArray('notafiscal_id','{notafiscal->id}');
        return implode(', ', $values);
    }

    public function set_pedido_evento_system_unit_to_string($pedido_evento_system_unit_to_string)
    {
        if(is_array($pedido_evento_system_unit_to_string))
        {
            $values = SystemUnit::where('id', 'in', $pedido_evento_system_unit_to_string)->getIndexedArray('name', 'name');
            $this->pedido_evento_system_unit_to_string = implode(', ', $values);
        }
        else
        {
            $this->pedido_evento_system_unit_to_string = $pedido_evento_system_unit_to_string;
        }

        $this->vdata['pedido_evento_system_unit_to_string'] = $this->pedido_evento_system_unit_to_string;
    }

    public function get_pedido_evento_system_unit_to_string()
    {
        if(!empty($this->pedido_evento_system_unit_to_string))
        {
            return $this->pedido_evento_system_unit_to_string;
        }
    
        $values = PedidoEvento::where('system_users_id', '=', $this->id)->getIndexedArray('system_unit_id','{system_unit->name}');
        return implode(', ', $values);
    }

    public function set_pedido_evento_system_users_to_string($pedido_evento_system_users_to_string)
    {
        if(is_array($pedido_evento_system_users_to_string))
        {
            $values = SystemUsers::where('id', 'in', $pedido_evento_system_users_to_string)->getIndexedArray('name', 'name');
            $this->pedido_evento_system_users_to_string = implode(', ', $values);
        }
        else
        {
            $this->pedido_evento_system_users_to_string = $pedido_evento_system_users_to_string;
        }

        $this->vdata['pedido_evento_system_users_to_string'] = $this->pedido_evento_system_users_to_string;
    }

    public function get_pedido_evento_system_users_to_string()
    {
        if(!empty($this->pedido_evento_system_users_to_string))
        {
            return $this->pedido_evento_system_users_to_string;
        }
    
        $values = PedidoEvento::where('system_users_id', '=', $this->id)->getIndexedArray('system_users_id','{system_users->name}');
        return implode(', ', $values);
    }

    public function set_system_user_program_permissoes_system_unit_to_string($system_user_program_permissoes_system_unit_to_string)
    {
        if(is_array($system_user_program_permissoes_system_unit_to_string))
        {
            $values = SystemUnit::where('id', 'in', $system_user_program_permissoes_system_unit_to_string)->getIndexedArray('name', 'name');
            $this->system_user_program_permissoes_system_unit_to_string = implode(', ', $values);
        }
        else
        {
            $this->system_user_program_permissoes_system_unit_to_string = $system_user_program_permissoes_system_unit_to_string;
        }

        $this->vdata['system_user_program_permissoes_system_unit_to_string'] = $this->system_user_program_permissoes_system_unit_to_string;
    }

    public function get_system_user_program_permissoes_system_unit_to_string()
    {
        if(!empty($this->system_user_program_permissoes_system_unit_to_string))
        {
            return $this->system_user_program_permissoes_system_unit_to_string;
        }
    
        $values = SystemUserProgramPermissoes::where('system_users_id', '=', $this->id)->getIndexedArray('system_unit_id','{system_unit->name}');
        return implode(', ', $values);
    }

    public function set_system_user_program_permissoes_system_users_to_string($system_user_program_permissoes_system_users_to_string)
    {
        if(is_array($system_user_program_permissoes_system_users_to_string))
        {
            $values = SystemUsers::where('id', 'in', $system_user_program_permissoes_system_users_to_string)->getIndexedArray('name', 'name');
            $this->system_user_program_permissoes_system_users_to_string = implode(', ', $values);
        }
        else
        {
            $this->system_user_program_permissoes_system_users_to_string = $system_user_program_permissoes_system_users_to_string;
        }

        $this->vdata['system_user_program_permissoes_system_users_to_string'] = $this->system_user_program_permissoes_system_users_to_string;
    }

    public function get_system_user_program_permissoes_system_users_to_string()
    {
        if(!empty($this->system_user_program_permissoes_system_users_to_string))
        {
            return $this->system_user_program_permissoes_system_users_to_string;
        }
    
        $values = SystemUserProgramPermissoes::where('system_users_id', '=', $this->id)->getIndexedArray('system_users_id','{system_users->name}');
        return implode(', ', $values);
    }

    public function set_system_user_program_permissoes_system_program_permissoes_to_string($system_user_program_permissoes_system_program_permissoes_to_string)
    {
        if(is_array($system_user_program_permissoes_system_program_permissoes_to_string))
        {
            $values = SystemProgramPermissoes::where('id', 'in', $system_user_program_permissoes_system_program_permissoes_to_string)->getIndexedArray('id', 'id');
            $this->system_user_program_permissoes_system_program_permissoes_to_string = implode(', ', $values);
        }
        else
        {
            $this->system_user_program_permissoes_system_program_permissoes_to_string = $system_user_program_permissoes_system_program_permissoes_to_string;
        }

        $this->vdata['system_user_program_permissoes_system_program_permissoes_to_string'] = $this->system_user_program_permissoes_system_program_permissoes_to_string;
    }

    public function get_system_user_program_permissoes_system_program_permissoes_to_string()
    {
        if(!empty($this->system_user_program_permissoes_system_program_permissoes_to_string))
        {
            return $this->system_user_program_permissoes_system_program_permissoes_to_string;
        }
    
        $values = SystemUserProgramPermissoes::where('system_users_id', '=', $this->id)->getIndexedArray('system_program_permissoes_id','{system_program_permissoes->id}');
        return implode(', ', $values);
    }

    public function set_xml_contador_system_user_to_string($xml_contador_system_user_to_string)
    {
        if(is_array($xml_contador_system_user_to_string))
        {
            $values = SystemUsers::where('id', 'in', $xml_contador_system_user_to_string)->getIndexedArray('name', 'name');
            $this->xml_contador_system_user_to_string = implode(', ', $values);
        }
        else
        {
            $this->xml_contador_system_user_to_string = $xml_contador_system_user_to_string;
        }

        $this->vdata['xml_contador_system_user_to_string'] = $this->xml_contador_system_user_to_string;
    }

    public function get_xml_contador_system_user_to_string()
    {
        if(!empty($this->xml_contador_system_user_to_string))
        {
            return $this->xml_contador_system_user_to_string;
        }
    
        $values = XmlContador::where('system_user_id', '=', $this->id)->getIndexedArray('system_user_id','{system_user->name}');
        return implode(', ', $values);
    }

    public function set_xml_contador_system_unit_to_string($xml_contador_system_unit_to_string)
    {
        if(is_array($xml_contador_system_unit_to_string))
        {
            $values = SystemUnit::where('id', 'in', $xml_contador_system_unit_to_string)->getIndexedArray('name', 'name');
            $this->xml_contador_system_unit_to_string = implode(', ', $values);
        }
        else
        {
            $this->xml_contador_system_unit_to_string = $xml_contador_system_unit_to_string;
        }

        $this->vdata['xml_contador_system_unit_to_string'] = $this->xml_contador_system_unit_to_string;
    }

    public function get_xml_contador_system_unit_to_string()
    {
        if(!empty($this->xml_contador_system_unit_to_string))
        {
            return $this->xml_contador_system_unit_to_string;
        }
    
        $values = XmlContador::where('system_user_id', '=', $this->id)->getIndexedArray('system_unit_id','{system_unit->name}');
        return implode(', ', $values);
    }

    /**
     * Return the user' group's
     * @return Collection of SystemGroup
     */
    public function getSystemUserGroups()
    {
        return parent::loadAggregate('SystemGroup', 'SystemUserGroup', 'system_user_id', 'system_group_id', $this->id);
    }

    /**
     * Return the user' unit's
     * @return Collection of SystemUnit
     */
    public function getSystemUserUnits()
    {
        return parent::loadAggregate('SystemUnit', 'SystemUserUnit', 'system_user_id', 'system_unit_id', $this->id);
    }

    /**
     * Return the user' program's
     * @return Collection of SystemProgram
     */
    public function getSystemUserPrograms()
    {
        return parent::loadAggregate('SystemProgram', 'SystemUserProgram', 'system_user_id', 'system_program_id', $this->id);
    }

    /**
     * Returns the frontpage name
     */
    public function get_frontpage_name()
    {
        // loads the associated object
        if (empty($this->frontpage))
            $this->frontpage = new SystemProgram($this->frontpage_id);

        // returns the associated object
        return $this->frontpage->name;
    }

    /**
     * Returns the unit
     */
    public function get_unit()
    {
        // loads the associated object
        if (empty($this->unit))
            $this->unit = new SystemUnit($this->system_unit_id);

        // returns the associated object
        return $this->unit;
    }

    /**
     * Add a Group to the user
     * @param $object Instance of SystemGroup
     */
    public function addSystemUserGroup(SystemGroup $systemgroup)
    {
        $object = new SystemUserGroup;
        $object->system_group_id = $systemgroup->id;
        $object->system_user_id = $this->id;
        $object->store();
    }

    /**
     * Add a Unit to the user
     * @param $object Instance of SystemUnit
     */
    public function addSystemUserUnit(SystemUnit $systemunit)
    {
        $object = new SystemUserUnit;
        $object->system_unit_id = $systemunit->id;
        $object->system_user_id = $this->id;
        $object->store();
    }

    /**
     * Add a program to the user
     * @param $object Instance of SystemProgram
     */
    public function addSystemUserProgram(SystemProgram $systemprogram)
    {
        $object = new SystemUserProgram;
        $object->system_program_id = $systemprogram->id;
        $object->system_user_id = $this->id;
        $object->store();
    }

    /**
     * Get user group ids
     */
    public function getSystemUserGroupIds( $as_string = false )
    {
        $groupids = array();
        $groups = $this->getSystemUserGroups();
        if ($groups)
        {
            foreach ($groups as $group)
            {
                $groupids[] = $group->id;
            }
        }
    
        if ($as_string)
        {
            return implode(',', $groupids);
        }
    
        return $groupids;
    }

    /**
     * Get user unit ids
     */
    public function getSystemUserUnitIds( $as_string = false )
    {
        $unitids = array();
        $units = $this->getSystemUserUnits();
        if ($units)
        {
            foreach ($units as $unit)
            {
                $unitids[] = $unit->id;
            }
        }
    
        if ($as_string)
        {
            return implode(',', $unitids);
        }
    
        return $unitids;
    }

    /**
     * Get user group names
     */
    public function getSystemUserGroupNames()
    {
        $groupnames = array();
        $groups = $this->getSystemUserGroups();
        if ($groups)
        {
            foreach ($groups as $group)
            {
                $groupnames[] = $group->name;
            }
        }
    
        return implode(',', $groupnames);
    }

    /**
     * Reset aggregates
     */
    public function clearParts()
    {
        SystemUserGroup::where('system_user_id', '=', $this->id)->delete();
        SystemUserUnit::where('system_user_id', '=', $this->id)->delete();
        SystemUserProgram::where('system_user_id', '=', $this->id)->delete();
    }

    /**
     * Delete the object and its aggregates
     * @param $id object ID
     */
    public function delete($id = NULL)
    {
        // delete the related System_userSystem_user_group objects
        $id = isset($id) ? $id : $this->id;
    
        SystemUserGroup::where('system_user_id', '=', $id)->delete();
        SystemUserUnit::where('system_user_id', '=', $id)->delete();
        SystemUserProgram::where('system_user_id', '=', $id)->delete();
    
        // delete the object itself
        parent::delete($id);
    }

    /**
     * Validate user login
     * @param $login String with user login
     */
    public static function validate($login)
    {
        $user = self::newFromLogin($login);
    
        if ($user instanceof SystemUsers)
        {
            if ($user->active == 'N')
            {
                throw new Exception(_t('Inactive user'));
            }
        }
        else
        {
            throw new Exception(_t('User not found'));
        }
    
        return $user;
    }

    /**
     * Authenticate the user
     * @param $login String with user login
     * @param $password String with user password
     * @returns TRUE if the password matches, otherwise throw Exception
     */
    public static function authenticate($login, $password)
    {
        $user = self::newFromLogin($login);
        if ($user->password !== md5($password))
        {
            throw new Exception(_t('Wrong password'));
        }
    
        return $user;
    }

    /**
     * Returns a SystemUser object based on its login
     * @param $login String with user login
     */
    static public function newFromLogin($login)
    {
        return SystemUsers::where('login', '=', $login)->first();
    }

    /**
     * Returns a SystemUser object based on its e-mail
     * @param $email String with user email
     */
    static public function newFromEmail($email)
    {
        return SystemUsers::where('email', '=', $email)->first();
    }

    /**
     * Return the programs the user has permission to run
     */
    public function getPrograms()
    {
        $programs = array();
    
        foreach( $this->getSystemUserGroups() as $group )
        {
            foreach( $group->getSystemPrograms() as $prog )
            {
                $programs[$prog->controller] = true;
            }
        }
            
        foreach( $this->getSystemUserPrograms() as $prog )
        {
            $programs[$prog->controller] = true;
        }
    
        return $programs;
    }

    /**
     * Return the programs the user has permission to run
     */
    public function getProgramsList()
    {
        $programs = array();
    
        foreach( $this->getSystemUserGroups() as $group )
        {
            foreach( $group->getSystemPrograms() as $prog )
            {
                $programs[$prog->controller] = $prog->name;
            }
        }
            
        foreach( $this->getSystemUserPrograms() as $prog )
        {
            $programs[$prog->controller] = $prog->name;
        }
    
        asort($programs);
        return $programs;
    }

    /**
     * Check if the user is within a group
     */
    public function checkInGroup( SystemGroup $group )
    {
        $user_groups = array();
        foreach( $this->getSystemUserGroups() as $user_group )
        {
            $user_groups[] = $user_group->id;
        }

        return in_array($group->id, $user_groups);
    }

    /**
     *
     */
    public static function getInGroups( $groups )
    {
        $collection = [];
        $users = self::all();
        if ($users)
        {
            foreach ($users as $user)
            {
                foreach ($groups as $group)
                {
                    if ($user->checkInGroup($group))
                    {
                        $collection[] = $user;
                    }
                }
            }
        }
        return $collection;
    }

    /**
     * Clone the entire object and related ones
     */
    public function cloneUser()
    {
        $groups   = $this->getSystemUserGroups();
        $units    = $this->getSystemUserUnits();
        $programs = $this->getSystemUserPrograms();
        unset($this->id);
        $this->name .= ' (clone)';
        $this->store();
        if ($groups)
        {
            foreach ($groups as $group)
            {
                $this->addSystemUserGroup( $group );
            }
        }
        if ($units)
        {
            foreach ($units as $unit)
            {
                $this->addSystemUserUnit( $unit );
            }
        }
        if ($programs)
        {
            foreach ($programs as $program)
            {
                $this->addSystemUserProgram( $program );
            }
        }
    }

            
}

