<?php

use CloudDfe\SdkPHP\Emitente;
use CloudDfe\SdkPHP\Nfe;
use CloudDfe\SdkPHP\Nfce;

class cloudDFE_NFE
{
   
    private static $database = 'kraftsoft';
    
    public static function retornaTipoAmbiente($tpamb)
    {
       if ($tpamb==NotafiscalTpamb::Producao)
       {
        return Emitente::AMBIENTE_PRODUCAO;   
       }
       else
       {
        return Emitente::AMBIENTE_HOMOLOGACAO;   
       }
    }

    public static function retornaToken($nota)
    {
        
        $filial = SystemUnit::find($nota->system_unit_id);
        
        if(!$filial){
            throw new Exception('Filial não encontrada!');
        }
        
        $api = ApiDocumentos::find($filial->nfe_api_documentos_id);

        if(!$api){
            throw new Exception('API não encontrada!');
        }
        
        return $api->token_api;    
        
    }
    
    public static function montaContingencia($nota)
    {   
        $dados = array();
        
        if(isset($nota->contingencia_data))
        {
              //  $dados["contingencia"] = [
               //     "data" => date("Y-m-d\TH:s:sP", strtotime($nota->contingencia_data)),
            //        "motivo" => $nota->contingencia_motivo,
             //       ];
             
                $dados = [
                    "data" => date("Y-m-d\TH:s:sP", strtotime($nota->contingencia_data)),
                    "motivo" => $nota->contingencia_motivo,
                    ];             

            return $dados;
        }
         
        return null;        
    }
    
    public static function montaExportacao($nota)
    {   
        // $dados = array();
        
        if(!isset($nota->cliente_cpfcnpj))
        {
                $dados = [
                    "uf_local_embarque" => $nota->exporta_ufsaidapais,
                    "local_embarque" => $nota->exporta_xlocexporta,
                    "local_despacho" => $nota->exporta_xlocdespacho,
                    ];

            return $dados;
        }
         
        return null;        
    }    
    
    public static function montaCPF_CNPJ_IdEstrangeiro($nota)
    {
        
        if(!isset($nota->cliente_cpfcnpj))
        {
           return '0000'.strval($nota->cliente_id);
        }
        else
        {
            return Funcoes::sonumeros($nota->cliente_cpfcnpj);
        }
    }
    
    public static function montaNotasReferenciadas($nota)
    {
        //FALTA AINDA TRATAR OS OUTROS TIPOS DE REFERENCIA, CUPOM FISCAL E CTE        
        
        $refnfe = NotafiscalRefnfe::where('notafiscal_id', '=', $nota->id)->load();
        
        $array = [];
        
        foreach($refnfe as $nfe)
        {

            $array[]['nfe']['chave'] = $nfe->refnfe;
            
        }
        

        return $array;
    }    
    
    public static function montaPessoasAutorizadas($nota)
    {
        $autxml = NotafiscalAutxml::where('notafiscal_id', '=', $nota->id)->load();
        
        $dados = array();
        
        foreach($autxml as $linha)
        {
            $cpf_cnpj = Funcoes::sonumeros($linha->cpf_cnpj);
            
            if (strlen(trim($cpf_cnpj)) > 11)
            {
                $desc_cpf_cnpj = "cnpj";     
            }
            else
            {
                $desc_cpf_cnpj = "cpf";
            }                 
            
            $dados[] = [
                $desc_cpf_cnpj => $cpf_cnpj
            ];
        }
        
        //echo "<pre>";
        //print_r($dados);
        //echo "</pre>";        
        
        return $dados;        
    }
    
    public static function montaCobranca($nota)
    {
        $count = NotafiscalDup::where('notafiscal_id', '=', $nota->id)->count();
        
        $dados = array();
        
        if($count > 0)
        {
            $dados["fatura"] = [
                             "numero" => $nota->fatura_numero,
                             "valor_original" => doubleval($nota->fatura_valor_original),
                             "valor_desconto" => doubleval($nota->fatura_valor_desconto),
                             "valor_liquido" => doubleval($nota->fatura_valor_liquido),
                            //  "duplicatas" => self::montaduplicatas($nota)
            ];

            $dados["duplicatas"] = self::montaduplicatas($nota);
                                
            return $dados;
        }
         
        return null;
    }

    public static function montaLocalretirada($nota)
    {

        if (strlen(trim(Funcoes::sonumeros($nota->retirada_cpfcnpj))) > 11)
        {
            $desc_cpf_cnpj = "cnpj";     
        }
        else
        {
            $desc_cpf_cnpj = "cpf";
        }
    
        if(!is_null($nota->retirada_xlgr) or !empty($nota->retirada_xlgr) or $nota->retirada_xlgr != ""){
            
            $dados = [
                $cpf_cnpj => Funcoes::sonumeros($nota->retirada_cpfcnpj),
                "nome" => $nota-> retirada_marcado,
                "inscricao_estadual" => "ISENTO",
                "email" => "",
                "endereco" => [
                    "logradouro" => $nota->retirada_xlgr,
                    "numero" => $nota->retirada_nro,
                    "complemento" => $nota->retirada_xcpl,
                    "bairro" => $nota->retirada_xbairro,
                    "codigo_municipio" => $nota->retirada_cmun,
                    "nome_municipio" => $nota->retirada_xmun,
                    "uf" => $nota->retirada_uf,
                    "cep" => "",
                    "codigo_pais" => "1058",
                    "nome_pais" => "Brasil",
                    "telefone" => "",
                ]
    
            ];
    
            return $dados; 
            
        }
        
        return null;
    }

    public static function montaLocalentrega($nota){

        if (strlen(trim(Funcoes::sonumeros($nota->entrega_cpfcnpj))) > 11)
        {
            $desc_cpf_cnpj = "cnpj";     
        }
        else
        {
            $desc_cpf_cnpj = "cpf";
        }
        
        if(!is_null($nota->entrega_xlgr) or !empty($nota->entrega_xlgr) or $nota->entrega_xlgr != ""){
            $dados = [
                $desc_cpf_cnpj => Funcoes::sonumeros($nota->entrega_cpfcnpj),
                "nome" => $nota->entrega_marcado,
                // "inscricao_estadual" => "ISENTO",
                "email" => "",
                "endereco" => [
                    "logradouro" => $nota->entrega_xlgr,
                    "numero" => $nota->entrega_nro,
                    "complemento" => $nota->entrega_xcpl,
                    "bairro" => $nota->entrega_xbairro,
                    "codigo_municipio" => strval($nota->entrega_cmun),
                    "nome_municipio" => $nota->entrega_xmun,
                    "uf" => $nota->entrega_uf,
                    "cep" => "",
                    "codigo_pais" => "1058",
                    "nome_pais" => "Brasil",
                    "telefone" => "",
                ]
    
            ];
    
            return $dados; 
            
        }
        
        return null;
    }
    
    public static function cartaCorrecao($chave, $justificativa)
    {
        
        try 
        {

            $nota = Notafiscal::where('chave', '=', $chave)->first();

            $params = [
                'ambiente' => self::retornaTipoAmbiente($nota->notafiscal_tpamb_id),
                'token' => self::retornaToken($nota),
                'options' => [
                    'debug' => false,
                    'timeout' => 60,
                    'port' => 443,
                    'http_version' => CURL_HTTP_VERSION_NONE
                ]
            ];
            
            $nfe = new Nfe($params);
            
            $payload = [
                'chave' => $chave,
                'justificativa' => $justificativa
            ];
            
            $resp = $nfe->correcao($payload);

            if($nota)
            {
                if($resp->codigo == 135)
                {
                    $nota->sn_carta_correcao   = 'S';
                    $nota->cce_protocolo       = $resp->protocolo;
                    $nota->cce_datahora_evento = date('Y-m-d H:i:s');
                    $nota->cce_sequencial      = $resp->numero_carta_correcao;
                    $nota->cce_correcao        = $justificativa;
                    $nota->cce_xml             = $resp->xml_carta_correcao;
                    $nota->cce_pdf             = $resp->pdf_carta_correcao;
                    $nota->store();
                }
            }
            
            funcaoNFE::salvarLog($nota->id, 'Retorno carta de correção: '.self::formataRetorno($resp));
            
            return $resp;
            
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }
    
    public static function criarNovaEmpresa($filial){
        
        
        try 
        {
            $params = [
                'ambiente' => self::retornaTipoAmbiente(),
                'token' => self::retornaToken(),
                'options' => [
                    'debug' => false,
                ]
            ];
            
            $emitente = new Emitente($params);
        
            $payload = [
                'nome' => $filial->name,
                'razao' => $filial->razao_social,
                "cnae" => '1234567',
                "crt" => 3,
                'ie' => $filial->inscricao_estadual,
                'im' => $filial->inscricao_municipal,
                'csc' => '9KLRH4IEMIQ58TKBOLRPHNDAN0SJEOKFK453',
                'cscid' => '2',
                'tar' => null,
                'login_prefeitura' => null,
                'senha_prefeitura' => null,
                'client_id_prefeitura' => null,
                'client_secret_prefeitura' =>null,
                'telefone' => '115555555',
                'email' => 'fulano@fulano.com.br',
                'rua' => 'AL JAPURUS',
                'numero' => '1345',
                'complemento' => 'Sala 111',
                'bairro' => 'BRAS',
                'municipio' => 'São Paulo',
                'cmun' => '3550308',
                'uf' => 'SP',
                'cep' => '02233000',
                'logo' => null
            ];
            
            //os payloads são sempre ARRAYS
            $resp = $emitente->atualiza($payload);
        
            echo "<pre>";
            print_r($resp); //imprime o objeto $resp em tela
            echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }
    
    public static function consultarDadosEmitente(){
        try 
        {
            $params = [
                'ambiente' => self::retornaTipoAmbiente(),
                'token' => self::retornaToken(),
                'options' => [
                    'debug' => false,
                ]
            ];
            
            $emitente = new Emitente($params);
        
            $resp = $emitente->mostra();
        
            echo "<pre>";
            print_r($resp); //imprime o objeto $resp em tela
            echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }

    public static function atualizaCertificado($filial){
        try 
        {
            $params = [
                'ambiente' => self::retornaTipoAmbiente(),
                'token' => self::retornaToken(),
                'options' => [
                    'debug' => false,
                ]
            ];
            
            $certificado = new Certificado($params);
        
            $payload = [
                'certificado' => base64_encode(file_get_contents($filial->certificadoA1)),
                'senha' => $filial->senha_certificado
            ];
            
            //os payloads são sempre ARRAYS
            $resp = $certificado->atualiza($payload);
        
            echo "<pre>";
            print_r($resp);
            echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public static function consultaCertificado(){
        try 
        {
            $params = [
                'ambiente' => self::retornaTipoAmbiente(),
                'token' => self::retornaToken(),
                'options' => [
                    'debug' => false,
                ]
            ];
        
            $certificado = new Certificado($params);
        
            $resp = $certificado->mostra();
        
         //   echo "<pre>";
          //  print_r($resp);
        //    echo "</pre>";
            
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }
    
    public static function montaPagamento($nota, $indicador_pagamento)
    {
        $pagamentos = NotafiscalPag::where('notafiscal_id', '=', $nota->id)->load();
        
        $array = array();
        
        foreach($pagamentos as $pag){
            $array[]= [
                "indicador_pagamento" => $indicador_pagamento,
                "meio_pagamento" => $pag->tpag,
                "valor" => doubleval($pag->vpag),
                "descricao_pagamento" => $pag->xpag
            ];
            
        }
        
        return $array;
    }
    
    public static function montaDuplicatas($nota)
    {
        $duplicatas = NotafiscalDup::where('notafiscal_id', '=', $nota->id)->load();
        
        $array = array();
        
        foreach($duplicatas as $dup)
        {
            array_push($array, ["data_vencimento" => $dup->dvenc, "valor" => doubleval($dup->vdup)]);
        }
        
        return $array;
    }
    
    public static function montaVolumes($nota){
        $volumes = NotafiscalFreteVolumes::where('notafiscal_id', '=', $nota->id)->load();
        
        $array = array();
        
        foreach($volumes as $vol){
            $array[] = [
                "quantidade" => strval($vol->qVol),
                "especie" => $vol->esp,
                "marca" => $vol->marca,
                "numero" => $vol->nVol,
                "peso_liquido" => $vol->pesol,
                "peso_bruto" => $vol->pesob,
                "lacres" => [
                    "numero" => $nota->transp_numeracao
                ]
            ];    
        }
        
        return $array;
    }
    
    public static function montaItensNota($nota)
    {
           TTransaction::open(self::$database);
        $items = NotafiscalItens::where('notafiscal_id', '=', $nota->id)->load();
        $natureza = NaturezaOperacao::find($nota->natureza_operacao_id);
                TTransaction::close();
        $array = array();
        
        $item_nota = 0;
        
        foreach ($items as $item) 
        {
            $item_nota ++;
            
            // $natureza = NaturezaOperacao::where('descricao', '=', $item->notafiscal->natureza_operacao)->first();
            //echo 'PFCP '.$item->fcp_pfcp !== '' ? doubleval($item->fcp_pfcp) : '';

            $array[] = [
                    "numero_item" => "$item_nota",
                    "codigo_produto" => $item->cprod,
                    "codigo_barras_comercial" => $item->cean,
                    "codigo_barras_tributavel" => $item->cean,
                    "descricao" => $item->xprod,
                    "codigo_ncm" => $item->ncm,
                    "cest" => $item->cest,
                    "cfop" => !is_null($item->cfop) ? $item->cfop : $natureza->cfop,
                    "unidade_comercial" => $item->ucom,
                    "quantidade_comercial" => doubleval($item->qcom),
                    "valor_unitario_comercial" => doubleval($item->vuncom),
                    "valor_bruto" => doubleval($item->vprod),
                    "unidade_tributavel" => $item->utrib,
                    "quantidade_tributavel" => doubleval($item->qtrib),
                    "valor_unitario_tributavel" => doubleval($item->vuntrib),
                    "origem" => strval($item->notafiscal_item_orig->codigo),
                    "inclui_no_total" => "1",
                    "imposto" => [
                        "valor_aproximado_tributos" => doubleval($item->vtottrib),
                        "icms" => [
                            "situacao_tributaria" => $item->icms_cst,
                            "aliquota_credito_simples" => doubleval($item->icms_pcredsn),
                            "valor_credito_simples" => doubleval($item->icms_vcredicmssn),
                            "modalidade_base_calculo" => strval($item->notafiscal_item_modbc->codigo),
                            "valor_base_calculo" => doubleval($item->icms_basecalc),
                            "modalidade_base_calculo_st" => $item->icms_modcst,
                            "aliquota_reducao_base_calculo" => doubleval($item->icms_predbc),
                            "aliquota" => doubleval($item->icms_aliquota),
                            "aliquota_final" => doubleval($item->icms_pst),
                            "valor" => doubleval($item->icms_valor),
                            "aliquota_margem_valor_adicionado_st" => doubleval($item->icms_pmvast),
                            "aliquota_reducao_base_calculo_st" => doubleval($item->icms_predbcst),
                            "valor_base_calculo_st" => doubleval($item->icms_vbcst),
                            "aliquota_st" => doubleval($item->icms_picmsst),
                            "valor_st" => doubleval($item->icms_vicmsst)
                        ],
                        "fcp" => !is_null($item->fcp_pfcp) ? [
                            "valor_base_calculo" => doubleval($item->fcp_vbcfcp), 
                            "aliquota" => doubleval($item->fcp_pfcp),
                            "valor" => doubleval($item->fcp_vfcp),
                            "valor_base_calculo_st" => doubleval($item->fcp_vbcfcpst),
                            "aliquota_st" => doubleval($item->fcp_pfcpst),
                            "valor_st" => doubleval($item->fcp_vfcpst),
                            "valor_base_calculo_retido_st" => doubleval($item->fcp_vbcfcpstret),
                            "aliquota_retido_st" => doubleval($item->fcp_pfcpstret),
                            "valor_retido_st" => doubleval($item->fcp_vfcpstret)
                            //"percentual_diferimento" => ??
                            //"valor_diferimento" => ??
                            //"valor_efetivo" => ??
                        ]: null,
                        "icms_interestadual" => [
                                           "valor_base_calculo_uf_destino" => !is_null($item->icms_vbcufdest) ? doubleval($item->icms_vbcufdest): 0,
                                           //"valor_fcp_base_calculo_uf_destino" => !is_null($item->icms_vbcfcpufdest) ? doubleval($item->icms_vbcfcpufdest): 0, 
                                           //"aliquota_fcp_uf_destino" => doubleval($item->icms_pfcpufdest), 
                                           //"valor_fcp_uf_destino" => doubleval($item->icms_vfcpufdest),     
                                           "aliquota_interna_uf_destino" => doubleval($item->icms_picmsufdest),     
                                           "aliquota_interestadual" => doubleval($item->icms_picmsinter),
                                           "aliquota_partilha" => doubleval($item->icms_picmsinterpart),
                                           "valor_uf_destino" => doubleval($item->icms_vicmsufdest),
                                           "valor_uf_remetente" => doubleval($item->icms_vicmsufremet)
                        ],
                        "pis" => [
                            "situacao_tributaria" => $item->pis_cst,
                            "valor_base_calculo" => doubleval($item->pis_valor_base),
                            "aliquota" => doubleval($item->pis_aliquota),
                            "valor" => doubleval($item->pis_valor_base)
                        ],
                        "cofins" => [
                            "situacao_tributaria" => $item->cofins_cst,
                            "valor_base_calculo" => doubleval($item->cofins_valor_base),
                            "aliquota" => doubleval($item->cofins_aliquota),
                            "valor" => doubleval($item->cofins_valor)
                        ]
                    ],
                    "valor_desconto" => doubleval($item->vdesc),
                    "valor_frete" => doubleval($item->vfrete),
                    "valor_seguro" => doubleval($item->vseg),
                    "valor_outras_despesas" => doubleval($item->voutro),
                    "informacoes_adicionais_item" => "Valor aproximado tributos R$: 9,43 (4,20%) Fonte: IBPT",
                    "pedido_compra" => [
                        "numero" => $item->xped,
                        "item" => $item->nitemped
                    ]
            ];
            
            
        }

        return $array;
    }
    
    public static function buscaNFEs(){
        try 
        {
            $params = [
                'ambiente' => self::retornaTipoAmbiente(),
                'token' => self::retornaToken(),
                'options' => [
                    'debug' => false,
                ]
            ];
            $nfe = new Nfe($params);
        
        
            //informar o período desejado de backup
            $payload = [
                //"numero_inicial" => 1710,
                //"numero_final" => 101002,
                //"serie" => 1,
                "data_inicial" => "2021-04-01",
                "data_final" => "2021-04-31",
                //"cancel_inicial" => "2019-12-01", // Cancelamento
                //"cancel_final" => "2019-12-31"
            ];
            //payload é sempre um ARRAY
            $resp = $nfe->busca($payload);
        
            //echo "<pre>";
            //print_r($resp);
            //echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }
    
    public static function formataRetorno($resp)
    {
       $str = "Cod. Sefaz: ".$resp->codigo;
       $str = $str." | Mensagem: ".$resp->mensagem;
       
       if (!$resp->sucesso)
       //if (!$resp->erros=='')
       {
           if ($resp->erros)
           {
               $str = $str." | Erro(s): ";
               
                foreach ($resp->erros as $erro)
                {   
                    $str = $str.'<pre>';
                    $str = $str."Campo: ".$erro->campo;
                    $str = $str."| Erro: ".$erro->erro;
                    $str = $str.'</pre>';
                }           
           }
       }
       else
       {
           if (!$resp->status=='')
           {       
            $str = $str." | Status: ".$resp->status;  
           }
       }
       
        //echo "<pre>";
        //print_r($resp);
        //echo "</pre>";       
       
       return $str;
    }
    
    public static function processa_nota_autorizada($nota, $resp)
    {
        $nota->codigo_sefaz = $resp->codigo;
        $nota->mensagem = $resp->mensagem;
        $nota->status = $resp->status;   
        $nota->xml = $resp->xml;
        $nota->protocolo_envio = $resp->protocolo;
        $nota->data_hora_envio = $resp->data_hora_evento;
        $nota->notafiscal_situacao_id = NotafiscalSituacao::Autorizada;                    
        $nota->store();        
    }
    
    public static function processa_nota_rejeitada($nota, $resp)
    {
        $nota->codigo_sefaz = $resp->codigo;
        $nota->mensagem = $resp->mensagem;
        // $nota->status = $resp->status;   
        $nota->store();        
    }
    

    public static function consultaNFE($numero, $apresenta_erro = true)
    {
        try 
        {
            $nota = Notafiscal::where('chave', '=', $numero)->first();
            
            if($nota)
            {
                $params = [
                    'ambiente' => self::retornaTipoAmbiente($nota->notafiscal_tpamb_id),
                    'token' => self::retornaToken($nota),
                    'options' => [
                        'debug' => false,
                    ]
                ];
                
                if($nota->notafiscal_mod_id == NotafiscalMod::NFe){
                    $nfe = new Nfe($params);    
                }else{
                    $nfe = new Nfce($params);
                }
                
            
                //informar o período desejado de backup
            
            
                $resp = $nfe->consulta([
                    'chave' => $numero
                ]);
                
                // var_dump($resp);
                
                $notafiscalsituacaoSefaz = NotafiscalSituacaoSefaz::where('codigo_sefaz', '=', $resp->codigo)->first();
                
                if($notafiscalsituacaoSefaz)
                {
                    $nota->notafiscal_situacao_id = $notafiscalsituacaoSefaz->notafiscal_situacao_id;
                }
                
                if($resp->codigo == 101)
                {
                    $nota->protocolo_cancelamento = $resp->protocolo;
                    $nota->datahora_cancelamento = $resp->data_hora_evento;
                    $nota->xml_cancelamento = $resp->xml;
                    
                    funcaoNFE::salvarLog($nota->id, 'Retorno consulta (cancelamento): '.self::formataRetorno($resp));
                }
                elseif($resp->codigo == 100)
                {
                    $nota->xml = $resp->xml;
                    $nota->protocolo_envio = $resp->protocolo;
                    $nota->data_hora_envio = $resp->data_hora_evento;
                    
                    funcaoNFE::salvarLog($nota->id, 'Retorno consulta (100): '.self::formataRetorno($resp));
                }
                else
                {

                    //  $retorno = [
                    //     'campo' => $resp->codigo,
                    //     'erro' => $resp->status,
                    //     'descricao' => $resp->mensagem,
                    //     'detalhes' => null
                    //  ];
                     
                    //  $erros = [$retorno];
                     
                    //  TSession::setValue('retorno_envio_nota', $erros);
                    //  TApplication::loadPage('RetornoCloudList', 'onShow');
                    if($apresenta_erro == true)
                    {
                        funcaoNFE::salvarLog($nota->id, 'Retorno consulta (erro): '.self::formataRetorno($resp));
                        
                        new TMessage('error', $resp->codigo." - ".$resp->mensagem);    
                    } 
                    
                }
                
                $nota->codigo_sefaz = $resp->codigo;
                $nota->mensagem = $resp->mensagem;
                
                if (isset($resp->status))
                {
                    $nota->status = $resp->status;
                }
                
                $nota->store();
                
            }
            
            // echo "<pre>";
            // print_r($resp);
            // echo "</pre>";

        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }
    
    public static function cancelaNFE($chave, $justificativa)
    {
        try 
        {
            $nota = Notafiscal::where('chave', '=', $chave)->first();

            if($nota){

                $params = [
                    'ambiente' => self::retornaTipoAmbiente($nota->notafiscal_tpamb_id),
                    'token' => self::retornaToken($nota),
                    'options' => [
                        'debug' => false,
                    ]
                ];
                
                if($nota->notafiscal_mod_id == NotafiscalMod::NFe)
                {
                    $nfe = new Nfe($params);    
                }
                else
                {
                    $nfe = new Nfce($params);
                }                
                

                $payload = [
                    'chave' => $chave,
                    'justificativa' => $justificativa
                ];
                
                $resp = $nfe->cancela($payload);
                
                funcaoNFE::salvarLog($nota->id, 'Retorno cancelamento: '.self::formataRetorno($resp));
                
                if($resp->codigo != 501)
                {                
                    $nota->justificativa_cancelamento = $justificativa;
                    $nota->codigo_sefaz = $resp->codigo;
                    $nota->status = 'Cancelado';
                    $nota->mensagem = $resp->mensagem;
                    $nota->protocolo_cancelamento = $resp->protocolo;
                    $nota->datahora_cancelamento = date('Y-m-d H:i:s');
                    $nota->xml_cancelamento = $resp->xml;
                    $nota->notafiscal_situacao_id = NotafiscalSituacao::Cancelada;
                    $nota->store();
                }
                
                return $resp;
            }
            
                
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }
    
    public static function pdfNFE($chave)
    {
        try 
        {
           $nota = Notafiscal::where('chave', '=', $chave)->first();
            
            if($nota)
            {
                $params = [
                    'ambiente' => self::retornaTipoAmbiente($nota->notafiscal_tpamb_id),
                    'token' => self::retornaToken($nota),
                    'options' => [
                        'debug' => false,
                    ]
                ];
                
                $nfe = new Nfe($params);

                $payload = [
                    'chave' => $chave
                ];
                
                $resp = $nfe->pdf($payload);
                
                $bin = base64_decode($resp->pdf , true);
                
                $document = 'tmp/' . uniqid() . '.pdf';
    
                file_put_contents($document, $bin);

                return $document;            
            }
            
            return null;
            
            // echo "<pre>";
            // print_r($resp);
            // echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }
    
    public static function pdfNFCe($chave)
    {
        try 
        {
            $nota = Notafiscal::where('chave', '=', $chave)->first();
            
            if($nota)
            {
                $params = [
                    'ambiente' => self::retornaTipoAmbiente($nota->notafiscal_tpamb_id),
                    'token' => self::retornaToken($nota),
                    'options' => [
                        'debug' => false,
                    ]
                ];
                
                $nfce = new Nfce($params);
            
            
                $payload = [
                    'chave' => $chave
                ];
                
                $resp = $nfce->pdf($payload);
                
                $bin = base64_decode($resp->pdf , true);
                
                $document = 'tmp/' . uniqid() . '.pdf';
    
                file_put_contents($document, $bin);

                return $document;            
            }
            
            return null;
            
            // echo "<pre>";
            // print_r($resp);
            // echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }    
    

    public static function xmlNFE($chave)
    {
        try 
        {
            $nota = Notafiscal::where('chave', '=', $chave)->first();
            
            if($nota)
            {
                $bin = base64_decode($nota->xml , true);
                
                $document = 'tmp/'.$chave.'.xml';
    
                file_put_contents($document, $bin);
                
                // parent::openFile($document);
            
                // header("Content-type: application/save; charset=iso-8859-1");
                // header('Content-Disposition: attachment; filename="' . basename($document) . '"');
                // header('Expires: 0');
                // header('Pragma: no-cache');
                // readfile($document);
                
                return $document;            
                
            }
            
            // return null;
            
            // echo "<pre>";
            // print_r($resp);
            // echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }
    
    public static function pdfCCe($chave)
    {
        try 
        {
            $nota = Notafiscal::where('chave', '=', $chave)->first();
            
            if($nota){
                $params = [
                    'ambiente' => self::retornaTipoAmbiente($nota->notafiscal_tpamb_id),
                    'token' => self::retornaToken($nota),
                    'options' => [
                        'debug' => false,
                    ]
                ];
                
                $nfe = new Nfe($params);
                
                $resp = $nfe->consulta([
                    'chave' => $chave
                ]);
                
                // echo "<pre>";
                // var_dump($resp);
                // echo "</pre>";
                
                $bin = base64_decode($resp->pdf_carta_correcao , true);
                
                $document = 'tmp/' . uniqid() . '.pdf';
    
                file_put_contents($document, $bin);
            
                
                return $document;            
                
            }
            
            return null;
            
            // echo "<pre>";
            // print_r($resp);
            // echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }   
    
    public static function criarNfce($nota)
    {
        try 
        {
                $params = [
                    'ambiente' => self::retornaTipoAmbiente($nota->notafiscal_tpamb_id),
                    'token' => self::retornaToken($nota),
                    'options' => [
                        'debug' => false,
                        'timeout' => 60,
                        'port' => 443,
                        'http_version' => CURL_HTTP_VERSION_NONE
                    ]
                ];
                
                $nfce = new Nfce($params);
                
                $date = date('Y-m-d H:i:s');
                // $dataemissao = date("Y-m-d\TH:s:sP", strtotime($date));
                // $dataemissao = date("Y-m-d\TH:s:sP", strtotime($nota->data_emissao));
                $dataemissao = date("Y-m-d\TH:i:sP");
                
                $nota->data_emissao = $date;
                $nota->data_saida = $date;
                $nota->store();

                $payload = [
                    "natureza_operacao" => $nota->natureza_operacao->descricao,
                    "serie" => $nota->serie,
                    "numero" => $nota->numero,
                    "data_emissao" =>  $dataemissao,
                    "data_saida" => $dataemissao,
                    "tipo_operacao" => $nota->nf_tipodocumento->valor,
                    "presenca_comprador" => $nota->notafiscal_indpres->valor,
                    "itens" => self::montaItensNfce($nota),
                    "frete" => [
                        "modalidade_frete" => $nota->notafiscal_modfrete->valor
                    ],
                    "pagamento" => [
                        "formas_pagamento" => self::montaPagamentoNfce($nota, $nota->notafiscal_indpag->valor)
                    ],
                    "informacoes_adicionais_contribuinte" => $nota->infcpl,
                    "pessoas_autorizadas" => self::montaPessoasAutorizadasNfce($nota)
                    
                ];
                
                $resp = $nfce->cria($payload);
                
                // echo '<pre>';
                // var_dump($resp);
                // echo '</pre>';
                
                if (isset($resp->chave))
                {
                    $nota->chave = $resp->chave;
                    $nota->store();
                }                

                //validações do envio...
                if ($resp->sucesso) 
                {
                    $chave = $resp->chave;
                    
                    // autorizado
                    funcaoNFE::salvarLog($nota->id, "Retorno Enviar (Autorizado): ".self::formataRetorno($resp)); 
                         
                    self::processa_nota_autorizada($nota, $resp);
                } 
                else if (in_array($resp->codigo, [5001, 5002]))
                {
                    // erro nos campos
                    funcaoNFE::salvarLog($nota->id, "Retorno Enviar (erro): ".self::formataRetorno($resp));                                 
                                    
                    self::processa_nota_rejeitada($nota, $resp);
                }
                else if ($resp->codigo == 5008 or $resp->codigo >= 7000)
                {
                    $chave = $resp->chave;
                    // >= 7000 erro de timeout ou de conexão
                    // 5008 documento já criado
                    //var_dump($resp);
                    
                    $payload = [
                        'chave' => $chave
                    ];
                    // recomendamos fazer a consulta pela chave para sincronizar o documento
                    $resp = $nfce->consulta($payload);
                    
                    if ($resp->sucesso)
                    {
                        // autorizado
                        funcaoNFE::salvarLog($nota->id, "Retorno Enviar (5008/7000-Autorizado): ".self::formataRetorno($resp)); 
                         
                        self::processa_nota_autorizada($nota, $resp);
                    } 
                    else
                    {
                        // rejeição
                        funcaoNFE::salvarLog($nota->id, "Retorno Enviar (5008/7000-Rejeição): ".self::formataRetorno($resp));                                 
                                
                        self::processa_nota_rejeitada($nota, $resp);
                    }
                }
                else
                {
                    // rejeição
                    funcaoNFE::salvarLog($nota->id, "Retorno Enviar (Rejeição): ".self::formataRetorno($resp));                                 
                                    
                    self::processa_nota_rejeitada($nota, $resp);
                }    
                
                // echo '<pre>';
                // var_dump($resp);
                // echo '</pre>';                
         
                    
        return $resp;
                
        } catch (Exception $e) 
        {
            var_dump($e->getMessage());
        }
    }
    
    public static function montaPessoasAutorizadasNfce($nota)
    {
        $autxml = NotafiscalAutxml::where('notafiscal_id', '=', $nota->id)->load();
        
        $dados = array();
        
        foreach($autxml as $linha)
        {
            $cpf_cnpj = Funcoes::sonumeros($linha->cpf_cnpj);
            
            if (strlen(trim($cpf_cnpj)) > 11)
            {
                $desc_cpf_cnpj = "cnpj";     
            }
            else
            {
                $desc_cpf_cnpj = "cpf";
            }                 
            
            $dados[] = [
                $desc_cpf_cnpj => $cpf_cnpj
            ];
        }
        
        //echo "<pre>";
        //print_r($dados);
        //echo "</pre>";        
        
        return $dados;        
    }
    
    public static function montaPagamentoNfce($nota, $indicador_pagamento)
    {
        $pagamentos = NotafiscalPag::where('notafiscal_id', '=', $nota->id)->load();
        
        $array = array();
        
        foreach($pagamentos as $pag)
        {
            $array[]= [
                "indicador_pagamento" => $indicador_pagamento,
                "meio_pagamento" => $pag->tpag,
                "valor" => doubleval($pag->vpag),
                "descricao_pagamento" => $pag->xpag,
                "tipo_integracao" => '2'
            ];
        }
        
        return $array;
    }
    
    public static function montaItensNfce($nota)
    {
        $items = NotafiscalItens::where('notafiscal_id', '=', $nota->id)->load();
        $natureza = NaturezaOperacao::find($nota->natureza_operacao_id);
        
        $array = array();
        
        $item_nota = 0;
        
        foreach ($items as $item) {
            $item_nota ++;
            
            // $natureza = NaturezaOperacao::where('descricao', '=', $item->notafiscal->natureza_operacao)->first();
            
            $array[] = [
                    "numero_item" => "$item_nota",
                    "codigo_produto" => $item->cprod,
                    "descricao" => $item->xprod,
                    "codigo_ncm" => $item->ncm,
                    "cfop" => !is_null($item->cfop) ? $item->cfop : $natureza->cfop,
                    "unidade_comercial" => $item->ucom,
                    "quantidade_comercial" => doubleval($item->qcom),
                    "valor_unitario_comercial" => doubleval($item->vuncom),
                    "valor_bruto" => doubleval($item->vprod),
                    "unidade_tributavel" => $item->utrib,
                    "quantidade_tributavel" => doubleval($item->qtrib),
                    "valor_unitario_tributavel" => doubleval($item->vuntrib),
                    "origem" => strval($item->notafiscal_item_orig->codigo),
                    "inclui_no_total" => "1",
                    "imposto" => [
                        "valor_total_tributos" => doubleval($item->vtottrib),
                        "icms" => [
                            "situacao_tributaria" => $item->icms_cst,
                            "aliquota_credito_simples" => doubleval($item->icms_pcredsn),
                            "valor_credito_simples" => doubleval($item->icms_vcredicmssn),
                            "modalidade_base_calculo" => $item->notafiscal_item_modbc->codigo,
                            "valor_base_calculo" => doubleval($item->icms_basecalc),
                            "modalidade_base_calculo_st" => $item->icms_modcst,
                            "aliquota_reducao_base_calculo" => doubleval($item->icms_predbc),
                            "aliquota" => doubleval($item->icms_aliquota),
                            "aliquota_final" => doubleval($item->icms_pst),
                            "valor" => doubleval($item->icms_valor),
                            "aliquota_margem_valor_adicionado_st" => doubleval($item->icms_pmvast),
                            "aliquota_reducao_base_calculo_st" => doubleval($item->icms_predbcst),
                            "valor_base_calculo_st" => doubleval($item->icms_vbcst),
                            "aliquota_st" => doubleval($item->icms_picmsst),
                            "valor_st" => doubleval($item->icms_vicmsst)
                        ],
                        "fcp" => [
                            "aliquota" => doubleval($item->fcp_pfcp)
                        ],
                        "pis" => [
                            "situacao_tributaria" => $item->pis_cst,
                            "valor_base_calculo" => doubleval($item->pis_valor_base),
                            "aliquota" => doubleval($item->pis_aliquota),
                            "valor" => doubleval($item->pis_valor_base)
                        ],
                        "cofins" => [
                            "situacao_tributaria" => $item->cofins_cst,
                            "valor_base_calculo" => doubleval($item->cofins_valor_base),
                            "aliquota" => doubleval($item->cofins_aliquota),
                            "valor" => doubleval($item->cofins_valor)
                        ]
                    ],
                    "valor_desconto" => doubleval($item->vdesc),
                    "valor_frete" => doubleval($item->vfrete),
                    "valor_seguro" => doubleval($item->vseg),
                    "valor_outras_despesas" => doubleval($item->voutro),
                    "informacoes_adicionais_item" => "Valor aproximado tributos R$: 9,43 (4,20%) Fonte: IBPT"

            ];
        }

        return $array;
    }
    
    public static function criarNFE($nota)
    {
        try 
        {
            
    

       
            $params = [
                'ambiente' => self::retornaTipoAmbiente($nota->notafiscal_tpamb_id),
                'token' => self::retornaToken($nota),
                'options' => [
                    'debug' => false,
                             ]
            ];
            
            $nfe = new Nfe($params);
    
            //informar o período desejado de backup
            
            $dataemissao = date("Y-m-d\TH:s:sP", strtotime($nota->data_emissao));
            $data_saida = date("Y-m-d\TH:s:sP", strtotime($nota->data_saida));
            
            if (strlen(trim(Funcoes::sonumeros($nota->cliente_cpfcnpj))) > 11)
            {
                $desc_cpf_cnpj = "cnpj";     
            }
            else
            {
                $desc_cpf_cnpj = "cpf";
            }

            if (strlen(trim(Funcoes::sonumeros($nota->transp_cnpj))) > 11)
            {
                $desc_transp_cpf_cnpj = "cnpj";     
            }
            else
            {
                $desc_transp_cpf_cnpj = "cpf";
            }
            
            if(!isset($nota->cliente_cpfcnpj))
            {
               $desc_cpf_cnpj = "id_estrangeiro"; 
            }            

            $paylod = [
                "contingencia" =>  self::montaContingencia($nota),
                "natureza_operacao" => $nota->natureza_operacao->descricao,
                "serie" => $nota->serie,
                "numero" => $nota->numero,
                "data_emissao" => $dataemissao,
                "data_entrada_saida" => $data_saida,
                "tipo_operacao" => $nota->nf_tipodocumento->valor,
                "local_destino" => $nota->notafiscaldest->valor,
                "finalidade_emissao" => $nota->notafiscal_finnfe->valor,
                "consumidor_final" => $nota->notafiscal_indfinal->valor,
                "presenca_comprador" => $nota->notafiscal_indpres->valor,
                // "intermediario" => [
                //     "indicador" => "0"
                // ],
                "notas_referenciadas" => self::montaNotasReferenciadas($nota),
                "destinatario" => [
                    $desc_cpf_cnpj => self::montaCPF_CNPJ_IdEstrangeiro($nota),
                    "nome" => trim($nota->cliente_nome),
                    "indicador_inscricao_estadual" => $nota->notafiscal_indiedest->valor,
                    "inscricao_estadual" => $nota->cliente_inscest,
                    "endereco" => [
                        "logradouro" => $nota->cliente_endereco,
                        "numero" => $nota->cliente_numero,
                        "bairro" => $nota->cliente_bairro,
                        "codigo_municipio" => strval($nota->cliente_cidade_codibge),
                        "nome_municipio" => $nota->cliente_cidade,
                        "uf" => $nota->cliente_estado,
                        "cep" => Funcoes::sonumeros($nota->cliente_cep),
                        "codigo_pais" => $nota->cliente_cpais, //"1058",
                        "nome_pais" => $nota->cliente_xpais, //"BRASIL",
                        "telefone" => Funcoes::sonumeros($nota->cliente_fone)
                    ]
                ],

                "local_retirada" => self::montaLocalretirada($nota),

                "local_entrega" => self::montaLocalentrega($nota),
                
                "itens" => self::montaItensNota($nota),

                "icms_base_calculo" => doubleval($nota->icms_basecalc),
                "icms_valor_total" => doubleval($nota->icms_valor),
                "valor_produtos_total" => doubleval($nota->vprod),
                "valor_frete" => doubleval($nota->frete_valor),
                "valor_seguro" => doubleval($nota->seguro_valor),
                "valor_desconto" => doubleval($nota->valor_total_produtos),
                "valor_pis" => doubleval($nota->pis_valor),
                "valor_cofins" => doubleval($nota->cofins_valor),
                "valor_outras_despesas" => doubleval($nota->outras_despesas),
                "valor_total" => doubleval($nota->valor_total_nota),
                "icms_valor_total_uf_dest" => doubleval($nota->vicmsufdest),                
                "icms_valor_total_uf_rem" => doubleval($nota->vicmsufdestremet),
                
                "frete" => [
                    "modalidade_frete" => $nota->notafiscal_modfrete->valor,
                    "transportador" => [
                        $desc_transp_cpf_cnpj = $nota->transp_cnpj,
                        "nome" => $nota->transp_nome,    
                        "inscricao_estadual" => $nota->transp_inscest,
                        "endereco" => $nota->transp_endereco,
                        "nome_municipio" => $nota->transp_cidade,
                        "uf" => $nota->transp_uf
                    ],
                    "veiculo" => [
                        "placa" => $nota->transp_placaveiculo,    
                        "uf" => $nota->transp_veiculouf,
                        "rntc" => $nota->transp_veiculoRNTC
                    ],
                    "volumes" => self::montaVolumes($nota)
                ],
                
                "cobranca" => self::montaCobranca($nota),
                
                "pagamento" => [
                    "formas_pagamento" => self::montaPagamento($nota, $nota->notafiscal_indpag->valor)
                ],
                
                "informacoes_adicionais_contribuinte" => $nota->infcpl,
                "informacoes_adicionais_fisco" => $nota->infadfisco,
                
                "exportacao" => self::montaExportacao($nota),
                
                "pessoas_autorizadas" => self::montaPessoasAutorizadas($nota)

            ];
            
            //  echo "<pre>";
            //  var_dump($paylod);
            //  echo "</pre>";
             
            //  exit;
            
            $resp = $nfe->cria($paylod);
            
            if (isset($resp->chave))
            {
                $nota->chave = $resp->chave;
                $nota->store();
            }
            
            if ($resp->sucesso) 
            {
                $chave = $resp->chave;
                
                // $nota->chave = $chave;
                // $nota->store();
                
                if ($resp->codigo == 5023)
                {
                    /**
                     * Em alguns casos a SEFAZ pode demorar mais do que esperado pela api
                     * para processar o lote, devido a trafego na rede ou sobrecarga de processamento
                     * então nesse caso quando vir codigo 5023 é necessario buscar a NFe pela chave de acesso
                     */
                    sleep(5);
                    $tentativa = 1;
                    
                    while ($tentativa <= 5)
                    {
                        $payload = [
                            'chave' => $chave
                        ];
                        
                        $resp = $nfe->consulta($payload);
                        
                        if ($resp->codigo != 5023)
                        {
                            if ($resp->sucesso)
                            {
                                // autorizado
                                funcaoNFE::salvarLog($nota->id, "Retorno Enviar (5023-Autorizado): ".self::formataRetorno($resp));  
                                
                                self::processa_nota_autorizada($nota, $resp);
                                
                                break;
                            }
                            else
                            {
                                // rejeição
                                funcaoNFE::salvarLog($nota->id, "Retorno Enviar (5023-Rejeição): ".self::formataRetorno($resp));   
                                
                                self::processa_nota_rejeitada($nota, $resp);
                                
                                break;
                            }
                        }
                        sleep(5);
                        $tentativa++;
                    }
                } 
                else
                {
                    // autorizado
                    funcaoNFE::salvarLog($nota->id, "Retorno Enviar (Autorizado): ".self::formataRetorno($resp)); 
                     
                    self::processa_nota_autorizada($nota, $resp);
                }
            } 
            else if (in_array($resp->codigo, [5001, 5002]))
            {
                // erro nos campos
                funcaoNFE::salvarLog($nota->id, "Retorno Enviar (erro): ".self::formataRetorno($resp));                                 
                                
                self::processa_nota_rejeitada($nota, $resp);
            }
            else if ($resp->codigo == 5008 or $resp->codigo >= 7000)
            {
                $chave = $resp->chave;
                // >= 7000 erro de timeout ou de conexão
                // 5008 documento já criado
                //var_dump($resp);
                
                $payload = [
                    'chave' => $chave
                ];
                // recomendamos fazer a consulta pela chave para sincronizar o documento
                $resp = $nfe->consulta($payload);
                
                if ($resp->sucesso)
                {
                    // autorizado
                    funcaoNFE::salvarLog($nota->id, "Retorno Enviar (5008/7000-Autorizado): ".self::formataRetorno($resp)); 
                     
                    self::processa_nota_autorizada($nota, $resp);
                } 
                else
                {
                    // rejeição
                    funcaoNFE::salvarLog($nota->id, "Retorno Enviar (5008/7000-Rejeição): ".self::formataRetorno($resp));                                 
                            
                    self::processa_nota_rejeitada($nota, $resp);
                }
            }
            else
            {
                // rejeição
                funcaoNFE::salvarLog($nota->id, "Retorno Enviar (Rejeição): ".self::formataRetorno($resp));                                 
                                
                self::processa_nota_rejeitada($nota, $resp);
            }

           // $nota_imprimir = Notafiscal::find($nota->id);
            
           // if($nota_imprimir)i
        //    {
                if($nota->codigo_sefaz == 100)
                {
                    self::pdfNFE($nota->chave);
                }
          //  }
        
            //echo "<pre>";
            //print_r($resp);
            //echo "</pre>";

            return $resp;
 
        } catch (\Exception $e) {
            echo $e->getMessage();
        }  
        
    }
                
}
