<?php

use CloudDfe\SdkPHP\Emitente;
use CloudDfe\SdkPHP\Mdfe;

class cloudDFe_MDFe
{
    private static $database = 'kraftsoft';
    
    public static function retornaTipoAmbiente($tpamb)
    {
       if ($tpamb==MdfeTpamb::Producao)
       {
        return Emitente::AMBIENTE_PRODUCAO;   
       }
       else
       {
        return Emitente::AMBIENTE_HOMOLOGACAO;   
       }
    }
    
    public static function retornaToken($mdfe)
    {
        
        $filial = SystemUnit::find($mdfe->system_unit_id);
        
        if(!$filial){
            throw new Exception('Filial não encontrada!');
        }
        
        $api = ApiDocumentos::find($filial->mdfe_api_documentos_id);

        if(!$api){
            throw new Exception('API não encontrada!');
        }
        
        return $api->token_api;    
        
    }    
    
    public static function Enviar($mdfe, $api_documentos_id)
    {
        
        try {
            $params = [
                'ambiente' => self::retornaTipoAmbiente($mdfe->mdfe_tpamb_id),
                'token' => self::retornaToken($mdfe),
                'options' => [
                    'debug' => false,
                             ]
            ];
            
            $mdfeapi = new Mdfe($params);            
            
            $dhemi = date("Y-m-d\TH:s:sP", strtotime($mdfe->dhEmi));
            $dhIniViagem = date("Y-m-d\TH:s:sP", strtotime($mdfe->dhIniViagem));
            
            // $qcte =  MdfeCtes::where('mdfe_infmundescarga_id' , 'in', "(select id from mdfe_infmundescarga where mdfe_id = $mdfe->id)" )->count();
            // $qnfe =  MdfeNfes::where('mdfe_infmundescarga_id' , 'in', "(select id from mdfe_infmundescarga where mdfe_id = $mdfe->id)" )->count();
            // $qmdfe =  MdfeMdfes::where('mdfe_infmundescarga_id' , 'in', "(select id from mdfe_infmundescarga where mdfe_id = $mdfe->id)" )->count();
            
            if(strlen(Funcoes::sonumeros($mdfe->seguro_cpfcnpj)) == 11){
                $nome_cpf_cnpj_seguro = 'CPF';
                $cpfnpj_seguro = Funcoes::sonumeros($mdfe->seguro_cpfcnpj); 
            }else{
                $nome_cpf_cnpj_seguro = 'CNPJ';
                $cpfnpj_seguro = Funcoes::sonumeros($mdfe->seguro_cpfcnpj); 
            }            
            

            $payload = [
                "tipo_operacao" => $mdfe->mdfe_tpemit->valor,
                "tipo_transporte" => $mdfe->mdfe_tptransp->valor,
                "numero" => $mdfe->nMDF,
                "serie" => $mdfe->serie,
                "data_emissao" => $dhemi,
                "uf_inicio" => $mdfe->fk_UFIni->sigla,
                "uf_fim" => $mdfe->fk_UFFim->sigla,
                "data_inicio_viagem" => $dhIniViagem,
                "canal_verde" => funcaoMDFe::ConverteSimNao($mdfe->indCanalVerde),
                "carregamento_posterior" => funcaoMDFe::ConverteSimNao($mdfe->indCarregaPosterior),
                "informacoes_adicionais_fisco" => $mdfe->infAdFisco,
                "informacoes_adicionais_contribuinte" => $mdfe->infCpl,
                "municipios_carregamento" => self::MontaInfCarrega($mdfe),
                "percursos" =>self::MontaInfPercurso($mdfe),
                "municipios_descarregamento" => self::MontaInfDescarga($mdfe),
                "seguros" => [
                    "responsavel" => $mdfe->mdfe_respSeg->valor,
                    "cnpj" => $mdfe->seguro_cpfcnpj,
                    "seguradora" => [
                        "nome" => $mdfe->seguradora_xSeg,
                        "cnpj" => $mdfe->seguradora_cnpj,
                        ]
                    ],
                "valores" => [
                    "valor_total_carga" => floatval($mdfe->vCarga),
                    "codigo_unidade_medida_peso_bruto" => $mdfe->mdfe_cunid->valor,
                    "peso_bruto" => floatval($mdfe->qCarga)
                ],
                "lacres" => self::MontaLacres($mdfe),
                "pessoas_autorizadas" => self::MontaXmlAuth($mdfe),
                "produto" => [
                    "tipo_carga" => $mdfe->mdfe_tpcarga->valor,
                    "descricao" => $mdfe->prod_xProd,
                    "codigo_barras" => $mdfe->prod_cEAN,
                    "codigo_ncm" => $mdfe->prod_NCM
                    ],
                "pagamentos" => self::MontaPagamentos($mdfe),   
                "modal_rodoviario" => [
                    "rntrc" => $mdfe->veic_rntrc,
                    "ciot" => [],
                    "contratante" => [],
                    "vale_pedagio" => [],
                    "veiculo" => [
                        "codigo" => strval($mdfe->veic_cint),
                        "placa" => $mdfe->veic_placa,
                        "renavam" => $mdfe->veic_renavam,
                        "tara" => $mdfe->veic_tara,
                        "tipo_rodado" => $mdfe->mdfe_tpRod->valor,
                        "tipo_carroceria" => $mdfe->mdfe_tpcar->valor,
                        "uf" => $mdfe->veic_estado->sigla,
                        "condutores" => self::MontaCondutores($mdfe)
                      ],
                    "reboque" => []
                ]
            ];
            
            $resp = $mdfeapi->cria($payload);
        
            // echo "<pre>";
            // print_r($resp);
            // echo "</pre>";
            
            if (isset($resp->chave))
            {            
                $mdfe->chMDF = $resp->chave;
            }
            
            $mdfe->api_documentos_id = $api_documentos_id->id;
            $mdfe->store();
            
            if ($resp->sucesso) 
            {
                $chave = $resp->chave;

                // autorizado
                funcaoNFE::salvarLog($mdfe->id, "Retorno Enviar (Autorizado): ".self::formataRetorno($resp)); 
                 
                funcaoMDFe::processa_mdfe_autorizado($mdfe, $resp);

            } 
            else if (in_array($resp->codigo, [5001, 5002]))
            {
                // erro nos campos
                funcaoMDFe::salvarLog($mdfe->id, "Retorno Enviar (erro): ".self::formataRetorno($resp));                                 
                                
                funcaoMDFe::processa_mdfe_rejeitado($mdfe, $resp);
            }
            else if ($resp->codigo == 5008 or $resp->codigo >= 7000)
            {
                $chave = $resp->chave;
                // >= 7000 erro de timeout ou de conexão
                // 5008 documento já criado
                //var_dump($resp);
                
                $payload = [
                    'chave' => $chave
                ];
                // recomendamos fazer a consulta pela chave para sincronizar o documento
                $resp = $mdfeapi->consulta($payload);
                
                if ($resp->sucesso)
                {
                    // autorizado
                    funcaoMDFe::salvarLog($mdfe->id, "Retorno Enviar (5008/7000-Autorizado): ".self::formataRetorno($resp)); 
                     
                    funcaoMDFe::processa_mdfe_autorizado($mdfe, $resp);
                } 
                else
                {
                    // rejeição
                    funcaoMDFe::salvarLog($mdfe->id, "Retorno Enviar (5008/7000-Rejeição): ".self::formataRetorno($resp));                                 
                            
                    funcaoMDFe::processa_mdfe_rejeitado($mdfe, $resp);
                }
            }
            else
            {
                // rejeição
                funcaoMDFe::salvarLog($mdfe->id, "Retorno Enviar (Rejeição): ".self::formataRetorno($resp));                                 
                                
                funcaoMDFe::processa_mdfe_rejeitado($mdfe, $resp);
            }

           // $nota_imprimir = Notafiscal::find($nota->id);
            
           // if($nota_imprimir)i
        //    {
                // if($mdfe->codigo_sefaz == 100)
                // {
                    // self::pdfMDFE($mdfe->chave);
                // }
          //  }
        

            return $resp;            


        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
        
    }
    
    public static function MontaInfCarrega($mdfe)
    {
        $array = array();
        
        foreach ($mdfe->getMdfeInfmuncarregas() as $item) 
        {
            $array[] = [
                "nome_municipio" => $item->xMunCarrega,    
                "codigo_municipio" => $item->cMunCarrega,
            ];    
        }
        
        return $array;
    }
    
    public static function MontaInfPercurso($mdfe)
    {
        $array = array();
        
        foreach ($mdfe->getMdfeInfpercursos() as $item) 
        {
            $array[] = [
                "uf" => $item->fk_UFPer->sigla,    
            ];    
        }
        
        return $array;
    }    
    
    public static function MontaInfDescarga($mdfe)
    {
        $array = array();
        
        foreach ($mdfe->getMdfeInfmundescargas() as $item)
        {
            $array[] = [
                "nome_municipio" => $item->xMunDescarga,    
                "codigo_municipio" => $item->cMunDescarga,
                "ctes" => self::MontaCtes($item->id),
                "nfes" => self::MontaNfes($item->id),
                "mdfes" => self::MontaMDFes($item->id),
            ];    
        }
        
        return $array;
    }   
    
    public static function MontaCtes($id)
    {
        $array = array(); 
        
        $descarga = MdfeInfmundescarga::find($id);
        
        foreach($descarga->getMdfeCtess() as $item)
        {
            $array[] = [
                "chave" => $item->chCTe,    
                "segundo_codigo_barras" => $item->SegCodBarra,
                "indicador_reentrega" => funcaoMDFe::ConverteSimNao($item->indReentrega)
            ];    
        }
       
        return $array;
    }   
    
    public static function MontaNfes($id)
    {
        $array = array(); 
        
        $descarga = MdfeInfmundescarga::find($id);
        
        foreach($descarga->getMdfeNfess() as $item)
        {
            $array[] = [
                "chave" => $item->chNFe,    
                "segundo_codigo_barras" => $item->SegCodBarra,
                "indicador_reentrega" => funcaoMDFe::ConverteSimNao($item->indReentrega)
            ];    
        }
       
        return $array;
    }   
    
    public static function MontaMDFes($id)
    {
        $array = array(); 
        
        $descarga = MdfeInfmundescarga::find($id);
        
        foreach($descarga->getMdfeMdfess() as $item)
        {
            $array[] = [
                "chave" => $item->chMDFe,    
                "indicador_reentrega" => funcaoMDFe::ConverteSimNao($item->indReentrega)
            ];    
        }
       
        return $array;
    }    
    
    public static function MontaLacres($mdfe)
    {
        $array = array();
        
        foreach ($mdfe->getMdfeLacress() as $item)
        {
            $array[] = [
                "nLacre" => $item->nLacre,    
            ];    
        }
        
        return $array;
    }    
    
    public static function MontaXmlAuth($mdfe)
    {
        $array = array();
        
        foreach ($mdfe->getMdfeXmlauts() as $item)
        {
            $documento = Funcoes::sonumeros($item->cpfcnpj);
            $cpfcnpj = strlen($documento) == 11 ? "cpf" : "cnpj";
            
            $array[] = [
                $cpfcnpj => $documento    
            ];    
        }
        
        return $array;
    }    
    
    public static function MontaParcelas($mdfe)
    {
        $array = array();
        
        foreach ($mdfe->getMdfeParcelass() as $item)
        {
            $array[] = [
                "numero" => $item->nParcela,
                "data_vencimento" => date("Y-m-d", strtotime($item->dVenc)),
                "valor" => $item->vParcela,
                "tipo_antecipacao" => $item->mdfe_tpantecip->valor
            ];    
        }
        
        return $array;
    }    
    
    public static function MontaComponentes($mdfe)
    {
        $array = array();
        
        foreach ($mdfe->getMdfeComponentess() as $item)
        {
            $array[] = [
                "tipo" => $item->mdfe_tpcomp->valor,
                "valor" => $item->vComp,
                "descricao" => $item->xComp
            ];    
        }
        
        return $array;
    }      
    
    public static function MontaCondutores($mdfe)
    {
        $array = array();
        
        foreach ($mdfe->getMdfeCondutors() as $item)
        {
            $array[] = [
                "nome" => $item->xnome, 
                "cpf" => $item->cpf,
            ];    
        }
        
        return $array;
    }    
    
    public static function MontaPagamentos($mdfe)
    {
        $array = array();
        
        if ($mdfe->pgto_cpfcnpj != '')
        {
        
            $array[] = [
                        "cnpj" => strlen($mdfe->pgto_cpfcnpj) == 11 ? "" : $mdfe->pgto_cpfcnpj,
                        "cpf" => strlen($mdfe->pgto_cpfcnpj) == 11 ? $mdfe->pgto_cpfcnpj : "",                    
                        "nome" => $mdfe->pgto_xNome,
                        "valor_contrato" => floatval($mdfe->pgto_vContrato),
                        "indicador_pagamento" => $mdfe->mdfe_indpag->valor,
                        "valor_adiantamento" => floatval($mdfe->pgto_vAdiant),
                        "indicador_antecipacao_adiantamento" => $mdfe->indAntecipaAdiant,
                        "parcelas" => self::MontaParcelas($mdfe),
                        "componentes" => self::MontaComponentes($mdfe),
                        "banco" => [
                            "codigo_banco" => $mdfe->pgto_codBanco,
                            "codigo_agencia" => $mdfe->pgto_codAgencia,
                            "cnpj" => $mdfe->pgto_CNPJIPEF,
                            "pix" => $mdfe->pgto_PIX
                            ]                         
                       ];         
        }

        return $array;
    }    
    

    public static function formataRetorno($resp)
    {
      $str = "Cod. Sefaz: ".$resp->codigo;
      $str = $str." | Mensagem: ".$resp->mensagem;
      
      if (isset($resp->erros))
      {
          $str = $str." | Erro(s): ";
          
          if (is_array($resp->erros)) 
          {
            foreach ($resp->erros as $erro)
            {   
                $str = $str.'<pre>';
                $str = $str."Campo: ".$erro->campo;
                $str = $str."| Erro: ".$erro->erro;
                $str = $str.'</pre>';
            }           
          }
          else
          {
                $str = $str.'<pre>';
                $str = $str."Erro: ".$resp->erros;
                $str = $str.'</pre>';              
          }
      }
       
      if (isset($resp->status))
      {       
        $str = $str." | Status: ".$resp->status;  
      }

      return $str;

    }    
    
    public static function PDF($mdfe)
    {
        try 
        {
        //   $mdfe = Mdfe::where('chave', '=', $chave)->first();
            
            if($mdfe)
            {
                $params = [
                    'ambiente' => self::retornaTipoAmbiente($mdfe->mdfe_tpamb_id),
                    'token' => self::retornaToken($mdfe),
                    'options' => [
                        'debug' => false,
                                 ]
                ];
                
                $mdfeapi = new Mdfe($params);

                $payload = [
                    'chave' => $mdfe->chMDF
                ];
                
                $resp = $mdfeapi->pdf($payload);
                
                $bin = base64_decode($resp->pdf , true);
                
                $document = 'tmp/' . uniqid() . '.pdf';
    
                file_put_contents($document, $bin);

                return $document;            
            }
            
            return null;
            
            // echo "<pre>";
            // print_r($resp);
            // echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }    
    
    public static function Cancelar($mdfe, $justificativa)
    {
        try 
        {
            // $mdfe = Mdfe::where('chave', '=', $chave)->first();

            if($mdfe)
            {
                $params = [
                    'ambiente' => self::retornaTipoAmbiente($mdfe->mdfe_tpamb_id),
                    'token' => self::retornaToken($mdfe),
                    'options' => [
                        'debug' => false,
                                 ]
                ];
                
                $mdfeapi = new Mdfe($params);

                $payload = [
                    'chave' => $mdfe->chMDF,
                    'justificativa' => $justificativa
                ];
                
                $resp = $mdfeapi->cancela($payload);
                
                funcaoMDFe::salvarLog($mdfe->id, 'Retorno cancelamento: '.self::formataRetorno($resp));
                
                if($resp->codigo != 5001)
                {                   
                    $mdfe->justificativa_cancelamento = $justificativa;
                    $mdfe->codigo_sefaz = $resp->codigo;
                    $mdfe->status = 'Cancelado';
                    $mdfe->mensagem_sefaz = $resp->mensagem;
                    $mdfe->protocolo_cancelamento = $resp->protocolo;
                    $mdfe->datahora_cancelamento = date('Y-m-d H:i:s');
                    $mdfe->xml_cancelamento = $resp->xml;
                    $mdfe->mdfe_situacao_id = MdfeSituacao::Cancelado; 
                    $mdfe->store();
                }
                
                return $resp;
            }
            
                
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    } 
    
    
    public static function XML($mdfe)
    {
        try 
        {
            // $mdfe = Mdfe::where('chave', '=', $chave)->first();
            
            if($mdfe)
            {
                $bin = base64_decode($mdfe->xml , true);
                
                $document = 'tmp/'.$mdfe->chMDF.'.xml';
    
                file_put_contents($document, $bin);
                
                // parent::openFile($document);
            
                // header("Content-type: application/save; charset=iso-8859-1");
                // header('Content-Disposition: attachment; filename="' . basename($document) . '"');
                // header('Expires: 0');
                // header('Pragma: no-cache');
                // readfile($document);
                
                return $document;            
                
            }
            
            // return null;
            
            // echo "<pre>";
            // print_r($resp);
            // echo "</pre>";
        
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }    
    
    public static function Encerra($mdfe)
    {
        try 
        {
            // $mdfe = Mdfe::where('chave', '=', $chave)->first();

            if($mdfe)
            {
                $params = [
                    'ambiente' => self::retornaTipoAmbiente($mdfe->mdfe_tpamb_id),
                    'token' => self::retornaToken($mdfe),
                    'options' => [
                        'debug' => false,
                                 ]
                ];
                
                $mdfeapi = new Mdfe($params);

                $payload = [
                    'chave' => $mdfe->chMDF,
                    'codigo_municipio' => $mdfe->cmunencerramento,
                    'data' => $mdfe->data_encerramento
                ];
                
                $resp = $mdfeapi->encerra($payload);
                
                funcaoMDFe::salvarLog($mdfe->id, 'MDf-e Encerrado: '.self::formataRetorno($resp));

                $mdfe->codigo_sefaz = $resp->codigo;
                $mdfe->status = 'Encerrado';
                $mdfe->mensagem_sefaz = $resp->mensagem;
                $mdfe->protocolo_encerramento = $resp->protocolo;
                $mdfe->datahora_encerramento = date('Y-m-d H:i:s');
                $mdfe->xml_encerramento = $resp->xml;
                $mdfe->mdfe_situacao_id = MdfeSituacao::Encerrado; 
                $mdfe->store();
            }
            
                
        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }     
    
    
    public static function Consulta($mdfe, $apresenta_erro = true)
    {
        try 
        {
            // $mdfe = Mdfe::where('chave', '=', $chave)->first();
            
            if($mdfe)
            {
                $params = [
                    'ambiente' => self::retornaTipoAmbiente($mdfe->mdfe_tpamb_id),
                    'token' => self::retornaToken($mdfe),
                    'options' => [
                        'debug' => false,
                                 ]
                ];
                
                $mdfeapi = new Mdfe($params);
                
                $resp = $mdfeapi->consulta([
                    'chave' => $mdfe->chMDF
                ]);
                
                return $resp;
                
/*                if($resp->codigo == 101)
                {
                    $nota->protocolo_cancelamento = $resp->protocolo;
                    $nota->datahora_cancelamento = $resp->data_hora_evento;
                    $nota->xml_cancelamento = $resp->xml;
                    
                    funcaoNFE::salvarLog($nota->id, 'Retorno consulta (cancelamento): '.self::formataRetorno($resp));
                }
                elseif($resp->codigo == 100)
                {
                    $nota->xml = $resp->xml;
                    $nota->protocolo_envio = $resp->protocolo;
                    $nota->data_hora_envio = $resp->data_hora_evento;
                    
                    funcaoNFE::salvarLog($nota->id, 'Retorno consulta (100): '.self::formataRetorno($resp));
                }
                else
                {

                    //  $retorno = [
                    //     'campo' => $resp->codigo,
                    //     'erro' => $resp->status,
                    //     'descricao' => $resp->mensagem,
                    //     'detalhes' => null
                    //  ];
                     
                    //  $erros = [$retorno];
                     
                    //  TSession::setValue('retorno_envio_nota', $erros);
                    //  TApplication::loadPage('RetornoCloudList', 'onShow');
                    if($apresenta_erro == true)
                    {
                        funcaoNFE::salvarLog($nota->id, 'Retorno consulta (erro): '.self::formataRetorno($resp));
                        
                        new TMessage('error', $resp->codigo." - ".$resp->mensagem);    
                    } 
                    
                }
                
                $nota->codigo_sefaz = $resp->codigo;
                $nota->mensagem = $resp->mensagem;
                $nota->status = $resp->status;
                $nota->store();
              */  
            } 
            
            // echo "<pre>";
            // print_r($resp);
            // echo "</pre>";

        } catch (\Exception $e) {
            echo $e->getMessage();
        }        
    }    

}
